@@ -44,12 +44,17 @@
 #undef XPM_H_NO_X11
 #undef FUNCPROTO
 #undef NARROWPROTO
+#undef HAVE_LIBREADLINE
 #undef ALWAYS_ROOT
 #undef STRICT_WINDOWS
 #undef GENERATING_MAC
 #undef HAVE_OPENTRANSPORT
 #undef PATH_SEPARATOR
 #undef SOCKET_ZERO_ISNT_STDIN
+#undef NONBLOCKING_SOCKETS
+#undef HAVE_FCNTL
+#undef HAVE_IOCTL
+#undef OPTION_FILE_NAME
 
 @BOTTOM@
 
