@@ -145,7 +145,7 @@ static int assess_danger_unit(struct city *pcity, struct unit *punit)
   int v, sailing;
   struct unit_type *ptype;
 
-  if (unit_flag(punit->type, F_SUBMARINE)) return(0);
+  if (unit_flag(punit->type, F_NO_LAND_ATTACK)) return(0);
   sailing = is_sailing_unit(punit);
   if (sailing && !is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN)) return(0);
 
@@ -631,7 +631,7 @@ static void kill_something_with(struct player *pplayer, struct city *pcity,
       is_terrain_near_tile(myunit->x, myunit->y, T_OCEAN))
     harborcity++;
 
-  ferryboat = unit_list_find(&pplayer->units, boatid);
+  ferryboat = player_find_unit_by_id(pplayer, boatid);
   if (ferryboat) boatspeed = (unit_flag(ferryboat->type, F_TRIREME) ? 6 : 12);
   else boatspeed = (get_invention(pplayer, game.rtech.nav) != TECH_KNOWN ? 6 : 12);
 
@@ -743,7 +743,7 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
       n = pdef->type;
       m = get_virtual_defense_power(v, n, x, y);
       if (pdef->veteran) m += m / 2;
-      if (pdef->activity == ACTIVITY_FORTIFY) m += m / 2;
+      if (pdef->activity == ACTIVITY_FORTIFIED) m += m / 2;
 /* attempting to recreate the rounding errors in get_total_defense_power -- Syela */
 
       m *= (myunit->id ? pdef->hp : unit_types[n].hp) * unit_types[n].firepower;
@@ -839,7 +839,7 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
         }
         if (is_sailing_unit(myunit) && improvement_exists(B_PORT)
 	    && !city_got_building(pcity, B_PORT)) {
-	  int tech = get_improvement_type(B_PORT)->tech_requirement;
+	  int tech = get_improvement_type(B_PORT)->tech_req;
           if (get_invention(pplayer, tech) == TECH_KNOWN) {
             choice->choice = B_PORT;
             choice->want = e;
@@ -851,20 +851,29 @@ did I realize the magnitude of my transgression.  How despicable. -- Syela */
   } 
 }
 
+/********************************************************************** 
+Checks if there is a port or a ship being build within d distance.
+***********************************************************************/
 static int port_is_within(struct player *pplayer, int d)
 {
   city_list_iterate(pplayer->cities, pcity)
-    if (city_got_building(pcity, B_PORT) &&
-        warmap.seacost[pcity->x][pcity->y] <= d) return 1;
-    if (!pcity->is_building_unit && pcity->currently_building == B_PORT &&
-        pcity->shield_stock >= improvement_value(B_PORT) &&
-        warmap.seacost[pcity->x][pcity->y] <= d) return 1;
-    if (!player_knows_improvement_tech(pplayer, B_PORT) &&
-        pcity->is_building_unit &&
-        is_water_unit(pcity->currently_building) &&
-        unit_types[pcity->currently_building].attack_strength >
-        unit_types[pcity->currently_building].transport_capacity) return 1;
+    if (warmap.seacost[pcity->x][pcity->y] <= d) {
+      if (city_got_building(pcity, B_PORT))
+	return 1;
+
+      if (!pcity->is_building_unit && pcity->currently_building == B_PORT
+	  && pcity->shield_stock >= improvement_value(B_PORT))
+	return 1;
+
+      if (!player_knows_improvement_tech(pplayer, B_PORT)
+	  && pcity->is_building_unit
+	  && is_water_unit(pcity->currently_building)
+	  && unit_types[pcity->currently_building].attack_strength >
+	  unit_types[pcity->currently_building].transport_capacity)
+	return 1;
+    }
   city_list_iterate_end;
+
   return 0;
 }
 
@@ -874,7 +883,6 @@ static int port_is_within(struct player *pplayer, int d)
     if want is 0 this advisor doesn't want anything
     type = 1 means unit, type = 0 means building
 ***********************************************************************/
-
 void military_advisor_choose_build(struct player *pplayer, struct city *pcity,
 				    struct ai_choice *choice)
 {
