@@ -13,5 +13,10 @@
 #ifndef FC__ADVSPACE_H
 #define FC__ADVSPACE_H
 
-int ai_spaceship_autoplace(struct player *pplayer, struct player_spaceship *ship);
-#endif
+struct player;
+struct player_spaceship;
+
+int ai_spaceship_autoplace(struct player *pplayer,
+			   struct player_spaceship *ship);
+
+#endif  /* FC__ADVSPACE_H */
