@@ -96,6 +96,9 @@ int ai_city_build_peaceful_unit(struct city *pcity)
 }
 #endif /* UNUSED */
 
+/************************************************************************** 
+...
+**************************************************************************/
 static void ai_manage_buildings(struct player *pplayer)
 { /* we have just managed all our cities but not chosen build for them yet */
   struct government *g = get_gov_pplayer(pplayer);
@@ -129,7 +132,7 @@ static void ai_manage_buildings(struct player *pplayer)
   if (g->index != game.default_government
       && g->index != game.government_when_anarchy) {
     for (i = 0; i < B_LAST; i++) {
-      j = improvement_types[i].tech_requirement;
+      j = improvement_types[i].tech_req;
       if (get_invention(pplayer, j) != TECH_KNOWN)
         pplayer->ai.tech_want[j] += values[i];
     }
@@ -171,7 +174,6 @@ static void ai_manage_buildings(struct player *pplayer)
 /************************************************************************** 
 ... change the build order.
 **************************************************************************/
-
 static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
 {
   struct ai_choice bestchoice, curchoice;
@@ -237,7 +239,7 @@ static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
       notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STOPPED,
                    _("Game: The %s have stopped building The %s in %s."),
                    get_nation_name_plural(pplayer->nation),
-                   get_imp_name_ex(pcity, pcity->currently_building),
+                   get_impr_name_ex(pcity, pcity->currently_building),
                    pcity->name);
 
     if (!bestchoice.type && (pcity->is_building_unit ||
@@ -246,7 +248,7 @@ static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
       notify_player_ex(0, pcity->x, pcity->y, E_WONDER_STARTED,
                     _("Game: The %s have started building The %s in %s."),
                     get_nation_name_plural(city_owner(pcity)->nation),
-                    get_imp_name_ex(pcity, bestchoice.choice), pcity->name);
+                    get_impr_name_ex(pcity, bestchoice.choice), pcity->name);
       pcity->currently_building = bestchoice.choice;
       pcity->is_building_unit    = (bestchoice.type > 0);
       generate_warmap(pcity, 0); /* need to establish distance to wondercity */
@@ -274,6 +276,9 @@ static void ai_city_choose_build(struct player *pplayer, struct city *pcity)
 I haven't seen them, but I want to somewhat prepare for them anyway. -- Syela */  
 }
 
+/************************************************************************** 
+...
+**************************************************************************/
 #ifdef GRAVEDANGERWORKS
 static int ai_city_defender_value(struct city *pcity, int a_type, int d_type)
 {
@@ -286,6 +291,9 @@ static int ai_city_defender_value(struct city *pcity, int a_type, int d_type)
 }
 #endif
 
+/************************************************************************** 
+...
+**************************************************************************/
 static void try_to_sell_stuff(struct player *pplayer, struct city *pcity)
 {
   int id;
@@ -298,18 +306,27 @@ static void try_to_sell_stuff(struct player *pplayer, struct city *pcity)
   }
 }
 
+/************************************************************************** 
+...
+**************************************************************************/
 static void ai_new_spend_gold(struct player *pplayer)
 {
   int buycost, id, cost, frugal = 0, trireme=0;
-  int total, build, did_upgrade;
+  int total, build, did_upgrade, reserve;
   struct ai_choice bestchoice;
   struct city *pcity = NULL;
 
+  reserve=ai_gold_reserve(pplayer);
+
   do {
+    if (reserve > pplayer->economic.gold)
+      break;
+
     bestchoice.want = 0;
     bestchoice.type = 0;
     bestchoice.choice = 0;
     city_list_iterate(pplayer->cities, acity)
+      if (acity->anarchy) continue;
       if (acity->ai.choice.want > bestchoice.want && ai_fuzzy(pplayer,1)) {
         bestchoice.choice = acity->ai.choice.choice;
         bestchoice.want = acity->ai.choice.want;
@@ -336,14 +353,11 @@ static void ai_new_spend_gold(struct player *pplayer)
           if (cost < buycost) { /* and the upgrade would be cheaper */
             if (cost < pplayer->economic.gold) { /* let's just upgrade */
               pplayer->economic.gold -= cost;
-              if (punit->hp==get_unit_type(punit->type)->hp)
-                punit->hp=get_unit_type(id)->hp;
               notify_player(pplayer,
-			    _("Game: %s upgraded to %s in %s for %d credits."),
+			    _("Game: %s upgraded to %s in %s for %d gold."),
 			    unit_types[punit->type].name, unit_types[id].name,
 			    pcity->name, cost);
-              punit->type = id;
-              send_unit_info(0, punit, 0);
+	      upgrade_unit(punit, id);
               send_player_info(pplayer, pplayer);
               bestchoice.want = 0; /* no reason to buy unit we already made */
               pcity->ai.choice.want = 0; /* or deal with this city again */
@@ -366,9 +380,10 @@ static void ai_new_spend_gold(struct player *pplayer)
 		   unit_types[bestchoice.choice].name);
               notify_player(pplayer, _("Game: %s disbanded in %s."),
 			    unit_types[punit->type].name, pcity->name);
+	      /* FIXME: should be handled in server... */
               pcity->shield_stock+=(get_unit_type(punit->type)->build_cost/2);
-              send_city_info(pplayer, pcity, 0);
-              wipe_unit(pplayer, punit);
+              send_city_info(pplayer, pcity);
+              wipe_unit_safe(punit, &myiter);
             }
           }
         }
@@ -465,14 +480,11 @@ static void ai_new_spend_gold(struct player *pplayer)
       if (cost < pplayer->economic.gold && ai_fuzzy(pplayer,1)) {
 	/* let's just upgrade */
         pplayer->economic.gold -= cost;
-        if (punit->hp==get_unit_type(punit->type)->hp)
-          punit->hp=get_unit_type(id)->hp;
+	upgrade_unit(punit, id);
         notify_player(pplayer,
-		      _("Game: %s upgraded to %s in %s for %d credits."),
+		      _("Game: %s upgraded to %s in %s for %d gold."),
 		      unit_types[punit->type].name, unit_types[id].name,
 		      pcity->name, cost);
-        punit->type = id;
-        send_unit_info(0, punit, 0);
         send_player_info(pplayer, pplayer);
       } /* end if upgrade */
     unit_list_iterate_end;
@@ -482,7 +494,6 @@ static void ai_new_spend_gold(struct player *pplayer)
 /**************************************************************************
  cities, build order and worker allocation stuff here..
 **************************************************************************/
-
 void ai_manage_cities(struct player *pplayer)
 {
   int i;
@@ -534,7 +545,6 @@ we don't rely on the seamap being current since we will recalculate. -- Syela */
 /**************************************************************************
 ...
 **************************************************************************/
-
 int city_get_buildings(struct city *pcity)
 {
   int b=0;
@@ -547,10 +557,10 @@ int city_get_buildings(struct city *pcity)
   }
   return b;
 }
+
 /**************************************************************************
 ... find a good (bad) tile to remove
 **************************************************************************/
-
 static int worst_elvis_tile(struct city *pcity, int x, int y, int bx, int by,
 			    int foodneed, int prodneed)
 {
@@ -567,7 +577,6 @@ static int worst_elvis_tile(struct city *pcity, int x, int y, int bx, int by,
 /************************************************************************** 
 ...
 **************************************************************************/
-
 static int is_defender_unit(int unit_type) 
 {
   return unit_has_role(unit_type, L_DEFEND_GOOD)
@@ -578,7 +587,6 @@ static int is_defender_unit(int unit_type)
 /************************************************************************** 
 ...
 **************************************************************************/
-
 int city_get_defenders(struct city *pcity)
 {
   int def=0;
@@ -597,7 +605,6 @@ int city_get_defenders(struct city *pcity)
 /************************************************************************** 
 ...
 **************************************************************************/
-
 int city_get_settlers(struct city *pcity)
 {
   int set=0;
@@ -612,7 +619,6 @@ int city_get_settlers(struct city *pcity)
 /************************************************************************** 
 ...
 **************************************************************************/
-
 int ai_in_initial_expand(struct player *pplayer)
 {
   int expand_cities [3] = {3, 5, 7};
@@ -1040,6 +1046,7 @@ void emergency_reallocate_workers(struct player *pplayer, struct city *pcity)
     if (ai_fix_unhappy(acity) && ai_fuzzy(pplayer,1))
       ai_scientists_taxmen(acity);
     freelog(LOG_DEBUG, "Readjusting workers in %s", acity->name);
-    send_city_info(city_owner(acity), acity, 1);
+    /* FIXME: should be handled seperately, and in server... */
+    send_city_info(city_owner(acity), acity);
   city_list_iterate_end;
 }
