@@ -21,6 +21,7 @@
 #include "government.h"
 #include "log.h"
 #include "map.h"
+#include "mem.h"
 #include "packets.h"
 #include "player.h"
 #include "rand.h"
@@ -51,6 +52,126 @@ extern struct move_cost_map warmap;
 static void ai_manage_barbarian_leader(struct player *pplayer, struct unit *leader);
 static void ai_manage_diplomat(struct player *pplayer, struct unit *pdiplomat);
 
+/**************************************************************************
+  Similar to is_my_zoc(), but with some changes:
+  - destination (x0,y0) need not be adjacent?
+  - don't care about some directions?
+  
+  Note this function only makes sense for ground units.
+**************************************************************************/
+static int could_be_my_zoc(struct unit *myunit, int x0, int y0)
+{
+  /* Fix to bizarre did-not-find bug.  Thanks, Katvrr -- Syela */
+  int ax, ay, k;
+  int owner=myunit->owner;
+
+  assert(is_ground_unit(myunit));
+  
+  if (same_pos(x0, y0, myunit->x, myunit->y))
+    return 0; /* can't be my zoc */
+  if (is_tiles_adjacent(x0, y0, myunit->x, myunit->y)
+      && !is_non_allied_unit_tile(map_get_tile(x0, y0), owner))
+    return 0;
+
+  for (k = 0; k < 8; k++) {
+    ax = map_adjust_x(myunit->x + DIR_DX[k]);
+    ay = map_adjust_y(myunit->y + DIR_DY[k]);
+    
+    if (map_get_terrain(ax,ay)!=T_OCEAN
+	&& is_non_allied_unit_tile(map_get_tile(ax,ay),owner))
+      return 0;
+  }
+  
+  return 1;
+  /* return was -1 as a flag value; I've moved -1 value into
+   * could_unit_move_to_tile(), since that's where it becomes
+   * distinct/relevant --dwp
+   */
+}
+
+/**************************************************************************
+  this WAS can_unit_move_to_tile with the notifys removed -- Syela 
+  but is now a little more complicated to allow non-adjacent tiles
+  returns:
+    0 if can't move
+    1 if zoc_ok
+   -1 if zoc could be ok?
+**************************************************************************/
+int could_unit_move_to_tile(struct unit *punit, int x0, int y0, int x, int y)
+{
+  struct tile *ptile,*ptile2;
+  struct city *pcity;
+
+  if(punit->activity!=ACTIVITY_IDLE &&
+     punit->activity!=ACTIVITY_GOTO && !punit->connecting)
+    return 0;
+
+  if(x<0 || x>=map.xsize || y<0 || y>=map.ysize)
+    return 0;
+  
+  if(!is_tiles_adjacent(x0, y0, x, y))
+    return 0; 
+
+  if(is_non_allied_unit_tile(map_get_tile(x,y),punit->owner))
+    return 0;
+
+  ptile=map_get_tile(x, y);
+  ptile2=map_get_tile(x0, y0);
+  if(is_ground_unit(punit)) {
+    /* Check condition 4 */
+    if(ptile->terrain==T_OCEAN &&
+       ground_unit_transporter_capacity(x, y, punit->owner) <= 0)
+	return 0;
+
+    /* Moving from ocean */
+    if(ptile2->terrain==T_OCEAN) {
+      /* Can't attack a city from ocean unless marines */
+      if(!unit_flag(punit->type, F_MARINES)
+	 && is_enemy_city_tile(map_get_tile(x, y), punit->owner)) {
+	return 0;
+      }
+    }
+  } else if(is_sailing_unit(punit)) {
+    if(ptile->terrain!=T_OCEAN && ptile->terrain!=T_UNKNOWN)
+      if(!is_allied_city_tile(map_get_tile(x, y), punit->owner))
+	return 0;
+  } 
+
+  pcity = map_get_city(x, y);
+  if (pcity && !is_allied_city_tile(map_get_tile(x, y), punit->owner)) {
+    if (is_air_unit(punit) || !is_military_unit(punit)) {
+      return 0;  
+    }
+  }
+
+  if (zoc_ok_move_gen(punit, x0, y0, x, y))
+    return 1;
+  else if (could_be_my_zoc(punit, x0, y0))
+    return -1;	/* flag value  */
+  else
+    return 0;
+}
+
+/********************************************************************** 
+  This is a much simplified form of assess_defense (see advmilitary.c),
+  but which doesn't use pcity->ai.wallvalue and just returns a boolean
+  value.  This is for use with "foreign" cities, especially non-ai
+  cities, where ai.wallvalue may be out of date or uninitialized --dwp
+***********************************************************************/
+static int has_defense(struct city *pcity)
+{
+  unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, punit) {
+    if (is_military_unit(punit) && get_defense_power(punit) && punit->hp) {
+      return 1;
+    }
+  }
+  unit_list_iterate_end;
+  return 0;
+}
+
+/********************************************************************** 
+  ...
+***********************************************************************/
 int unit_move_turns(struct unit *punit, int x, int y)
 {
   int m, d;
@@ -78,24 +199,31 @@ int unit_move_turns(struct unit *punit, int x, int y)
 **************************************************************************/
 static int tile_is_accessible(struct unit *punit, int x, int y)
 {
-  int ii[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
-  int jj[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
-  int k;
+  int dir, x1, y1;
   
   if (unit_really_ignores_zoc(punit))  return 1;
   if (is_my_zoc(punit, x, y))          return 1;
-  for (k = 0; k < 8; k++)
-    if (map_get_terrain(x+ii[k], y+jj[k]) != T_OCEAN
-	&& is_my_zoc(punit, x + ii[k], y + jj[k]))  return 1;
+
+  for (dir = 0; dir < 8; dir++) {
+    x1 = x + DIR_DX[dir];
+    y1 = y + DIR_DY[dir];
+    if (!normalize_map_pos(&x1, &y1))
+      continue;
+
+    if (map_get_terrain(x1, y1) != T_OCEAN
+	&& is_my_zoc(punit, x1, y1))
+      return 1;
+  }
+
   return 0;
 }
  
 /**************************************************************************
-... do something sensible with the unit...
+Explores unknown territory, finds huts.
+Returns whether there is any more territory to be explored.
 **************************************************************************/
-
-void ai_manage_explorer(struct player *pplayer, struct unit *punit)
-{ /* this is smarter than the old one, but much slower too. -- Syela */
+int ai_manage_explorer(struct player *pplayer, struct unit *punit)
+{
   int i, j, d, f, x, y, con, dest_x=0, dest_y=0, best, lmt, cur, a, b;
   int ok, ct = punit->moves_left + 3;
   struct city *pcity;
@@ -103,7 +231,7 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
 			 stumble on huts it fuzzily ignored */
 
   if (punit->activity != ACTIVITY_IDLE)
-    handle_unit_activity_request(pplayer, punit, ACTIVITY_IDLE);
+    handle_unit_activity_request(punit, ACTIVITY_IDLE);
 
   x = punit->x; y = punit->y;
   if (is_ground_unit(punit)) con = map_get_continent(x, y);
@@ -141,7 +269,7 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
       punit->goto_dest_y = dest_y;
       set_unit_activity(punit, ACTIVITY_GOTO);
       do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
-      return; /* maybe have moves left but I want to return anyway. */
+      return 1; /* maybe have moves left but I want to return anyway. */
     }
   }
 
@@ -157,10 +285,11 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
     x = punit->x; y = punit->y;
     for (i = -1; i <= 1; i++) {
       for (j = -1; j <= 1; j++) {
+	struct tile *ptile = map_get_tile(x + i, y + j);
         ok = (unit_flag(punit->type, F_TRIREME) ? 0 : 1);
         if (map_get_continent(x + i, y + j) == con &&
-	    !is_enemy_unit_tile(x + i, y + j, punit->owner) &&
-	    !map_get_city(x + i, y + j)) {
+	    !is_non_allied_unit_tile(ptile, punit->owner) &&
+	    (!ptile->city || is_allied_city_tile(ptile, punit->owner))) {
           cur = 0;
           for (a = i - 1; a <= i + 1; a++) {
             for (b = j - 1; b <= j + 1; b++) {
@@ -186,11 +315,11 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
     } /* end i */
     if (best) {
       handle_unit_move_request(pplayer, punit, dest_x, dest_y, FALSE);
-      if(!unit_list_find(&pplayer->units, id)) return; /* died */
+      if(!player_find_unit_by_id(pplayer, id)) return 0; /* died */
     }
     ct--; /* trying to avoid loops */
   }
-  if (!punit->moves_left) return;
+  if (!punit->moves_left) return 1;
 /* END PART TWO */
 
 /* BEGIN PART THREE: Go towards unexplored territory */
@@ -198,9 +327,11 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
   best = 0;
   for (x = 0; x < map.xsize; x++) {
     for (y = 0; y < map.ysize; y++) {
-      if (map_get_continent(x, y) == con && map_get_known(x, y, pplayer) &&
-	  !is_enemy_unit_tile(x, y, punit->owner) && !map_get_city(x, y) &&
-	  tile_is_accessible(punit, x, y)) {
+      struct tile *ptile = map_get_tile(x,y);
+      if (map_get_continent(x, y) == con && map_get_known(x, y, pplayer)
+	  && !is_non_allied_unit_tile(ptile, punit->owner)
+	  && (!ptile->city || is_allied_city_tile(ptile, punit->owner))
+	  && tile_is_accessible(punit, x, y)) {
         cur = 0;
         for (a = -1; a <= 1; a++)
           for (b = -1; b <= 1; b++)
@@ -223,13 +354,13 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
     punit->goto_dest_y = dest_y;
     set_unit_activity(punit, ACTIVITY_GOTO);
     do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
-    return;
+    return 1;
   }
 /* END PART THREE */
 
   freelog(LOG_DEBUG, "%s's %s at (%d,%d) failed to explore.",
 	  pplayer->name, unit_types[punit->type].name, punit->x, punit->y);
-  handle_unit_activity_request(pplayer, punit, ACTIVITY_IDLE);
+  handle_unit_activity_request(punit, ACTIVITY_IDLE);
   if (pplayer->ai.control && is_military_unit(punit)) {
     pcity = find_city_by_id(punit->homecity);
     if (pcity && map_get_continent(pcity->x, pcity->y) == con)
@@ -244,7 +375,7 @@ void ai_manage_explorer(struct player *pplayer, struct unit *punit)
       }
     }
   }
-/*  if (!is_military_unit(punit)) wipe_unit(pplayer, punit); */
+  return 0;
 }
 
 /**************************************************************************
@@ -283,11 +414,11 @@ static int should_unit_change_homecity(struct player *pplayer,
       punit->ai.charge = 0; /* Very important, or will not stay -- Syela */
 /* change homecity to this city */
       unit_list_insert(&pcity->units_supported, punit);
-      if ((pcity=city_list_find_id(&pplayer->cities, punit->homecity)))
+      if ((pcity=player_find_city_by_id(pplayer, punit->homecity)))
             unit_list_unlink(&pcity->units_supported, punit);
 
       punit->homecity = map_get_city(punit->x, punit->y)->id;
-      send_unit_info(pplayer, punit, 0);
+      send_unit_info(pplayer, punit);
 /* homecity changed */
       return(1);
     }
@@ -364,7 +495,7 @@ static void invasion_funct(struct unit *punit, int dest, int n, int which)
     for (i = x - n; i <= x + n; i++) {
       pcity = map_get_city(i, j);
       if (pcity && pcity->owner != punit->owner)
-        if (dest || punit->activity != ACTIVITY_GOTO || !assess_defense(pcity))
+        if (dest || punit->activity != ACTIVITY_GOTO || !has_defense(pcity))
           pcity->ai.invasion |= which;
     }
   }
@@ -436,6 +567,9 @@ static int reinforcements_cost(struct unit *punit, int x, int y)
 }
 #endif
 
+/*************************************************************************
+...
+**************************************************************************/
 static int is_my_turn(struct unit *punit, struct unit *pdef)
 {
   int val = unit_belligerence_primitive(punit), i, j, cur, d;
@@ -466,15 +600,13 @@ explore. It prefers tiles in the following order:
 3. Enemy units weaker than the unit
 4. Land barbarians also like unfrastructure tiles (for later pillage)
 If none of the following is there, nothing is chosen.
-**************************************************************************/
 
-/* work of Syela - mostly to fix the ZOC/goto strangeness */
+work of Syela - mostly to fix the ZOC/goto strangeness
+**************************************************************************/
 static int ai_military_findvictim(struct player *pplayer, struct unit *punit,
 				  int *dest_x, int *dest_y)
 {
-  int xx[3], yy[3], x, y, x1, y1, k;
-  int ii[8] = { 0, 1, 2, 0, 2, 0, 1, 2 };
-  int jj[8] = { 0, 0, 0, 1, 1, 2, 2, 2 };
+  int x, y, x1, y1, k;
   int best = 0, a, b, c, d, e, f;
   struct unit *pdef;
   struct unit *patt;
@@ -493,11 +625,12 @@ static int ai_military_findvictim(struct player *pplayer, struct unit *punit,
     unit_list_iterate_end;
   } /* ferryboats do not attack.  no. -- Syela */
 
-  map_calc_adjacent_xy(x, y, xx, yy);
-
   for (k = 0; k < 8; k++) {
-    x1 = xx[ii[k]];
-    y1 = yy[jj[k]];
+    x1 = x + DIR_DX[k];
+    y1 = y + DIR_DY[k];
+    if (!normalize_map_pos(&x1, &y1))
+      continue;
+
     pdef = get_defender(pplayer, punit, x1, y1);
     if (pdef) {
       patt = get_attacker(pplayer, punit, x1, y1);
@@ -567,9 +700,12 @@ bodyguarding catapult - patt will resolve this bug nicely -- Syela */
   return(best);
 }
 
+/*************************************************************************
+...
+**************************************************************************/
 static void ai_military_bodyguard(struct player *pplayer, struct unit *punit)
 {
-  struct unit *aunit = unit_list_find(&pplayer->units, punit->ai.charge);
+  struct unit *aunit = player_find_unit_by_id(pplayer, punit->ai.charge);
   struct city *acity = find_city_by_id(punit->ai.charge);
   int x, y, id = punit->id, i;
 
@@ -606,32 +742,36 @@ static void ai_military_bodyguard(struct player *pplayer, struct unit *punit)
     aunit->ai.bodyguard = id;
 }
 
+/*************************************************************************
+...
+**************************************************************************/
 int find_beachhead(struct unit *punit, int dest_x, int dest_y, int *x, int *y)
 {
-  int ii[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
-  int jj[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
-  int i, j, k, l, ok, t, fu, best = 0;
+  int x1, y1, dir, l, ok, t, fu, best = 0;
+
+  for (dir = 0; dir < 8; dir++) {
+    x1 = dest_x + DIR_DX[dir];
+    y1 = dest_y + DIR_DY[dir];
+    if (!normalize_map_pos(&x1, &y1))
+      continue;
 
-  for (k = 0; k < 8; k++) {
-    i = map_adjust_x(dest_x + ii[k]);
-    j = map_adjust_y(dest_y + jj[k]);
     ok = 0; fu = 0;
-    t = map_get_terrain(i, j);
-    if (warmap.seacost[i][j] <= 6 * THRESHOLD && t != T_OCEAN) { /* accessible beachhead */
+    t = map_get_terrain(x1, y1);
+    if (warmap.seacost[x1][y1] <= 6 * THRESHOLD && t != T_OCEAN) { /* accessible beachhead */
       for (l = 0; l < 8 && !ok; l++) {
-        if (map_get_terrain(i + ii[l], j + jj[l]) == T_OCEAN) {
+        if (map_get_terrain(x1 + DIR_DX[l], y1 + DIR_DY[l]) == T_OCEAN) {
           fu++;
-          if (is_my_zoc(punit, i + ii[l], j + jj[l])) ok++;
+          if (is_my_zoc(punit, x1 + DIR_DX[l], y1 + DIR_DY[l])) ok++;
         }
       }
       if (ok) { /* accessible beachhead with zoc-ok water tile nearby */
         ok = get_tile_type(t)->defense_bonus;
-	if (map_get_special(i, j) & S_RIVER)
+	if (map_get_special(x1, y1) & S_RIVER)
 	  ok += (ok * terrain_control.river_defense_bonus) / 100;
         if (get_tile_type(t)->movement_cost * 3 <
             unit_types[punit->type].move_rate) ok *= 8;
-        ok += (6 * THRESHOLD - warmap.seacost[i][j]);
-        if (ok > best) { best = ok; *x = i; *y = j; }
+        ok += (6 * THRESHOLD - warmap.seacost[x1][y1]);
+        if (ok > best) { best = ok; *x = x1; *y = y1; }
       }
     }
   }
@@ -644,7 +784,6 @@ find_beachhead() works only when city is not further that 1 tile from
 the sea. But Sea Raiders might want to attack cities inland.
 So this finds the nearest land tile on the same continent as the city.
 **************************************************************************/
-
 static void find_city_beach( struct city *pc, struct unit *punit, int *x, int *y)
 {
   int i, j;
@@ -669,7 +808,6 @@ static void find_city_beach( struct city *pc, struct unit *punit, int *x, int *y
 /**************************************************************************
 ...
 **************************************************************************/
-
 static int ai_military_gothere(struct player *pplayer, struct unit *punit,
 			       int dest_x, int dest_y)
 {
@@ -725,7 +863,7 @@ static int ai_military_gothere(struct player *pplayer, struct unit *punit,
         punit->goto_dest_y = by;
 	set_unit_activity(punit, ACTIVITY_GOTO);
         do_unit_goto(pplayer,punit, GOTO_MOVE_ANY);
-        if (!unit_list_find(&pplayer->units, id)) return(-1); /* died */
+        if (!player_find_unit_by_id(pplayer, id)) return(-1); /* died */
       }
       ptile = map_get_tile(punit->x, punit->y);
       ferryboat = unit_list_find(&ptile->units, punit->ai.ferryboat);
@@ -743,7 +881,7 @@ static int ai_military_gothere(struct player *pplayer, struct unit *punit,
           punit->goto_dest_x = dest_x;
           punit->goto_dest_y = dest_y;
           set_unit_activity(punit, ACTIVITY_SENTRY); /* anything but GOTO!! */
-          if (!is_transporter_with_free_space(pplayer, punit->x, punit->y)) {
+          if (ground_unit_transporter_capacity(punit->x, punit->y, pplayer->player_no) <= 0) {
 	    freelog(LOG_DEBUG, "All aboard!");
 	    /* perhaps this should only require two passengers */
             unit_list_iterate(ptile->units, mypass)
@@ -754,6 +892,7 @@ static int ai_military_gothere(struct player *pplayer, struct unit *punit,
               }
             unit_list_iterate_end; /* passengers are safely stowed away */
             do_unit_goto(pplayer, ferryboat, GOTO_MOVE_ANY);
+	    if (!player_find_unit_by_id(pplayer, boatid)) return(-1); /* died */
             set_unit_activity(punit, ACTIVITY_IDLE);
           } /* else wait, we can GOTO later. */
         }
@@ -764,7 +903,7 @@ static int ai_military_gothere(struct player *pplayer, struct unit *punit,
        (real_map_distance(punit->x, punit->y, dest_x, dest_y) < 3 &&
        (!punit->ai.bodyguard || unit_list_find(&(map_get_tile(punit->x,
        punit->y)->units), punit->ai.bodyguard) || (dcity &&
-       !assess_defense(dcity)))))) {
+       !has_defense(dcity)))))) {
 /* if we are on a boat, disembark only if we are within two tiles of
 our target, and either 1) we don't need a bodyguard, 2) we have a
 bodyguard, or 3) we are going to an empty city.  Previously, cannons
@@ -809,7 +948,7 @@ handled properly.  There should be a way to do it with dir_ok but I'm tired now.
 		    punit->x, punit->y, dest_x, dest_y);
     }
   }
-  if (unit_list_find(&pplayer->units, id)) { /* didn't die */
+  if (player_find_unit_by_id(pplayer, id)) { /* didn't die */
     punit->ai.ai_role = AIUNIT_NONE; /* in case we need to change */
     if (x != punit->x || y != punit->y) return 1; /* moved */
     else return 0; /* didn't move, didn't die */
@@ -817,6 +956,9 @@ handled properly.  There should be a way to do it with dir_ok but I'm tired now.
   return(-1); /* died */
 }
 
+/*************************************************************************
+...
+**************************************************************************/
 int unit_can_defend(int type)
 {
   if (unit_types[type].move_type != LAND_MOVING) return 0; /* temporary kluge */
@@ -840,6 +982,9 @@ int old_unit_can_defend(int type)
 }
 #endif
 
+/*************************************************************************
+...
+**************************************************************************/
 int look_for_charge(struct player *pplayer, struct unit *punit, struct unit **aunit, struct city **acity)
 {
   int d, def, val = 0, u;
@@ -919,7 +1064,7 @@ void ai_military_findjob(struct player *pplayer,struct unit *punit)
   }
 
   if (punit->ai.charge) { /* I am a bodyguard */
-    aunit = unit_list_find(&pplayer->units, punit->ai.charge);
+    aunit = player_find_unit_by_id(pplayer, punit->ai.charge);
     acity = find_city_by_id(punit->ai.charge);
 
     if ((aunit && aunit->ai.bodyguard && unit_vulnerability_virtual(punit) >
@@ -1011,10 +1156,13 @@ void ai_military_gohome(struct player *pplayer,struct unit *punit)
       do_unit_goto(pplayer,punit,GOTO_MOVE_ANY);
     }
   } else {
-    handle_unit_activity_request(pplayer, punit, ACTIVITY_FORTIFY);
+    handle_unit_activity_request(punit, ACTIVITY_FORTIFYING);
   }
 }
 
+/*************************************************************************
+...
+**************************************************************************/
 int find_something_to_kill(struct player *pplayer, struct unit *punit, int *x, int *y)
 {
   int a=0, b, c, d, e, m, n, v, i, f, a0, b0, ab, g;
@@ -1036,6 +1184,8 @@ learning steam engine, even though ironclads would be very useful. -- Syela */
 /* this is horrible, but I need to do something like this somewhere. -- Syela */
   for (i = 0; i < game.nplayers; i++) {
     aplayer = &game.players[i];
+    if (aplayer == pplayer) continue;
+    /* AI will try to conquer only enemy cities. -- Nb */
     city_list_iterate(aplayer->cities, acity)
       city_reinforcements_cost_and_value(acity, punit);
       acity->ai.invasion = 0;
@@ -1091,7 +1241,7 @@ learning steam engine, even though ironclads would be very useful. -- Syela */
 		punit->x, punit->y, a);
 
   if (is_ground_unit(punit)) boatid = find_boat(pplayer, &bx, &by, 2);
-  if (boatid) ferryboat = unit_list_find(&pplayer->units, boatid);
+  if (boatid) ferryboat = player_find_unit_by_id(pplayer, boatid);
   if (ferryboat) really_generate_warmap(map_get_city(ferryboat->x, ferryboat->y),
                        ferryboat, SEA_MOVING);
 
@@ -1323,7 +1473,6 @@ This seems to do the attack. First find victim on the neighbouring tiles.
 If no enemies nearby find_something_to_kill() anywhere else. If there is
 nothing to kill, sailing units go home, others explore.
 **************************************************************************/
-
 void ai_military_attack(struct player *pplayer,struct unit *punit)
 { /* rewritten by Syela - old way was crashy and not smart (nor is this) */
   int dest_x, dest_y; 
@@ -1401,11 +1550,12 @@ void ai_manage_caravan(struct player *pplayer, struct unit *punit)
   if (punit->activity != ACTIVITY_IDLE)
     return;
   if (punit->ai.ai_role == AIUNIT_NONE) {
-    if ((pcity = wonder_on_continent(pplayer, map_get_continent(punit->x, punit->y))) && build_points_left(pcity) > (pcity->shield_surplus*2)) {
+    if ((pcity = wonder_on_continent(pplayer, map_get_continent(punit->x, punit->y))) &&
+	build_points_left(pcity) > (pcity->shield_surplus*2)) {
       if (!same_pos(pcity->x, pcity->y, punit->x, punit->y)) {
         if (!punit->moves_left) return;
 	auto_settler_do_goto(pplayer,punit, pcity->x, pcity->y);
-        handle_unit_activity_request(pplayer, punit, ACTIVITY_IDLE);
+        handle_unit_activity_request(punit, ACTIVITY_IDLE);
       } else {
       /*
        * We really don't want to just drop all caravans in immediately.
@@ -1419,7 +1569,7 @@ void ai_manage_caravan(struct player *pplayer, struct unit *punit)
     }
      else {
        /* A caravan without a home?  Kinda strange, but it might happen.  */
-       pcity=city_list_find_id(&pplayer->cities, punit->homecity);
+       pcity=player_find_city_by_id(pplayer, punit->homecity);
        city_list_iterate(pplayer->cities,pdest)
          if (pcity && can_establish_trade_route(pcity,pdest) &&
             map_same_continent(pcity->x, pcity->y, pdest->x, pdest->y)) {
@@ -1432,7 +1582,7 @@ void ai_manage_caravan(struct player *pplayer, struct unit *punit)
            }
          }
        city_list_iterate_end;
-       pcity=city_list_find_id(&pplayer->cities, best_city);
+       pcity=player_find_city_by_id(pplayer, best_city);
        if (pcity) {
          if (!same_pos(pcity->x, pcity->y, punit->x, punit->y)) {
            if (!punit->moves_left) return;
@@ -1494,9 +1644,9 @@ static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
       x = punit->goto_dest_x; y = punit->goto_dest_y;
       if (find_nearest_friendly_port(punit))
 	do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
-      if (!unit_list_find(&pplayer->units, id)) return; /* oops! */
+      if (!player_find_unit_by_id(pplayer, id)) return; /* oops! */
       punit->goto_dest_x = x; punit->goto_dest_y = y;
-      send_unit_info(pplayer, punit, 0); /* to get the crosshairs right -- Syela */
+      send_unit_info(pplayer, punit); /* to get the crosshairs right -- Syela */
     } else {
       freelog(LOG_DEBUG, "Ferryboat %d@(%d,%d) stalling.",
 		    punit->id, punit->x, punit->y);
@@ -1508,7 +1658,7 @@ static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
 
   /* check if barbarian boat is empty and so not needed - the crew has landed */
   if( is_barbarian(pplayer) && unit_list_size(&map_get_tile(punit->x, punit->y)->units)<2 ) {
-    wipe_unit(pplayer,punit);
+    wipe_unit(punit);
     return;
   }
 
@@ -1531,7 +1681,7 @@ static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
   unit_list_iterate(pplayer->units, aunit)
 /*    if (aunit->ai.ferryboat == punit->id && warmap.seacost[aunit->x][aunit->y] < best) {*/
     if (aunit->ai.ferryboat && warmap.seacost[aunit->x][aunit->y] < best &&
-          !is_transporter_with_free_space(pplayer, aunit->x, aunit->y) &&
+          ground_unit_transporter_capacity(aunit->x, aunit->y, pplayer->player_no) <= 0 &&
 	  ai_fuzzy(pplayer,1)) {
       freelog(LOG_DEBUG, "Found a friend %d@(%d, %d)",
 		    aunit->id, aunit->x, aunit->y);
@@ -1546,7 +1696,7 @@ static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
     punit->goto_dest_y = y;
     set_unit_activity(punit, ACTIVITY_GOTO);
     do_unit_goto(pplayer,punit,GOTO_MOVE_ANY);
-    if ((punit = unit_list_find(&pplayer->units, id)))
+    if ((punit = player_find_unit_by_id(pplayer, id)))
       set_unit_activity(punit, ACTIVITY_IDLE);
     return;
   }
@@ -1574,7 +1724,6 @@ static void ai_manage_ferryboat(struct player *pplayer, struct unit *punit)
 /**************************************************************************
 decides what to do with a military unit.
 **************************************************************************/
-
 void ai_manage_military(struct player *pplayer,struct unit *punit)
 {
   int id;
@@ -1582,7 +1731,7 @@ void ai_manage_military(struct player *pplayer,struct unit *punit)
   id = punit->id;
 
   if (punit->activity != ACTIVITY_IDLE)
-    handle_unit_activity_request(pplayer, punit, ACTIVITY_IDLE);
+    handle_unit_activity_request(punit, ACTIVITY_IDLE);
 
   punit->ai.ai_role = AIUNIT_NONE;
   /* was getting a bad bug where a settlers caused a defender to leave home */
@@ -1622,7 +1771,7 @@ void ai_manage_military(struct player *pplayer,struct unit *punit)
       ai_military_bodyguard(pplayer, punit);
       break;
     case AIUNIT_PILLAGE:
-      handle_unit_activity_request(pplayer, punit, ACTIVITY_PILLAGE);
+      handle_unit_activity_request(punit, ACTIVITY_PILLAGE);
       return; /* when you pillage, you have moves left, avoid later fortify */
       break;
     case AIUNIT_EXPLORE:
@@ -1633,14 +1782,14 @@ void ai_manage_military(struct player *pplayer,struct unit *punit)
   if ((punit = find_unit_by_id(id))) {
     if (punit->activity != ACTIVITY_IDLE &&
         punit->activity != ACTIVITY_GOTO)
-      handle_unit_activity_request(pplayer, punit, ACTIVITY_IDLE); 
+      handle_unit_activity_request(punit, ACTIVITY_IDLE); 
 
     if (punit->moves_left) {
       if (unit_list_find(&(map_get_tile(punit->x, punit->y)->units),
           punit->ai.ferryboat))
-        handle_unit_activity_request(pplayer, punit, ACTIVITY_SENTRY);
+        handle_unit_activity_request(punit, ACTIVITY_SENTRY);
       else 
-        handle_unit_activity_request(pplayer, punit, ACTIVITY_FORTIFY);
+        handle_unit_activity_request(punit, ACTIVITY_FORTIFYING);
     } /* better than doing nothing */
   }
 }
@@ -1650,7 +1799,6 @@ void ai_manage_military(struct player *pplayer,struct unit *punit)
   they are not in cities, and they are far from any enemy units. It is to 
   remove barbarians that do not engage into any activity for a long time.
 **************************************************************************/
-
 static int unit_can_be_retired(struct unit *punit)
 {
   int x, y, x1;
@@ -1660,7 +1808,7 @@ static int unit_can_be_retired(struct unit *punit)
     return 0;
   }
 
-  if( is_friendly_city_tile( punit->x, punit->y, punit->owner) )
+  if (is_allied_city_tile(map_get_tile(punit->x, punit->y), punit->owner))
     return 0;
 
   /* check if there is enemy nearby */
@@ -1669,8 +1817,8 @@ static int unit_can_be_retired(struct unit *punit)
       if( y < 0 || y > map.ysize )
         continue;
       x1 = map_adjust_x(x);
-      if( is_enemy_city_tile(x1,y,punit->owner) ||
-          is_enemy_unit_tile(x1,y,punit->owner) )
+      if (is_enemy_city_tile(map_get_tile(x1, y), punit->owner) ||
+          is_enemy_unit_tile(map_get_tile(x1, y), punit->owner))
         return 0;
     }
 
@@ -1681,15 +1829,13 @@ static int unit_can_be_retired(struct unit *punit)
  manage one unit
  Careful: punit may have been destroyed upon return from this routine!
 **************************************************************************/
-
-static void ai_manage_unit(struct player *pplayer, struct unit *punit,
-			   struct genlist_iterator *iter)
+static void ai_manage_unit(struct player *pplayer, struct unit *punit)
 {
   /* retire useless barbarian units here, before calling the management
      function */
   if( is_barbarian(pplayer) ) {
     if( unit_can_be_retired(punit) && myrand(100) > 90 ) {
-      wipe_unit_safe(0, punit, iter);
+      wipe_unit(punit);
       return;
     }
     if( !is_military_unit(punit)
@@ -1732,29 +1878,59 @@ static void ai_manage_unit(struct player *pplayer, struct unit *punit,
     return;
   }
   /* should never get here */
-  freelog(LOG_DEBUG, "Error: At end of ai_manage_unit().");
 }
 
 /**************************************************************************
  do all the gritty nitty chess like analysis here... (argh)
 **************************************************************************/
-
 void ai_manage_units(struct player *pplayer) 
 {
   static struct timer *t = NULL;      /* alloc once, never free */
+  static int *unitids = NULL;         /* realloc'd, never freed */
+  static int max_units = 0;
+
+  int count, index;
+  struct unit *punit;
+  char *type_name;
 
   t = renew_timer_start(t, TIMER_CPU, TIMER_DEBUG);
 
   freelog(LOG_DEBUG, "Managing units for %s", pplayer->name);
-  unit_list_iterate(pplayer->units, punit) {
-    freelog(LOG_DEBUG, "Managing %s's %s %d@(%d,%d)", pplayer->name,
-		  unit_types[punit->type].name, punit->id, punit->x, punit->y);
-    ai_manage_unit(pplayer, punit, &myiter);
-    /* Note punit might be gone!! */
-    freelog(LOG_DEBUG, "Finished managing %s's unit", pplayer->name);
+
+  count = genlist_size(&(pplayer->units.list));
+  if (count > 0) {
+    if (max_units < count) {
+      max_units = count;
+      unitids = fc_realloc(unitids, max_units * sizeof(int));
+    }
+    index = 0;
+    unit_list_iterate(pplayer->units, punit) {
+      unitids[index++] = punit->id;
+    }
+    unit_list_iterate_end;
+
+    for (index = 0; index < count; index++) {
+      punit = player_find_unit_by_id(pplayer, unitids[index]);
+      if (!punit) {
+	freelog(LOG_DEBUG, "Can't manage %s's dead unit %d", pplayer->name,
+		unitids[index]);
+	continue;
+      }
+      type_name = unit_types[punit->type].name;
+
+      freelog(LOG_DEBUG, "Managing %s's %s %d@(%d,%d)", pplayer->name,
+	      type_name, unitids[index], punit->x, punit->y);
+
+      ai_manage_unit(pplayer, punit);
+      /* Note punit might be gone!! */
+
+      freelog(LOG_DEBUG, "Finished managing %s's %s %d", pplayer->name,
+	      type_name, unitids[index]);
+    }
   }
-  unit_list_iterate_end;
-  freelog(LOG_DEBUG, "Managed units successfully.");
+
+  freelog(LOG_DEBUG, "Managed %s's units successfully.", pplayer->name);
+
   if (timer_in_use(t)) {
     freelog(LOG_VERBOSE, "%s's units consumed %g milliseconds.",
 	    pplayer->name, 1000.0*read_timer_seconds(t));
@@ -1831,7 +2007,7 @@ int is_ai_simple_military(int type)
 {
   return !unit_flag(type, F_NONMIL)
     && !unit_flag(type, F_MISSILE)
-    && !unit_flag(type, F_SUBMARINE) /* not caught by capacity for civ1 */
+    && !unit_flag(type, F_NO_LAND_ATTACK)
     && !(get_unit_type(type)->transport_capacity >= 8);
 }
 
@@ -1855,7 +2031,7 @@ static void ai_manage_diplomat(struct player *pplayer, struct unit *pdiplomat)
   struct player *aplayer = NULL;
 
   if (pdiplomat->activity != ACTIVITY_IDLE)
-    handle_unit_activity_request(pplayer, pdiplomat, ACTIVITY_IDLE);
+    handle_unit_activity_request(pdiplomat, ACTIVITY_IDLE);
 
   pcity = map_get_city(pdiplomat->x, pdiplomat->y);
 
@@ -1936,6 +2112,7 @@ static void ai_manage_diplomat(struct player *pplayer, struct unit *pdiplomat)
     if (!ctarget && aplayer) {
       /* No enemy cities are useful.  Check our own. -AJS */
       city_list_iterate(aplayer->cities, acy)
+	if (continent != map_get_continent(acy->x, acy->y)) continue;
 	if (!count_diplomats_on_tile(acy->x, acy->y)) {
 	  ctarget=acy;
 	  dist=real_map_distance(pdiplomat->x, pdiplomat->y, acy->x, acy->y);
@@ -1981,7 +2158,6 @@ static void ai_manage_diplomat(struct player *pplayer, struct unit *pdiplomat)
 Barbarian leader tries to stack with other barbarian units, and if it's
 not possible it runs away. When on coast, it may disappear with 33% chance.
 **************************************************************************/
-
 static void ai_manage_barbarian_leader(struct player *pplayer, struct unit *leader)
 {
   int con = map_get_continent(leader->x, leader->y), i, x, y, dx, dy,
@@ -1992,7 +2168,7 @@ static void ai_manage_barbarian_leader(struct player *pplayer, struct unit *lead
   if (leader->moves_left == 0 || 
       (map_get_terrain(leader->x, leader->y) != T_OCEAN &&
        unit_list_size(&(map_get_tile(leader->x, leader->y)->units)) > 1) ) {
-      handle_unit_activity_request(pplayer, leader, ACTIVITY_SENTRY);
+      handle_unit_activity_request(leader, ACTIVITY_SENTRY);
       return;
   }
   /* the following takes much CPU time and could be avoided */
@@ -2015,7 +2191,7 @@ static void ai_manage_barbarian_leader(struct player *pplayer, struct unit *lead
       && !same_pos(closest_unit->x, closest_unit->y, leader->x, leader->y)
       && map_same_continent(leader->x, leader->y, closest_unit->x, closest_unit->y)) {
     auto_settler_do_goto(pplayer, leader, closest_unit->x, closest_unit->y);
-    handle_unit_activity_request(pplayer, leader, ACTIVITY_IDLE);
+    handle_unit_activity_request(leader, ACTIVITY_IDLE);
     return; /* sticks better to own units with this -- jk */
   }
 
@@ -2044,13 +2220,13 @@ static void ai_manage_barbarian_leader(struct player *pplayer, struct unit *lead
   if( is_at_coast(leader->x, leader->y) && !leader->fuel) {
     if(myrand(3) == 0) {
       freelog(LOG_DEBUG, "Barbarian leader disappeared at %d %d", leader->x, leader->y);
-      wipe_unit(pplayer,leader);
+      wipe_unit(leader);
       return;
     }
   }
 
   if (closest_unit == NULL) {
-    handle_unit_activity_request(pplayer, leader, ACTIVITY_IDLE);
+    handle_unit_activity_request(leader, ACTIVITY_IDLE);
     freelog(LOG_DEBUG, "Barbarian leader: No enemy.");
     return;
   }
@@ -2078,7 +2254,7 @@ static void ai_manage_barbarian_leader(struct player *pplayer, struct unit *lead
 
     if (same_pos(leader->x, leader->y, safest_x, safest_y)) {
       freelog(LOG_DEBUG, "Barbarian leader reached the safest position.");
-      handle_unit_activity_request(pplayer, leader, ACTIVITY_IDLE);
+      handle_unit_activity_request(leader, ACTIVITY_IDLE);
       return;
     }
 
