@@ -24,11 +24,11 @@ enum ai_unit_task { AIUNIT_NONE, AIUNIT_AUTO_SETTLER, AIUNIT_BUILD_CITY,
                     AIUNIT_PILLAGE };
 
 void ai_manage_units(struct player *pplayer); 
-
+int could_unit_move_to_tile(struct unit *punit, int x0, int y0, int x, int y);
 int look_for_charge(struct player *pplayer, struct unit *punit,
                     struct unit **aunit, struct city **acity);
 
-void ai_manage_explorer(struct player *pplayer,struct unit *punit);
+int ai_manage_explorer(struct player *pplayer,struct unit *punit);
 void ai_manage_military(struct player *pplayer,struct unit *punit);
 void ai_military_findjob(struct player *pplayer,struct unit *punit);
 void ai_military_gohome(struct player *pplayer,struct unit *punit);
