@@ -14,6 +14,7 @@
 #define FC__CLIMISC_H
 
 struct city;
+struct Clause;
 
 void client_remove_player(int plr_id);
 void client_remove_city(struct city *pcity);
@@ -23,5 +24,14 @@ void climap_init_continents(void);
 void climap_update_continents(int x, int y);
 void client_change_all(int x, int y);
 
+void format_duration(char *buffer, int buffer_size, int duration);
+
+void client_diplomacy_clause_string(char *buf, int bufsiz,
+				    struct Clause *pclause);
+
+int client_research_sprite(void);
+int client_warming_sprite(void);
+int client_cooling_sprite(void);
+
 #endif  /* FC__CLIMISC_H */
 
