@@ -79,6 +79,19 @@ struct connection aconnection;
 
 extern char metaserver[];
 
+/**************************************************************************
+...
+**************************************************************************/
+static void close_socket_callback(struct connection *pc)
+{
+  append_output_window(_("Lost connection to server!"));
+  freelog(LOG_NORMAL, "lost connection to server");
+  close(pc->sock);
+  remove_net_input();
+  popdown_races_dialog(); 
+  set_client_state(CLIENT_PRE_GAME_STATE);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -137,6 +150,7 @@ int connect_to_server(char *name, char *hostname, int port,
   /* gui-dependent details now in gui_main.c: */
   add_net_input(aconnection.sock);
   
+  close_socket_set_callback(close_socket_callback);
 
   /* now send join_request package */
 
@@ -170,7 +184,7 @@ void disconnect_from_server(void)
 **************************************************************************/
 void input_from_server(int fid)
 {
-  if(read_socket_data(fid, &aconnection.buffer)>0) {
+  if(read_socket_data(fid, &aconnection.buffer)>=0) {
     int type;
     char *packet;
 
@@ -179,12 +193,7 @@ void input_from_server(int fid)
     }
   }
   else {
-    append_output_window(_("Lost connection to server!"));
-    freelog(LOG_NORMAL, "lost connection to server");
-    close(fid);
-    remove_net_input();
-    popdown_races_dialog(); 
-    set_client_state(CLIENT_PRE_GAME_STATE);
+    close_socket_callback(&aconnection);
   }
 }
 
@@ -316,8 +325,8 @@ struct server_list *create_server_list(char *errbuf, int n_errbuf)
   server_list = fc_malloc(sizeof(struct server_list));
   server_list_init(server_list);
 
-  while(fgets(str,512,f)!=NULL)  {
-    if(!strncmp(str,"<TR BGCOLOR",11))  {
+  while(fgets(str, 512, f) != NULL) {
+    if((0 == strncmp(str, "<TR BGCOLOR",11)) && strchr(str, '\n')) {
       char *name,*port,*version,*status,*players,*metastring;
       char *p;
       struct server *pserver = (struct server*)fc_malloc(sizeof(struct server));
