@@ -19,23 +19,31 @@
 #include "capability.h"
 #include "fcintl.h"
 #include "log.h"
+#include "mem.h"
 
 #include "chatline_g.h"
 #include "citydlg_g.h"
 #include "dialogs_g.h"
+#include "gui_main_g.h"
 #include "mapctrl_g.h"
 #include "mapview_g.h"
 #include "menu_g.h"
 
 #include "civclient.h"
 #include "options.h"
+#include "tilespec.h"
 
 #include "control.h"
 
 extern struct connection aconnection;
 
+
+/* gui-dep code may adjust depending on tile size etc: */
+int num_units_below = MAX_NUM_UNITS_BELOW;
+
+
 /* unit_focus points to the current unit in focus */
-struct unit *punit_focus;
+static struct unit *punit_focus;
 
 /* set high, if the player has selected goto */
 /* actually, set to id of unit goto-ing (id is non-zero) */
@@ -50,8 +58,35 @@ int paradrop_state;
 /* set high if the player has selected connect */
 int connect_state;
 
+/* units involved in current combat */
+static struct unit *punit_attacking;
+static struct unit *punit_defending;
+
+/*************************************************************************/
+
 static struct unit *find_best_focus_candidate(void);
 
+/**************************************************************************
+...
+**************************************************************************/
+void handle_advance_focus(struct packet_generic_integer *packet)
+{
+  struct unit *punit = find_unit_by_id(packet->value);
+  if (punit && punit_focus == punit)
+    advance_unit_focus();
+}
+
+/**************************************************************************
+Center on the focus unit, if off-screen and auto_center_on_unit is true.
+**************************************************************************/
+void auto_center_on_focus_unit(void)
+{
+  if (punit_focus &&
+      auto_center_on_unit &&
+      !tile_visible_and_not_on_border_mapcanvas(punit_focus->x, punit_focus->y))
+    center_tile_mapcanvas(punit_focus->x, punit_focus->y);
+}
+
 /**************************************************************************
 note: punit can be NULL
 We make sure that the previous focus unit is refreshed, if necessary,
@@ -65,14 +100,10 @@ void set_unit_focus(struct unit *punit)
   punit_focus=punit;
 
   if(punit) {
-    raise_unit_top(punit);
-    if(auto_center_on_unit && 
-       !tile_visible_and_not_on_border_mapcanvas(punit->x, punit->y))
-      center_tile_mapcanvas(punit->x, punit->y);
+    auto_center_on_focus_unit();
 
     punit->focus_status=FOCUS_AVAIL;
     refresh_tile_mapcanvas(punit->x, punit->y, 1);
-    put_cross_overlay_tile(punit->x, punit->y);
   }
   
   /* avoid the old focus unit disappearing: */
@@ -97,14 +128,21 @@ void set_unit_focus_no_center(struct unit *punit)
   punit_focus=punit;
 
   if(punit) {
-    raise_unit_top(punit);
     refresh_tile_mapcanvas(punit->x, punit->y, 1);
     punit->focus_status=FOCUS_AVAIL;
   }
 }
 
-
-
+/**************************************************************************
+The only difference is that here we draw the "cross".
+**************************************************************************/
+void set_unit_focus_and_select(struct unit *punit)
+{
+  set_unit_focus(punit);
+  if (punit) {
+    put_cross_overlay_tile(punit->x, punit->y);
+  }
+}
 
 /**************************************************************************
 If there is no unit currently in focus, or if the current unit in
@@ -123,7 +161,6 @@ void update_unit_focus(void)
   }
 }
 
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -162,7 +199,7 @@ void advance_unit_focus(void)
       }
     }
   }
-  
+
   /* We have to do this ourselves, and not rely on set_unit_focus(),
    * because above we change punit_focus directly.
    */
@@ -170,6 +207,12 @@ void advance_unit_focus(void)
     refresh_tile_mapcanvas(punit_old_focus->x, punit_old_focus->y, 1);
 
   set_unit_focus(punit_focus);
+
+  /* Handle auto-turn-done mode:  If a unit was in focus (did move),
+   * but now none are (no more to move), then fake a Turn Done keypress.
+   */
+  if(auto_turn_done && punit_old_focus!=NULL && punit_focus==NULL)
+    key_end_turn();
 }
 
 /**************************************************************************
@@ -207,6 +250,219 @@ static struct unit *find_best_focus_candidate(void)
   return best_candidate;
 }
 
+/**************************************************************************
+Return a pointer to a visible unit, if there is one.
+**************************************************************************/
+struct unit *find_visible_unit(struct tile *ptile)
+{
+  struct unit *panyowned = NULL, *panyother = NULL, *ptptother = NULL;
+
+  /* If no units here, return nothing. */
+  if (unit_list_size(&ptile->units)==0) {
+    return NULL;
+  }
+
+  /* If a unit is attacking we should show that on top */
+  if (punit_attacking && map_get_tile(punit_attacking->x,punit_attacking->y) == ptile) {
+    unit_list_iterate(ptile->units, punit)
+      if(punit == punit_attacking) return punit;
+    unit_list_iterate_end;
+  }
+
+  /* If a unit is defending we should show that on top */
+  if (punit_defending && map_get_tile(punit_defending->x,punit_defending->y) == ptile) {
+    unit_list_iterate(ptile->units, punit)
+      if(punit == punit_defending) return punit;
+    unit_list_iterate_end;
+  }
+
+  /* If the unit in focus is at this tile, show that on top */
+  if (punit_focus && map_get_tile(punit_focus->x,punit_focus->y) == ptile) {
+    unit_list_iterate(ptile->units, punit)
+      if(punit == punit_focus) return punit;
+    unit_list_iterate_end;
+  }
+
+  /* If a city is here, return nothing (unit hidden by city). */
+  if (ptile->city) {
+    return NULL;
+  }
+
+  /* Iterate through the units to find the best one we prioritize this way:
+       1: owned transporter.
+       2: any owned unit
+       3: any transporter
+       4: any unit
+     (always return first in stack). */
+  unit_list_iterate(ptile->units, punit)
+    if (unit_owner(punit) == game.player_ptr) {
+      if (get_transporter_capacity(punit)) {
+	return punit;
+      } else if (!panyowned) {
+	panyowned = punit;
+      }
+    } else if (!ptptother &&
+	       player_can_see_unit(game.player_ptr, punit)) {
+      if (get_transporter_capacity(punit)) {
+	ptptother = punit;
+      } else if (!panyother) {
+	panyother = punit;
+      }
+    }
+  unit_list_iterate_end;
+
+  return (panyowned ? panyowned : (ptptother ? ptptother : panyother));
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void set_units_in_combat(struct unit *pattacker, struct unit *pdefender)
+{
+  punit_attacking = pattacker;
+  punit_defending = pdefender;
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void blink_active_unit(void)
+{
+  static int is_shown;
+  struct unit *punit;
+
+  if((punit=get_unit_in_focus())) {
+    if(is_shown) {
+      set_focus_unit_hidden_state(1);
+      refresh_tile_mapcanvas(punit->x, punit->y, 1);
+      set_focus_unit_hidden_state(0);
+    } else {
+      refresh_tile_mapcanvas(punit->x, punit->y, 1);
+    }
+    is_shown=!is_shown;
+  }
+}
+
+/**************************************************************************
+  Add punit to queue of caravan arrivals, and popup a window for the
+  next arrival in the queue, if there is not already a popup, and
+  re-checking that a popup is appropriate.
+  If punit is NULL, just do for the next arrival in the queue.
+**************************************************************************/
+void process_caravan_arrival(struct unit *punit)
+{
+  static struct genlist arrival_queue;
+  static int is_init_arrival_queue = 0;
+  int *p_id;
+
+  /* arrival_queue is a list of individually malloc-ed ints with
+     punit.id values, for units which have arrived. */
+
+  if (!is_init_arrival_queue) {
+    genlist_init(&arrival_queue);
+    is_init_arrival_queue = 1;
+  }
+
+  if (punit) {
+    p_id = fc_malloc(sizeof(int));
+    *p_id = punit->id;
+    genlist_insert(&arrival_queue, p_id, -1);
+  }
+
+  /* There can only be one dialog at a time: */
+  if (caravan_dialog_is_open()) {
+    return;
+  }
+  
+  while (genlist_size(&arrival_queue)) {
+    int id;
+    
+    p_id = genlist_get(&arrival_queue, 0);
+    genlist_unlink(&arrival_queue, p_id);
+    id = *p_id;
+    free(p_id);
+    punit = player_find_unit_by_id(game.player_ptr, id);
+
+    if (punit && (unit_can_help_build_wonder_here(punit)
+		  || unit_can_est_traderoute_here(punit))
+	&& (!game.player_ptr->ai.control || ai_popup_windows)) {
+      struct city *pcity_dest = map_get_city(punit->x, punit->y);
+      struct city *pcity_homecity = find_city_by_id(punit->homecity);
+      if (pcity_dest && pcity_homecity) {
+	popup_caravan_dialog(punit, pcity_homecity, pcity_dest);
+	return;
+      }
+    }
+  }
+}
+
+/**************************************************************************
+  Add punit/pcity to queue of diplomat arrivals, and popup a window for
+  the next arrival in the queue, if there is not already a popup, and
+  re-checking that a popup is appropriate.
+  If punit is NULL, just do for the next arrival in the queue.
+**************************************************************************/
+void process_diplomat_arrival(struct unit *pdiplomat, int victim_id)
+{
+  static struct genlist arrival_queue;
+  static int is_init_arrival_queue = 0;
+  int *p_ids;
+
+  /* arrival_queue is a list of individually malloc-ed int[2]s with
+     punit.id and pcity.id values, for units which have arrived. */
+
+  if (!is_init_arrival_queue) {
+    genlist_init(&arrival_queue);
+    is_init_arrival_queue = 1;
+  }
+
+  if (pdiplomat && victim_id) {
+    p_ids = fc_malloc(2*sizeof(int));
+    p_ids[0] = pdiplomat->id;
+    p_ids[1] = victim_id;
+    genlist_insert(&arrival_queue, p_ids, -1);
+  }
+
+  /* There can only be one dialog at a time: */
+  if (diplomat_dialog_is_open()) {
+    return;
+  }
+
+  while (genlist_size(&arrival_queue)) {
+    int diplomat_id, victim_id;
+    struct city *pcity;
+    struct unit *punit;
+
+    p_ids = genlist_get(&arrival_queue, 0);
+    genlist_unlink(&arrival_queue, p_ids);
+    diplomat_id = p_ids[0];
+    victim_id = p_ids[1];
+    free(p_ids);
+    pdiplomat = player_find_unit_by_id(game.player_ptr, diplomat_id);
+    pcity = find_city_by_id(victim_id);
+    punit = find_unit_by_id(victim_id);
+
+    if (!pdiplomat || !unit_flag(pdiplomat->type, F_DIPLOMAT))
+      continue;
+
+    if (punit
+	&& is_diplomat_action_available(pdiplomat, DIPLOMAT_ANY_ACTION,
+					punit->x, punit->y)
+	&& diplomat_can_do_action(pdiplomat, DIPLOMAT_ANY_ACTION,
+				  punit->x, punit->y)) {
+      popup_diplomat_dialog(pdiplomat, punit->x, punit->y);
+      return;
+    } else if (pcity
+	       && is_diplomat_action_available(pdiplomat, DIPLOMAT_ANY_ACTION,
+					       pcity->x, pcity->y)
+	       && diplomat_can_do_action(pdiplomat, DIPLOMAT_ANY_ACTION,
+					 pcity->x, pcity->y)) {
+      popup_diplomat_dialog(pdiplomat, pcity->x, pcity->y);
+      return;
+    }
+  }
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -299,35 +555,30 @@ void request_unit_build_city(struct unit *punit)
 }
 
 /**************************************************************************
-...
+  This function is called whenever the player pressed an arrow key.
 
-  No need to do caravan-specific stuff here any more: server does trade
-  routes to enemy cities automatically, and for friendly cities we wait
-  for the unit to enter the city.  --dwp
-  
+  We do NOT take into account that punit might be a caravan or a diplomat
+  trying to move into a city, or a diplomat going into a tile with a unit;
+  the server will catch those cases and send the client a package to pop up
+  a dialog. (the server code has to be there anyway as goto's are entirely
+  in the server)
 **************************************************************************/
 void request_move_unit_direction(struct unit *punit, int dx, int dy)
 {
   int dest_x, dest_y;
   struct unit req_unit;
 
-  dest_x=map_adjust_x(punit->x+dx);
-  dest_y=punit->y+dy;   /* not adjusting on purpose*/   /* why? --dwp */
- 
-  if(unit_flag(punit->type, F_DIPLOMAT) &&
-          is_diplomat_action_available(punit, DIPLOMAT_ANY_ACTION,
-                                       dest_x, dest_y)) {
-    if (diplomat_can_do_action(punit, DIPLOMAT_ANY_ACTION, dest_x, dest_y)) {
-      popup_diplomat_dialog(punit, dest_x, dest_y);
-    } else {
-      append_output_window(_("Game: You don't have enough movement left."));
-    }
+  dest_x = map_adjust_x(punit->x+dx);
+  dest_y = punit->y+dy; /* Not adjusted since if it needed to be adjusted it
+			   would mean that we tried to move off the map... */
+
+  /* Catches attempts to move off map */
+  if (!is_real_tile(dest_x, dest_y))
     return;
-  }
-  
-  req_unit=*punit;
-  req_unit.x=dest_x;
-  req_unit.y=dest_y;
+
+  req_unit = *punit;
+  req_unit.x = dest_x;
+  req_unit.y = dest_y;
   send_move_unit(&req_unit);
 }
 
@@ -356,6 +607,49 @@ void request_new_unit_activity_targeted(struct unit *punit, enum unit_activity a
   send_unit_info(&req_unit);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void request_unit_selected(struct unit *punit)
+{
+  struct packet_unit_info info;
+
+  info.id=punit->id;
+  info.owner=punit->owner;
+  info.x=punit->x;
+  info.y=punit->y;
+  info.homecity=punit->homecity;
+  info.veteran=punit->veteran;
+  info.type=punit->type;
+  info.movesleft=punit->moves_left;
+  info.activity=ACTIVITY_IDLE;
+  info.activity_target=0;
+  info.select_it=1;
+  info.packet_use = UNIT_INFO_IDENTITY;
+
+  send_packet_unit_info(&aconnection, &info);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+void request_unit_sentry(struct unit *punit)
+{
+  if(punit->activity!=ACTIVITY_SENTRY &&
+     can_unit_do_activity(punit, ACTIVITY_SENTRY))
+    request_new_unit_activity(punit, ACTIVITY_SENTRY);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+void request_unit_fortify(struct unit *punit)
+{
+  if(punit->activity!=ACTIVITY_FORTIFYING &&
+     can_unit_do_activity(punit, ACTIVITY_FORTIFYING))
+    request_new_unit_activity(punit, ACTIVITY_FORTIFYING);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -373,7 +667,7 @@ void request_unit_pillage(struct unit *punit)
   what = get_preferred_pillage(pspresent & (~psworking));
   would = what | map_get_infrastructure_prerequisite (what);
 
-  if ((game.civstyle==2) &&
+  if ((game.rgame.pillage_select) &&
       ((pspresent & (~(psworking | would))) != S_NO_SPECIAL)) {
     popup_pillage_dialog(punit, pspresent & (~psworking));
   } else {
@@ -489,7 +783,7 @@ void request_unit_paradrop(struct unit *punit)
     append_output_window(_("Game: Only paratrooper units can do this."));
     return;
   }
-  if(!can_unit_paradropped(punit))
+  if(!can_unit_paradrop(punit))
     return;
 
   paradrop_state=1;
@@ -508,16 +802,36 @@ void request_toggle_map_grid(void)
   update_map_canvas_visible();
 }
 
+/**************************************************************************
+ Toggle display of city names
+**************************************************************************/
+void request_toggle_city_names(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_city_names ^= 1;
+  update_map_canvas_visible();
+}
+
+/**************************************************************************
+ Toggle display of city productions
+**************************************************************************/
+void request_toggle_city_productions(void)
+{
+  if (get_client_state() != CLIENT_GAME_RUNNING_STATE)
+    return;
+
+  draw_city_productions ^= 1;
+  update_map_canvas_visible();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
 void do_move_unit(struct unit *punit, struct packet_unit_info *pinfo)
 {
-  int x, y, was_carried, was_teleported;
-     
-  was_carried=(pinfo->movesleft==punit->moves_left && 
-	       (map_get_terrain(punit->x, punit->y)==T_OCEAN ||
-		map_get_terrain(pinfo->x, pinfo->y)==T_OCEAN));
+  int x, y, was_teleported;
   
   was_teleported=!is_tiles_adjacent(punit->x, punit->y, pinfo->x, pinfo->y);
   x=punit->x;
@@ -527,7 +841,7 @@ void do_move_unit(struct unit *punit, struct packet_unit_info *pinfo)
 
   unit_list_unlink(&map_get_tile(x, y)->units, punit);
 
-  if(!was_carried)
+  if(!pinfo->carried)
     refresh_tile_mapcanvas(x, y, was_teleported);
   
   if(game.player_idx==punit->owner && punit->activity!=ACTIVITY_GOTO && 
@@ -535,15 +849,14 @@ void do_move_unit(struct unit *punit, struct packet_unit_info *pinfo)
      !tile_visible_and_not_on_border_mapcanvas(pinfo->x, pinfo->y))
     center_tile_mapcanvas(pinfo->x, pinfo->y);
 
-  if(!was_carried && !was_teleported) {
+  if(!pinfo->carried && !was_teleported) {
     int dx=pinfo->x - x;
     if(dx>1) dx=-1;
     else if(dx<-1)
       dx=1;
     if(smooth_move_units)
       move_unit_map_canvas(punit, x, y, dx, pinfo->y - punit->y);
-    /*else*/
-      refresh_tile_mapcanvas(x, y, 1);
+    refresh_tile_mapcanvas(x, y, 1);
   }
     
   punit->x=pinfo->x;
@@ -557,14 +870,14 @@ void do_move_unit(struct unit *punit, struct packet_unit_info *pinfo)
       continue;
     for(x=punit->x-2; x<punit->x+3; ++x) { 
       unit_list_iterate(map_get_tile(x, y)->units, pu)
-	if(unit_flag(pu->type, F_SUBMARINE)) {
+	if(unit_flag(pu->type, F_PARTIAL_INVIS)) {
 	  refresh_tile_mapcanvas(map_adjust_x(pu->x), y, 1);
 	}
       unit_list_iterate_end
     }
   }
   
-  if(!was_carried)
+  if(!pinfo->carried && tile_is_known(punit->x,punit->y) == TILE_KNOWN)
     refresh_tile_mapcanvas(punit->x, punit->y, 1);
 
   if(get_unit_in_focus()==punit) update_menus();
@@ -581,7 +894,7 @@ int do_goto(int xtile, int ytile)
   if(goto_state) {
     struct unit *punit;
 
-    if((punit=unit_list_find(&game.player_ptr->units, goto_state))) {
+    if((punit=player_find_unit_by_id(game.player_ptr, goto_state))) {
       struct packet_unit_request req;
       req.unit_id=punit->id;
       req.name[0]='\0';
@@ -614,7 +927,7 @@ void do_map_click(int xtile, int ytile)
   if(goto_state) { 
     struct unit *punit;
 
-    if((punit=unit_list_find(&game.player_ptr->units, goto_state))) {
+    if((punit=player_find_unit_by_id(game.player_ptr, goto_state))) {
       struct packet_unit_request req;
 
       if(paradrop_state) {
@@ -623,6 +936,7 @@ void do_map_click(int xtile, int ytile)
         nuke_state=0;
         paradrop_state=0;
 	connect_state=0;
+	update_unit_info_label(punit);
         return;
       }
 
@@ -671,18 +985,81 @@ void do_map_click(int xtile, int ytile)
     struct unit *punit=unit_list_get(&ptile->units, 0);
     if(game.player_idx==punit->owner) {
       if(can_unit_do_activity(punit, ACTIVITY_IDLE)) {
-        /* struct unit *old_focus=get_unit_in_focus(); */
-        request_new_unit_activity(punit, ACTIVITY_IDLE);
-        /* this is now done in set_unit_focus: --dwp */
-        /* if(old_focus)
-             refresh_tile_mapcanvas(old_focus->x, old_focus->y, 1); */
-        set_unit_focus(punit);
+	request_unit_selected(punit);
       }
     }
   }
   else if(unit_list_size(&ptile->units)>=2) {
-    if(unit_list_get(&ptile->units, 0)->owner==game.player_idx)
-      popup_unit_select_dialog(ptile);
+    unit_list_iterate(ptile->units, punit)
+      if (punit->owner == game.player_idx) {
+	popup_unit_select_dialog(ptile);
+	return;
+      }
+    unit_list_iterate_end;
+  }
+}
+
+/**************************************************************************
+  Update unit icons (and arrow) in the information display, for specified
+  punit as the active unit and other units on the same square.  In practice
+  punit is almost always (or maybe strictly always?) the focus unit.
+  
+  Static vars store some info on current (ie previous) state, to avoid
+  unnecessary redraws; initialise to "flag" values to always redraw first
+  time.  In principle we _might_ need more info (eg ai.control, connecting),
+  but in practice this is enough?
+  
+  Used to store unit_ids for below units, to use for callbacks (now done
+  inside gui-dep set_unit_icon()), but even with ids here they would not
+  be enough information to know whether to redraw -- instead redraw every
+  time.  (Could store enough info to know, but is it worth it?)
+**************************************************************************/
+void update_unit_pix_label(struct unit *punit)
+{
+  static enum unit_activity prev_activity = ACTIVITY_UNKNOWN;
+  static Unit_Type_id prev_unit_type = U_LAST;
+  static int prev_hp = -1;	         /* or could store ihp cf tilespec.c */
+  
+  int i;
+  
+  if(punit) {
+    if(punit->type != prev_unit_type
+       || punit->activity != prev_activity
+       || punit->hp != prev_hp) {
+      set_unit_icon(-1, punit);
+      prev_unit_type = punit->type;
+      prev_activity = punit->activity;
+      prev_hp = punit->hp;
+    }
+
+    i = 0;			/* index into unit_below_canvas */
+    unit_list_iterate(map_get_tile(punit->x, punit->y)->units, aunit) {
+      if (aunit != punit) {
+	if (i < num_units_below) {
+	  set_unit_icon(i, aunit);
+	}
+	i++;
+      }
+    }
+    unit_list_iterate_end;
+    
+    if (i > num_units_below) {
+      set_unit_icons_more_arrow(1);
+    } else {
+      set_unit_icons_more_arrow(0);
+      for(; i < num_units_below; i++) {
+	set_unit_icon(i, NULL);
+      }
+    }
+  }
+  else {
+    prev_unit_type = U_LAST;
+    prev_activity = ACTIVITY_UNKNOWN;
+    prev_hp = -1;
+    for(i=-1; i<num_units_below; i++) {
+      set_unit_icon(i, NULL);
+    }
+    set_unit_icons_more_arrow(0);
   }
 }
 
@@ -755,7 +1132,7 @@ void key_cancel_action(void)
   if(goto_state) {
     struct unit *punit;
 
-    punit=unit_list_find(&game.player_ptr->units, goto_state);
+    punit=player_find_unit_by_id(game.player_ptr, goto_state);
 
     goto_state=0;
     nuke_state=0;
@@ -766,6 +1143,22 @@ void key_cancel_action(void)
   }
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void key_city_names_toggle(void)
+{
+  request_toggle_city_names();
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void key_city_productions_toggle(void)
+{
+  request_toggle_city_productions();
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -945,14 +1338,24 @@ void key_unit_done(void)
   request_unit_move_done();
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void key_unit_fallout(void)
+{
+  if(get_unit_in_focus())
+    if(can_unit_do_activity(punit_focus, ACTIVITY_FALLOUT))
+      request_new_unit_activity(punit_focus, ACTIVITY_FALLOUT);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
 void key_unit_fortify(void)
 {
   if(get_unit_in_focus())
-    if(can_unit_do_activity(punit_focus, ACTIVITY_FORTIFY))
-      request_new_unit_activity(punit_focus, ACTIVITY_FORTIFY);
+    if(can_unit_do_activity(punit_focus, ACTIVITY_FORTIFYING))
+      request_new_unit_activity(punit_focus, ACTIVITY_FORTIFYING);
 }
 
 /**************************************************************************
@@ -1017,7 +1420,7 @@ void key_unit_nuke(void)
 void key_unit_paradrop(void)
 {
   if(get_unit_in_focus())
-    if(can_unit_paradropped(punit_focus))
+    if(can_unit_paradrop(punit_focus))
       request_unit_paradrop(punit_focus);
 }
 
