@@ -22,12 +22,16 @@ void do_unit_connect(struct unit *punit, int x, int y);
 int do_goto(int xtile, int ytile);
 void do_map_click(int xtile, int ytile);
 
+void handle_advance_focus(struct packet_generic_integer *packet);
 void request_center_focus_unit(void);
 void request_move_unit_direction(struct unit *punit, int dx, int dy); /* used by key_xxx */
 void request_new_unit_activity(struct unit *punit, enum unit_activity act);
 void request_new_unit_activity_targeted(struct unit *punit, enum unit_activity act,
 					int tgt);
+void request_unit_selected(struct unit *punit);
 void request_unit_pillage(struct unit *punit);
+void request_unit_sentry(struct unit *punit);
+void request_unit_fortify(struct unit *punit);
 void request_unit_auto(struct unit *punit);
 void request_unit_build_city(struct unit *punit);
 void request_unit_caravan_action(struct unit *punit, enum packet_type action);
@@ -43,16 +47,29 @@ void request_unit_upgrade(struct unit *punit);
 void request_unit_wait(struct unit *punit);
 void request_unit_wakeup(struct unit *punit);
 void request_toggle_map_grid(void);
+void request_toggle_city_names(void);
+void request_toggle_city_productions(void);
 
 void wakeup_sentried_units(int x, int y);
 
+void auto_center_on_focus_unit(void);
 void advance_unit_focus(void);
 struct unit *get_unit_in_focus(void);
 void set_unit_focus(struct unit *punit);
 void set_unit_focus_no_center(struct unit *punit);
+void set_unit_focus_and_select(struct unit *punit);
 void update_unit_focus(void);
+struct unit *find_visible_unit(struct tile *ptile);
+void set_units_in_combat(struct unit *pattacker, struct unit *pdefender);
+void blink_active_unit(void);
+void update_unit_pix_label(struct unit *punit);
+
+void process_caravan_arrival(struct unit *punit);
+void process_diplomat_arrival(struct unit *punit, int victim_id);
 
 void key_cancel_action(void);
+void key_city_names_toggle(void);
+void key_city_productions_toggle(void);
 void key_end_turn(void);
 void key_map_grid_toggle(void);
 void key_move_north(void);
@@ -72,6 +89,7 @@ void key_unit_build_wonder(void);
 void key_unit_connect(void);
 void key_unit_disband(void);
 void key_unit_done(void);
+void key_unit_fallout(void);
 void key_unit_fortify(void);
 void key_unit_fortress(void);
 void key_unit_goto(void);
@@ -90,5 +108,10 @@ void key_unit_unload(void);
 void key_unit_wait(void);
 void key_unit_wakeup_others(void);
 
+/* don't change this unless you also put more entries in data/Freeciv */
+#define MAX_NUM_UNITS_BELOW 4
+
+extern int num_units_below;
+
 
 #endif  /* FC__CONTROL_H */
