@@ -58,8 +58,6 @@ extern int map_view_x0, map_view_y0;
 extern GdkGC *fill_bg_gc;
 extern GdkGC *civ_gc;
 
-extern GtkStyle *city_dialog_style;
-
 #define NUM_UNITS_SHOWN  12
 #define NUM_CITIZENS_SHOWN 25
 
@@ -89,11 +87,11 @@ struct city_dialog {
   GtkWidget *present_unit_label;
   GtkWidget *present_unit_boxes		[NUM_UNITS_SHOWN];
   GtkWidget *present_unit_pixmaps	[NUM_UNITS_SHOWN];
-  GtkWidget *change_list;
+  GtkWidget *change_shell, *change_list;
   GtkWidget *rename_input;
   GtkWidget *worklist_shell;
   
-  enum improvement_type_id sell_id;
+  Impr_Type_id sell_id;
   
   int citizen_type[NUM_CITIZENS_SHOWN];
   int support_unit_ids[NUM_UNITS_SHOWN];
@@ -101,8 +99,6 @@ struct city_dialog {
   char improvlist_names[B_LAST+1][64];
   char *improvlist_names_ptrs[B_LAST+1];
   
-  char *change_list_names_ptrs[B_LAST+1+U_LAST+1+1];
-  char change_list_names[B_LAST+1+U_LAST+1][200];
   int change_list_ids[B_LAST+1+U_LAST+1];
   int change_list_num_improvements;
 
@@ -234,7 +230,7 @@ void refresh_unit_city_dialogs(struct unit *punit)
   struct city *pcity_sup, *pcity_pre;
   struct city_dialog *pdialog;
 
-  pcity_sup=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+  pcity_sup=player_find_city_by_id(game.player_ptr, punit->homecity);
   pcity_pre=map_get_city(punit->x, punit->y);
   
   if(pcity_sup && (pdialog=get_city_dialog(pcity_sup)))
@@ -311,17 +307,19 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 {
   int i;
   struct city_dialog *pdialog;
-  GtkWidget *box, *table, *frame, *vbox, *scrolled, *hbox;
+  GtkWidget *box, *frame, *vbox, *scrolled, *hbox;
   GtkAccelGroup *accel=gtk_accel_group_new();
 
   pdialog=fc_malloc(sizeof(struct city_dialog));
   pdialog->pcity=pcity;
+  pdialog->change_shell=0;
 
   pdialog->shell=gtk_dialog_new();
   gtk_signal_connect(GTK_OBJECT(pdialog->shell),"delete_event",
 	GTK_SIGNAL_FUNC(city_dialog_delete_callback),(gpointer)pdialog);
   gtk_window_set_position(GTK_WINDOW(pdialog->shell), GTK_WIN_POS_MOUSE);
   gtk_accel_group_attach(accel, GTK_OBJECT(pdialog->shell));
+  gtk_widget_set_name(pdialog->shell, "Freeciv");
 
   gtk_window_set_title(GTK_WINDOW(pdialog->shell), pcity->name);
 
@@ -370,25 +368,25 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
 
   pdialog->production_label=gtk_label_new("\n\n");
   gtk_box_pack_start(GTK_BOX(box),pdialog->production_label, TRUE, TRUE, 0);
-  gtk_widget_set_style (pdialog->production_label, city_dialog_style);
+  gtk_widget_set_name(pdialog->production_label, "city label");
   gtk_label_set_justify (GTK_LABEL (pdialog->production_label),
 			 GTK_JUSTIFY_LEFT);
 
   pdialog->output_label=gtk_label_new("\n\n");
   gtk_box_pack_start(GTK_BOX(box),pdialog->output_label, TRUE, TRUE, 0);
-  gtk_widget_set_style (pdialog->output_label, city_dialog_style);
+  gtk_widget_set_name(pdialog->output_label, "city label");
   gtk_label_set_justify (GTK_LABEL (pdialog->output_label),
 			 GTK_JUSTIFY_LEFT);
 
   pdialog->storage_label=gtk_label_new("");
   gtk_box_pack_start(GTK_BOX(box),pdialog->storage_label, TRUE, TRUE, 0);
-  gtk_widget_set_style (pdialog->storage_label, city_dialog_style);
+  gtk_widget_set_name(pdialog->storage_label, "city label");
   gtk_label_set_justify (GTK_LABEL (pdialog->storage_label),
 			 GTK_JUSTIFY_LEFT);
 
   pdialog->pollution_label=gtk_label_new("");
   gtk_box_pack_start(GTK_BOX(box),pdialog->pollution_label, TRUE, TRUE, 0);
-  gtk_widget_set_style (pdialog->pollution_label, city_dialog_style);
+  gtk_widget_set_name(pdialog->pollution_label, "city label");
   gtk_label_set_justify (GTK_LABEL (pdialog->pollution_label),
 			 GTK_JUSTIFY_LEFT);
   
@@ -416,17 +414,20 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
   pdialog->building_label=gtk_frame_new(NULL);
   gtk_box_pack_start(GTK_BOX(box), pdialog->building_label, FALSE, FALSE, 0);
   
-  table=gtk_table_new(3, 1, FALSE);
-  gtk_container_add(GTK_CONTAINER(pdialog->building_label), table);
+  hbox=gtk_hbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(pdialog->building_label), hbox);
 
-  pdialog->progress_label=gtk_label_new("");
-  gtk_table_attach_defaults(GTK_TABLE(table), pdialog->progress_label,0,1,1,2);
+  pdialog->progress_label=gtk_progress_bar_new();
+  gtk_progress_bar_set_bar_style(GTK_PROGRESS_BAR (pdialog->progress_label),
+				 GTK_PROGRESS_CONTINUOUS);
+  gtk_progress_set_show_text(GTK_PROGRESS (pdialog->progress_label), TRUE);
+  gtk_box_pack_start(GTK_BOX(hbox), pdialog->progress_label, FALSE, FALSE, 1);
 
   pdialog->buy_command=gtk_accelbutton_new(_("_Buy"), accel);
-  gtk_table_attach_defaults(GTK_TABLE(table), pdialog->buy_command,1,2,1,2);
+  gtk_box_pack_start(GTK_BOX(hbox), pdialog->buy_command, TRUE, TRUE, 0);
 
   pdialog->change_command=gtk_accelbutton_new(_("_Change"), accel);
-  gtk_table_attach_defaults(GTK_TABLE(table), pdialog->change_command,2,3,1,2);
+  gtk_box_pack_start(GTK_BOX(hbox), pdialog->change_command, TRUE, TRUE, 0);
   
   pdialog->improvement_list=gtk_clist_new(1);
   gtk_clist_set_column_width(GTK_CLIST(pdialog->improvement_list), 0,
@@ -578,7 +579,7 @@ struct city_dialog *create_city_dialog(struct city *pcity, int make_modal)
   genlist_insert(&dialog_list, pdialog, 0);
 
   for(i=0; i<B_LAST+1; i++)
-    pdialog->improvlist_names_ptrs[i]=0;
+      pdialog->improvlist_names_ptrs[i]=0;
 
   for(i=0; i<NUM_CITIZENS_SHOWN; i++)
     pdialog->citizen_type[i]=-1;
@@ -613,14 +614,17 @@ void activate_callback(GtkWidget *w, gpointer data)
   struct city_dialog *pdialog = (struct city_dialog *)data;
   int x=pdialog->pcity->x,y=pdialog->pcity->y;
   struct unit_list *punit_list = &map_get_tile(x,y)->units;
+  struct unit *pmyunit = NULL;
 
   if(unit_list_size(punit_list))  {
     unit_list_iterate((*punit_list), punit) {
       if(game.player_idx==punit->owner) {
-       request_new_unit_activity(punit, ACTIVITY_IDLE);
+	pmyunit = punit;
+	request_new_unit_activity(punit, ACTIVITY_IDLE);
       }
     } unit_list_iterate_end;
-    set_unit_focus(unit_list_get(punit_list, 0));
+    if (pmyunit)
+      set_unit_focus(pmyunit);
   }
 }
 
@@ -664,7 +668,7 @@ static void present_units_activate_callback(GtkWidget *w, gpointer data)
 {
   struct unit *punit;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)))
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
     activate_unit(punit);
   destroy_message_dialog(w);
 }
@@ -680,7 +684,7 @@ static void present_units_activate_close_callback(GtkWidget *w, gpointer data)
 
   destroy_message_dialog(w);
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)))  {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data))) {
     activate_unit(punit);
     if((pcity=map_get_city(punit->x, punit->y)))
       if((pdialog=get_city_dialog(pcity)))
@@ -688,6 +692,30 @@ static void present_units_activate_close_callback(GtkWidget *w, gpointer data)
   }
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static void present_units_sentry_callback(GtkWidget *w, gpointer data)
+{
+  struct unit *punit;
+
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
+    request_unit_sentry(punit);
+  destroy_message_dialog(w);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+static void present_units_fortify_callback(GtkWidget *w, gpointer data)
+{
+  struct unit *punit;
+
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
+    request_unit_fortify(punit);
+  destroy_message_dialog(w);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -699,9 +727,9 @@ static void supported_units_activate_close_callback(GtkWidget *w, gpointer data)
 
   destroy_message_dialog(w);
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)))  {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data))) {
     activate_unit(punit);
-    if((pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity)))
+    if((pcity=player_find_city_by_id(game.player_ptr, punit->homecity)))
       if((pdialog=get_city_dialog(pcity)))
 	close_city_dialog(pdialog);
   }
@@ -714,7 +742,7 @@ static void present_units_disband_callback(GtkWidget *w, gpointer data)
 {
   struct unit *punit;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)))
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
     request_unit_disband(punit);
 
   destroy_message_dialog(w);
@@ -727,7 +755,7 @@ static void present_units_homecity_callback(GtkWidget *w, gpointer data)
 {
   struct unit *punit;
   
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)))
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)))
     request_unit_change_homecity(punit);
 
   destroy_message_dialog(w);
@@ -750,11 +778,12 @@ gint p_units_middle_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
   struct city *pcity;
   struct city_dialog *pdialog;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)) &&
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)) &&
      (pcity=map_get_city(punit->x, punit->y)) &&
-     (pdialog=get_city_dialog(pcity)) && ev->button==2) {
+     (pdialog=get_city_dialog(pcity)) && (ev->button==2 || ev->button==3)) {
     activate_unit(punit);
-    close_city_dialog(pdialog);
+    if (ev->button==2)
+      close_city_dialog(pdialog);
   }
 
   return TRUE;
@@ -769,18 +798,19 @@ gint s_units_middle_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
   struct city *pcity;
   struct city_dialog *pdialog;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)) &&
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)) &&
      (pcity=find_city_by_id(punit->homecity)) &&
-     (pdialog=get_city_dialog(pcity)) && ev->button==2) {
+     (pdialog=get_city_dialog(pcity)) && (ev->button==2 || ev->button==3)) {
     activate_unit(punit);
-    close_city_dialog(pdialog);
+    if (ev->button==2)
+      close_city_dialog(pdialog);
   }
 
   return TRUE;
 }
 
 /****************************************************************
-...
+Pop-up menu to change attributes of units, ex. change homecity.
 *****************************************************************/
 gint present_units_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
 {
@@ -789,31 +819,46 @@ gint present_units_callback(GtkWidget *w, GdkEventButton *ev, gpointer data)
   struct city_dialog *pdialog;
   GtkWidget *wd;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)) &&
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)) &&
      (pcity=map_get_city(punit->x, punit->y)) &&
      (pdialog=get_city_dialog(pcity))) {
 
-    if (ev->button==2)
+    if (ev->button==2 || ev->button==3)
       return FALSE;
 
     wd=popup_message_dialog(pdialog->shell, 
 			   /*"presentunitsdialog"*/_("Unit Commands"), 
 			   unit_description(punit),
-			   _("Activate unit"),
+			   _("_Activate unit"),
 			     present_units_activate_callback, punit->id,
-			   _("Activate unit, close dialog"),
+			   _("Activate unit, _close dialog"),
 			     present_units_activate_close_callback, punit->id,
-			   _("Disband unit"),
+			   _("_Sentry unit"),
+			     present_units_sentry_callback, punit->id,
+			   _("_Fortify unit"),
+			     present_units_fortify_callback, punit->id,
+			   _("_Disband unit"),
 			     present_units_disband_callback, punit->id,
-			   _("Make new homecity"),
+			   _("Make new _homecity"),
 			     present_units_homecity_callback, punit->id,
-			   _("Upgrade unit"),
+			   _("_Upgrade unit"),
 			     upgrade_callback, punit->id,
-			   _("Cancel"),
+			   _("_Cancel"),
 			     present_units_cancel_callback, 0, 
 			   NULL);
+    if (punit->activity == ACTIVITY_SENTRY
+	|| !can_unit_do_activity(punit, ACTIVITY_SENTRY)) {
+      message_dialog_button_set_sensitive(wd, "button2", FALSE);
+    }
+    if (punit->activity == ACTIVITY_FORTIFYING
+	|| !can_unit_do_activity(punit, ACTIVITY_FORTIFYING)) {
+      message_dialog_button_set_sensitive(wd, "button3", FALSE);
+    }
+    if (punit->homecity == pcity->id) {
+      message_dialog_button_set_sensitive(wd, "button5", FALSE);
+    }
     if (can_upgrade_unittype(game.player_ptr,punit->type) == -1) {
-      message_dialog_button_set_sensitive(wd, "button4", FALSE);
+      message_dialog_button_set_sensitive(wd, "button6", FALSE);
     }
   }
   return TRUE;
@@ -960,6 +1005,7 @@ void city_dialog_update_building(struct city_dialog *pdialog)
   char buf[32], buf2[64];
   int turns;
   struct city *pcity=pdialog->pcity;
+  gfloat pct;
   
   gtk_widget_set_sensitive(pdialog->buy_command, !pcity->did_buy);
   gtk_widget_set_sensitive(pdialog->sell_command, !pcity->did_sell);
@@ -973,6 +1019,9 @@ void city_dialog_update_building(struct city_dialog *pdialog)
  		get_unit_type(pcity->currently_building)->build_cost,
  		turns);
     sz_strlcpy(buf2, get_unit_type(pcity->currently_building)->name);
+    pct = (gfloat)pcity->shield_stock /
+	  (get_unit_type(pcity->currently_building)->build_cost + 0.1);
+    pct = CLAMP(pct, 0.0, 1.0);		
   } else {
     if(pcity->currently_building==B_CAPITAL)  {
       /* Capitalization is special, you can't buy it or finish making it */
@@ -981,6 +1030,7 @@ void city_dialog_update_building(struct city_dialog *pdialog)
 		    _("%3d/XXX XXX turns"),
 		  pcity->shield_stock);
       gtk_widget_set_sensitive(pdialog->buy_command, FALSE);
+      pct = 1.0;
     } else {
       turns = city_turns_to_build (pcity, pcity->currently_building, FALSE);
       my_snprintf(buf, sizeof(buf),
@@ -989,12 +1039,17 @@ void city_dialog_update_building(struct city_dialog *pdialog)
 		  pcity->shield_stock, 
 		  get_improvement_type(pcity->currently_building)->build_cost,
 		  turns);
+      pct = (gfloat)pcity->shield_stock /
+	    (get_improvement_type(pcity->currently_building)->build_cost + 0.1);
+      pct = CLAMP(pct, 0.0, 1.0);		
     }
-    sz_strlcpy(buf2, get_imp_name_ex(pcity, pcity->currently_building));
+    sz_strlcpy(buf2, get_impr_name_ex(pcity, pcity->currently_building));
   }    
-    
+
   gtk_frame_set_label(GTK_FRAME(pdialog->building_label), buf2);
-  gtk_set_label(pdialog->progress_label, buf);
+  gtk_progress_set_percentage(GTK_PROGRESS (pdialog->progress_label), pct);
+  gtk_progress_set_format_string(GTK_PROGRESS (pdialog->progress_label), buf);
+
   gtk_set_label(pdialog->worklist_label,
 		worklist_is_empty(pcity->worklist) ?
 		_("(list empty)") :
@@ -1173,23 +1228,23 @@ static gint support_units_callback(GtkWidget *w, GdkEventButton *ev,
   struct city *pcity;
   struct city_dialog *pdialog;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data)) &&
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data)) &&
      (pcity=find_city_by_id(punit->homecity)) &&
      (pdialog=get_city_dialog(pcity))) {
 
-    if (ev->button==2)
+    if (ev->button==2 || ev->button==3)
       return FALSE;
 
     popup_message_dialog(pdialog->shell,
     	   /*"supportunitsdialog"*/ _("Unit Commands"), 
     	   unit_description(punit),
-    	   _("Activate unit"),
+    	   _("_Activate unit"),
     	     present_units_activate_callback, punit->id,
-    	   _("Activate unit, close dialog"),
+    	   _("Activate unit, _close dialog"),
     	     supported_units_activate_close_callback, punit->id, /* act+c */
-    	   _("Disband unit"),
+    	   _("_Disband unit"),
     	     present_units_disband_callback, punit->id,
-    	   _("Cancel"),
+    	   _("_Cancel"),
     	     present_units_cancel_callback, 0, 0);
     }
   return TRUE;
@@ -1203,6 +1258,7 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
 					int unitid)
 {
   int i;
+  struct unit_list *plist;
   struct genlist_iterator myiter;
   struct unit *punit;
 
@@ -1213,8 +1269,14 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
     if(i==NUM_UNITS_SHOWN)
       unitid=0;
   }
-  
-  genlist_iterator_init(&myiter, &pdialog->pcity->units_supported.list, 0);
+
+  if(pdialog->pcity->owner != game.player_idx) {
+    plist = &(pdialog->pcity->info_units_supported);
+  } else {
+    plist = &(pdialog->pcity->units_supported);
+  }
+
+  genlist_iterator_init(&myiter, &(plist->list), 0);
 
   for(i=0; i<NUM_UNITS_SHOWN&&ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter),i++) {
     punit=(struct unit*)ITERATOR_PTR(myiter);
@@ -1256,6 +1318,7 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
 void city_dialog_update_present_units(struct city_dialog *pdialog, int unitid)
 {
   int i;
+  struct unit_list *plist;
   struct genlist_iterator myiter;
   struct unit *punit;
   
@@ -1267,9 +1330,14 @@ void city_dialog_update_present_units(struct city_dialog *pdialog, int unitid)
       unitid=0;
   }
 
-  genlist_iterator_init(&myiter, 
-	&map_get_tile(pdialog->pcity->x, pdialog->pcity->y)->units.list, 0);
-  
+  if(pdialog->pcity->owner != game.player_idx) {
+    plist = &(pdialog->pcity->info_units_present);
+  } else {
+    plist = &(map_get_tile(pdialog->pcity->x, pdialog->pcity->y)->units);
+  }
+
+  genlist_iterator_init(&myiter, &(plist->list), 0);
+
   for(i=0; i<NUM_UNITS_SHOWN&&ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter),i++) {
     punit=(struct unit*)ITERATOR_PTR(myiter);
     
@@ -1331,10 +1399,10 @@ void city_dialog_update_improvement_list(struct city_dialog *pdialog)
     if(pdialog->pcity->improvements[i]) {
       if(!pdialog->improvlist_names_ptrs[n] ||
 	 strcmp(pdialog->improvlist_names_ptrs[n],
-		get_imp_name_ex(pdialog->pcity, i)))
+		get_impr_name_ex(pdialog->pcity, i)))
 	flag=1;
       sz_strlcpy(pdialog->improvlist_names[n],
-		 get_imp_name_ex(pdialog->pcity, i));
+		 get_impr_name_ex(pdialog->pcity, i));
       pdialog->improvlist_names_ptrs[n]=pdialog->improvlist_names[n];
       n++;
     }
@@ -1498,7 +1566,7 @@ void buy_callback(GtkWidget *w, gpointer data)
     name=get_unit_type(pdialog->pcity->currently_building)->name;
   }
   else {
-    name=get_imp_name_ex(pdialog->pcity, pdialog->pcity->currently_building);
+    name=get_impr_name_ex(pdialog->pcity, pdialog->pcity->currently_building);
   }
   value=city_buy_cost(pdialog->pcity);
 
@@ -1508,8 +1576,8 @@ void buy_callback(GtkWidget *w, gpointer data)
 	    name, value, game.player_ptr->economic.gold);
 
     popup_message_dialog(pdialog->shell, /*"buydialog"*/ _("Buy It!"), buf,
-			 _("Yes"), buy_callback_yes, pdialog,
-			 _("No"), buy_callback_no, 0, 0);
+			 _("_Yes"), buy_callback_yes, pdialog,
+			 _("_No"), buy_callback_no, 0, 0);
   }
   else {
     my_snprintf(buf, sizeof(buf),
@@ -1529,7 +1597,7 @@ void unitupgrade_callback_yes(GtkWidget *w, gpointer data)
 {
   struct unit *punit;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data))) {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data))) {
     request_unit_upgrade(punit);
   }
   destroy_message_dialog(w);
@@ -1555,7 +1623,7 @@ void upgrade_callback(GtkWidget *w, gpointer data)
   int ut1,ut2;
   int value;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)data))) {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)data))) {
     ut1=punit->type;
     /* printf("upgrade_callback for %s\n", unit_types[ut1].name); */
 
@@ -1578,9 +1646,9 @@ void upgrade_callback(GtkWidget *w, gpointer data)
 	       value, game.player_ptr->economic.gold);
 	popup_message_dialog(toplevel, 
 			     /*"upgradedialog"*/_("Upgrade Obsolete Units"), buf,
-			     _("Yes"),
+			     _("_Yes"),
 			       unitupgrade_callback_yes, (gpointer)(punit->id),
-			     _("No"),
+			     _("_No"),
 			       unitupgrade_callback_no, 0,
 			     NULL);
       } else {
@@ -1623,6 +1691,7 @@ static void change_to_callback(GtkWidget *w, gpointer data)
     send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
   }
   gtk_widget_destroy(w->parent->parent->parent);
+  pdialog->change_shell=0;
   gtk_widget_set_sensitive(pdialog->shell, TRUE);
 }
 
@@ -1636,6 +1705,7 @@ static void change_no_callback(GtkWidget *w, gpointer data)
   pdialog=(struct city_dialog *)data;
   
   gtk_widget_destroy(w->parent->parent->parent);
+  pdialog->change_shell=0;
   gtk_widget_set_sensitive(pdialog->shell, TRUE);
 }
 
@@ -1678,6 +1748,7 @@ static gint change_deleted_callback(GtkWidget *w, GdkEvent *ev, gpointer data)
   
   pdialog=(struct city_dialog *)data;
   
+  pdialog->change_shell=0;
   gtk_widget_set_sensitive(pdialog->shell, TRUE);
   return FALSE;
 }
@@ -1700,34 +1771,45 @@ void change_callback(GtkWidget *w, gpointer data)
   GtkWidget *cshell, *button, *scrolled;
   struct city_dialog *pdialog;
   int i, n, turns;
-  static gchar *title_[1] = { N_("Select new production") };
+  static gchar *title_[4] = { N_("Type"), N_("Info"), N_("Cost"), N_("Turns") };
   static gchar **title = NULL;
   GtkAccelGroup *accel=gtk_accel_group_new();
+  char *row   [4];
+  char  buf   [4][64];
 
-  if (!title) title = intl_slist(1, title_);
+  if (!title) title = intl_slist(4, title_);
   
   pdialog=(struct city_dialog *)data;
   
   cshell=gtk_dialog_new();
   gtk_signal_connect(GTK_OBJECT(cshell),"delete_event",
 	GTK_SIGNAL_FUNC(change_deleted_callback),pdialog);
+  pdialog->change_shell=cshell;
 
   gtk_window_set_position (GTK_WINDOW(cshell), GTK_WIN_POS_MOUSE);
   gtk_accel_group_attach(accel, GTK_OBJECT(cshell));
   gtk_window_set_title(GTK_WINDOW(cshell), _("Change Production"));
 
-  pdialog->change_list=gtk_clist_new_with_titles(1, title);
+  pdialog->change_list=gtk_clist_new_with_titles(4, title);
   gtk_clist_column_titles_passive(GTK_CLIST(pdialog->change_list));
   scrolled = gtk_scrolled_window_new(NULL, NULL);
   gtk_container_add(GTK_CONTAINER(scrolled), pdialog->change_list);
 
+  for (i=0; i<4; i++)
+    gtk_clist_set_column_auto_resize(GTK_CLIST (pdialog->change_list), i, TRUE);
+
+  gtk_clist_set_column_justification(GTK_CLIST (pdialog->change_list), 2,
+				     GTK_JUSTIFY_RIGHT);
+  gtk_clist_set_column_justification(GTK_CLIST (pdialog->change_list), 3,
+				     GTK_JUSTIFY_RIGHT);
+
   /* Set up the doubleclick-on-list-item handler */
   gtk_signal_connect(GTK_OBJECT(pdialog->change_list), "select_row",
 		     GTK_SIGNAL_FUNC(change_list_callback), pdialog);
 
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled),
 	GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_widget_set_usize(scrolled, 280, 350);
+  gtk_widget_set_usize(scrolled, 310, 350);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(cshell)->vbox), scrolled,
 	TRUE, TRUE, 0);
   
@@ -1757,23 +1839,47 @@ void change_callback(GtkWidget *w, gpointer data)
 
   gtk_widget_set_sensitive(pdialog->shell, FALSE);
 
+  for(i=0; i<4; i++)
+    row[i]=buf[i];
+
+  gtk_clist_freeze(GTK_CLIST(pdialog->change_list));
+  gtk_clist_clear(GTK_CLIST(pdialog->change_list));
+
   for(i=0, n=0; i<B_LAST; i++)
     if(can_build_improvement(pdialog->pcity, i)) {
-      if (i==B_CAPITAL) { /* Total and turns left meaningless for captialization */
-	my_snprintf(pdialog->change_list_names[n],
-		    sizeof(pdialog->change_list_names[n]),
-		    "%s (XX)",
-		    get_imp_name_ex(pdialog->pcity, i));
+      if (i==B_CAPITAL) { /* Total & turns left meaningless on capitalization */
+	my_snprintf(buf[0], sizeof(buf[0]), get_improvement_type(i)->name);
+	buf[1][0]='\0';
+	my_snprintf(buf[2], sizeof(buf[2]), "--");
+	my_snprintf(buf[3], sizeof(buf[3]), "--");
       } else {
 	turns = city_turns_to_build (pdialog->pcity, i, FALSE);
-	my_snprintf(pdialog->change_list_names[n],
-		    sizeof(pdialog->change_list_names[n]),
-		    turns == 1 ? _("%s (%d) %3d turn") : _("%s (%d) %3d turns"),
-		    get_imp_name_ex(pdialog->pcity, i),
-		    get_improvement_type(i)->build_cost,
+	my_snprintf(buf[0], sizeof(buf[0]), get_improvement_type(i)->name);
+
+        /* from city.c get_impr_name_ex() */
+        if (wonder_replacement(pdialog->pcity, i))
+        {
+	  my_snprintf(buf[1], sizeof(buf[1]), "*");
+        }
+        else
+        {
+	  char *state = "";
+
+          if (is_wonder(i))
+          {
+	    state = _("Wonder");
+            if (game.global_wonders[i])	state = _("Built");
+            if (wonder_obsolete(i))	state = _("Obsolete");
+          }
+	  my_snprintf(buf[1], sizeof(buf[1]), state);
+        }
+
+	my_snprintf(buf[2], sizeof(buf[2]), "%d",
+		    get_improvement_type(i)->build_cost);
+	my_snprintf(buf[3], sizeof(buf[3]), "%d",
 		    turns);
       }
-      pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
+      gtk_clist_append(GTK_CLIST(pdialog->change_list), row);
       pdialog->change_list_ids[n++]=i;
     }
   
@@ -1782,25 +1888,26 @@ void change_callback(GtkWidget *w, gpointer data)
 
   for(i=0; i<game.num_unit_types; i++)
     if(can_build_unit(pdialog->pcity, i)) {
+      struct unit_type *ptype;
+
       turns = city_turns_to_build (pdialog->pcity, i, TRUE);
-      my_snprintf(pdialog->change_list_names[n],
-		  sizeof(pdialog->change_list_names[n]),
-		  turns == 1 ? _("%s (%d) %3d turn") : _("%s (%d) %3d turns"),
-		  get_unit_name(i),
-		  get_unit_type(i)->build_cost,
-		  turns);
-      pdialog->change_list_names_ptrs[n]=pdialog->change_list_names[n];
+      my_snprintf(buf[0], sizeof(buf[0]), unit_name(i));
+      
+      /* from unit.h get_unit_name() */
+      ptype = get_unit_type(i);
+      if (ptype->fuel)
+	my_snprintf(buf[1], sizeof(buf[1]), "%d/%d/%d(%d)",
+		    ptype->attack_strength, ptype->defense_strength,
+		    ptype->move_rate / 3, (ptype->move_rate / 3) * ptype->fuel);
+      else
+        my_snprintf(buf[1], sizeof(buf[1]), "%d/%d/%d",
+		    ptype->attack_strength, ptype->defense_strength,
+		    ptype->move_rate / 3);
+      my_snprintf(buf[2], sizeof(buf[2]), "%d", get_unit_type(i)->build_cost);
+      my_snprintf(buf[3], sizeof(buf[3]), "%d", turns);
+      gtk_clist_append(GTK_CLIST(pdialog->change_list), row);
       pdialog->change_list_ids[n++]=i;
     }
-  
-  pdialog->change_list_names_ptrs[n]=0;
-
-  gtk_clist_freeze(GTK_CLIST(pdialog->change_list));
-  gtk_clist_clear(GTK_CLIST(pdialog->change_list));
-
-  for(i=0; i<n; i++)
-    gtk_clist_append(GTK_CLIST(pdialog->change_list),
-		     &pdialog->change_list_names_ptrs[i]);
 
   gtk_clist_thaw(GTK_CLIST(pdialog->change_list));
 
@@ -1947,12 +2054,12 @@ void sell_callback(GtkWidget *w, gpointer data)
 	  
 	  pdialog->sell_id=i;
 	  my_snprintf(buf, sizeof(buf), _("Sell %s for %d gold?"), 
-		  get_imp_name_ex(pdialog->pcity, i),
+		  get_impr_name_ex(pdialog->pcity, i),
 		  improvement_value(i));
 
 	  popup_message_dialog(pdialog->shell, /*"selldialog"*/ _("Sell It!"), buf,
-			       _("Yes"), sell_callback_yes, pdialog,
-			       _("No"), sell_callback_no, pdialog, 0);
+			       _("_Yes"), sell_callback_yes, pdialog,
+			       _("_No"), sell_callback_no, pdialog, 0);
 	  return;
 	}
 	n++;
@@ -1974,6 +2081,18 @@ void close_city_dialog(struct city_dialog *pdialog)
   if (pdialog->worklist_shell)
     gtk_widget_destroy(pdialog->worklist_shell);
 
+  if (pdialog->change_shell)
+    gtk_widget_destroy(pdialog->change_shell);
+
+  unit_list_iterate(pdialog->pcity->info_units_supported, psunit) {
+    free(psunit);
+  } unit_list_iterate_end;
+  unit_list_unlink_all(&(pdialog->pcity->info_units_supported));
+  unit_list_iterate(pdialog->pcity->info_units_present, psunit) {
+    free(psunit);
+  } unit_list_iterate_end;
+  unit_list_unlink_all(&(pdialog->pcity->info_units_present));
+
   gtk_widget_destroy(pdialog->shell);
   free(pdialog);
 }
@@ -2002,15 +2121,15 @@ multiple city dialogs.
 GtkWidget *create_cityopt_dialog(char *city_name);
 void cityopt_ok_command_callback(GtkWidget *w, gpointer data);
 void cityopt_cancel_command_callback(GtkWidget *w, gpointer data);
-void cityopt_newcit_triggle_callback(GtkWidget *w, gpointer data);
+void cityopt_newcit_radio_callback(GtkWidget *w, gpointer data);
 
-char *newcitizen_labels[] = { N_("Workers"), N_("Scientists"), N_("Taxmen") };
+char *ncitizen_labels[] = { N_("Elvises"), N_("Scientists"), N_("Taxmen") };
 
 static GtkWidget *cityopt_shell = 0;
-static GtkWidget *cityopt_triggle;
+static GtkWidget *cityopt_radio[3];	/* cityopt_ncitizen_radio */
 static GtkWidget *cityopt_toggles[NUM_CITYOPT_TOGGLES];
 static int cityopt_city_id = 0;
-static int newcitizen_index;
+static int ncitizen_idx;
 
 /****************************************************************
 ...
@@ -2032,14 +2151,14 @@ void cityopt_callback(GtkWidget *w, gpointer data)
 	state);
   }
   if (pcity->city_options & (1<<CITYO_NEW_EINSTEIN)) {
-    newcitizen_index = 1;
+    ncitizen_idx = 1;
   } else if (pcity->city_options & (1<<CITYO_NEW_TAXMAN)) {
-    newcitizen_index = 2;
+    ncitizen_idx = 2;
   } else {
-    newcitizen_index = 0;
+    ncitizen_idx = 0;
   }
-  gtk_label_set_text (GTK_LABEL (GTK_BIN (cityopt_triggle)->child),
-	_(newcitizen_labels[newcitizen_index]));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cityopt_radio[ncitizen_idx]),
+			       TRUE);
   
   cityopt_city_id = pcity->id;
 
@@ -2053,8 +2172,10 @@ void cityopt_callback(GtkWidget *w, gpointer data)
 **************************************************************************/
 GtkWidget *create_cityopt_dialog(char *city_name)
 {
-  GtkWidget *shell, *label, *ok, *cancel;
+  GtkWidget *shell, *label, *frame, *vbox, *ok, *cancel;
   GtkAccelGroup *accel=gtk_accel_group_new();
+  GSList *group=NULL;
+  int i;
 
   shell = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (shell), _("City Options"));
@@ -2062,13 +2183,22 @@ GtkWidget *create_cityopt_dialog(char *city_name)
   gtk_accel_group_attach(accel, GTK_OBJECT(shell));
 
   label = gtk_label_new (city_name);
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(shell)->vbox),
-	label, FALSE, FALSE, 0);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(shell)->vbox), label, FALSE, FALSE, 0);
 
 
-  cityopt_triggle = gtk_check_button_new_with_label (_("Scientists"));
-  gtk_box_pack_start (GTK_BOX (GTK_DIALOG(shell)->vbox),
-	cityopt_triggle, FALSE, FALSE, 0);
+  frame = gtk_frame_new(_("New citizens are"));
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(shell)->vbox), frame, FALSE, FALSE, 4);
+  vbox = gtk_vbox_new(FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(frame), vbox);
+
+  for (i=0; i<3; i++) {
+    cityopt_radio[i]=gtk_radio_button_new_with_label(group,
+						_(ncitizen_labels[i]));
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cityopt_radio[i]), FALSE);
+    group=gtk_radio_button_group(GTK_RADIO_BUTTON(cityopt_radio[i]));
+    gtk_box_pack_start(GTK_BOX(vbox), cityopt_radio[i],
+		       FALSE, FALSE, 0);
+  }
 
   /* NOTE: the ordering here is deliberately out of order;
      want toggles[] to be in enum city_options order, but
@@ -2117,8 +2247,10 @@ GtkWidget *create_cityopt_dialog(char *city_name)
   gtk_signal_connect(GTK_OBJECT(cancel), "clicked",
 	GTK_SIGNAL_FUNC(cityopt_cancel_command_callback), shell);
 
-  gtk_signal_connect(GTK_OBJECT(cityopt_triggle), "toggled",
-	GTK_SIGNAL_FUNC(cityopt_newcit_triggle_callback), NULL);
+  for (i=0; i<3; i++) {
+    gtk_signal_connect(GTK_OBJECT(cityopt_radio[i]), "toggled",
+	GTK_SIGNAL_FUNC(cityopt_newcit_radio_callback), GINT_TO_POINTER(i));
+  }
  
   gtk_widget_show_all(GTK_DIALOG(shell)->vbox);
   gtk_widget_show_all(GTK_DIALOG(shell)->action_area);
@@ -2144,19 +2276,19 @@ void cityopt_ok_command_callback(GtkWidget *w, gpointer data)
 
   if (pcity) {
     struct packet_generic_values packet;
-    int i, new;
+    int i, new_options;
     
-    new = 0;
+    new_options = 0;
     for(i=0; i<NUM_CITYOPT_TOGGLES; i++)  {
-      if (GTK_TOGGLE_BUTTON(cityopt_toggles[i])->active) new |= (1<<i);
+      if (GTK_TOGGLE_BUTTON(cityopt_toggles[i])->active) new_options |= (1<<i);
     }
-    if (newcitizen_index == 1) {
-      new |= (1<<CITYO_NEW_EINSTEIN);
-    } else if (newcitizen_index == 2) {
-      new |= (1<<CITYO_NEW_TAXMAN);
+    if (ncitizen_idx == 1) {
+      new_options |= (1<<CITYO_NEW_EINSTEIN);
+    } else if (ncitizen_idx == 2) {
+      new_options |= (1<<CITYO_NEW_TAXMAN);
     }
     packet.value1 = cityopt_city_id;
-    packet.value2 = new;
+    packet.value2 = new_options;
     send_packet_generic_values(&aconnection, PACKET_CITY_OPTIONS,
 			      &packet);
   }
@@ -2165,17 +2297,11 @@ void cityopt_ok_command_callback(GtkWidget *w, gpointer data)
 }
 
 /**************************************************************************
- Changes the label of the toggle widget to between newcitizen_labels
- and increments (mod 3) newcitizen_index.
+ Sets newcitizen_index based on the selected radio button.
 **************************************************************************/
-void cityopt_newcit_triggle_callback(GtkWidget *w, gpointer data)
+void cityopt_newcit_radio_callback(GtkWidget *w, gpointer data)
 {
-  newcitizen_index++;
-  if (newcitizen_index>=3) {
-    newcitizen_index = 0;
-  }
-  gtk_label_set_text (GTK_LABEL (GTK_BIN (cityopt_triggle)->child),
-	_(newcitizen_labels[newcitizen_index]));
+  ncitizen_idx = GPOINTER_TO_INT(data);
 }
 
 /**************************************************************************
