@@ -39,6 +39,7 @@ extern int  server_port;
 static GtkWidget *iname, *ihost, *iport;
 static GtkWidget *connw, *quitw;
 
+extern GtkWidget *turn_done_button;
 extern GtkWidget *toplevel;
 
 static GtkWidget *dialog;
@@ -128,7 +129,8 @@ void gui_server_connect(void)
   int i;
 
   if (!titles) titles = intl_slist(6, titles_);
-  
+
+  gtk_widget_set_sensitive(turn_done_button, FALSE);
   gtk_widget_set_sensitive(toplevel, FALSE);
 
   dialog=gtk_dialog_new();
@@ -270,7 +272,7 @@ static int get_meta_list(GtkWidget *list, char *errbuf, int n_errbuf)
     sz_strlcpy(buf[0], pserver->name);
     sz_strlcpy(buf[1], pserver->port);
     sz_strlcpy(buf[2], pserver->version);
-    sz_strlcpy(buf[3], pserver->status);
+    sz_strlcpy(buf[3], _(pserver->status));
     sz_strlcpy(buf[4], pserver->players);
     sz_strlcpy(buf[5], pserver->metastring);
 
