@@ -32,6 +32,7 @@
 #include "mem.h"
 #include "packets.h"
 #include "player.h"
+#include "rand.h"
 #include "support.h"
 
 #include "chatline.h"
@@ -68,6 +69,11 @@ GtkWidget	**races_toggles=NULL,		/* toggle race */
                 **city_style_toggles = NULL,
 		*races_name;			/* leader name */
 GList           *leader_strings = NULL;
+GList           *sorted_races_list = NULL; /* contains a list of race
+					      ids sorted by the race
+					      name. Is valid as long
+					      as the races_dialog is
+					      poped up. */
 /******************************************************************/
 GtkWidget *spy_tech_shell;
 GtkWidget *spy_advances_list, *spy_advances_list_label;
@@ -107,6 +113,7 @@ void races_name_callback	( GtkWidget *w, gpointer data );
 void city_style_toggles_callback ( GtkWidget *w, gpointer data );
 
 int selected_nation;
+int selected_leader;
 int selected_sex;
 int selected_city_style;
 int city_style_idx[64];        /* translation table basic style->city_style  */
@@ -122,6 +129,7 @@ int unit_to_use_to_pillage;
 int caravan_city_id;
 int caravan_unit_id;
 
+int diplomat_dialog_open = 0;
 int diplomat_id;
 int diplomat_target_id;
 
@@ -130,8 +138,6 @@ struct unit *punit_caravan;
 
 static GtkWidget *caravan_dialog;
 
-extern GtkStyle *notify_dialog_style;
-
 static int is_showing_unit_connect_dialog = FALSE;
 static int unit_to_use_to_connect;
 static int connect_unit_x;
@@ -166,6 +172,7 @@ void popup_notify_dialog(char *caption, char *headline, char *lines)
   gtk_signal_connect( GTK_OBJECT(notify_dialog_shell),"delete_event",
 	GTK_SIGNAL_FUNC(deleted_callback),NULL );
   gtk_accel_group_attach(accel, GTK_OBJECT(notify_dialog_shell));
+  gtk_widget_set_name(notify_dialog_shell, "Freeciv");
 
   gtk_window_set_title( GTK_WINDOW( notify_dialog_shell ), caption );
   
@@ -176,8 +183,8 @@ void popup_notify_dialog(char *caption, char *headline, char *lines)
   notify_headline = gtk_label_new( headline);   
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(notify_dialog_shell)->vbox ),
 	notify_headline, FALSE, FALSE, 0 );
+  gtk_widget_set_name(notify_headline, "notify label");
 
-  gtk_widget_set_style( notify_headline, notify_dialog_style );
   gtk_label_set_justify( GTK_LABEL( notify_headline ), GTK_JUSTIFY_LEFT );
   gtk_misc_set_alignment(GTK_MISC(notify_headline), 0.0, 0.0);
 
@@ -188,7 +195,7 @@ void popup_notify_dialog(char *caption, char *headline, char *lines)
   gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (notify_scrolled),
 					notify_label);
 
-  gtk_widget_set_style( notify_label, notify_dialog_style );
+  gtk_widget_set_name(notify_label, "notify label");
   gtk_label_set_justify( GTK_LABEL( notify_label ), GTK_JUSTIFY_LEFT );
   gtk_misc_set_alignment(GTK_MISC(notify_label), 0.0, 0.0);
 
@@ -400,8 +407,8 @@ void popup_bribe_dialog(struct unit *punit)
 		_("Bribe unit for %d gold?\nTreasury contains %d gold."), 
 		punit->bribe_cost, game.player_ptr->economic.gold);
     popup_message_dialog(toplevel, /*"diplomatbribedialog"*/_("Bribe Enemy Unit"), buf,
-			_("Yes"), diplomat_bribe_yes_callback, 0,
-			_("No"), diplomat_bribe_no_callback, 0, 0);
+			_("_Yes"), diplomat_bribe_yes_callback, 0,
+			_("_No"), diplomat_bribe_no_callback, 0, 0);
   } else {
     my_snprintf(buf, sizeof(buf),
 		_("Bribing the unit costs %d gold.\n"
@@ -419,7 +426,8 @@ void popup_bribe_dialog(struct unit *punit)
 static void diplomat_sabotage_callback(GtkWidget *w, gpointer data)
 {
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      find_city_by_id(diplomat_target_id)) { 
     struct packet_diplomat_action req;
@@ -432,27 +440,31 @@ static void diplomat_sabotage_callback(GtkWidget *w, gpointer data)
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
+
 /****************************************************************
 ...
 *****************************************************************/
 static void diplomat_investigate_callback(GtkWidget *w, gpointer data)
 {
- 
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      (find_city_by_id(diplomat_target_id))) { 
     struct packet_diplomat_action req;
-    
+
     req.action_type=DIPLOMAT_INVESTIGATE;
     req.diplomat_id=diplomat_id;
     req.target_id=diplomat_target_id;
-    
+
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -475,20 +487,21 @@ static void spy_sabotage_unit_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 static void diplomat_embassy_callback(GtkWidget *w, gpointer data)
 {
- 
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      (find_city_by_id(diplomat_target_id))) { 
     struct packet_diplomat_action req;
-    
+
     req.action_type=DIPLOMAT_EMBASSY;
     req.diplomat_id=diplomat_id;
     req.target_id=diplomat_target_id;
-    
+
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -496,8 +509,8 @@ static void diplomat_embassy_callback(GtkWidget *w, gpointer data)
 *****************************************************************/
 static void spy_poison_callback(GtkWidget *w, gpointer data)
 {
-
   destroy_message_dialog(w);
+  diplomat_dialog_open=0;
 
   if(find_unit_by_id(diplomat_id) &&
      (find_city_by_id(diplomat_target_id))) {
@@ -510,6 +523,7 @@ static void spy_poison_callback(GtkWidget *w, gpointer data)
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -518,19 +532,21 @@ static void spy_poison_callback(GtkWidget *w, gpointer data)
 static void diplomat_steal_callback(GtkWidget *w, gpointer data)
 {
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      find_city_by_id(diplomat_target_id)) { 
     struct packet_diplomat_action req;
-    
+
     req.action_type=DIPLOMAT_STEAL;
     req.diplomat_id=diplomat_id;
     req.target_id=diplomat_target_id;
     req.value=0;
-    
+
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -543,6 +559,8 @@ static void spy_close_tech_callback(GtkWidget *w, gpointer data)
      gtk_widget_set_sensitive(toplevel, TRUE);
    gtk_widget_destroy(spy_tech_shell);
    spy_tech_shell=0;
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -555,6 +573,8 @@ static void spy_close_sabotage_callback(GtkWidget *w, gpointer data)
      gtk_widget_set_sensitive(toplevel, TRUE);
    gtk_widget_destroy(spy_sabotage_shell);
    spy_sabotage_shell=0;
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -606,6 +626,7 @@ static void spy_steal_callback(GtkWidget *w, gpointer data)
   
   if(!steal_advance){
     freelog(LOG_NORMAL,"Bug in spy steal tech code.");
+    process_diplomat_arrival(NULL, 0);
     return;
   }
   
@@ -620,6 +641,8 @@ static void spy_steal_callback(GtkWidget *w, gpointer data)
 
     send_packet_diplomat_action(&aconnection, &req);
   }
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -632,6 +655,7 @@ static void spy_sabotage_callback(GtkWidget *w, gpointer data)
   
   if(!sabotage_improvement){
     freelog(LOG_NORMAL, "Bug in spy sabotage code");
+    process_diplomat_arrival(NULL, 0);
     return;
   }
   
@@ -646,6 +670,8 @@ static void spy_sabotage_callback(GtkWidget *w, gpointer data)
 
     send_packet_diplomat_action(&aconnection, &req);
   }
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -723,7 +749,7 @@ static int create_advances_list(struct player *pplayer,
 
     if(j > 0) {
       row[0] = _("At Spy's Discretion");
-      gtk_clist_append(GTK_CLIST(spy_improvements_list), row);
+      gtk_clist_append(GTK_CLIST(spy_advances_list), row);
       advance_type[j++] = game.num_tech_types;
     }
   }
@@ -809,7 +835,7 @@ static int create_improvements_list(struct player *pplayer,
 
   for(i=0; i<B_LAST; i++) {
     if(i != B_PALACE && pcity->improvements[i] && !is_wonder(i)) {
-      row[0] = get_imp_name_ex(pcity, i);
+      row[0] = get_impr_name_ex(pcity, i);
       gtk_clist_append(GTK_CLIST(spy_improvements_list), row);
       improvement_type[j++] = i;
     }  
@@ -848,6 +874,7 @@ has happened to the city during latency.  Therefore we must initialize
 pvictim to NULL and account for !pvictim in create_advances_list. -- Syela */
   
   destroy_message_dialog(w);
+  diplomat_dialog_open=0;
 
   if(!spy_tech_shell){
     spy_tech_shell_is_modal=1;
@@ -860,18 +887,36 @@ pvictim to NULL and account for !pvictim in create_advances_list. -- Syela */
 }
 
 /****************************************************************
-...
+ Requests up-to-date list of improvements, the return of
+ which will trigger the popup_sabotage_dialog() function.
 *****************************************************************/
-static void spy_sabotage_popup(GtkWidget *w, gpointer data)
+static void spy_request_sabotage_list(GtkWidget *w, gpointer data)
 {
-  struct city *pvcity = find_city_by_id(diplomat_target_id);
-  
   destroy_message_dialog(w);
+  diplomat_dialog_open=0;
 
+  if(find_unit_by_id(diplomat_id) &&
+     (find_city_by_id(diplomat_target_id))) {
+    struct packet_diplomat_action req;
+
+    req.action_type = SPY_GET_SABOTAGE_LIST;
+    req.diplomat_id = diplomat_id;
+    req.target_id = diplomat_target_id;
+
+    send_packet_diplomat_action(&aconnection, &req);
+  }
+}
+
+/****************************************************************
+ Pops-up the Spy sabotage dialog, upon return of list of
+ available improvements requested by the above function.
+*****************************************************************/
+void popup_sabotage_dialog(struct city *pcity)
+{
   if(!spy_sabotage_shell){
     spy_sabotage_shell_is_modal=1;
 
-    create_improvements_list(game.player_ptr, pvcity, spy_sabotage_shell_is_modal);
+    create_improvements_list(game.player_ptr, pcity, spy_sabotage_shell_is_modal);
     gtk_set_relative_position (toplevel, spy_sabotage_shell, 10, 10);
 
     gtk_widget_show(spy_sabotage_shell);
@@ -892,6 +937,8 @@ static void diplomat_incite_yes_callback(GtkWidget *w, gpointer data)
   send_packet_diplomat_action(&aconnection, &req);
   
   destroy_message_dialog(w);
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -900,8 +947,9 @@ static void diplomat_incite_yes_callback(GtkWidget *w, gpointer data)
 static void diplomat_incite_no_callback(GtkWidget *w, gpointer data)
 {
   destroy_message_dialog(w);
-}
 
+  process_diplomat_arrival(NULL, 0);
+}
 
 
 /****************************************************************
@@ -913,17 +961,17 @@ static void diplomat_incite_callback(GtkWidget *w, gpointer data)
   struct packet_generic_integer packet;
 
   destroy_message_dialog(w);
+  diplomat_dialog_open=0;
   
   if(find_unit_by_id(diplomat_id) && 
      (pcity=find_city_by_id(diplomat_target_id))) { 
     packet.value = diplomat_target_id;
     send_packet_generic_integer(&aconnection, PACKET_INCITE_INQ, &packet);
   }
-
 }
 
 /****************************************************************
-...  Popup the yes/no dialog for inciting, since we know the cost now
+Popup the yes/no dialog for inciting, since we know the cost now
 *****************************************************************/
 void popup_incite_dialog(struct city *pcity)
 {
@@ -935,8 +983,8 @@ void popup_incite_dialog(struct city *pcity)
 		pcity->incite_revolt_cost, game.player_ptr->economic.gold);
    diplomat_target_id = pcity->id;
    popup_message_dialog(toplevel, /*"diplomatrevoltdialog"*/_("Incite a Revolt!"), buf,
-		       _("Yes"), diplomat_incite_yes_callback, 0,
-		       _("No"), diplomat_incite_no_callback, 0, 0);
+		       _("_Yes"), diplomat_incite_yes_callback, 0,
+		       _("_No"), diplomat_incite_no_callback, 0, 0);
   } else {
     my_snprintf(buf, sizeof(buf),
 		_("Inciting a revolt costs %d gold.\n"
@@ -955,6 +1003,9 @@ void popup_incite_dialog(struct city *pcity)
 static void diplomat_cancel_callback(GtkWidget *w, gpointer data)
 {
   destroy_message_dialog(w);
+  diplomat_dialog_open=0;
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 
@@ -966,23 +1017,27 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
   struct city *pcity;
   struct unit *ptunit;
   GtkWidget *shl;
+  char buf[128];
 
   diplomat_id=punit->id;
-  
+
   if((pcity=map_get_city(dest_x, dest_y))){
-    
-    /* Spy/Diplomat acting against a city */ 
-    
+    /* Spy/Diplomat acting against a city */
+
     diplomat_target_id=pcity->id;
+    my_snprintf(buf, sizeof(buf),
+		_("Your %s has arrived at %s.\nWhat is your command?"),
+		unit_name(punit->type), pcity->name);
+
     if(!unit_flag(punit->type, F_SPY)){
-      shl=popup_message_dialog(toplevel, /*"diplomatdialog"*/_(" Choose Your Diplomat's Strategy"), 
-         		     _("Sir, the diplomat is waiting for your command"),
-         		     _("Establish embassy"), diplomat_embassy_callback, 0,
-         		     _("Investigate City"), diplomat_investigate_callback, 0,
-         		     _("Sabotage city"), diplomat_sabotage_callback, 0,
-         		     _("Steal technology"), diplomat_steal_callback, 0,
-         		     _("Incite a revolt"), diplomat_incite_callback, 0,
-         		     _("Cancel"), diplomat_cancel_callback, 0,
+      shl=popup_message_dialog(toplevel, /*"diplomatdialog"*/
+			       _(" Choose Your Diplomat's Strategy"), buf,
+         		     _("Establish _Embassy"), diplomat_embassy_callback, 0,
+         		     _("_Investigate City"), diplomat_investigate_callback, 0,
+         		     _("_Sabotage City"), diplomat_sabotage_callback, 0,
+         		     _("Steal _Technology"), diplomat_steal_callback, 0,
+         		     _("Incite a _Revolt"), diplomat_incite_callback, 0,
+         		     _("_Cancel"), diplomat_cancel_callback, 0,
          		     0);
       
       if(!diplomat_can_do_action(punit, DIPLOMAT_EMBASSY, dest_x, dest_y))
@@ -996,15 +1051,15 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_x, dest_y))
        message_dialog_button_set_sensitive(shl,"button4",FALSE);
     }else{
-       shl=popup_message_dialog(toplevel, /*"spydialog"*/_("Choose Your Spy's Strategy"),
- 			      _("Sir, the spy is waiting for your command"),
- 			      _("Establish Embassy"), diplomat_embassy_callback, 0,
- 			      _("Investigate City (free)"), diplomat_investigate_callback, 0,
- 			      _("Poison City"), spy_poison_callback,0,
- 			      _("Industrial Sabotage"), spy_sabotage_popup, 0,
- 			      _("Steal Technology"), spy_steal_popup, 0,
- 			      _("Incite a Revolt"), diplomat_incite_callback, 0,
- 			      _("Cancel"), diplomat_cancel_callback, 0,
+       shl=popup_message_dialog(toplevel, /*"spydialog"*/
+				_("Choose Your Spy's Strategy"), buf,
+ 			      _("Establish _Embassy"), diplomat_embassy_callback, 0,
+ 			      _("_Investigate City (free)"), diplomat_investigate_callback, 0,
+ 			      _("_Poison City"), spy_poison_callback,0,
+ 			      _("Industrial _Sabotage"), spy_request_sabotage_list, 0,
+ 			      _("Steal _Technology"), spy_steal_popup, 0,
+ 			      _("Incite a _Revolt"), diplomat_incite_callback, 0,
+ 			      _("_Cancel"), diplomat_cancel_callback, 0,
  			      0);
  
       if(!diplomat_can_do_action(punit, DIPLOMAT_EMBASSY, dest_x, dest_y))
@@ -1020,6 +1075,8 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_x, dest_y))
        message_dialog_button_set_sensitive(shl,"button5",FALSE);
      }
+
+    diplomat_dialog_open=1;
    }else{ 
      if((ptunit=unit_list_get(&map_get_tile(dest_x, dest_y)->units, 0))){
        /* Spy/Diplomat acting against a unit */ 
@@ -1030,9 +1087,9 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
                               (!unit_flag(punit->type, F_SPY))?
  			      _("Sir, the diplomat is waiting for your command"):
  			      _("Sir, the spy is waiting for your command"),
- 			      _("Bribe Enemy Unit"), diplomat_bribe_callback, 0,
- 			      _("Sabotage Enemy Unit"), spy_sabotage_unit_callback, 0,
- 			      _("Cancel"), diplomat_cancel_callback, 0,
+ 			      _("_Bribe Enemy Unit"), diplomat_bribe_callback, 0,
+ 			      _("_Sabotage Enemy Unit"), spy_sabotage_unit_callback, 0,
+ 			      _("_Cancel"), diplomat_cancel_callback, 0,
  			      0);
         
        if(!diplomat_can_do_action(punit, DIPLOMAT_BRIBE, dest_x, dest_y))
@@ -1043,6 +1100,14 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
   }
 }
 
+/****************************************************************
+...
+*****************************************************************/
+int diplomat_dialog_is_open(void)
+{
+  return diplomat_dialog_open;
+}
+
 
 /****************************************************************
 ...
@@ -1124,9 +1189,9 @@ void popup_caravan_dialog(struct unit *punit,
   caravan_dialog=popup_message_dialog(toplevel,
 			   /*"caravandialog"*/_("Your Caravan Has Arrived"), 
 			   buf,
-			   _("Establish traderoute"),caravan_establish_trade_callback, 0,
-			   _("Help build Wonder"),caravan_help_build_wonder_callback, 0,
-			   _("Keep moving"),caravan_keep_moving_callback, 0,
+			   _("Establish _Traderoute"),caravan_establish_trade_callback, 0,
+			   _("Help build _Wonder"),caravan_help_build_wonder_callback, 0,
+			   _("_Keep moving"),caravan_keep_moving_callback, 0,
 			   0);
   
   if(!can_establish_trade_route(phomecity, pdestcity))
@@ -1142,6 +1207,14 @@ void popup_caravan_dialog(struct unit *punit,
   }
 }
 
+/****************************************************************
+...
+*****************************************************************/
+int caravan_dialog_is_open(void)
+{
+  return BOOL_VAL(caravan_dialog);
+}
+
 
 /****************************************************************
 ...
@@ -1182,6 +1255,10 @@ void popup_government_dialog(void)
     );
 
     gtk_window_set_title(GTK_WINDOW(dshell), _("Choose Your New Government"));
+#if 0
+    gtk_widget_realize(dshell);
+    gdk_window_set_decorations(dshell->window, GDK_DECOR_BORDER);
+#endif
 
     vbox = gtk_vbox_new(0,TRUE);
     gtk_container_add(GTK_CONTAINER(dshell),vbox);
@@ -1245,8 +1322,8 @@ void popup_revolution_dialog(void)
 {
   popup_message_dialog(toplevel, /*"revolutiondialog"*/_("Revolution!"), 
 		       _("You say you wanna revolution?"),
-		       _("Yes"),revolution_callback_yes, 0,
-		       _("No"),revolution_callback_no, 0, 
+		       _("_Yes"),revolution_callback_yes, 0,
+		       _("_No"),revolution_callback_no, 0, 
 		       0);
 }
 
@@ -1342,7 +1419,7 @@ handle buttons in unit connect dialog
 static void unit_connect_callback (GtkWidget *w, gpointer data)
 {
   struct unit *punit;
-  int activity = (int)data;
+  int activity = GPOINTER_TO_INT(data);
 
   if (!is_showing_unit_connect_dialog) {
     destroy_message_dialog(w);
@@ -1463,6 +1540,7 @@ GtkWidget *popup_message_dialog(GtkWidget *parent, char *dialogname,
 {
   va_list args;
   GtkWidget *dshell, *button, *dlabel, *vbox;
+  GtkAccelGroup *accel = gtk_accel_group_new();
   void (*fcb)(GtkWidget *, gpointer);
   gpointer data;
   char *name;
@@ -1473,10 +1551,10 @@ GtkWidget *popup_message_dialog(GtkWidget *parent, char *dialogname,
   
   dshell=gtk_window_new(GTK_WINDOW_TOPLEVEL);
   gtk_window_set_position (GTK_WINDOW(dshell), GTK_WIN_POS_MOUSE);
+  gtk_accel_group_attach(accel, GTK_OBJECT(dshell));
 
   gtk_signal_connect( GTK_OBJECT(dshell),"delete_event",
 	GTK_SIGNAL_FUNC(popup_mes_del_callback),(gpointer)parent );
-
   gtk_window_set_title( GTK_WINDOW(dshell), dialogname );
 
   vbox = gtk_vbox_new(0,TRUE);
@@ -1497,12 +1575,15 @@ GtkWidget *popup_message_dialog(GtkWidget *parent, char *dialogname,
     data=va_arg(args, gpointer);
     my_snprintf(button_name, sizeof(button_name), "button%d", i++);
     
-    button=gtk_button_new_with_label(name);
+    button=gtk_accelbutton_new(name, accel);
     gtk_box_pack_start( GTK_BOX( vbox ), button, TRUE, FALSE, 0 );
     
     gtk_object_set_data( GTK_OBJECT( dshell ), button_name, button );
 
     gtk_signal_connect(GTK_OBJECT(button),"clicked",GTK_SIGNAL_FUNC(fcb),data );
+
+    if (i==1)
+      gtk_widget_grab_focus(button);
   }
   
   va_end(args);
@@ -1540,8 +1621,8 @@ static void unit_select_all_callback(GtkWidget *w, gpointer data)
   unit_select_dialog_shell=0;
   
   for(i=0; i<unit_select_no; i++) {
-    struct unit *punit=unit_list_find(&game.player_ptr->units, 
-				      unit_select_ids[i]);
+    struct unit *punit = player_find_unit_by_id(game.player_ptr,
+						unit_select_ids[i]);
     if(punit) {
       request_new_unit_activity(punit, ACTIVITY_IDLE);
       set_unit_focus(punit);
@@ -1554,7 +1635,7 @@ static void unit_select_all_callback(GtkWidget *w, gpointer data)
 **************************************************************************/
 static void unit_select_callback(GtkWidget *w, struct unit *punit)
 {
-  if(punit) {
+  if(punit && punit->owner == game.player_idx) {
     request_new_unit_activity(punit, ACTIVITY_IDLE);
     set_unit_focus(punit);
   }
@@ -1627,7 +1708,7 @@ void popup_unit_select_dialog(struct tile *ptile)
 
     unit_select_ids[i]=punit->id;
 
-    pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+    pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
 
     my_snprintf(buffer, sizeof(buffer), "%s(%s)\n%s",
 	    punittemp->name, 
@@ -1706,21 +1787,89 @@ void popdown_races_dialog(void)
     gtk_widget_set_sensitive (toplevel, TRUE);
     gtk_widget_destroy (races_dialog_shell);
     races_dialog_shell = NULL;
+    g_list_free(sorted_races_list);
+    sorted_races_list = NULL;
   } /* else there is no dialog shell to destroy */
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static gint cmp_func(gconstpointer a_p, gconstpointer b_p)
+{
+  return strcmp(get_nation_name(GPOINTER_TO_INT(a_p)),
+		get_nation_name(GPOINTER_TO_INT(b_p)));
+}
+
+/****************************************************************
+Selectes a leader and the appropriate sex.
+Updates the gui elements and the selected_* variables.
+*****************************************************************/
+static void select_random_leader()
+{
+  int j, leader_num;
+  char **leaders;
+
+  g_list_free(leader_strings);
+  leader_strings = NULL;
+
+  /* fill leader names combo box */
+  leaders = get_nation_leader_names( selected_nation, &leader_num);
+  for( j=0; j<leader_num; j++) {
+    leader_strings = g_list_append(leader_strings, leaders[j]);
+  }
+  gtk_combo_set_value_in_list( GTK_COMBO(races_name), FALSE, FALSE);
+  gtk_combo_set_popdown_strings( GTK_COMBO(races_name), leader_strings);
+
+  /* initalize leader names */
+  selected_leader = myrand(leader_num);
+  gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(races_name)->entry),
+		      leaders[selected_leader]);
+
+  /* initalize leader sex */
+  selected_sex = get_nation_leader_sex(selected_nation,
+				       leaders[selected_leader]);
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
+			races_sex_toggles[selected_sex?0:1]), TRUE);
+}
+
+/****************************************************************
+Selectes a random race and the appropriate city style.
+Updates the gui elements and the selected_* variables.
+*****************************************************************/
+static void select_random_race()
+{
+  /* try to find a free nation */
+  while(1) {
+    selected_nation = myrand(game.playable_nation_count);
+    if(GTK_WIDGET_SENSITIVE(races_toggles[g_list_index(sorted_races_list,
+				       GINT_TO_POINTER(selected_nation))]))
+      break;
+  }
+
+  /* initalize nation toggle array */
+  gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON(
+		 races_toggles[g_list_index(sorted_races_list,
+			    GINT_TO_POINTER(selected_nation))] ), TRUE );
+
+  /* initalize city style */
+  selected_city_style =
+    city_style_ridx[get_nation_city_style(selected_nation)];
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
+			city_style_toggles[selected_city_style] ), TRUE );
+}
+
 /****************************************************************
 ...
 *****************************************************************/
 void create_races_dialog(void)
 {
-  int       per_row = 4;
-  int       i, leader_num;
+  int       per_row = 5;
+  int       i;
   GSList    *group = NULL;
   GSList    *sgroup = NULL;
   GSList    *cgroup = NULL;
   GtkWidget *f, *fs, *fa;
-  char **leaders;
 
   races_dialog_shell = gtk_dialog_new();
     gtk_signal_connect( GTK_OBJECT(races_dialog_shell),"delete_event",
@@ -1743,27 +1892,23 @@ void create_races_dialog(void)
   races_toggles = fc_calloc( game.playable_nation_count, sizeof(GtkWidget*) );
 
   for(i=0; i<game.playable_nation_count; i++) {
-    races_toggles[i]= gtk_radio_button_new_with_label( group, get_nation_name(i) );
-    gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( races_toggles[i] ), FALSE );
+    sorted_races_list=g_list_append(sorted_races_list,GINT_TO_POINTER(i));
+  }
+  sorted_races_list=g_list_sort(sorted_races_list,cmp_func);
+  for(i=0; i<g_list_length(sorted_races_list); i++) {
+    gint nat_id=GPOINTER_TO_INT(g_list_nth_data(sorted_races_list,i));
+    races_toggles[i]=gtk_radio_button_new_with_label(group,
+						     get_nation_name(nat_id));
+    gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(races_toggles[i]), FALSE);
     group = gtk_radio_button_group( GTK_RADIO_BUTTON( races_toggles[i] ) );
-    gtk_table_attach_defaults( GTK_TABLE(races_toggles_form), races_toggles[i],
-			i%per_row,i%per_row+1,i/per_row,i/per_row+1 );
+    gtk_table_attach_defaults(GTK_TABLE(races_toggles_form),races_toggles[i],
+			      i%per_row, i%per_row+1, i/per_row, i/per_row+1);
   }
 
   /* ------- nation leader combo ------- */
 
   races_name = gtk_combo_new();
 
-  leaders = get_nation_leader_names(0, &leader_num);
-  g_list_free(leader_strings);
-  leader_strings = 0;
-  for(i=0; i<leader_num; i++)
-    leader_strings = g_list_append(leader_strings, leaders[i]);
-
-  gtk_combo_set_value_in_list( GTK_COMBO(races_name), FALSE, FALSE);
-  gtk_combo_set_popdown_strings( GTK_COMBO(races_name), leader_strings);
-  gtk_entry_set_text( GTK_ENTRY(GTK_COMBO(races_name)->entry), leaders[0]);
-
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG( races_dialog_shell )->vbox ),
 	races_name, FALSE, FALSE, 0 );
   GTK_WIDGET_SET_FLAGS( races_name, GTK_CAN_DEFAULT );
@@ -1779,19 +1924,21 @@ void create_races_dialog(void)
   gtk_container_add( GTK_CONTAINER( fs ), races_sex_toggles_form ); 
 
   races_sex_toggles[0]= gtk_radio_button_new_with_label( sgroup, _("Male") );
-  gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( races_sex_toggles[0] ), FALSE );
+  gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( races_sex_toggles[0] ),
+			       FALSE );
   sgroup = gtk_radio_button_group( GTK_RADIO_BUTTON( races_sex_toggles[0] ) );
-  gtk_table_attach_defaults( GTK_TABLE(races_sex_toggles_form), races_sex_toggles[0],
-			     0,1,0,1); 
+  gtk_table_attach_defaults( GTK_TABLE(races_sex_toggles_form),
+			     races_sex_toggles[0],0,1,0,1); 
   races_sex_toggles[1]= gtk_radio_button_new_with_label( sgroup, _("Female") );
-  gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( races_sex_toggles[1] ), FALSE );
+  gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( races_sex_toggles[1] ),
+			       FALSE );
   sgroup = gtk_radio_button_group( GTK_RADIO_BUTTON( races_sex_toggles[1] ) );
-  gtk_table_attach_defaults( GTK_TABLE(races_sex_toggles_form), races_sex_toggles[1],
-			     1,2,0,1); 
+  gtk_table_attach_defaults( GTK_TABLE(races_sex_toggles_form),
+			     races_sex_toggles[1],1,2,0,1); 
 
   /* ------- city style toggles ------- */
 
-/* find out styles that can be used at the game beginning */
+  /* find out styles that can be used at the game beginning */
    
   for(i=0,b_s_num=0; i<game.styles_count && i<64; i++) {
     if(city_styles[i].techreq == A_NONE) {
@@ -1812,9 +1959,11 @@ void create_races_dialog(void)
   gtk_container_add( GTK_CONTAINER( fa ), city_style_toggles_form ); 
 
   for(i=0; i<b_s_num; i++) {
-      city_style_toggles[i] = gtk_radio_button_new_with_label(
-                                           cgroup, city_styles[city_style_idx[i]].name);
-      gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( city_style_toggles[i] ), FALSE );
+      city_style_toggles[i] =
+	gtk_radio_button_new_with_label(cgroup,
+					city_styles[city_style_idx[i]].name);
+      gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( city_style_toggles[i] ),
+				   FALSE );
       cgroup = gtk_radio_button_group( GTK_RADIO_BUTTON( city_style_toggles[i] ) );
       gtk_table_attach_defaults( GTK_TABLE(city_style_toggles_form),
 				 city_style_toggles[i],
@@ -1827,24 +1976,23 @@ void create_races_dialog(void)
   GTK_WIDGET_SET_FLAGS( races_ok_command, GTK_CAN_DEFAULT );
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG( races_dialog_shell )->action_area ),
 	races_ok_command, TRUE, TRUE, 0 );
-  
-  /* if(has_capability("dconn_in_sel_nat", aconnection.capability)) */
-  {
-    races_disc_command = gtk_button_new_with_label( "Disconnect" );
-    GTK_WIDGET_SET_FLAGS( races_disc_command, GTK_CAN_DEFAULT );
-    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( races_dialog_shell )->action_area ),
- 			races_disc_command, TRUE, TRUE, 0 );
-    races_quit_command = gtk_button_new_with_label( "Quit" );
-    GTK_WIDGET_SET_FLAGS( races_quit_command, GTK_CAN_DEFAULT );
-    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( races_dialog_shell )->action_area ),
- 			races_quit_command, TRUE, TRUE, 0 );
-  }
+
+  races_disc_command = gtk_button_new_with_label( _("Disconnect") );
+  GTK_WIDGET_SET_FLAGS( races_disc_command, GTK_CAN_DEFAULT );
+  gtk_box_pack_start( GTK_BOX( GTK_DIALOG( races_dialog_shell )->action_area ),
+		      races_disc_command, TRUE, TRUE, 0 );
+
+  races_quit_command = gtk_button_new_with_label( _("Quit") );
+  GTK_WIDGET_SET_FLAGS( races_quit_command, GTK_CAN_DEFAULT );
+  gtk_box_pack_start( GTK_BOX( GTK_DIALOG( races_dialog_shell )->action_area ),
+		      races_quit_command, TRUE, TRUE, 0 );
 
   /* ------- connect callback functions ------- */
 
-  for(i=0; i<game.playable_nation_count; i++)
+   for(i=0; i<g_list_length(sorted_races_list); i++)
 	gtk_signal_connect( GTK_OBJECT( races_toggles[i] ), "toggled",
-	    GTK_SIGNAL_FUNC( races_toggles_callback ), NULL );
+                           GTK_SIGNAL_FUNC( races_toggles_callback ),
+                           g_list_nth_data(sorted_races_list, i));
 
   for(i=0; i<2; i++)
         gtk_signal_connect( GTK_OBJECT( races_sex_toggles[i] ), "toggled",
@@ -1854,32 +2002,23 @@ void create_races_dialog(void)
         gtk_signal_connect( GTK_OBJECT( city_style_toggles[i] ), "toggled",
 	    GTK_SIGNAL_FUNC( city_style_toggles_callback ), NULL );
 
-  gtk_signal_connect( GTK_OBJECT( GTK_COMBO(races_name)->list ), "selection_changed",
-                      GTK_SIGNAL_FUNC( races_name_callback ), NULL );
+  gtk_signal_connect( GTK_OBJECT( GTK_COMBO(races_name)->list ), 
+		      "selection_changed",
+		      GTK_SIGNAL_FUNC( races_name_callback ), NULL );
 
   gtk_signal_connect( GTK_OBJECT( races_ok_command ), "clicked",
 			GTK_SIGNAL_FUNC( races_buttons_callback ), NULL );
-  
-  /* if(has_capability("dconn_in_sel_nat", aconnection.capability)) */
-  {
-    gtk_signal_connect( GTK_OBJECT( races_disc_command ), "clicked",
-			GTK_SIGNAL_FUNC( races_buttons_callback ), NULL );
-    gtk_signal_connect( GTK_OBJECT( races_quit_command ), "clicked",
-			GTK_SIGNAL_FUNC( races_buttons_callback ), NULL );
-  }
+
+  gtk_signal_connect( GTK_OBJECT( races_disc_command ), "clicked",
+		      GTK_SIGNAL_FUNC( races_buttons_callback ), NULL );
+
+  gtk_signal_connect( GTK_OBJECT( races_quit_command ), "clicked",
+		      GTK_SIGNAL_FUNC( races_buttons_callback ), NULL );
 
   /* ------- set initial selections ------- */
 
-  selected_nation = 0;
-  gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( races_toggles[0] ), TRUE );
-
-  selected_sex = get_nation_leader_sex( 0, leaders[0]); 
-  gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( 
-                               races_sex_toggles[selected_sex?0:1]), TRUE); 
-
-  selected_city_style = city_style_ridx[get_nation_city_style(0)];
-  gtk_toggle_button_set_state( GTK_TOGGLE_BUTTON( 
-                               city_style_toggles[selected_city_style] ), TRUE );
+  select_random_race();
+  select_random_leader();
 
   gtk_widget_show_all( GTK_DIALOG(races_dialog_shell)->vbox );
   gtk_widget_show_all( GTK_DIALOG(races_dialog_shell)->action_area );
@@ -1901,30 +2040,37 @@ void races_dialog_returnkey(Widget w, XEvent *event, String *params,
 **************************************************************************/
 void races_toggles_set_sensitive(int bits1, int bits2)
 {
-  int i, selected, mybits;
+  int i, mybits;
 
   mybits=bits1;
 
   for(i=0; i<game.playable_nation_count && i<32; i++) {
-    if(mybits&1)
-      gtk_widget_set_sensitive( races_toggles[i], FALSE );
-    else
-      gtk_widget_set_sensitive( races_toggles[i], TRUE );
+    if(mybits&1) {
+      gtk_widget_set_sensitive( races_toggles[g_list_index(sorted_races_list,
+						   GINT_TO_POINTER(i))], FALSE );
+      if(i==selected_nation)
+	select_random_race();
+    } else {
+      gtk_widget_set_sensitive( races_toggles[g_list_index(sorted_races_list,
+						   GINT_TO_POINTER(i))], TRUE );
+    }
     mybits>>=1;
   }
 
   mybits=bits2;
 
   for(i=32; i<game.playable_nation_count; i++) {
-    if(mybits&1)
-      gtk_widget_set_sensitive( races_toggles[i], FALSE );
-    else
-      gtk_widget_set_sensitive( races_toggles[i], TRUE );
+    if(mybits&1) {
+      gtk_widget_set_sensitive( races_toggles[g_list_index(sorted_races_list,
+						   GINT_TO_POINTER(i))], FALSE );
+      if(i==selected_nation)
+	select_random_race();
+    } else {
+      gtk_widget_set_sensitive( races_toggles[g_list_index(sorted_races_list,
+						   GINT_TO_POINTER(i))], TRUE );
+    }
     mybits>>=1;
   }
-
-  if((selected=races_buttons_get_current())==-1)
-     return;
 }
 
 /**************************************************************************
@@ -1945,32 +2091,15 @@ void races_name_callback( GtkWidget *w, gpointer data )
 /**************************************************************************
 ...
 **************************************************************************/
-void races_toggles_callback( GtkWidget *w, gpointer data )
+void races_toggles_callback( GtkWidget *w, gpointer race_id_p )
 {
-  int i, j, leader_count;
-  char **leaders;
+  selected_nation = GPOINTER_TO_INT(race_id_p);
 
-  g_list_free(leader_strings);
-  leader_strings = 0;
+  select_random_leader();
 
-  for(i=0; i<game.playable_nation_count; i++)
-    if(w==races_toggles[i]) {
-      leaders = get_nation_leader_names( i, &leader_count);
-      for( j=0; j<leader_count; j++) {
-        leader_strings = g_list_append(leader_strings, leaders[j]);
-      }
-      gtk_combo_set_value_in_list( GTK_COMBO(races_name), FALSE, FALSE);
-      gtk_combo_set_popdown_strings( GTK_COMBO(races_name), leader_strings);
-      gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(races_name)->entry), leaders[0]);
-      selected_nation = i;
-      selected_sex = get_nation_leader_sex(i,leaders[0]);
-      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON( 
-                 races_sex_toggles[selected_sex?0:1]), TRUE);
-      selected_city_style = city_style_ridx[get_nation_city_style(i)];
-      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
-                 city_style_toggles[selected_city_style] ), TRUE );
-      return;
-    }
+  selected_city_style = city_style_ridx[get_nation_city_style(selected_nation)];
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(
+				city_style_toggles[selected_city_style] ), TRUE );
 }
 
 /**************************************************************************
@@ -2068,7 +2197,6 @@ void races_buttons_callback( GtkWidget *w, gpointer data )
 }
 
 
-
 /**************************************************************************
   Destroys its widget.  Usefull for a popdown callback on pop-ups that
   won't get resused.
@@ -2078,6 +2206,7 @@ void destroy_me_callback( GtkWidget *w, gpointer data)
   gtk_widget_destroy(w);
 }
 
+
 /**************************************************************************
   Adjust tax rates from main window
 **************************************************************************/
@@ -2112,56 +2241,3 @@ void taxrates_callback( GtkWidget *w, GdkEventButton *event, gpointer data )
   send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_RATES);
 
 }
-
-/**************************************************************************
-  Add punit to queue of caravan arrivals, and popup a window for the
-  next arrival in the queue, if there is not already a popup, and
-  re-checking that a popup is appropriate.
-  If punit is NULL, just do for the next arrival in the queue.
-**************************************************************************/
-void process_caravan_arrival(struct unit *punit)
-{
-  static struct genlist arrival_queue;
-  static int is_init_arrival_queue = 0;
-  int *p_id;
-
-  /* arrival_queue is a list of individually malloc-ed ints with
-     punit.id values, for units which have arrived. */
-
-  if (!is_init_arrival_queue) {
-    genlist_init(&arrival_queue);
-    is_init_arrival_queue = 1;
-  }
-
-  if (punit) {
-    p_id = fc_malloc(sizeof(int));
-    *p_id = punit->id;
-    genlist_insert(&arrival_queue, p_id, -1);
-  }
-
-  /* There can only be one dialog at a time: */
-  if (caravan_dialog) {
-    return;
-  }
-  
-  while (genlist_size(&arrival_queue)) {
-    int id;
-    
-    p_id = genlist_get(&arrival_queue, 0);
-    genlist_unlink(&arrival_queue, p_id);
-    id = *p_id;
-    free(p_id);
-    punit = unit_list_find(&game.player_ptr->units, id);
-
-    if (punit && (unit_can_help_build_wonder_here(punit)
-		  || unit_can_est_traderoute_here(punit))
-	&& (!game.player_ptr->ai.control || ai_popup_windows)) {
-      struct city *pcity_dest = map_get_city(punit->x, punit->y);
-      struct city *pcity_homecity = find_city_by_id(punit->homecity);
-      if (pcity_dest && pcity_homecity) {
-	popup_caravan_dialog(punit, pcity_homecity, pcity_dest);
-	return;
-      }
-    }
-  }
-}
