@@ -32,6 +32,7 @@
 #include "support.h"
 
 #include "chatline.h"
+#include "climisc.h"
 #include "clinet.h"
 #include "diptreaty.h"
 #include "gui_stuff.h"
@@ -63,7 +64,8 @@ struct Diplomacy_dialog {
   GtkWidget *dip_gold_frame1;
   GtkWidget *dip_gold_entry0;
   GtkWidget *dip_gold_entry1;
-  
+  GtkWidget *dip_pact_menu;
+
   GtkWidget *dip_label;
   GtkWidget *dip_clauselabel;
   GtkWidget *dip_clauselist;
@@ -94,6 +96,9 @@ void diplomacy_dialog_erase_clause_callback(GtkWidget *w, gpointer data);
 void diplomacy_dialog_accept_callback(GtkWidget *w, gpointer data);
 void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data);
 void diplomacy_dialog_city_callback(GtkWidget *w, gpointer data);
+void diplomacy_dialog_ceasefire_callback(GtkWidget *w, gpointer data);
+void diplomacy_dialog_peace_callback(GtkWidget *w, gpointer data);
+void diplomacy_dialog_alliance_callback(GtkWidget *w, gpointer data);
 void close_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog);
 static gint diplomacy_dialog_mbutton_callback(GtkWidget *w, GdkEvent *event);
@@ -216,35 +221,44 @@ static int fill_diplomacy_tech_menu(GtkWidget *popupmenu,
 }
 
 /****************************************************************
-
 Creates a sorted list of plr0's cities, excluding the capital and
 any cities not visible to plr1.  This means that you can only trade 
 cities visible to requesting player.  
 
 			    - Kris Bubendorfer
 *****************************************************************/
-/* FIXME - this one should be sorted -vasco */
 static int fill_diplomacy_city_menu(GtkWidget *popupmenu, 
 				    struct player *plr0, struct player *plr1)
 {
-  int flag=0;
+  int i = 0, j = 0, n = city_list_size(&plr0->cities);
+  struct city **city_list_ptrs;
+  if (n>0) {
+    city_list_ptrs = fc_malloc(sizeof(struct city*)*n);
+  } else {
+    city_list_ptrs = NULL;
+  }
 
   city_list_iterate(plr0->cities, pcity) {
     if(!city_got_effect(pcity, B_PALACE)){
-      GtkWidget *item=gtk_menu_item_new_with_label(pcity->name);
+      city_list_ptrs[i] = pcity;
+      i++;
+    }
+  } city_list_iterate_end;
+
+  qsort(city_list_ptrs, i, sizeof(struct city*), city_name_compare);
+  
+  for(j=0; j<i; j++) {
+      GtkWidget *item=gtk_menu_item_new_with_label(city_list_ptrs[j]->name);
 
       gtk_menu_append(GTK_MENU(popupmenu),item);
       gtk_signal_connect(GTK_OBJECT(item),"activate",
         GTK_SIGNAL_FUNC(diplomacy_dialog_city_callback),
-        (gpointer)(pcity->id*1024 + plr0->player_no*32 + plr1->player_no));
-      flag=1;
-    }
-  } city_list_iterate_end;
-
-  return flag;
+        (gpointer)(city_list_ptrs[j]->id*1024 + plr0->player_no*32 + plr1->player_no));
+  }
+  free(city_list_ptrs);
+  return i;
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
@@ -402,7 +416,36 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_box_pack_start(GTK_BOX(pdialog->dip_vbox1),
 	pdialog->dip_gold_frame1, FALSE,FALSE,2);
 
-  
+
+  /* Start of pact button insertion */
+
+  pdialog->dip_pact_menu=gtk_menu_new();
+
+  item=gtk_menu_item_new_with_label(_("Cease-fire"));
+  gtk_menu_append(GTK_MENU(pdialog->dip_pact_menu),item);
+  gtk_signal_connect(GTK_OBJECT(item),"activate",
+	GTK_SIGNAL_FUNC(diplomacy_dialog_ceasefire_callback),(gpointer)pdialog);
+
+  item=gtk_menu_item_new_with_label(_("Peace"));
+  gtk_menu_append(GTK_MENU(pdialog->dip_pact_menu),item);
+  gtk_signal_connect(GTK_OBJECT(item),"activate",
+	GTK_SIGNAL_FUNC(diplomacy_dialog_peace_callback),(gpointer)pdialog);
+
+  item=gtk_menu_item_new_with_label(_("Alliance"));
+  gtk_menu_append(GTK_MENU(pdialog->dip_pact_menu),item);
+  gtk_signal_connect(GTK_OBJECT(item),"activate",
+	GTK_SIGNAL_FUNC(diplomacy_dialog_alliance_callback),(gpointer)pdialog);
+
+  gtk_widget_show_all(pdialog->dip_pact_menu);
+
+  button=gtk_button_new_with_label(_("Pacts"));
+  gtk_box_pack_start(GTK_BOX(pdialog->dip_vbox0),button, FALSE,FALSE,2);
+  gtk_signal_connect_object(GTK_OBJECT(button), "event",
+	GTK_SIGNAL_FUNC(diplomacy_dialog_mbutton_callback),
+	GTK_OBJECT(pdialog->dip_pact_menu));
+
+  /* End of pact button insertion */
+
   my_snprintf(buf, sizeof(buf),
 	       _("This Eternal Treaty\n"
 		 "marks the results of the diplomatic work between\n"
@@ -439,6 +482,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);
 
   pdialog->dip_acceptthumb0=gtk_pixmap_new(get_thumb_pixmap(0),NULL);
+  gtk_pixmap_set_build_insensitive(GTK_PIXMAP(pdialog->dip_acceptthumb0),FALSE);
   gtk_table_attach_defaults(GTK_TABLE(table),pdialog->dip_acceptthumb0,1,2,0,1);
   gtk_widget_show(pdialog->dip_acceptthumb0);
 
@@ -447,6 +491,7 @@ struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_table_attach_defaults(GTK_TABLE(table),label,2,3,0,1);
 
   pdialog->dip_acceptthumb1=gtk_pixmap_new(get_thumb_pixmap(0),NULL);
+  gtk_pixmap_set_build_insensitive(GTK_PIXMAP(pdialog->dip_acceptthumb1),FALSE);
   gtk_table_attach_defaults(GTK_TABLE(table),pdialog->dip_acceptthumb1,3,4,0,1);
   gtk_widget_show(pdialog->dip_acceptthumb1);
   
@@ -507,32 +552,7 @@ void update_diplomacy_dialog(struct Diplomacy_dialog *pdialog)
   
   for(i=0; i<MAX_NUM_CLAUSES && ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
     struct Clause *pclause=(struct Clause *)ITERATOR_PTR(myiter);
-    
-    switch(pclause->type) {
-     case CLAUSE_ADVANCE:
-      my_snprintf(buf, sizeof(buf), _("The %s give %s"),
-	      get_nation_name_plural(pclause->from->nation),
-	      advances[pclause->value].name);
-      break;
-    case CLAUSE_CITY:
-      my_snprintf(buf, sizeof(buf), _("The %s give %s"),
-	      get_nation_name_plural(pclause->from->nation),
-	      find_city_by_id(pclause->value)->name);
-      break;
-     case CLAUSE_GOLD:
-      my_snprintf(buf, sizeof(buf), _("The %s give %d gold"),
-	      get_nation_name_plural(pclause->from->nation),
-	      pclause->value);
-      break;
-     case CLAUSE_MAP: 
-      my_snprintf(buf, sizeof(buf), _("The %s give their worldmap"),
-	      get_nation_name_plural(pclause->from->nation));
-      break;
-     case CLAUSE_SEAMAP: 
-      my_snprintf(buf, sizeof(buf), _("The %s give their seamap"),
-	      get_nation_name_plural(pclause->from->nation));
-      break;
-    }
+    client_diplomacy_clause_string(buf, sizeof(buf), pclause);
     gtk_clist_append(GTK_CLIST(pdialog->dip_clauselist),row);
     i++;
   }
@@ -685,7 +705,47 @@ void diplomacy_dialog_seamap_callback(GtkWidget *w, gpointer data)
 			     &pa);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static void diplomacy_dialog_add_pact_clause(GtkWidget *w, gpointer data,
+					     int type)
+{
+  struct Diplomacy_dialog *pdialog=(struct Diplomacy_dialog *)data;
+  struct packet_diplomacy_info pa;
+  
+  pa.plrno0 = pdialog->treaty.plr0->player_no;
+  pa.plrno1 = pdialog->treaty.plr1->player_no;
+  pa.clause_type = type;
+  pa.plrno_from = pdialog->treaty.plr0->player_no;
+  pa.value = 0;
+  send_packet_diplomacy_info(&aconnection, PACKET_DIPLOMACY_CREATE_CLAUSE,
+			     &pa);
+}
 
+/****************************************************************
+...
+*****************************************************************/
+void diplomacy_dialog_ceasefire_callback(GtkWidget *w, gpointer data)
+{
+  diplomacy_dialog_add_pact_clause(w, data, CLAUSE_CEASEFIRE);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+void diplomacy_dialog_peace_callback(GtkWidget *w, gpointer data)
+{
+  diplomacy_dialog_add_pact_clause(w, data, CLAUSE_PEACE);
+}
+
+/****************************************************************
+...
+*****************************************************************/
+void diplomacy_dialog_alliance_callback(GtkWidget *w, gpointer data)
+{
+  diplomacy_dialog_add_pact_clause(w, data, CLAUSE_ALLIANCE);
+}
 
 
 
