@@ -14,6 +14,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -53,15 +54,13 @@
 #include "optiondlg.h"
 #include "options.h"
 #include "spaceshipdlg.h"
+#include "resources.h"
 #include "tilespec.h"
 
 #include "gui_main.h"
 
 #include "freeciv.ico"
 
-#define NOTIFY_DIALOG_FONT	"-b&h-lucidatypewriter-bold-r-normal-*-12-*-*-*-*-*-*-*"
-#define FIXED_10_BFONT		"-b&h-lucidatypewriter-bold-r-normal-*-10-*-*-*-*-*-*-*"
-
 /*void file_quit_cmd_callback( GtkWidget *widget, gpointer data )*/
 void game_rates( GtkWidget *widget, gpointer data );
 
@@ -105,23 +104,25 @@ GdkGC *		mask_bg_gc;
 GdkPixmap *	mask_bitmap;
 
 GdkFont *	main_font;
+GdkFont *	city_productions_font;
 
 GtkWidget *	main_frame_civ_name;
 GtkWidget *	main_label_info;
-/*
 GtkWidget *	main_label_info_ebox;
-*/
 GtkText *	main_message_area;
 
 GtkWidget *	econ_label			[10];
 GtkWidget *	bulb_label;
 GtkWidget *	sun_label;
+GtkWidget *	flake_label;
 GtkWidget *	government_label;
 GtkWidget *	timeout_label;
 GtkWidget *	turn_done_button;
 
 GtkWidget *	unit_pixmap;
+GtkWidget *     unit_pixmap_button;
 GtkWidget *	unit_below_pixmap		[MAX_NUM_UNITS_BELOW];
+GtkWidget *     unit_below_pixmap_button        [MAX_NUM_UNITS_BELOW];
 GtkWidget *	more_arrow_pixmap;
 
 gint		gtk_interval_id;
@@ -133,20 +134,15 @@ GtkWidget *	inputline;
 GtkWidget *	map_horizontal_scrollbar, *map_vertical_scrollbar;
 gint		gdk_input_id;
 
-GtkStyle *	notify_dialog_style;
-GtkStyle *	city_dialog_style;
-
 GdkWindow *	root_window;
 
+/* ids of the units icons in information display: (or 0) */
+static int unit_ids[MAX_NUM_UNITS_BELOW];  
+
 
 enum Display_color_type		display_color_type;
 
-extern int	num_units_below;
-
-#ifdef UNUSED  /* used in commented-out code */
 static gint show_info_popup(GtkWidget *w, GdkEventButton *ev);
-#endif
-
 static gint timer_callback(gpointer data);
 
 
@@ -235,46 +231,79 @@ static gint keyboard_handler(GtkWidget *widget, GdkEventKey *event)
     return TRUE;
 }
 
-static void tearoff_callback (GtkWidget *but, GtkWidget *box)
+static gint tearoff_delete(GtkWidget *w, GdkEvent *ev, GtkWidget *box)
 {
-    GtkWidget *w, *p;
+  GtkWidget *p;
 
-    if (GTK_TOGGLE_BUTTON (but)->active)
-    {
-	w = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_window_set_position(GTK_WINDOW(w), GTK_WIN_POS_MOUSE);
+  gtk_widget_hide(w);
+  p = ((GtkBoxChild *)GTK_BOX(box)->children->data)->widget;
+  gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(p), FALSE);
 
-	gtk_object_set_user_data (GTK_OBJECT (w), box->parent);
-	gtk_widget_reparent (box, w);
-	gtk_widget_show (w);
-    }
-    else
-    {
-        w = box->parent;
-	p = gtk_object_get_user_data (GTK_OBJECT (w));
-	gtk_widget_reparent (box, p);
-	gtk_widget_destroy (w);
-    }
+  p = gtk_object_get_user_data(GTK_OBJECT (w));
+  gtk_widget_reparent(box, p);
+  return TRUE;
 }
 
-static GtkWidget *detached_widget_new(GtkWidget **avbox)
+static void tearoff_callback (GtkWidget *but, GtkWidget *box)
 {
-    GtkWidget *ahbox, *b, *sep;
+  GtkWidget *w;
 
-    ahbox = gtk_hbox_new( FALSE, 2 );
+  if (GTK_TOGGLE_BUTTON(but)->active)
+  {
+    w = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+    gtk_window_set_title(GTK_WINDOW(w), _("Freeciv"));
+    gtk_window_set_position(GTK_WINDOW(w), GTK_WIN_POS_MOUSE);
+    gtk_signal_connect(GTK_OBJECT(w), "delete_event",
+    		       GTK_SIGNAL_FUNC(tearoff_delete), box);
 
-    b = gtk_toggle_button_new();
-    gtk_box_pack_start( GTK_BOX( ahbox ), b, FALSE, FALSE, 0 );
-    gtk_signal_connect( GTK_OBJECT(b), "clicked",
-			GTK_SIGNAL_FUNC( tearoff_callback ), ahbox);
+    gtk_object_set_user_data(GTK_OBJECT (w), box->parent);
+    gtk_widget_reparent(box, w);
+    gtk_widget_show(w);
+  }
+  else
+  {
+    w = box->parent;
+    tearoff_delete(w, NULL, box);
+    gtk_widget_destroy(w);
+  }
+}
 
-    /* cosmetic effects */
-    sep = gtk_vseparator_new();
-    gtk_container_add( GTK_CONTAINER( b ), sep );
+static GtkWidget *detached_widget_new(void)
+{
+  return gtk_hbox_new(FALSE, 2);
+}
 
-    *avbox = gtk_vbox_new( FALSE, 0 );
-    gtk_box_pack_start( GTK_BOX( ahbox ), *avbox, TRUE, TRUE, 0 );
-    return ahbox;
+static GtkWidget *detached_widget_fill(GtkWidget *ahbox)
+{
+  GtkWidget *b, *sep, *avbox;
+
+  b = gtk_toggle_button_new();
+  gtk_box_pack_start( GTK_BOX( ahbox ), b, FALSE, FALSE, 0 );
+  gtk_signal_connect( GTK_OBJECT(b), "clicked",
+        	      GTK_SIGNAL_FUNC( tearoff_callback ), ahbox);
+
+  /* cosmetic effects */
+  sep = gtk_vseparator_new();
+  gtk_container_add( GTK_CONTAINER( b ), sep );
+
+  avbox = gtk_vbox_new( FALSE, 0 );
+  gtk_box_pack_start( GTK_BOX( ahbox ), avbox, TRUE, TRUE, 0 );
+  return avbox;
+}
+
+static void select_pixmap_callback(GtkWidget *w, GdkEvent *ev, int i) 
+{
+  struct unit *punit;
+
+  if (i == -1)  /* unit currently selected */
+    return;
+  if (unit_ids[i] == 0) /* no unit displayed at this place */
+    return;
+  punit=find_unit_by_id(unit_ids[i]);
+  if(punit) { /* should always be true at this point */
+     request_new_unit_activity(punit, ACTIVITY_IDLE);
+     set_unit_focus(punit);
+  }
 }
 
 /**************************************************************************
@@ -286,6 +315,7 @@ static void setup_widgets(void)
   GtkWidget	      *hbox;
   GtkWidget	      *vbox1;
   GtkWidget           *avbox;
+  GtkWidget           *ahbox;
   GtkWidget	      *frame;
 
   GtkWidget	      *menubar;
@@ -309,7 +339,9 @@ static void setup_widgets(void)
   vbox1 = gtk_vbox_new( FALSE, 3 );
   gtk_box_pack_start( GTK_BOX( hbox ), vbox1, FALSE, FALSE, 0 );
 
-  gtk_container_add(GTK_CONTAINER(vbox1), detached_widget_new (&avbox));
+  ahbox = detached_widget_new();
+  gtk_container_add(GTK_CONTAINER(vbox1), ahbox);
+  avbox = detached_widget_fill(ahbox);
 
   overview_canvas	      = gtk_drawing_area_new();
 
@@ -317,14 +349,16 @@ static void setup_widgets(void)
         			      | GDK_BUTTON_PRESS_MASK );
 
   gtk_drawing_area_size( GTK_DRAWING_AREA( overview_canvas ), 160, 100 );
-  gtk_box_pack_start(GTK_BOX(avbox), overview_canvas, TRUE, TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(avbox), overview_canvas, FALSE, FALSE, 0);
 
   gtk_signal_connect( GTK_OBJECT( overview_canvas ), "expose_event",
         	      (GtkSignalFunc) overview_canvas_expose, NULL );
   gtk_signal_connect( GTK_OBJECT( overview_canvas ), "button_press_event",
         	      (GtkSignalFunc) butt_down_overviewcanvas, NULL );
 
-  gtk_container_add(GTK_CONTAINER(vbox1), detached_widget_new (&avbox));
+  ahbox = detached_widget_new();
+  gtk_container_add(GTK_CONTAINER(vbox1), ahbox);
+  avbox = detached_widget_fill(ahbox);
 
   {   /* Info on player's civilization */
       GtkWidget *vbox4;
@@ -336,19 +370,16 @@ static void setup_widgets(void)
 
       vbox4 = gtk_vbox_new(FALSE, 0);
       gtk_container_add(GTK_CONTAINER(frame), vbox4);
-/*
+
       main_label_info_ebox = gtk_event_box_new ();
       gtk_widget_set_events (main_label_info_ebox, GDK_BUTTON_PRESS_MASK);
 
       gtk_box_pack_start(GTK_BOX(vbox4), main_label_info_ebox, FALSE, FALSE, 0);
-*/
+
       main_label_info = gtk_label_new("\n\n\n\n");
-      gtk_box_pack_start(GTK_BOX(vbox4), main_label_info, FALSE, FALSE, 0);
-/*
       gtk_container_add (GTK_CONTAINER (main_label_info_ebox), main_label_info);
       gtk_signal_connect (GTK_OBJECT(main_label_info_ebox),"button_press_event",
 			  GTK_SIGNAL_FUNC (show_info_popup), NULL);
-*/
   }
 
   { /* production status */
@@ -358,7 +389,7 @@ static void setup_widgets(void)
       box4 = gtk_hbox_new( FALSE, 0 );
       gtk_box_pack_start(GTK_BOX(avbox), box4, FALSE, FALSE, 0);
 
-      table = gtk_table_new(10, 2, FALSE);
+      table = gtk_table_new(10, 3, FALSE);
       gtk_table_set_row_spacing(GTK_TABLE(table), 0, 0);
       gtk_table_set_col_spacing(GTK_TABLE(table), 0, 0);
       gtk_box_pack_start(GTK_BOX(box4), table, TRUE, FALSE, 0);
@@ -373,41 +404,49 @@ static void setup_widgets(void)
           gtk_signal_connect( GTK_OBJECT( ebox ), "button_press_event",
               GTK_SIGNAL_FUNC(taxrates_callback), (gpointer)i);
 
-          econ_label[i] = gtk_pixmap_new(sprites.warming[0]->pixmap, NULL);
+          econ_label[i] = gtk_pixmap_new(get_citizen_pixmap(i<5?1:2), NULL);
+	  gtk_pixmap_set_build_insensitive(GTK_PIXMAP(econ_label[i]), FALSE);
           gtk_container_add( GTK_CONTAINER( ebox ), econ_label[i] );
       }
 
       bulb_label      = gtk_pixmap_new(sprites.bulb[0]->pixmap, NULL);
+      gtk_pixmap_set_build_insensitive(GTK_PIXMAP(bulb_label), FALSE);
       sun_label       = gtk_pixmap_new(sprites.warming[0]->pixmap, NULL);
+      gtk_pixmap_set_build_insensitive(GTK_PIXMAP(sun_label), FALSE);
+      flake_label     = gtk_pixmap_new(sprites.cooling[0]->pixmap, NULL);
+      gtk_pixmap_set_build_insensitive(GTK_PIXMAP(flake_label), FALSE);
       government_label= gtk_pixmap_new(sprites.citizen[7]->pixmap, NULL);
+      gtk_pixmap_set_build_insensitive(GTK_PIXMAP(government_label), FALSE);
       timeout_label   = gtk_label_new("");
 
-      for (i=5; i<9; i++)
+      for (i=0; i<5; i++)
       {
           frame = gtk_frame_new(NULL);
           gtk_widget_set_usize(frame, SMALL_TILE_WIDTH, SMALL_TILE_HEIGHT);
 
-	  if (i==8)
-            gtk_table_attach_defaults(GTK_TABLE(table), frame, i, i + 2, 1, 2);
+	  if (i==4)
+            gtk_table_attach_defaults(GTK_TABLE(table), frame, i, i + 6, 1, 2);
 	  else
             gtk_table_attach_defaults(GTK_TABLE(table), frame, i, i + 1, 1, 2);
 
           switch (i)
           {
-          case 5: gtk_container_add( GTK_CONTAINER( frame ), bulb_label );
+          case 0: gtk_container_add( GTK_CONTAINER( frame ), bulb_label );
               break;
-          case 6: gtk_container_add( GTK_CONTAINER( frame ), sun_label );
+          case 1: gtk_container_add( GTK_CONTAINER( frame ), sun_label );
               break;
-          case 7: gtk_container_add( GTK_CONTAINER( frame ), government_label );
+          case 2: gtk_container_add( GTK_CONTAINER( frame ), flake_label );
               break;
-          case 8: gtk_container_add( GTK_CONTAINER( frame ), timeout_label );
+          case 3: gtk_container_add( GTK_CONTAINER( frame ), government_label );
+              break;
+          case 4: gtk_container_add( GTK_CONTAINER( frame ), timeout_label );
               break;
           }
       }
 
-  turn_done_button = gtk_button_new_with_label( _("Turn Done") );
-  gtk_widget_set_style(turn_done_button, gtk_style_copy(turn_done_button->style));
-  gtk_table_attach_defaults(GTK_TABLE(table), turn_done_button, 0, 5, 1, 2);
+      turn_done_button = gtk_button_new_with_label( _("Turn Done") );
+      gtk_widget_set_style(turn_done_button, gtk_style_copy(turn_done_button->style));
+      gtk_table_attach_defaults(GTK_TABLE(table), turn_done_button, 0, 10, 2, 3);
   }
  
   { /* selected unit status */
@@ -430,13 +469,25 @@ static void setup_widgets(void)
     gtk_table_set_col_spacings(GTK_TABLE(table), 2);
 
     unit_pixmap=gtk_pixcomm_new(root_window, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
-    gtk_table_attach_defaults( GTK_TABLE(table), unit_pixmap, 0, 1, 0, 1 );
     gtk_pixcomm_clear(GTK_PIXCOMM(unit_pixmap), TRUE);
+    unit_pixmap_button=gtk_event_box_new();
+    gtk_container_add(GTK_CONTAINER(unit_pixmap_button),unit_pixmap);
+    gtk_table_attach_defaults( GTK_TABLE(table), unit_pixmap_button, 
+       0, 1, 0, 1 );
+    gtk_signal_connect(GTK_OBJECT(unit_pixmap_button), "button_press_event",
+       GTK_SIGNAL_FUNC(select_pixmap_callback),(gpointer)(-1));
 
     for (i=0; i<num_units_below; i++)
     {
       unit_below_pixmap[i]=gtk_pixcomm_new(root_window, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
-      gtk_table_attach_defaults(GTK_TABLE(table), unit_below_pixmap[i],
+      unit_below_pixmap_button[i]=gtk_event_box_new();
+      gtk_container_add(GTK_CONTAINER(unit_below_pixmap_button[i]),
+        unit_below_pixmap[i]);
+      gtk_signal_connect(GTK_OBJECT(unit_below_pixmap_button[i]), 
+         "button_press_event",
+        GTK_SIGNAL_FUNC(select_pixmap_callback),(gpointer)(i));
+      
+      gtk_table_attach_defaults(GTK_TABLE(table), unit_below_pixmap_button[i],
     	  i, i+1, 1, 2);
       gtk_widget_set_usize(unit_below_pixmap[i],
             NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
@@ -444,6 +495,7 @@ static void setup_widgets(void)
     }
 
     more_arrow_pixmap=gtk_pixmap_new(sprites.right_arrow->pixmap, NULL);
+    gtk_pixmap_set_build_insensitive(GTK_PIXMAP(more_arrow_pixmap), FALSE);
     gtk_table_attach_defaults(GTK_TABLE(table), more_arrow_pixmap, 4, 5, 1, 2);
   }
 
@@ -484,8 +536,9 @@ static void setup_widgets(void)
   {   /* the message window */
       GtkWidget *text;
 
-      gtk_paned_pack2(GTK_PANED(paned), detached_widget_new (&avbox),
-								TRUE, TRUE);
+      ahbox = detached_widget_new();
+      gtk_paned_pack2(GTK_PANED(paned), ahbox, TRUE, TRUE);
+      avbox = detached_widget_fill(ahbox);
 
       hbox = gtk_hbox_new(FALSE, 0);
       gtk_box_pack_start(GTK_BOX(avbox), hbox, TRUE, TRUE, 0);
@@ -511,7 +564,7 @@ static void setup_widgets(void)
       gtk_box_pack_start (GTK_BOX(hbox), text_scrollbar, FALSE, FALSE, 0);
       gtk_widget_realize (text_scrollbar);
 
-      append_output_window(
+      set_output_window_text(
       _("Freeciv is free software and you are welcome to distribute copies of"
       " it\nunder certain conditions; See the \"Copying\" item on the Help"
       " menu.\nNow.. Go give'em hell!") );
@@ -525,6 +578,7 @@ static void setup_widgets(void)
 /*  gtk_key_snooper_install(keyboard_handler, NULL);*/
 
   gtk_widget_show_all(vbox);
+  gtk_widget_hide(more_arrow_pixmap);
 }
 
 /**************************************************************************
@@ -532,32 +586,38 @@ static void setup_widgets(void)
 **************************************************************************/
 void ui_main(int argc, char **argv)
 {
-  GdkBitmap	      *icon_bitmap;
+  GdkBitmap *icon_bitmap;
+  gchar *homedir, *rc_file;
 
   parse_options(argc, argv);
 
+  /* tell GTK+ library which locale */
+#ifdef ENABLE_NLS
+  gtk_set_locale();
+#endif
+
   /* GTK withdraw gtk options */
   /* Process GTK arguments */
   gtk_init(&argc, &argv);
 
+  /* Load resources */
+  gtk_rc_parse_string(fallback_resources);
+
+  homedir = g_get_home_dir();	/* should i gfree() this also? --vasc */
+  rc_file = g_strdup_printf("%s%s%s", homedir, G_DIR_SEPARATOR_S, "freeciv.rc");
+  gtk_rc_parse(rc_file);
+  g_free (rc_file);
+
+
   display_color_type=get_visual();
 
   toplevel = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_realize (toplevel);
+  gtk_widget_set_name(toplevel, "Freeciv");
 
   root_window=toplevel->window;
   init_color_system();
 
-  notify_dialog_style=gtk_style_new();
-  gdk_font_unref (notify_dialog_style->font);
-  notify_dialog_style->font=gdk_font_load (NOTIFY_DIALOG_FONT);
-  gdk_font_ref (notify_dialog_style->font);
-
-  city_dialog_style=gtk_style_new();
-  gdk_font_unref (city_dialog_style->font);
-  city_dialog_style->font=gdk_font_load (FIXED_10_BFONT);
-  gdk_font_ref (city_dialog_style->font);
-
   gtk_signal_connect( GTK_OBJECT(toplevel),"delete_event",
       GTK_SIGNAL_FUNC(gtk_main_quit),NULL );
 
@@ -576,6 +636,12 @@ void ui_main(int argc, char **argv)
       exit(1);
   }
 
+  if (!(city_productions_font = gdk_font_load(city_productions_font_name))) {
+    freelog(LOG_FATAL, "failed loading font: %s",
+	    city_productions_font_name);
+    exit(1);
+  }
+
   fill_bg_gc = gdk_gc_new(root_window);
   fill_tile_gc = gdk_gc_new(root_window);
   gdk_gc_set_fill(fill_tile_gc, GDK_STIPPLED);
@@ -676,9 +742,9 @@ void add_net_input(int sock)
 void remove_net_input(void)
 {
   gdk_input_remove(gdk_input_id);
+  gdk_window_set_cursor(root_window, 0);
 }
 
-#ifdef UNUSED  /* used in show_info_popup() below */
 /**************************************************************************
 ...
 **************************************************************************/
@@ -690,9 +756,7 @@ show_info_button_release(GtkWidget *w, GdkEventButton *event)
   gtk_widget_destroy(w);
   return FALSE;
 }
-#endif /* UNUSED */
 
-#ifdef UNUSED  /* used in commented-out code */
 /**************************************************************************
 ...
 **************************************************************************/
@@ -736,9 +800,16 @@ static gint show_info_popup(GtkWidget *w, GdkEventButton *ev)
   }
   return TRUE;
 }
-#endif /* UNUSED */
 
 
+/**************************************************************************
+...
+**************************************************************************/
+void sound_bell(void)
+{
+  gdk_beep();
+}
+
 
 /**************************************************************************
 ...
@@ -749,9 +820,6 @@ void enable_turn_done_button(void)
     user_ended_turn();
   gtk_widget_set_sensitive(turn_done_button,
 		!game.player_ptr->ai.control||ai_manual_turn_done);
-
-  if(sound_bell_at_new_turn)
-    gdk_beep();
 }
 
 
@@ -795,11 +863,58 @@ static gint timer_callback(gpointer data)
     
     if(flip) {
       update_timeout_label();
-    if(seconds_to_turndone)
-        seconds_to_turndone--;
+      if(seconds_to_turndone > 0)
+	seconds_to_turndone--;
+      else
+	seconds_to_turndone = 0;
     }
     
     flip=!flip;
   }
   return TRUE;
 }
+
+/**************************************************************************
+  Set one of the unit icons in information area based on punit.
+  Use punit==NULL to clear icon.
+  Index 'idx' is -1 for "active unit", or 0 to (num_units_below-1) for
+  units below.  Also updates unit_ids[idx] for idx>=0.
+**************************************************************************/
+void set_unit_icon(int idx, struct unit *punit)
+{
+  GtkWidget *w;
+  
+  assert(idx>=-1 && idx<num_units_below);
+  if (idx == -1) {
+    w = unit_pixmap;
+  } else {
+    w = unit_below_pixmap[idx];
+    unit_ids[idx] = punit ? punit->id : 0;
+  }
+  
+  if (punit) {
+    gtk_pixcomm_clear(GTK_PIXCOMM(w), FALSE);
+    put_unit_gpixmap(punit, GTK_PIXCOMM(w), 0, 0);
+  } else {
+    gtk_pixcomm_clear(GTK_PIXCOMM(w), TRUE);
+  }
+  
+}
+
+/**************************************************************************
+  Set the "more arrow" for the unit icons to on(1) or off(0).
+  Maintains a static record of current state to avoid unnecessary redraws.
+  Note initial state should match initial gui setup (off).
+**************************************************************************/
+void set_unit_icons_more_arrow(int onoff)
+{
+  static int showing = 0;
+  if (onoff==1 && !showing) {
+    gtk_widget_show(more_arrow_pixmap);
+    showing = 1;
+  }
+  else if(onoff==0 && showing) {
+    gtk_widget_hide(more_arrow_pixmap);
+    showing = 0;
+  }
+}
