@@ -55,9 +55,6 @@ int city_workers_color=COLOR_STD_WHITE;
 extern GtkWidget *map_canvas;
 extern GtkWidget *toplevel;
 extern GtkWidget *turn_done_button;
-extern int smooth_move_units; /* from options.o */
-extern int auto_center_on_unit;
-extern int draw_map_grid;
 
 /**************************************************************************
 ...
@@ -116,7 +113,7 @@ static void popit(GdkEventButton *event, int xtile, int ytile)
   struct unit *punit;
   struct tile *ptile=map_get_tile(xtile, ytile);
 
-  if(ptile->known>=TILE_KNOWN) {
+  if(ptile->known>=TILE_KNOWN_FOGGED) {
     p=gtk_window_new(GTK_WINDOW_POPUP);
     b=gtk_vbox_new(FALSE, 0);
     gtk_container_add(GTK_CONTAINER(p), b);
@@ -135,7 +132,7 @@ static void popit(GdkEventButton *event, int xtile, int ytile)
     }
     
     if((pcity=map_get_city(xtile, ytile))) {
-      my_snprintf(s, sizeof(s), "City: %s(%s)", pcity->name, 
+      my_snprintf(s, sizeof(s), _("City: %s(%s)"), pcity->name, 
 		  get_nation_name(game.players[pcity->owner].nation));
       gtk_widget_new(GTK_TYPE_LABEL, "GtkWidget::parent", b,
 				     "GtkLabel::label", s,
@@ -159,13 +156,13 @@ static void popit(GdkEventButton *event, int xtile, int ytile)
       count++;
     }
     
-    if((punit=player_find_visible_unit(game.player_ptr, ptile)) && !pcity) {
+    if((punit=find_visible_unit(ptile)) && !pcity) {
       char cn[64];
       struct unit_type *ptype=get_unit_type(punit->type);
       cn[0]='\0';
       if(punit->owner==game.player_idx) {
 	struct city *pcity;
-	pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+	pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
 	if(pcity)
 	  my_snprintf(cn, sizeof(cn), "/%s", pcity->name);
       }
@@ -182,10 +179,10 @@ static void popit(GdkEventButton *event, int xtile, int ytile)
 	  my_snprintf(uc, sizeof(uc), _("  (%d more)"),
 		      unit_list_size(&ptile->units) - 1);
 	}
-        my_snprintf(s, sizeof(s), "A:%d D:%d FP:%d HP:%d/%d%s%s",
+        my_snprintf(s, sizeof(s), _("A:%d D:%d FP:%d HP:%d/%d%s%s"),
 		    ptype->attack_strength, 
 		    ptype->defense_strength, ptype->firepower, punit->hp, 
-		    ptype->hp, punit->veteran?" V":"", uc);
+		    ptype->hp, punit->veteran?_(" V"):"", uc);
 
         if(punit->activity==ACTIVITY_GOTO || punit->connecting)  {
 	  cross_head->x = punit->goto_dest_x;
@@ -193,7 +190,7 @@ static void popit(GdkEventButton *event, int xtile, int ytile)
 	  cross_head++;
         }
       } else {
-        my_snprintf(s, sizeof(s), "A:%d D:%d FP:%d HP:%d0%%",
+        my_snprintf(s, sizeof(s), _("A:%d D:%d FP:%d HP:%d0%%"),
 		    ptype->attack_strength, 
 		    ptype->defense_strength, ptype->firepower, 
 		    (punit->hp*100/ptype->hp + 9)/10 );
@@ -282,8 +279,10 @@ gint butt_down_mapcanvas(GtkWidget *widget, GdkEventButton *event)
   xtile=map_adjust_x(map_view_x0+event->x/NORMAL_TILE_WIDTH);
   ytile=map_adjust_y(map_view_y0+event->y/NORMAL_TILE_HEIGHT);
 
-	if(event->button==1) 
+  if(event->button==1) {
     do_map_click(xtile, ytile);
+    gtk_widget_grab_focus(turn_done_button);
+  }
   else if((event->button==2)||(event->state&GDK_CONTROL_MASK))
     popit(event, xtile, ytile);
   else
@@ -373,6 +372,9 @@ gint butt_down_overviewcanvas(GtkWidget *widget, GdkEventButton *event)
 {
   int xtile, ytile;
 
+  if (event->type != GDK_BUTTON_PRESS)
+    return TRUE; /* Double-clicks? Triple-clicks? No thanks! */
+
   xtile=event->x/2-(map.xsize/2-(map_view_x0+map_canvas_store_twidth/2));
   ytile=event->y/2;
   
@@ -427,10 +429,6 @@ gint key_city_workers(GtkWidget *widget, GdkEventKey *ev)
 void focus_to_next_unit(void)
 {
   advance_unit_focus();
-/*
-  goto_state=0;
-  nuke_state=0;
-  set_unit_focus(punit_focus); */  /* done in advance_unit_focus */
 }
 
 /**************************************************************************
