@@ -34,6 +34,7 @@
 #include "timing.h"
 
 #include "civclient.h"
+#include "climisc.h"
 #include "colors.h"
 #include "control.h" /* set_unit_focus_no_center and get_unit_in_focus */
 #include "graphics.h"
@@ -44,10 +45,24 @@
 
 #include "mapview.h"
 
-extern int		seconds_to_turndone;
+/*
+The bottom row of the map was sometimes hidden.
 
-/* adjusted depending on tile size: */
-int num_units_below = MAX_NUM_UNITS_BELOW;
+As of now the top left corner is always aligned with the tiles. This is what
+causes the problem in the first place. The ideal solution would be to align the
+window with the bottom left tiles if you tried to center the window on a tile
+closer than (screen_tiles_height/2 -1) to the south pole.
+
+But, for now, I just grepped for occurences where the ysize (or the values
+derived from it) were used, and those places that had relevance to drawing the
+map, and I added 1 (using the EXTRA_BOTTOM_ROW constant).
+
+-Thue
+*/
+#define EXTRA_BOTTOM_ROW 1
+
+
+extern int		seconds_to_turndone;
 
 extern int goto_state;
 extern int paradrop_state;
@@ -58,16 +73,13 @@ extern GtkWidget *	main_label_info;
 extern GtkWidget *	econ_label			[10];
 extern GtkWidget *	bulb_label;
 extern GtkWidget *	sun_label;
+extern GtkWidget *	flake_label;
 extern GtkWidget *	government_label;
 extern GtkWidget *	timeout_label;
 extern GtkWidget *	turn_done_button;
 
 extern GtkWidget *	unit_info_frame, *unit_info_label;
 
-extern GtkWidget *	unit_pixmap;
-extern GtkWidget *	unit_below_pixmap		[MAX_NUM_UNITS_BELOW];
-extern GtkWidget *	more_arrow_pixmap;
-
 extern GtkWidget *	map_canvas;			/* GtkDrawingArea */
 extern GtkWidget *	overview_canvas;		/* GtkDrawingArea */
 
@@ -106,14 +118,13 @@ extern GdkGC *		mask_fg_gc;
 extern GdkGC *		mask_bg_gc;
 
 extern GdkFont *	main_font;
-
+extern GdkFont *city_productions_font;
 
 static void pixmap_put_overlay_tile(GdkPixmap *pixmap,
 				    int x, int y, struct Sprite *ssprite);
 static void put_overlay_tile_gpixmap(GtkPixcomm *pixmap,
 				     int x, int y, struct Sprite *ssprite);
-static void show_city_names(void);
-
+static void show_city_descriptions(void);
 
 /* the intro picture is held in this pixmap, which is scaled to
    the screen size */
@@ -139,18 +150,19 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
   struct unit *losing_unit = (hp0 == 0 ? punit0 : punit1);
   int i;
 
-  set_unit_focus_no_center(punit0);
-  set_unit_focus_no_center(punit1);
-  
   if (!do_combat_animation) {
     punit0->hp = hp0;
     punit1->hp = hp1;
+
+    set_units_in_combat(NULL, NULL);
     refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
     refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
 
     return;
   }
 
+  set_units_in_combat(punit0, punit1);
+
   do {
     anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
 
@@ -189,49 +201,17 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
     usleep_since_timer_start(anim_timer, 20000);
   }
 
+  set_units_in_combat(NULL, NULL);
   refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
   refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
 }
 
 
-/**************************************************************************
-...
-**************************************************************************/
-void blink_active_unit(void)
-{
-  static int is_shown;
-  struct unit *punit;
-  
-  if((punit=get_unit_in_focus())) {
-    struct tile *ptile;
-    ptile=map_get_tile(punit->x, punit->y);
-
-    if(is_shown) {
-      struct unit_list units;
-      units=ptile->units;
-      unit_list_init(&ptile->units);
-      refresh_tile_mapcanvas(punit->x, punit->y, 1);
-      ptile->units=units;
-    }
-    else {
-      /* make sure that the blinking unit is always on the top */      
-      unit_list_unlink(&ptile->units, punit);
-      unit_list_insert(&ptile->units, punit);
-      refresh_tile_mapcanvas(punit->x, punit->y, 1);
-    }
-      
-    is_shown=!is_shown;
-  }
-}
-
-
 /**************************************************************************
 ...
 **************************************************************************/
 void set_overview_dimensions(int x, int y)
 {
-  gtk_widget_set_usize(overview_canvas, 2*x, 2*y);
-
   overview_canvas_store_width=2*x;
   overview_canvas_store_height=2*y;
 
@@ -241,6 +221,14 @@ void set_overview_dimensions(int x, int y)
   overview_canvas_store	= gdk_pixmap_new(root_window,
 			  overview_canvas_store_width,
 			  overview_canvas_store_height, -1);
+
+  gdk_gc_set_foreground(fill_bg_gc, colors_standard[COLOR_STD_BLACK]);
+  gdk_draw_rectangle(overview_canvas_store, fill_bg_gc, TRUE,
+		     0, 0,
+		     overview_canvas_store_width, overview_canvas_store_height);
+
+  gtk_widget_set_usize(overview_canvas, 2*x, 2*y);
+  update_map_canvas_scrollbars_size();
 }
 
 
@@ -276,7 +264,10 @@ void update_timeout_label(void)
 {
   char buffer[512];
 
-  my_snprintf(buffer, sizeof(buffer), "%d", seconds_to_turndone);
+  if (game.timeout <= 0)
+    sz_strlcpy(buffer, Q_("?timeout:off"));
+  else
+    format_duration(buffer, sizeof(buffer), seconds_to_turndone);
   gtk_set_label(timeout_label, buffer);
 }
 
@@ -301,12 +292,10 @@ void update_info_label( void )
 
   gtk_set_label(main_label_info, buffer);
 
-  if (game.heating>7) game.heating=7;
-  set_bulb_sol_government(8*game.player_ptr->research.researched/
-                         (research_time(game.player_ptr)+1),
-                         game.heating, 
-                         game.player_ptr->government);
-
+  set_indicator_icons(client_research_sprite(),
+		      client_warming_sprite(),
+		      client_cooling_sprite(),
+		      game.player_ptr->government);
 
   d=0;
   for(;d<(game.player_ptr->economic.luxury)/10;d++)
@@ -325,14 +314,21 @@ void update_info_label( void )
 }
 
 /**************************************************************************
-...
+  Update the information label which gives info on the current unit and the
+  square under the current unit, for specified unit.  Note that in practice
+  punit is almost always (or maybe strictly always?) the focus unit.
+  Clears label if punit is NULL.
+  Also updates the cursor for the map_canvas (this is related because the
+  info label includes a "select destination" prompt etc).
+  Also calls update_unit_pix_label() to update the icons for units on this
+  square.
 **************************************************************************/
 void update_unit_info_label(struct unit *punit)
 {
   if(punit) {
     char buffer[512];
     struct city *pcity;
-    pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+    pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
 
     my_snprintf(buffer, sizeof(buffer), "%s %s", 
             get_unit_type(punit->type)->name,
@@ -368,88 +364,6 @@ void update_unit_info_label(struct unit *punit)
 }
 
 
-/**************************************************************************
-...
-**************************************************************************/
-void update_unit_pix_label(struct unit *punit)
-{
-  int i;
-  /* what initialises these statics? */
-  static enum unit_activity uactivity = ACTIVITY_UNKNOWN;
-  static int utemplate = U_LAST;
-  static int unit_ids[MAX_NUM_UNITS_BELOW];
-  static int showing_arrow=0;
-  struct genlist_iterator myiter;
-  
-  if(punit) {
-    if(punit->type!=utemplate || punit->activity!=uactivity) {
-      gtk_pixcomm_clear(GTK_PIXCOMM(unit_pixmap), FALSE); /* STG */
-      put_unit_gpixmap(punit, GTK_PIXCOMM(unit_pixmap), 0, 0);
-      utemplate=punit->type;
-      uactivity=punit->activity;
-    }
-    genlist_iterator_init(&myiter, 
-			  &(map_get_tile(punit->x, punit->y)->units.list), 0);
-    
-    for(i=0; i<num_units_below && ITERATOR_PTR(myiter); i++) {
-      int id;
-      id=ITERATOR_PTR(myiter) ? ((struct unit *)ITERATOR_PTR(myiter))->id : 0;
-      if(id==punit->id) {
-	ITERATOR_NEXT(myiter);
-	i--;
-	continue;
-      }
-      
-      /* IS THIS INTENTIONAL?? - mjd */
-      if(1 || unit_ids[i]!=id) {
-	if(id) {
-	  gtk_pixcomm_clear(GTK_PIXCOMM(unit_below_pixmap[i]), FALSE); /* STG */
-	  put_unit_gpixmap((struct unit *)ITERATOR_PTR(myiter),
-			  GTK_PIXCOMM(unit_below_pixmap[i]),
-			  0, 0);
-	}
-	else
-	  gtk_pixcomm_clear(GTK_PIXCOMM(unit_below_pixmap[i]), TRUE);
-
-	  
-	unit_ids[i]=id;
-      }
-      ITERATOR_NEXT(myiter);
-    }
-
-    
-    for(; i<num_units_below; i++) {
-      gtk_pixcomm_clear(GTK_PIXCOMM(unit_below_pixmap[i]), TRUE);
-      unit_ids[i]=0;
-    }
-
-    
-    if(ITERATOR_PTR(myiter)) {
-      if(!showing_arrow) {
-	gtk_widget_show(more_arrow_pixmap);
-	showing_arrow=1;
-      }
-    }
-    else {
-      if(showing_arrow) {
-	gtk_widget_hide(more_arrow_pixmap);
-	showing_arrow=0;
-      }
-    }
-  }
-  else {
-    gtk_pixcomm_clear(GTK_PIXCOMM(unit_pixmap), TRUE);
-    utemplate=U_LAST;
-    uactivity=ACTIVITY_UNKNOWN;
-    for(i=0; i<num_units_below; i++) {
-      gtk_pixcomm_clear(GTK_PIXCOMM(unit_below_pixmap[i]), TRUE);
-      unit_ids[i]=0;
-    }
-    gtk_widget_hide(more_arrow_pixmap);
-    showing_arrow=0;
-  }
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -481,22 +395,23 @@ SPRITE *get_citizen_sprite(int frame)
 /**************************************************************************
 ...
 **************************************************************************/
-void set_bulb_sol_government(int bulb, int sol, int government)
+void set_indicator_icons(int bulb, int sol, int flake, int gov)
 {
   struct Sprite *gov_sprite;
-  
+
   bulb = CLIP(0, bulb, NUM_TILES_PROGRESS-1);
-  sol  = CLIP(0, sol, NUM_TILES_PROGRESS-1);
+  sol = CLIP(0, sol, NUM_TILES_PROGRESS-1);
+  flake = CLIP(0, flake, NUM_TILES_PROGRESS-1);
 
   gtk_pixmap_set(GTK_PIXMAP(bulb_label), sprites.bulb[bulb]->pixmap, NULL);
-
   gtk_pixmap_set(GTK_PIXMAP(sun_label), sprites.warming[sol]->pixmap, NULL);
-  
+  gtk_pixmap_set(GTK_PIXMAP(flake_label), sprites.cooling[flake]->pixmap, NULL);
+
   if (game.government_count==0) {
     /* not sure what to do here */
     gov_sprite = sprites.citizen[7]; 
   } else {
-    gov_sprite = get_government(government)->sprite;
+    gov_sprite = get_government(gov)->sprite;
   }
   gtk_pixmap_set(GTK_PIXMAP(government_label), gov_sprite->pixmap, NULL);
 }
@@ -555,62 +470,83 @@ int tile_visible_mapcanvas(int x, int y)
 **************************************************************************/
 int tile_visible_and_not_on_border_mapcanvas(int x, int y)
 {
-  return (y>=map_view_y0+1 && y<map_view_y0+map_canvas_store_theight-1 &&
-	  ((x>=map_view_x0+1 && x<map_view_x0+map_canvas_store_twidth-1) ||
-	   (x+map.xsize>=map_view_x0+1 && 
-	    x+map.xsize<map_view_x0+map_canvas_store_twidth-1)));
+  return ((y>=map_view_y0+2 || (y >= map_view_y0 && map_view_y0 == 0))
+	  && (y<map_view_y0+map_canvas_store_theight-2 ||
+	      (y<map_view_y0+map_canvas_store_theight &&
+	       map_view_y0 + map_canvas_store_theight-EXTRA_BOTTOM_ROW == map.ysize))
+	  && ((x>=map_view_x0+2 && x<map_view_x0+map_canvas_store_twidth-2) ||
+	      (x+map.xsize>=map_view_x0+2
+	       && x+map.xsize<map_view_x0+map_canvas_store_twidth-2)));
 }
 
 /**************************************************************************
-...
+Animates punit's "smooth" move from (x0,y0) to (x0+dx,y0+dy).
+Note: Works only for adjacent-square moves.
+(Tiles need not be square.)
 **************************************************************************/
 void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
 {
+  static struct timer *anim_timer = NULL; 
   int dest_x, dest_y;
-  int i;
 
-  dest_x=map_adjust_x(x0+dx);
-  dest_y=map_adjust_y(y0+dy);
+  /* only works for adjacent-square moves */
+  if ((dx < -1) || (dx > 1) || (dy < -1) || (dy > 1) ||
+      ((dx == 0) && (dy == 0))) {
+    return;
+  }
 
-  if(player_can_see_unit(game.player_ptr, punit) && (
-     tile_visible_mapcanvas(x0, y0) || 
-     tile_visible_mapcanvas(dest_x, dest_y))) {
-    int x, y;
+  dest_x = map_adjust_x(x0+dx);
+  dest_y = map_adjust_y(y0+dy);
 
-    gdk_draw_rectangle(single_tile_pixmap, civ_gc, TRUE, 0, 0, -1, -1);
-    put_unit_pixmap(punit, single_tile_pixmap, 0, 0);
+  if (player_can_see_unit(game.player_ptr, punit) &&
+      (tile_visible_mapcanvas(x0, y0) ||
+       tile_visible_mapcanvas(dest_x, dest_y))) {
+    int i, steps;
+    int start_x, start_y;
+    int this_x, this_y;
 
-    if(x0>=map_view_x0)
-      x=(x0-map_view_x0)*NORMAL_TILE_WIDTH;
-    else
-      x=(map.xsize-map_view_x0+x0)*NORMAL_TILE_WIDTH;
+    if (smooth_move_unit_steps < 2) {
+      steps = 2;
+    } else if (smooth_move_unit_steps >
+	       MIN(NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT)) {
+      steps = MIN(NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    } else {
+      steps = smooth_move_unit_steps;
+    }
 
-    y=(y0-map_view_y0)*NORMAL_TILE_HEIGHT;
+    if(x0 >= map_view_x0) {
+      start_x = (x0 - map_view_x0) * NORMAL_TILE_WIDTH;
+    } else {
+      start_x = (map.xsize - map_view_x0 + x0) * NORMAL_TILE_WIDTH;
+    }
+    start_y = (y0 - map_view_y0) * NORMAL_TILE_HEIGHT;
 
-    for(i=0; i<NORMAL_TILE_WIDTH; i++) {
-      if(dy>0)
-	gdk_draw_pixmap( map_canvas->window, civ_gc, map_canvas_store,
-		  x, y, x, y, NORMAL_TILE_WIDTH, 1 );
-      else if(dy<0)
-	gdk_draw_pixmap( map_canvas->window, civ_gc, map_canvas_store,
-		  x, y+NORMAL_TILE_HEIGHT-1, x, y+NORMAL_TILE_HEIGHT-1,
-		  NORMAL_TILE_WIDTH, 1 );
-      
-      if(dx>0)
-	gdk_draw_pixmap( map_canvas->window, civ_gc, map_canvas_store,
-		  x, y, x, y, 1, NORMAL_TILE_HEIGHT );
-      else if(dx<0)
-	gdk_draw_pixmap( map_canvas->window, civ_gc, map_canvas_store,
-		  x+NORMAL_TILE_WIDTH-1, y, x+NORMAL_TILE_WIDTH-1, y, 1,
-		  NORMAL_TILE_HEIGHT );
+    this_x = start_x;
+    this_y = start_y;
 
-      x+=dx; y+=dy;
-      
-      gdk_draw_pixmap( map_canvas->window, civ_gc, single_tile_pixmap,
-		  0, 0, x, y, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT );
+    for (i = 1; i <= steps; i++) {
+      anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
 
-      gdk_flush ();
-/*      poll(0, 0, 1);*/	/* this is an evil hack */
+      gdk_draw_pixmap(map_canvas->window, civ_gc, map_canvas_store,
+		      this_x, this_y, this_x, this_y,
+		      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+
+      this_x = start_x + (dx * ((i * NORMAL_TILE_WIDTH) / steps));
+      this_y = start_y + (dy * ((i * NORMAL_TILE_HEIGHT) / steps));
+
+      gdk_draw_pixmap(single_tile_pixmap, civ_gc, map_canvas_store,
+		      this_x, this_y, 0, 0,
+		      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+      put_unit_pixmap(punit, single_tile_pixmap, 0, 0);
+
+      gdk_draw_pixmap(map_canvas->window, civ_gc, single_tile_pixmap,
+		      0, 0, this_x, this_y,
+		      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+
+      gdk_flush();
+      if (i < steps) {
+	usleep_since_timer_start(anim_timer, 10000);
+      }
     }
   }
 }
@@ -633,8 +569,9 @@ void center_tile_mapcanvas(int x, int y)
 
   new_map_view_x0=map_adjust_x(x-map_canvas_store_twidth/2);
   new_map_view_y0=map_adjust_y(y-map_canvas_store_theight/2);
-  if(new_map_view_y0>map.ysize-map_canvas_store_theight)
-     new_map_view_y0=map_adjust_y(map.ysize-map_canvas_store_theight);
+  if(new_map_view_y0>map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight)
+    new_map_view_y0=
+      map_adjust_y(map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight);
 
   map_view_x0=new_map_view_x0;
   map_view_y0=new_map_view_y0;
@@ -668,32 +605,11 @@ gint overview_canvas_expose( GtkWidget *widget, GdkEventExpose *event )
 **************************************************************************/
 static void set_overview_tile_foreground_color(int x, int y)
 {
-  struct tile *ptile=map_get_tile(x, y);
-  struct unit *punit;
-  struct city *pcity;
-  
-  if(!ptile->known)
-    gdk_gc_set_foreground (fill_bg_gc, colors_standard[COLOR_STD_BLACK]);
-  else if((punit=player_find_visible_unit(game.player_ptr, ptile))) {
-    if(punit->owner==game.player_idx)
-      gdk_gc_set_foreground (fill_bg_gc, colors_standard[COLOR_STD_YELLOW]);
-    else
-      gdk_gc_set_foreground (fill_bg_gc, colors_standard[COLOR_STD_RED]);
-  }
-  else if((pcity=map_get_city(x, y))) {
-    if(pcity->owner==game.player_idx)
-      gdk_gc_set_foreground (fill_bg_gc, colors_standard[COLOR_STD_WHITE]);
-    else
-      gdk_gc_set_foreground (fill_bg_gc, colors_standard[COLOR_STD_CYAN]);
-  }
-  else if(ptile->terrain==T_OCEAN)
-    gdk_gc_set_foreground (fill_bg_gc, colors_standard[COLOR_STD_OCEAN]);
-  else
-    gdk_gc_set_foreground (fill_bg_gc, colors_standard[COLOR_STD_GROUND]);
+  gdk_gc_set_foreground (fill_bg_gc,
+			 colors_standard[overview_tile_color(x, y)]);
 }
 
 
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -717,10 +633,11 @@ void refresh_overview_canvas(void)
 **************************************************************************/
 void overview_update_tile(int x, int y)
 {
-  int pos=x+map.xsize/2-(map_view_x0+map_canvas_store_twidth/2);
+  int pos = x + map.xsize/2 - (map_view_x0 + map_canvas_store_twidth/2);
   
-  if(pos>=map.xsize)
-    pos-=map.xsize;
+  pos %= map.xsize;
+  if (pos < 0)
+    pos += map.xsize;
   
   set_overview_tile_foreground_color(x, y);
   gdk_draw_rectangle( overview_canvas_store, fill_bg_gc, TRUE, x*2, y*2,
@@ -800,10 +717,11 @@ gint map_canvas_expose( GtkWidget *widget, GdkEventExpose *event )
   		    tile_height*NORMAL_TILE_HEIGHT,
   		    -1 );
   				 
-    gdk_draw_rectangle( map_canvas_store, fill_bg_gc, TRUE,
-  		    0, 0,
-  		    NORMAL_TILE_WIDTH*map_canvas_store_twidth,
-  		    NORMAL_TILE_HEIGHT*map_canvas_store_theight );
+    gdk_gc_set_foreground(fill_bg_gc, colors_standard[COLOR_STD_BLACK]);
+    gdk_draw_rectangle(map_canvas_store, fill_bg_gc, TRUE,
+		       0, 0,
+		       NORMAL_TILE_WIDTH*map_canvas_store_twidth,
+		       NORMAL_TILE_HEIGHT*map_canvas_store_theight );
     update_map_canvas_scrollbars_size();
     map_resized=TRUE;
   }
@@ -847,6 +765,7 @@ gint map_canvas_expose( GtkWidget *widget, GdkEventExpose *event )
 	gdk_draw_pixmap( map_canvas->window, civ_gc, map_canvas_store,
 		event->area.x, event->area.y, event->area.x, event->area.y,
 		event->area.width, event->area.height );
+	show_city_descriptions();
       }
     }
     refresh_overview_canvas();
@@ -876,10 +795,10 @@ void update_map_canvas(int tile_x, int tile_y, int width, int height,
 		width*NORMAL_TILE_WIDTH,
 		height*NORMAL_TILE_HEIGHT );
 
-    if(width==map_canvas_store_twidth && height==map_canvas_store_theight) {
-      show_city_names();
+    if (width == map_canvas_store_twidth
+	&& height == map_canvas_store_theight) {
+      show_city_descriptions();
     }
-    
   }
 }
 
@@ -907,7 +826,7 @@ void update_map_canvas_scrollbars_size(void)
 {
   map_hadj=gtk_adjustment_new(-1, 0, map.xsize, 1,
 	   map_canvas_store_twidth, map_canvas_store_twidth);
-  map_vadj=gtk_adjustment_new(-1, 0, map.ysize, 1,
+  map_vadj=gtk_adjustment_new(-1, 0, map.ysize+EXTRA_BOTTOM_ROW, 1,
 	   map_canvas_store_theight, map_canvas_store_theight);
   gtk_range_set_adjustment(GTK_RANGE(map_horizontal_scrollbar),
 	GTK_ADJUSTMENT(map_hadj));
@@ -920,87 +839,99 @@ void update_map_canvas_scrollbars_size(void)
 	GTK_SIGNAL_FUNC(scrollbar_jump_callback), (gpointer)FALSE);
 }
 
+/**************************************************************************
+ Update display of descriptions associated with cities on the main map.
+**************************************************************************/
+void update_city_descriptions(void)
+{
+  update_map_canvas(0, 0, map_canvas_store_twidth,
+		    map_canvas_store_theight, 1);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
-static void show_city_names(void)
+static void draw_shadowed_string(GdkDrawable * drawable,
+				 GdkFont * font,
+				 GdkGC * black_gc,
+				 GdkGC * white_gc,
+				 gint x, gint y, const gchar * string)
+{
+  gdk_draw_string(drawable, font, black_gc, x + 1, y + 1, string);
+  gdk_draw_string(drawable, font, white_gc, x, y, string);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static void show_city_descriptions(void)
 {
   int x, y;
+  static char buffer[512];
 
-  for(y=0; y<map_canvas_store_theight; ++y) { 
-    int ry=map_view_y0+y;
-    for(x=0; x<map_canvas_store_twidth; ++x) { 
-      int rx=(map_view_x0+x)%map.xsize;
+  if (!draw_city_names && !draw_city_productions)
+    return;
+
+  for (y = 0; y < map_canvas_store_theight; ++y) {
+    int ry = map_view_y0 + y;
+    for (x = 0; x < map_canvas_store_twidth; ++x) {
+      int rx = (map_view_x0 + x) % map.xsize;
       struct city *pcity;
-      if((pcity=map_get_city(rx, ry))) {
-	int w=gdk_string_width(main_font, pcity->name);
-						  /*FIXME*/	
-	gdk_draw_string( map_canvas->window, main_font,
+      if ((pcity = map_get_city(rx, ry))) {
+	int w;
+
+	if (draw_city_names) {
+	  my_snprintf(buffer, sizeof(buffer), "%s", pcity->name);
+	  w = gdk_string_width(main_font, buffer);
+	  draw_shadowed_string(map_canvas->window, main_font,
 		    toplevel->style->black_gc,
-		    x*NORMAL_TILE_WIDTH+NORMAL_TILE_WIDTH/2-w/2+1,
-		    (y+1)*NORMAL_TILE_HEIGHT+main_font->ascent+1,
-		    pcity->name );
-	gdk_draw_string( map_canvas->window, main_font,
 		    toplevel->style->white_gc,
-		    x*NORMAL_TILE_WIDTH+NORMAL_TILE_WIDTH/2-w/2, 
-		    (y+1)*NORMAL_TILE_HEIGHT+main_font->ascent,
-		    pcity->name );
+			       x * NORMAL_TILE_WIDTH +
+			       NORMAL_TILE_WIDTH / 2 - w / 2,
+			       (y + 1) * NORMAL_TILE_HEIGHT +
+			       main_font->ascent, buffer);
+	}
+
+	if (draw_city_productions && (pcity->owner==game.player_idx)) {
+	  int turns, y_offset;
+	  struct unit_type *punit_type;
+	  struct impr_type *pimprovement_type;
+
+	  turns = city_turns_to_build(pcity, pcity->currently_building,
+				      pcity->is_building_unit);
+
+	  if (pcity->is_building_unit) {
+	    punit_type = get_unit_type(pcity->currently_building);
+	    my_snprintf(buffer, sizeof(buffer), "%s %d",
+			punit_type->name, turns);
+	  } else {
+	    pimprovement_type =
+		get_improvement_type(pcity->currently_building);
+	    if (pcity->currently_building == B_CAPITAL) {
+	      sz_strlcpy(buffer, pimprovement_type->name);
+	    } else {
+	      my_snprintf(buffer, sizeof(buffer), "%s %d",
+			  pimprovement_type->name, turns);
+	    }
+	  }
+	  if (draw_city_names)
+	    y_offset = main_font->ascent + main_font->descent;
+	  else
+	    y_offset = 0;
+	  w = gdk_string_width(city_productions_font, buffer);
+	  draw_shadowed_string(map_canvas->window, city_productions_font,
+			       toplevel->style->black_gc,
+			       toplevel->style->white_gc,
+			       x * NORMAL_TILE_WIDTH +
+			       NORMAL_TILE_WIDTH / 2 - w / 2,
+			       (y + 1) * NORMAL_TILE_HEIGHT +
+			       main_font->ascent + y_offset, buffer);
+	}
       }
     }
   }
 }
 
-#ifdef UNUSED
-
-/**************************************************************************
- Can be removed
-**************************************************************************/
-void put_city_pixmap(struct city *pcity, GdkPixmap *pm, int xtile, int ytile)
-{
-  SPRITE *mysprite;
-
-  if(use_solid_color_behind_units) {
-    gdk_gc_set_foreground( fill_bg_gc,
-			   colors_standard[player_color(city_owner(pcity))]);
-    gdk_draw_rectangle( pm, fill_bg_gc, TRUE,
-		    xtile*NORMAL_TILE_WIDTH, ytile*NORMAL_TILE_HEIGHT, 
-		    NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT );
-  }
-  else if(!flags_are_transparent) {	/* observe transparency here, too! */
-    mysprite=get_city_nation_flag_sprite(pcity);
-    gdk_draw_pixmap( pm, civ_gc, mysprite->pixmap,
-		0, 0,
-		xtile*NORMAL_TILE_WIDTH, ytile*NORMAL_TILE_HEIGHT,
-		mysprite->width, mysprite->height );
-  } else {
-    pixmap_put_overlay_tile(pm, xtile, ytile, get_city_nation_flag_sprite(pcity));
-  }
-  
-
-  pixmap_put_overlay_tile(pm, xtile, ytile,
-			  (city_got_citywalls(pcity)
-			   ? sprites.tx.city_walls
-			   : sprites.tx.city));
-
-  if(genlist_size(&((map_get_tile(pcity->x, pcity->y))->units.list)) > 0)
-    pixmap_put_overlay_tile(pm, xtile, ytile, sprites.city.occupied);
-
-  if(pcity->size>=10) {
-    pixmap_put_overlay_tile(pm, xtile, ytile,
-			    sprites.city.size_tens[pcity->size/10]);
-  }
-  pixmap_put_overlay_tile(pm, xtile, ytile, sprites.city.size[pcity->size%10]);
-
-  if(map_get_special(pcity->x, pcity->y) & S_POLLUTION)
-    pixmap_put_overlay_tile(pm, xtile, ytile, sprites.tx.pollution);
-
-  if(city_unhappy(pcity))
-    pixmap_put_overlay_tile(pm, xtile, ytile, sprites.city.disorder);
-
-}
-
-#endif
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -1110,30 +1041,33 @@ void put_unit_gpixmap_city_overlays(struct unit *punit, GtkPixcomm *p)
     put_overlay_tile_gpixmap(p, 0, 1, sprites.upkeep.unhappy[unhappy-1]);
 }
 
-
 /**************************************************************************
 ...
 **************************************************************************/
 void put_nuke_mushroom_pixmaps(int abs_x0, int abs_y0)
 {
   int x, y;
-  
-  for(y=0; y<3; y++)
+
+  for(y=0; y<3; y++) {
     for(x=0; x<3; x++) {
+      int map_x = map_canvas_adjust_x(x-1+abs_x0)*NORMAL_TILE_WIDTH;
+      int map_y = map_canvas_adjust_y(y-1+abs_y0)*NORMAL_TILE_HEIGHT;
       struct Sprite *mysprite = sprites.explode.nuke[y][x];
 
-      gdk_draw_pixmap( map_canvas->window, civ_gc, mysprite->pixmap,
-		0, 0,
-		map_canvas_adjust_x(x-1+abs_x0)*NORMAL_TILE_WIDTH, 
-		map_canvas_adjust_y(y-1+abs_y0)*NORMAL_TILE_HEIGHT,
-		NORMAL_TILE_WIDTH,
-		NORMAL_TILE_HEIGHT );
+      gdk_draw_pixmap(single_tile_pixmap, civ_gc, map_canvas_store,
+		      map_x, map_y, 0, 0,
+		      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+      pixmap_put_overlay_tile(single_tile_pixmap, 0, 0, mysprite);
+      gdk_draw_pixmap(map_canvas->window, civ_gc, single_tile_pixmap,
+		      0, 0, map_x, map_y,
+		      NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
     }
+  }
 
-  gdk_flush ();
+  gdk_flush();
   sleep(1);
 
-  update_map_canvas(map_canvas_adjust_x(abs_x0-1), 
+  update_map_canvas(map_canvas_adjust_x(abs_x0-1),
                     map_canvas_adjust_y(abs_y0-1),
 		    3, 3, 1);
 }
@@ -1199,7 +1133,7 @@ void pixmap_put_tile(GdkDrawable *pm, int x, int y, int abs_x0, int abs_y0,
                       sprites[0]->width, sprites[0]->height);
     } else
     {
-      /* normally when use_solid_color_behind_units */
+      /* normally when solid_color_behind_units */
       struct city *pcity;
       struct player *pplayer=NULL;
 
@@ -1209,7 +1143,7 @@ void pixmap_put_tile(GdkDrawable *pm, int x, int y, int abs_x0, int abs_y0,
         struct tile *ptile;
         struct unit *punit;
         ptile=map_get_tile(abs_x0, abs_y0);
-        if((punit=player_find_visible_unit(game.player_ptr, ptile)))
+        if ((punit=find_visible_unit(ptile)))
           pplayer = &game.players[punit->owner];
 
       } else
@@ -1263,7 +1197,7 @@ void pixmap_put_tile(GdkDrawable *pm, int x, int y, int abs_x0, int abs_y0,
 
   } else
   {
-    /* tile is unknow */
+    /* tile is unknown */
     pixmap_put_black_tile(pm,x,y);
   }
 }
@@ -1382,8 +1316,10 @@ void scrollbar_jump_callback(GtkAdjustment *adj, gpointer hscrollbar)
   else {
     map_view_y0=percent;
     map_view_y0=(map_view_y0<0) ? 0 : map_view_y0;
-    map_view_y0=(map_view_y0>map.ysize-map_canvas_store_theight) ? 
-       map.ysize-map_canvas_store_theight : map_view_y0;
+    map_view_y0=
+      (map_view_y0>map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight) ? 
+	map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight :
+	map_view_y0;
   }
 
   if(last_map_view_x0!=map_view_x0 || last_map_view_y0!=map_view_y0) {
