@@ -81,6 +81,8 @@ enum MenuID {
   MENU_KINGDOM_REVOLUTION,
 
   MENU_VIEW_SHOW_MAP_GRID,
+  MENU_VIEW_SHOW_CITY_NAMES,
+  MENU_VIEW_SHOW_CITY_PRODUCTIONS,
   MENU_VIEW_CENTER_VIEW,
 
   MENU_ORDER_AUTO_SETTLER,
@@ -94,6 +96,7 @@ enum MenuID {
   MENU_ORDER_ROAD,
   MENU_ORDER_CONNECT,
   MENU_ORDER_POLLUTION,
+  MENU_ORDER_FALLOUT,
   MENU_ORDER_FORTIFY,
   MENU_ORDER_SENTRY,
   MENU_ORDER_PILLAGE,
@@ -214,7 +217,16 @@ static void view_menu_callback(gpointer callback_data, guint callback_action,
 {
   switch(callback_action) {
   case MENU_VIEW_SHOW_MAP_GRID:
-    key_map_grid_toggle ();
+    if (draw_map_grid ^ GTK_CHECK_MENU_ITEM(widget)->active)
+      key_map_grid_toggle();
+    break;
+  case MENU_VIEW_SHOW_CITY_NAMES:
+    if (draw_city_names ^ GTK_CHECK_MENU_ITEM(widget)->active)
+      key_city_names_toggle();
+    break;
+  case MENU_VIEW_SHOW_CITY_PRODUCTIONS:
+    if (draw_city_productions ^ GTK_CHECK_MENU_ITEM(widget)->active)
+      key_city_productions_toggle();
     break;
   case MENU_VIEW_CENTER_VIEW:
     center_on_unit();
@@ -267,18 +279,22 @@ static void orders_menu_callback(gpointer callback_data,
       request_unit_connect();
     break;
    case MENU_ORDER_POLLUTION:
-    if(can_unit_paradropped(get_unit_in_focus()))
+    if(can_unit_paradrop(get_unit_in_focus()))
       key_unit_paradrop();
     else
       key_unit_pollution();
     break;
+   case MENU_ORDER_FALLOUT:
+    if(get_unit_in_focus())
+      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_FALLOUT);
+    break;
    case MENU_ORDER_HOMECITY:
     if(get_unit_in_focus())
       request_unit_change_homecity(get_unit_in_focus());
     break;
    case MENU_ORDER_FORTIFY:
     if(get_unit_in_focus())
-      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_FORTIFY);
+      request_new_unit_activity(get_unit_in_focus(), ACTIVITY_FORTIFYING);
     break;
    case MENU_ORDER_SENTRY:
     if(get_unit_in_focus())
@@ -519,6 +535,12 @@ static GtkItemFactoryEntry menu_items[]	=
     0,					"<Tearoff>"			      },
   { "/" N_("View") "/" N_("Map Grid"),		"<control>g",	view_menu_callback,
     MENU_VIEW_SHOW_MAP_GRID,		"<CheckItem>"			      },
+  { "/" N_("View") "/" N_("City Names"),	"<control>n",	view_menu_callback,
+    MENU_VIEW_SHOW_CITY_NAMES,		"<CheckItem>"			      },
+  { "/" N_("View") "/" N_("City Productions"),	"<control>p",	view_menu_callback,
+    MENU_VIEW_SHOW_CITY_PRODUCTIONS,	"<CheckItem>"			      },
+  { "/" N_("View") "/sep1",		NULL,		NULL,
+    0,					"<Separator>"			      },
   { "/" N_("View") "/" N_("Center View"),	"c",		view_menu_callback,
     MENU_VIEW_CENTER_VIEW						      },
 
@@ -542,6 +564,8 @@ static GtkItemFactoryEntry menu_items[]	=
     MENU_ORDER_AIRBASE								      },
   { "/" N_("Orders") "/" N_("Clean Pollution"), "p",		orders_menu_callback,
     MENU_ORDER_POLLUTION						      },
+  { "/" N_("Orders") "/" N_("Clean Nuclear Fallout"), "n",	orders_menu_callback,
+    MENU_ORDER_FALLOUT							      },
   { "/" N_("Orders") "/sep1",		NULL,		NULL,
     0,					"<Separator>"			      },
   { "/" N_("Orders") "/" N_("Fortify"),	"f",		orders_menu_callback,
@@ -616,37 +640,37 @@ static GtkItemFactoryEntry menu_items[]	=
     0,					"<LastBranch>"			      },
   { "/" N_("Help") "/tearoff1",		NULL,		NULL,
     0,					"<Tearoff>"			      },
-  { "/" N_("Help") "/" N_("Help Languages"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Languages"),		NULL,		help_menu_callback,
     MENU_HELP_LANGUAGES							      },
-  { "/" N_("Help") "/" N_("Help Connecting"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Connecting"),	NULL,		help_menu_callback,
     MENU_HELP_CONNECTING						      },
-  { "/" N_("Help") "/" N_("Help Controls"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Controls"),		NULL,		help_menu_callback,
     MENU_HELP_CONTROLS							      },
-  { "/" N_("Help") "/" N_("Help Chatline"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Chatline"),		NULL,		help_menu_callback,
     MENU_HELP_CHATLINE							      },
-  { "/" N_("Help") "/" N_("Help Playing"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Playing"),		NULL,		help_menu_callback,
     MENU_HELP_PLAYING							      },
   { "/" N_("Help") "/sep1",		NULL,		NULL,
     0,					"<Separator>"			      },
-  { "/" N_("Help") "/" N_("Help Improvements"), NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Improvements"),	NULL,		help_menu_callback,
     MENU_HELP_IMPROVEMENTS						      },
-  { "/" N_("Help") "/" N_("Help Units"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Units"),		NULL,		help_menu_callback,
     MENU_HELP_UNITS							      },
-  { "/" N_("Help") "/" N_("Help Combat"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Combat"),		NULL,		help_menu_callback,
     MENU_HELP_COMBAT							      },
-  { "/" N_("Help") "/" N_("Help ZOC"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("ZOC"),		NULL,		help_menu_callback,
     MENU_HELP_ZOC							      },
-  { "/" N_("Help") "/" N_("Help Technology"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Technology"),	NULL,		help_menu_callback,
     MENU_HELP_TECH							      },
-  { "/" N_("Help") "/" N_("Help Terrain"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Terrain"),		NULL,		help_menu_callback,
     MENU_HELP_TERRAIN							      },
-  { "/" N_("Help") "/" N_("Help Wonders"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Wonders"),		NULL,		help_menu_callback,
     MENU_HELP_WONDERS							      },
-  { "/" N_("Help") "/" N_("Help Government"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Government"),	NULL,		help_menu_callback,
     MENU_HELP_GOVERNMENT						      },
-  { "/" N_("Help") "/" N_("Help Happiness"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Happiness"),		NULL,		help_menu_callback,
     MENU_HELP_HAPPINESS							      },
-  { "/" N_("Help") "/" N_("Help Space Race"),	NULL,		help_menu_callback,
+  { "/" N_("Help") "/" N_("Space Race"),	NULL,		help_menu_callback,
     MENU_HELP_SPACE_RACE						      },
   { "/" N_("Help") "/sep2",		NULL,		NULL,
     0,					"<Separator>"			      },
@@ -746,6 +770,26 @@ static void menus_set_sensitive(const char *path, int sensitive)
   gtk_widget_set_sensitive(item, sensitive);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static void menus_set_active(const char *path, int active)
+{
+  GtkWidget *item;
+
+  path = translate_menu_path(path);
+
+  if (!(item = gtk_item_factory_get_widget(item_factory, path))) {
+    freelog(LOG_VERBOSE,
+	    "Can't set active for non-existent menu %s.", path);
+    return;
+  }
+
+  if (GTK_IS_MENU(item))
+    item = gtk_menu_get_attach_widget(GTK_MENU(item));
+  gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(item), active);
+}
+
 #ifdef UNUSED 
 /****************************************************************
 ...
@@ -837,6 +881,10 @@ void update_menus(void)
     menus_set_sensitive("<main>/Reports/Spaceship",
 			(game.player_ptr->spaceship.state!=SSHIP_NONE));
 
+    menus_set_active("<main>/View/Map Grid", draw_map_grid);
+    menus_set_active("<main>/View/City Names", draw_city_names);
+    menus_set_active("<main>/View/City Productions", draw_city_productions);
+
     if((punit=get_unit_in_focus())) {
       char *chgfmt = _("Change to %s");
       char *transfmt = _("Transform to %s");
@@ -871,9 +919,11 @@ void update_menus(void)
 			  can_unit_do_connect(punit, ACTIVITY_IDLE));
       menus_set_sensitive("<main>/Orders/Clean Pollution",
 			   can_unit_do_activity(punit, ACTIVITY_POLLUTION) ||
-			   can_unit_paradropped(punit));
+			   can_unit_paradrop(punit));
+      menus_set_sensitive("<main>/Orders/Clean Nuclear Fallout",
+			   can_unit_do_activity(punit, ACTIVITY_FALLOUT));
       menus_set_sensitive("<main>/Orders/Fortify",
-			   can_unit_do_activity(punit, ACTIVITY_FORTIFY));
+			   can_unit_do_activity(punit, ACTIVITY_FORTIFYING));
       menus_set_sensitive("<main>/Orders/Sentry",
 			   can_unit_do_activity(punit, ACTIVITY_SENTRY));
       menus_set_sensitive("<main>/Orders/Pillage",
