@@ -59,7 +59,7 @@ GtkWidget *science_dialog_shell=NULL;
 GtkWidget *science_label;
 GtkWidget *science_current_label, *science_goal_label;
 GtkWidget *science_change_menu_button, *science_goal_menu_button;
-GtkWidget *science_list, *science_help_toggle;
+GtkWidget *science_list[4], *science_help_toggle;
 int science_dialog_shell_is_modal;
 int science_dialog_popupmenu;
 GtkWidget *popupmenu, *goalmenu;
@@ -131,17 +131,18 @@ void update_report_dialogs(void)
 ****************************************************************/
 char *get_report_title(char *report_name)
 {
-  char buf[512];
-  
+  static char buf[512];
+
   my_snprintf(buf, sizeof(buf), _("%s\n%s of the %s\n%s %s: %s"),
 	  report_name,
 	  get_government_name(game.player_ptr->government),
 	  get_nation_name_plural(game.player_ptr->nation),
-	  get_ruler_title(game.player_ptr->government, game.player_ptr->is_male, game.player_ptr->nation),
+	  get_ruler_title(game.player_ptr->government,
+			  game.player_ptr->is_male, game.player_ptr->nation),
 	  game.player_ptr->name,
 	  textyear(game.year));
 
-  return create_centered_string(buf);
+  return buf;
 }
 
 
@@ -174,6 +175,7 @@ void create_science_dialog(int make_modal)
   GtkWidget *frame, *hbox, *w;
   GtkAccelGroup *accel=gtk_accel_group_new();
   int i;
+  char text[512], rate[128];
 
   science_dialog_shell = gtk_dialog_new();
   gtk_signal_connect( GTK_OBJECT(science_dialog_shell),"delete_event",
@@ -183,9 +185,11 @@ void create_science_dialog(int make_modal)
   gtk_window_set_title (GTK_WINDOW(science_dialog_shell), _("Science Report"));
 
   report_title=get_report_title(_("Science Advisor"));
-
-  science_label = gtk_label_new(report_title);
-  free(report_title);
+  sz_strlcpy(text, report_title);
+  my_snprintf(rate, sizeof(rate), _("\n(%d turns/advance)"),
+	      tech_turns_to_advance(game.player_ptr));
+  sz_strlcat(text, rate);
+  science_label = gtk_label_new(text);
 
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(science_dialog_shell)->vbox ),
         science_label, FALSE, FALSE, 0 );
@@ -195,7 +199,7 @@ void create_science_dialog(int make_modal)
         frame, FALSE, FALSE, 0 );
 
   hbox = gtk_hbox_new( TRUE, 5 );
-  gtk_container_add(GTK_CONTAINER(frame),hbox);
+  gtk_container_add(GTK_CONTAINER(frame), hbox);
 
   science_change_menu_button = gtk_option_menu_new();
   gtk_box_pack_start( GTK_BOX( hbox ), science_change_menu_button,TRUE, TRUE, 0 );
@@ -230,9 +234,17 @@ void create_science_dialog(int make_modal)
   w = gtk_label_new("");
   gtk_box_pack_start( GTK_BOX( hbox ), w,TRUE, FALSE, 0 );
 
-  science_list = gtk_clist_new(4);
-  gtk_box_pack_start( GTK_BOX( GTK_DIALOG(science_dialog_shell)->vbox ),
-		science_list, TRUE, TRUE, 0 );
+  hbox = gtk_hbox_new(TRUE, 0);
+  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(science_dialog_shell)->vbox),
+		     hbox, TRUE, TRUE, 0);
+
+  for (i=0; i<4; i++) {
+    science_list[i] = gtk_clist_new(1);
+    gtk_box_pack_start(GTK_BOX(hbox), science_list[i], TRUE, TRUE, 0);
+    gtk_clist_set_column_auto_resize (GTK_CLIST (science_list[i]), 0, TRUE);
+    gtk_signal_connect(GTK_OBJECT(science_list[i]), "select_row",
+		       GTK_SIGNAL_FUNC(science_help_callback), NULL);
+  }
 
   close_command = gtk_button_new_with_label(_("Close"));
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(science_dialog_shell)->action_area ),
@@ -243,18 +255,12 @@ void create_science_dialog(int make_modal)
   gtk_signal_connect( GTK_OBJECT( close_command ), "clicked",
         GTK_SIGNAL_FUNC( science_close_callback ), NULL);
 
-  gtk_signal_connect(GTK_OBJECT(science_list), "select_row",
-	GTK_SIGNAL_FUNC(science_help_callback), NULL);
-
   gtk_widget_show_all( GTK_DIALOG(science_dialog_shell)->vbox );
   gtk_widget_show_all( GTK_DIALOG(science_dialog_shell)->action_area );
 
   gtk_widget_add_accelerator(close_command, "clicked",
 	accel, GDK_Escape, 0, GTK_ACCEL_VISIBLE);
 
-  for(i=0; i<4; i++)
-    gtk_clist_set_column_auto_resize (GTK_CLIST (science_list), i, TRUE);
-
   science_dialog_update();
 }
 
@@ -274,15 +280,15 @@ void science_change_callback(GtkWidget *widget, gpointer data)
     /* Following is to make the menu go back to the current research;
      * there may be a better way to do this?  --dwp */
     science_dialog_update();
-  }
-  else {  
-      my_snprintf(text, sizeof(text), "%d/%d",
-	      game.player_ptr->research.researched, 
-	      research_time(game.player_ptr));
-      gtk_set_label(science_current_label,text);
-  
-      packet.tech=to;
-      send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_RESEARCH);
+  } else {
+    my_snprintf(text, sizeof(text), "%d/%d",
+		game.player_ptr->research.researched,
+		research_time(game.player_ptr));
+
+    gtk_set_label(science_current_label,text);
+    
+    packet.tech=to;
+    send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_RESEARCH);
   }
 }
 
@@ -328,15 +334,15 @@ void science_close_callback(GtkWidget *widget, gpointer data)
 /****************************************************************
 ...
 *****************************************************************/
-
 void science_help_callback(GtkWidget *w, gint row, gint column)
 {
-  char *s;
-
-  gtk_clist_get_text(GTK_CLIST(science_list), row, column, &s);
+  gtk_clist_unselect_row(GTK_CLIST(w), row, column);
 
   if (GTK_TOGGLE_BUTTON(science_help_toggle)->active)
   {
+    char *s;
+
+    gtk_clist_get_text(GTK_CLIST(w), row, column, &s);
     if (*s != '\0')
       popup_help_dialog_typed(s, HELP_TECH);
     else
@@ -344,49 +350,80 @@ void science_help_callback(GtkWidget *w, gint row, gint column)
   }
 }
 
+/****************************************************************
+...
+*****************************************************************/
+static gint cmp_func(gconstpointer a_p, gconstpointer b_p)
+{
+  gchar *a_str, *b_str;
+  gchar text_a[512], text_b[512];
+  gint a=(gint)a_p, b=(gint)b_p;
+
+  if(a < game.num_tech_types) {
+    a_str=advances[a].name;
+  } else {
+    my_snprintf(text_a,sizeof(text_a), _("Researching Future Tech. %d"),
+		a - game.num_tech_types);
+    a_str=text_a;
+  }
+
+  if(b < game.num_tech_types) {
+    b_str=advances[b].name;
+  } else {
+    my_snprintf(text_b,sizeof(text_b), _("Researching Future Tech. %d"),
+		b - game.num_tech_types);
+    b_str=text_b;
+  }
+
+  return strcmp(a_str,b_str);
+}
+
 /****************************************************************
 ...
 *****************************************************************/
 void science_dialog_update(void)
 {
   if(science_dialog_shell) {
-  char text[512];
-    int i, j, hist, n;
-    char *report_title;
-    static char *row	[4];
-    GtkWidget *item;
-    
+  char text[512], rate[128];
+  int i, j, hist;
+  char *report_title;
+  static char *row	[1];
+  GtkWidget *item;
+  GList *sorting_list = NULL;
+
   if(delay_report_update) return;
   report_title=get_report_title(_("Science Advisor"));
-  gtk_set_label(science_label, report_title);
-  free(report_title);
+  sz_strlcpy(text, report_title);
+  my_snprintf(rate, sizeof(rate), _("\n(%d turns/advance)"),
+	      tech_turns_to_advance(game.player_ptr));
+  sz_strlcat(text, rate);
+  gtk_set_label(science_label, text);
 
-  gtk_clist_freeze(GTK_CLIST(science_list));
-  gtk_clist_clear(GTK_CLIST(science_list));
-  for(n=0, i=A_FIRST; i<game.num_tech_types; i++)
-  {
-    if ((get_invention(game.player_ptr, i)==TECH_KNOWN))
-      n++;
-  }
-  for(j=0, i=A_FIRST; i<game.num_tech_types; i++)
-  {
-    if ((get_invention(game.player_ptr, i)==TECH_KNOWN)){
-      if (j==4)
-        j=0;
-      row[j++] = advances[i].name;
-
-      if (j==4)
-        gtk_clist_append( GTK_CLIST(science_list), row);
-      }
-  }
-  if (j!=4)
-  {
-    for (;j<4; j++)
-      row[j]="";
-    gtk_clist_append( GTK_CLIST(science_list), row );
+  for (i=0; i<4; i++) {
+    gtk_clist_freeze(GTK_CLIST(science_list[i]));
+    gtk_clist_clear(GTK_CLIST(science_list[i]));
   }
 
-  gtk_clist_thaw(GTK_CLIST(science_list));
+  /* collect all researched techs in sorting_list */
+  for(i=A_FIRST; i<game.num_tech_types; i++) {
+    if ((get_invention(game.player_ptr, i)==TECH_KNOWN)) {
+      sorting_list = g_list_append(sorting_list,(gpointer)i);
+    }
+  }
+
+  /* sort them, and install them in the list */
+  sorting_list = g_list_sort(sorting_list, cmp_func);
+  for(i=0; i<g_list_length(sorting_list); i++) {
+    j = (gint)g_list_nth_data(sorting_list, i);
+    row[0] = advances[j].name;
+    gtk_clist_append(GTK_CLIST(science_list[i%4]), row);
+  }
+  g_list_free(sorting_list);
+  sorting_list = NULL;
+
+  for (i=0; i<4; i++) {
+    gtk_clist_thaw(GTK_CLIST(science_list[i]));
+  }
 
   gtk_widget_destroy(popupmenu);
   popupmenu = gtk_menu_new();
@@ -394,39 +431,59 @@ void science_dialog_update(void)
   my_snprintf(text, sizeof(text), "%d/%d",
 	      game.player_ptr->research.researched,
 	      research_time(game.player_ptr));
+
   gtk_set_label(science_current_label,text);
 
+  /* collect all techs which are reachable in the next step
+   * hist will hold afterwards the techid of the current choice
+   */
   hist=0;
-  if (game.player_ptr->research.researching!=A_NONE)
-  {
-    for(i=A_FIRST, j=0; i<game.num_tech_types; i++)
-    {
+  if (game.player_ptr->research.researching!=A_NONE) {
+    for(i=A_FIRST; i<game.num_tech_types; i++) {
       if(get_invention(game.player_ptr, i)!=TECH_REACHABLE)
 	continue;
 
       if (i==game.player_ptr->research.researching)
-	hist=j;
-
-      item = gtk_menu_item_new_with_label(advances[i].name);
-      gtk_menu_append(GTK_MENU(popupmenu), item);
-
-      gtk_signal_connect(GTK_OBJECT(item), "activate",
-	GTK_SIGNAL_FUNC(science_change_callback ), (gpointer)i);
-      j++;
+	hist=i;
+      sorting_list = g_list_append(sorting_list,(gpointer)i);
     }
+  } else {
+    sorting_list = g_list_append(sorting_list, 
+				 (gpointer)(game.num_tech_types+
+					    (game.player_ptr->future_tech)
+					    +1));
   }
-  else
-  {
-    my_snprintf(text, sizeof(text), _("Researching Future Tech. %d"),
-		((game.player_ptr->future_tech)+1));
 
-    item = gtk_menu_item_new_with_label(text);
+  /* sort the list and build from it the menu */
+  sorting_list = g_list_sort(sorting_list, cmp_func);
+  for (i = 0; i < g_list_length(sorting_list); i++) {
+    gchar *data;
+
+    if((gint)g_list_nth_data(sorting_list, i) < game.num_tech_types) {
+      data=advances[(gint)g_list_nth_data(sorting_list, i)].name;
+    } else {
+      my_snprintf(text, sizeof(text), _("Researching Future Tech. %d"),
+		  (gint)g_list_nth_data(sorting_list, i)
+		  - game.num_tech_types);
+      data=text;
+    }
+
+    item = gtk_menu_item_new_with_label(data);
     gtk_menu_append(GTK_MENU(popupmenu), item);
+    if (strlen(data) > 0)
+      gtk_signal_connect(GTK_OBJECT(item), "activate",
+			 GTK_SIGNAL_FUNC(science_change_callback),
+			 (gpointer) g_list_nth_data(sorting_list, i));
   }
-  gtk_widget_show_all(popupmenu);
-  gtk_menu_set_active(GTK_MENU(popupmenu), hist);
-  gtk_option_menu_set_menu(GTK_OPTION_MENU(science_change_menu_button), popupmenu);
 
+  gtk_widget_show_all(popupmenu);
+  gtk_menu_set_active(GTK_MENU(popupmenu),
+		      g_list_index(sorting_list,(gpointer)hist));
+  g_list_free(sorting_list);
+  sorting_list = NULL;
+
+  gtk_option_menu_set_menu(GTK_OPTION_MENU(science_change_menu_button), 
+			   popupmenu);
 
   gtk_widget_destroy(goalmenu);
   goalmenu = gtk_menu_new();
@@ -435,35 +492,46 @@ void science_dialog_update(void)
 	      tech_goal_turns(game.player_ptr, game.player_ptr->ai.tech_goal));
   gtk_set_label(science_goal_label,text);
 
-  if (game.player_ptr->ai.tech_goal==A_NONE)
-  {
+  if (game.player_ptr->ai.tech_goal==A_NONE) {
     item = gtk_menu_item_new_with_label(advances[A_NONE].name);
     gtk_menu_append(GTK_MENU(goalmenu), item);
   }
 
+  /* collect all techs which are reachable in under 11 steps
+   * hist will hold afterwards the techid of the current choice
+   */
   hist=0;
-  for(i=A_FIRST, j=0; i<game.num_tech_types; i++)
-  {
+  for(i=A_FIRST; i<game.num_tech_types; i++) {
     if(get_invention(game.player_ptr, i) != TECH_KNOWN &&
        advances[i].req[0] != A_LAST && advances[i].req[1] != A_LAST &&
-       tech_goal_turns(game.player_ptr, i) < 11)
-    {
+       tech_goal_turns(game.player_ptr, i) < 11) {
       if (i==game.player_ptr->ai.tech_goal)
-        hist=j;
-
-      item = gtk_menu_item_new_with_label(advances[i].name);
-      gtk_menu_append(GTK_MENU(goalmenu), item);
-
-      gtk_signal_connect(GTK_OBJECT(item), "activate",
-        GTK_SIGNAL_FUNC(science_goal_callback), (gpointer)i);
-      j++;
+	hist=i;
+      sorting_list = g_list_append(sorting_list,(gpointer)i);
     }
   }
+
+  /* sort the list and build from it the menu */
+  sorting_list = g_list_sort(sorting_list, cmp_func);
+  for (i = 0; i < g_list_length(sorting_list); i++) {
+    gchar *data=advances[(gint)g_list_nth_data(sorting_list, i)].name;
+
+    item = gtk_menu_item_new_with_label(data);
+    gtk_menu_append(GTK_MENU(goalmenu), item);
+    gtk_signal_connect(GTK_OBJECT(item), "activate",
+		       GTK_SIGNAL_FUNC(science_goal_callback),
+		       (gpointer) g_list_nth_data(sorting_list, i));
+  }
+
   gtk_widget_show_all(goalmenu);
-  gtk_menu_set_active(GTK_MENU(goalmenu), hist);
-  gtk_option_menu_set_menu(GTK_OPTION_MENU(science_goal_menu_button), goalmenu);
+  gtk_menu_set_active(GTK_MENU(goalmenu), 
+		      g_list_index(sorting_list,(gpointer)hist));
+  g_list_free(sorting_list);
+  sorting_list = NULL;
+
+  gtk_option_menu_set_menu(GTK_OPTION_MENU(science_goal_menu_button), 
+			   goalmenu);
   }
-  
 }
 
 
@@ -679,7 +747,6 @@ void trade_report_dialog_update(void)
     
     report_title=get_report_title(_("Trade Advisor"));
     gtk_set_label(trade_label, report_title);
-    free(report_title);
 
     gtk_clist_freeze(GTK_CLIST(trade_list));
     gtk_clist_clear(GTK_CLIST(trade_list));
@@ -784,7 +851,6 @@ void create_activeunits_report_dialog(int make_modal)
   activeunits_label = gtk_label_new(report_title);
   gtk_box_pack_start( GTK_BOX( GTK_DIALOG(activeunits_dialog_shell)->vbox ),
         activeunits_label, FALSE, FALSE, 0 );
-  free(report_title);
 
   activeunits_list = gtk_clist_new_with_titles( AU_COL, titles );
   gtk_clist_column_titles_passive(GTK_CLIST(activeunits_list));
@@ -953,7 +1019,6 @@ void activeunits_report_dialog_update(void)
 
     report_title=get_report_title(_("Military Report"));
     gtk_set_label(activeunits_label, report_title);
-    free(report_title);
 
     gtk_clist_freeze(GTK_CLIST(activeunits_list));
     gtk_clist_clear(GTK_CLIST(activeunits_list));
