@@ -76,9 +76,19 @@ static void xaw_key_center_on_unit(Widget w, XEvent *event, String *argv, Cardin
   center_on_unit();
 }
 
-static void xaw_key_chatline_return(Widget w, XEvent *event, String *argv, Cardinal *argc)
+static void xaw_key_chatline_send(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  chatline_key_return(w);
+  chatline_key_send(w);
+}
+
+static void xaw_key_city_names_toggle(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  key_city_names_toggle();
+}
+
+static void xaw_key_city_productions_toggle(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  key_city_productions_toggle();
 }
 
 static void xaw_key_city_workers(Widget w, XEvent *event, String *argv, Cardinal *argc)
@@ -86,34 +96,34 @@ static void xaw_key_city_workers(Widget w, XEvent *event, String *argv, Cardinal
   mapctrl_key_city_workers(event);
 }
 
-static void xaw_key_dialog_city_return(Widget w, XEvent *event, String *argv, Cardinal *argc)
+static void xaw_key_dialog_city_close(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  citydlg_key_return(w);
+  citydlg_key_close(w);
 }
 
-static void xaw_key_dialog_connect_return(Widget w, XEvent *event, String *argv, Cardinal *argc)
+static void xaw_key_dialog_connect_connect(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  connectdlg_key_return(w);
+  connectdlg_key_connect(w);
 }
 
-static void xaw_key_dialog_diplo_return(Widget w, XEvent *event, String *argv, Cardinal *argc)
+static void xaw_key_dialog_diplo_gold(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  diplodlg_key_return(w);
+  diplodlg_key_gold(w);
 }
 
-static void xaw_key_dialog_input_return(Widget w, XEvent *event, String *argv, Cardinal *argc)
+static void xaw_key_dialog_input_ok(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  inputdlg_key_return(w);
+  inputdlg_key_ok(w);
 }
 
-static void xaw_key_dialog_races_return(Widget w, XEvent *event, String *argv, Cardinal *argc)
+static void xaw_key_dialog_races_ok(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  racesdlg_key_return(w);
+  racesdlg_key_ok(w);
 }
 
-static void xaw_key_dialog_spaceship_return(Widget w, XEvent *event, String *argv, Cardinal *argc)
+static void xaw_key_dialog_spaceship_close(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
-  spaceshipdlg_key_return(w);
+  spaceshipdlg_key_close(w);
 }
 
 static void xaw_key_end_turn(Widget w, XEvent *event, String *argv, Cardinal *argc)
@@ -358,6 +368,12 @@ static void xaw_key_unit_done(Widget w, XEvent *event, String *argv, Cardinal *a
     key_unit_done();
 }
 
+static void xaw_key_unit_fallout(Widget w, XEvent *event, String *argv, Cardinal *argc)
+{
+  if(is_menu_item_active(MENU_ORDER, MENU_ORDER_FALLOUT))
+    key_unit_fallout();
+}
+
 static void xaw_key_unit_fortify(Widget w, XEvent *event, String *argv, Cardinal *argc)
 {
   if(is_menu_item_active(MENU_ORDER, MENU_ORDER_FORTIFY))
@@ -563,14 +579,16 @@ static XtActionsRec Actions[] = {
   { "btn-show-info-popup",  xaw_btn_show_info_popup },
   { "key-cancel-action", xaw_key_cancel_action },
   { "key-center-on-unit", xaw_key_center_on_unit },
-  { "key-chatline-return", xaw_key_chatline_return },
+  { "key-chatline-send", xaw_key_chatline_send },
+  { "key-city-names-toggle", xaw_key_city_names_toggle },
+  { "key-city-productions-toggle", xaw_key_city_productions_toggle },
   { "key-city-workers", xaw_key_city_workers },
-  { "key-dialog-city-return", xaw_key_dialog_city_return },
-  { "key-dialog-connect-return", xaw_key_dialog_connect_return },
-  { "key-dialog-diplo-return", xaw_key_dialog_diplo_return },
-  { "key-dialog-input-return", xaw_key_dialog_input_return },
-  { "key-dialog-races-return", xaw_key_dialog_races_return },
-  { "key-dialog-spaceship-return", xaw_key_dialog_spaceship_return },
+  { "key-dialog-city-close", xaw_key_dialog_city_close },
+  { "key-dialog-connect-connect", xaw_key_dialog_connect_connect },
+  { "key-dialog-diplo-gold", xaw_key_dialog_diplo_gold },
+  { "key-dialog-input-ok", xaw_key_dialog_input_ok },
+  { "key-dialog-races-ok", xaw_key_dialog_races_ok },
+  { "key-dialog-spaceship-close", xaw_key_dialog_spaceship_close },
   { "key-end-turn", xaw_key_end_turn },
   { "key-focus-to-next-unit", xaw_key_focus_to_next_unit },
   { "key-map-grid-toggle", xaw_key_map_grid_toggle },
@@ -608,6 +626,7 @@ static XtActionsRec Actions[] = {
   { "key-unit-connect", xaw_key_unit_connect },
   { "key-unit-disband", xaw_key_unit_disband },
   { "key-unit-done", xaw_key_unit_done },
+  { "key-unit-fallout", xaw_key_unit_fallout },
   { "key-unit-fortify", xaw_key_unit_fortify },
   { "key-unit-fortify-or-fortress", xaw_key_unit_fortify_or_fortress },
   { "key-unit-fortress", xaw_key_unit_fortress },
