@@ -23,6 +23,7 @@
 #include <X11/Xaw/AsciiText.h>
 
 #include "fcintl.h"
+#include "log.h"
 #include "mem.h"
 #include "packets.h"
 #include "shared.h"		/* wordwrap_string() */
@@ -35,12 +36,10 @@
 
 extern Widget inputline_text, outputwindow_text;
 
-#define OUTPUT_MAXLEN 95
-
 /**************************************************************************
 ...
 **************************************************************************/
-void chatline_key_return(Widget w)
+void chatline_key_send(Widget w)
 {
   struct packet_generic_message apacket;
   String theinput;
@@ -71,16 +70,38 @@ void chatline_key_return(Widget w)
  instead of the start of line as below, but that didn't help.)  So we
  split the line ourselves.  I'm just using a fixed length split; should
  perhaps check and use width of output window (but font size?)  -dwp
+
+ Now uses window's font size and width.  Assumes fixed-width font.  --jjm
 **************************************************************************/
 void append_output_window(char *astring)
 {
+  static int m_width=0;
+
+  Dimension windowwth;
+  int maxlinelen;
   String theoutput;
   char *newout, *rmcr;
   char *input_string = astring;
 
-  if (strlen(astring) > OUTPUT_MAXLEN) {
+  if (!m_width) {
+    XFontStruct *out_font;
+    XtVaGetValues(outputwindow_text, XtNfont, &out_font, NULL);
+    if (out_font)
+      m_width=XTextWidth(out_font, "M", 1);
+    else
+      m_width=10;
+  }
+
+  XtVaGetValues(outputwindow_text, XtNwidth, &windowwth, NULL);
+  maxlinelen=(windowwth/m_width)-1;
+  if (maxlinelen<32) maxlinelen=32;
+
+  freelog(LOG_DEBUG, "m_width %d windowwth %d maxlinelen %d",
+	  m_width, (int)windowwth, maxlinelen);
+
+  if (strlen(astring) > maxlinelen) {
     astring = mystrdup(astring);
-    wordwrap_string(astring, OUTPUT_MAXLEN);
+    wordwrap_string(astring, maxlinelen);
   }
   
   XtVaGetValues(outputwindow_text, XtNstring, &theoutput, NULL);
