@@ -117,7 +117,7 @@ struct city_dialog {
   Widget rename_input;
   Widget worklist_shell;
   
-  enum improvement_type_id sell_id;
+  Impr_Type_id sell_id;
   
   int *citizen_type;
   int support_unit_base;
@@ -416,7 +416,7 @@ void refresh_unit_city_dialogs(struct unit *punit)
   struct city *pcity_sup, *pcity_pre;
   struct city_dialog *pdialog;
 
-  pcity_sup=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+  pcity_sup=player_find_city_by_id(game.player_ptr, punit->homecity);
   pcity_pre=map_get_city(punit->x, punit->y);
   
   if(pcity_sup && (pdialog=get_city_dialog(pcity_sup)))
@@ -1097,14 +1097,17 @@ void activate_callback(Widget w, XtPointer client_data,
   struct city_dialog *pdialog = (struct city_dialog *)client_data;
   int x=pdialog->pcity->x,y=pdialog->pcity->y;
   struct unit_list *punit_list = &map_get_tile(x,y)->units;
+  struct unit *pmyunit = NULL;
 
   if( unit_list_size(punit_list) )  {
     unit_list_iterate((*punit_list), punit) {
       if(game.player_idx==punit->owner) {
 	request_new_unit_activity(punit, ACTIVITY_IDLE);
+	pmyunit = punit;
       }
     } unit_list_iterate_end;
-    set_unit_focus(unit_list_get(punit_list, 0));
+    if (pmyunit)
+      set_unit_focus(pmyunit);
   }
 }
 
@@ -1181,7 +1184,7 @@ static void present_units_activate_callback(Widget w, XtPointer client_data,
   struct city *pcity;
   struct city_dialog *pdialog;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data)))  {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))  {
     activate_unit(punit);
     if((pcity=map_get_city(punit->x, punit->y)))
       if((pdialog=get_city_dialog(pcity)))
@@ -1202,7 +1205,7 @@ static void supported_units_activate_callback(Widget w, XtPointer client_data,
   struct city *pcity;
   struct city_dialog *pdialog;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data)))  {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))  {
     activate_unit(punit);
     if((pcity=map_get_city(punit->x, punit->y)))
       if((pdialog=get_city_dialog(pcity)))
@@ -1226,7 +1229,7 @@ static void present_units_activate_close_callback(Widget w,
 
   destroy_message_dialog(w);
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data)))  {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))  {
     activate_unit(punit);
     if((pcity=map_get_city(punit->x, punit->y)))
       if((pdialog=get_city_dialog(pcity)))
@@ -1247,15 +1250,44 @@ static void supported_units_activate_close_callback(Widget w,
 
   destroy_message_dialog(w);
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data)))  {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))  {
     activate_unit(punit);
-    if((pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity)))
+    if((pcity=player_find_city_by_id(game.player_ptr, punit->homecity)))
       if((pdialog=get_city_dialog(pcity)))
 	close_city_dialog(pdialog);
   }
 }
 
 
+/****************************************************************
+...
+*****************************************************************/
+static void present_units_sentry_callback(Widget w, XtPointer client_data, 
+					   XtPointer call_data)
+{
+  struct unit *punit;
+
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
+    request_unit_sentry(punit);
+
+  destroy_message_dialog(w);
+}
+
+
+/****************************************************************
+...
+*****************************************************************/
+static void present_units_fortify_callback(Widget w, XtPointer client_data, 
+					   XtPointer call_data)
+{
+  struct unit *punit;
+
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
+    request_unit_fortify(punit);
+
+  destroy_message_dialog(w);
+}
+
 
 /****************************************************************
 ...
@@ -1265,7 +1297,7 @@ static void present_units_disband_callback(Widget w, XtPointer client_data,
 {
   struct unit *punit;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data)))
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
     request_unit_disband(punit);
 
   destroy_message_dialog(w);
@@ -1280,7 +1312,7 @@ static void present_units_homecity_callback(Widget w, XtPointer client_data,
 {
   struct unit *punit;
   
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data)))
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
     request_unit_change_homecity(punit);
 
   destroy_message_dialog(w);
@@ -1309,28 +1341,46 @@ void present_units_callback(Widget w, XtPointer client_data,
   Widget wd;
   XEvent *e = (XEvent*)call_data;
   
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data)) &&
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)) &&
      (pcity=map_get_city(punit->x, punit->y)) &&
      (pdialog=get_city_dialog(pcity))) {
     
-    if(e->type==ButtonRelease && e->xbutton.button==2)  {
+    if(e->type==ButtonRelease && e->xbutton.button==Button2)  {
       activate_unit(punit);
       close_city_dialog(pdialog);
       return;
     }
+    if(e->type==ButtonRelease && e->xbutton.button==Button3)  {
+      activate_unit(punit);
+      return;
+    }
 
     wd=popup_message_dialog(pdialog->shell, 
 			    "presentunitsdialog", 
 			    unit_description(punit),
 			    present_units_activate_callback, punit->id, 1,
 			    present_units_activate_close_callback, punit->id, 1,
+			    present_units_sentry_callback, punit->id, 1,
+			    present_units_fortify_callback, punit->id, 1,
 			    present_units_disband_callback, punit->id, 1,
 			    present_units_homecity_callback, punit->id, 1,
 			    upgrade_callback, punit->id, 1,
 			    present_units_cancel_callback, 0, 0, 
 			    NULL);
+
+    if (punit->activity == ACTIVITY_SENTRY
+	|| !can_unit_do_activity(punit, ACTIVITY_SENTRY)) {
+      XtSetSensitive(XtNameToWidget(wd, "*button2"), FALSE);
+    }
+    if (punit->activity == ACTIVITY_FORTIFYING
+	|| !can_unit_do_activity(punit, ACTIVITY_FORTIFYING)) {
+      XtSetSensitive(XtNameToWidget(wd, "*button3"), FALSE);
+    }
+    if (punit->homecity == pcity->id) {
+      XtSetSensitive(XtNameToWidget(wd, "*button5"), FALSE);
+    }
     if (can_upgrade_unittype(game.player_ptr,punit->type) == -1) {
-      XtSetSensitive(XtNameToWidget(wd, "*button4"), FALSE);
+      XtSetSensitive(XtNameToWidget(wd, "*button6"), FALSE);
     }
   }
 }
@@ -1469,7 +1519,7 @@ void city_dialog_update_building(struct city_dialog *pdialog)
   xaw_set_label(pdialog->building_label,
 		pcity->is_building_unit ?
 		  get_unit_type(pcity->currently_building)->name :
-		  get_imp_name_ex(pcity, pcity->currently_building));
+		  get_impr_name_ex(pcity, pcity->currently_building));
 
   get_contents_of_progress(pdialog, buf, sizeof(buf));
   xaw_set_label(pdialog->progress_label, buf);
@@ -1635,14 +1685,18 @@ static void support_units_callback(Widget w, XtPointer client_data,
   struct city_dialog *pdialog;
   XEvent *e = (XEvent*)call_data;
   
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data)))
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data)))
     if((pcity=find_city_by_id(punit->homecity)))
       if((pdialog=get_city_dialog(pcity)))  {
-	if(e->type==ButtonRelease && e->xbutton.button==2)  {
+	if(e->type==ButtonRelease && e->xbutton.button==Button2)  {
 	  activate_unit(punit);
 	  close_city_dialog(pdialog);
 	  return;
 	}
+	if(e->type==ButtonRelease && e->xbutton.button==Button3)  {
+	  activate_unit(punit);
+	  return;
+	}
 	popup_message_dialog(pdialog->shell,
 			     "supportunitsdialog", 
 			     unit_description(punit),
@@ -1696,15 +1750,22 @@ static int units_scroll_maintenance(int nunits, int nshow, int *base,
 void city_dialog_update_supported_units(struct city_dialog *pdialog, 
 					int unitid)
 {
+  struct unit_list *plist;
   int i, adj_base;
   struct genlist_iterator myiter;
   struct unit *punit;
   Widget pixcomm;
 
+  if(pdialog->pcity->owner != game.player_idx) {
+    plist = &(pdialog->pcity->info_units_supported);
+  } else {
+    plist = &(pdialog->pcity->units_supported);
+  }
+
   adj_base=
     units_scroll_maintenance
     (
-     pdialog->pcity->units_supported.list.nelements,
+     plist->list.nelements,
      pdialog->num_units_shown,
      &(pdialog->support_unit_base),
      pdialog->support_unit_next_command,
@@ -1712,8 +1773,8 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
     );
 
   genlist_iterator_init(&myiter,
-	&pdialog->pcity->units_supported.list,
-	pdialog->support_unit_base);
+			&(plist->list),
+			pdialog->support_unit_base);
 
   for(i=0;
       i<pdialog->num_units_shown && ITERATOR_PTR(myiter);
@@ -1751,15 +1812,22 @@ void city_dialog_update_supported_units(struct city_dialog *pdialog,
 *****************************************************************/
 void city_dialog_update_present_units(struct city_dialog *pdialog, int unitid)
 {
+  struct unit_list *plist;
   int i, adj_base;
   struct genlist_iterator myiter;
   struct unit *punit;
   Widget pixcomm;
 
+  if(pdialog->pcity->owner != game.player_idx) {
+    plist = &(pdialog->pcity->info_units_present);
+  } else {
+    plist = &(map_get_tile(pdialog->pcity->x, pdialog->pcity->y)->units);
+  }
+
   adj_base=
     units_scroll_maintenance
     (
-     map_get_tile(pdialog->pcity->x, pdialog->pcity->y)->units.list.nelements,
+     plist->list.nelements,
      pdialog->num_units_shown,
      &(pdialog->present_unit_base),
      pdialog->present_unit_next_command,
@@ -1767,8 +1835,8 @@ void city_dialog_update_present_units(struct city_dialog *pdialog, int unitid)
     );
 
   genlist_iterator_init(&myiter, 
-	&map_get_tile(pdialog->pcity->x, pdialog->pcity->y)->units.list,
-	pdialog->present_unit_base);
+			&(plist->list),
+			pdialog->present_unit_base);
 
   for(i=0;
       i<pdialog->num_units_shown && ITERATOR_PTR(myiter);
@@ -1828,10 +1896,10 @@ void city_dialog_update_improvement_list(struct city_dialog *pdialog)
   for(i=0, n=0, flag=0; i<B_LAST; ++i)
     if(pdialog->pcity->improvements[i]) {
       if(!pdialog->improvlist_names_ptrs[n] ||
-	 strcmp(pdialog->improvlist_names_ptrs[n], get_imp_name_ex(pdialog->pcity, i)))
+	 strcmp(pdialog->improvlist_names_ptrs[n], get_impr_name_ex(pdialog->pcity, i)))
 	flag=1;
       sz_strlcpy(pdialog->improvlist_names[n],
-		 get_imp_name_ex(pdialog->pcity, i));
+		 get_impr_name_ex(pdialog->pcity, i));
       pdialog->improvlist_names_ptrs[n]=pdialog->improvlist_names[n];
       n++;
     }
@@ -1993,7 +2061,7 @@ void buy_callback(Widget w, XtPointer client_data, XtPointer call_data)
     name=get_unit_type(pdialog->pcity->currently_building)->name;
   }
   else {
-    name=get_imp_name_ex(pdialog->pcity, pdialog->pcity->currently_building);
+    name=get_impr_name_ex(pdialog->pcity, pdialog->pcity->currently_building);
   }
   value=city_buy_cost(pdialog->pcity);
  
@@ -2025,7 +2093,7 @@ void unitupgrade_callback_yes(Widget w, XtPointer client_data, XtPointer call_da
 {
   struct unit *punit;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data))) {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data))) {
     request_unit_upgrade(punit);
   }
   destroy_message_dialog(w);
@@ -2051,7 +2119,7 @@ void upgrade_callback(Widget w, XtPointer client_data, XtPointer call_data)
   int ut1,ut2;
   int value;
 
-  if((punit=unit_list_find(&game.player_ptr->units, (size_t)client_data))) {
+  if((punit=player_find_unit_by_id(game.player_ptr, (size_t)client_data))) {
     ut1 = punit->type;
     /* printf("upgrade_callback for %s\n", unit_types[ut1].name); */
 
@@ -2271,13 +2339,13 @@ void change_callback(Widget w, XtPointer client_data, XtPointer call_data)
 	my_snprintf(pdialog->change_list_names[n],
 		    sizeof(pdialog->change_list_names[n]),
 		    "%s (XX)",
-		    get_imp_name_ex(pdialog->pcity, i));
+		    get_impr_name_ex(pdialog->pcity, i));
       } else {
 	turns = city_turns_to_build (pdialog->pcity, i, FALSE);
 	my_snprintf(pdialog->change_list_names[n],
 		    sizeof(pdialog->change_list_names[n]),
 		    turns == 1 ? _("%s (%d) %d turn") : _("%s (%d) %d turns"),
-		    get_imp_name_ex(pdialog->pcity, i),
+		    get_impr_name_ex(pdialog->pcity, i),
 		    get_improvement_type(i)->build_cost,
 		    turns);
       }
@@ -2449,7 +2517,7 @@ void sell_callback(Widget w, XtPointer client_data, XtPointer call_data)
 	  
 	  pdialog->sell_id=i;
 	  my_snprintf(buf, sizeof(buf), _("Sell %s for %d gold?"), 
-		      get_imp_name_ex(pdialog->pcity, i),
+		      get_impr_name_ex(pdialog->pcity, i),
 		      improvement_value(i));
 
 	  popup_message_dialog(pdialog->shell, "selldialog", buf,
@@ -2481,6 +2549,15 @@ void close_city_dialog(struct city_dialog *pdialog)
   free(pdialog->support_unit_pixcomms);
   free(pdialog->present_unit_pixcomms);
 
+  unit_list_iterate(pdialog->pcity->info_units_supported, psunit) {
+    free(psunit);
+  } unit_list_iterate_end;
+  unit_list_unlink_all(&(pdialog->pcity->info_units_supported));
+  unit_list_iterate(pdialog->pcity->info_units_present, psunit) {
+    free(psunit);
+  } unit_list_iterate_end;
+  unit_list_unlink_all(&(pdialog->pcity->info_units_present));
+
   if(pdialog->is_modal)
     XtSetSensitive(toplevel, TRUE);
   free(pdialog);
@@ -2489,7 +2566,7 @@ void close_city_dialog(struct city_dialog *pdialog)
 /****************************************************************
 ...
 *****************************************************************/
-void citydlg_key_return(Widget w)
+void citydlg_key_close(Widget w)
 {
   citydlg_msg_close(XtParent(XtParent(w)));
 }
@@ -2697,21 +2774,21 @@ void cityopt_ok_command_callback(Widget w, XtPointer client_data,
 
   if (pcity) {
     struct packet_generic_values packet;
-    int i, new;
+    int i, new_options;
     Boolean b;
     
-    new = 0;
+    new_options = 0;
     for(i=0; i<NUM_CITYOPT_TOGGLES; i++)  {
       XtVaGetValues(cityopt_toggles[i], XtNstate, &b, NULL);
-      if (b) new |= (1<<i);
+      if (b) new_options |= (1<<i);
     }
     if (newcitizen_index == 1) {
-      new |= (1<<CITYO_NEW_EINSTEIN);
+      new_options |= (1<<CITYO_NEW_EINSTEIN);
     } else if (newcitizen_index == 2) {
-      new |= (1<<CITYO_NEW_TAXMAN);
+      new_options |= (1<<CITYO_NEW_TAXMAN);
     }
     packet.value1 = cityopt_city_id;
-    packet.value2 = new;
+    packet.value2 = new_options;
     send_packet_generic_values(&aconnection, PACKET_CITY_OPTIONS,
 			       &packet);
   }
