@@ -26,6 +26,7 @@
 #include <X11/Xaw/AsciiText.h>  
 #include <X11/Xaw/List.h>
 
+#include "fcintl.h"
 #include "mem.h"     /* mystrdup() */
 #include "support.h"
 #include "version.h"
@@ -40,6 +41,7 @@
 
 /* extern AppResources appResources; */
 extern Widget toplevel;
+extern Widget turn_done_button;
 
 /* in civclient.c; FIXME hardcoded sizes */
 extern char name[];
@@ -77,14 +79,13 @@ void gui_server_connect(void)
 {
   Widget shell, form;
   char buf[512];
-  
-  XtTranslations textfieldtranslations;
-  
+
+  XtSetSensitive(turn_done_button, FALSE);
   XtSetSensitive(toplevel, FALSE);
-  
+
   I_T(shell=XtCreatePopupShell("connectdialog", transientShellWidgetClass,
 			       toplevel, NULL, 0));
-  
+
   form=XtVaCreateManagedWidget("cform", formWidgetClass, shell, NULL);
 
   I_LW(XtVaCreateManagedWidget("cheadline", labelWidgetClass, form, NULL));
@@ -119,22 +120,15 @@ void gui_server_connect(void)
   XtAddCallback(metaw, XtNcallback, connect_meta_callback, (XtPointer)shell);
 
   xaw_set_relative_position(toplevel, shell, 30, 0);
+
   XtPopup(shell, XtGrabNone);
-
-  textfieldtranslations = 
-    XtParseTranslationTable("<Key>Return: key-dialog-connect-return()");
-  XtOverrideTranslations(form, textfieldtranslations);
-  XtOverrideTranslations(iname, textfieldtranslations);
-  XtOverrideTranslations(ihost, textfieldtranslations);
-  XtOverrideTranslations(iport, textfieldtranslations);
-
   XtSetKeyboardFocus(toplevel, shell);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-void connectdlg_key_return(Widget w)
+void connectdlg_key_connect(Widget w)
 {
   x_simulate_button_click(connw);
 }
@@ -196,15 +190,16 @@ void create_meta_dialog(Widget caller)
   Widget shell, form, label, list, update, close;
   Dimension width;
 
-  shell=XtCreatePopupShell("metadialog", transientShellWidgetClass,
-			   toplevel, NULL, 0);
+  I_T(shell=XtCreatePopupShell("metadialog", transientShellWidgetClass,
+			       toplevel, NULL, 0));
   meta_dialog_shell=shell;
 
   form=XtVaCreateManagedWidget("metaform", formWidgetClass, shell, NULL);
 
   I_L(label=XtVaCreateManagedWidget("legend", labelWidgetClass, form, NULL));
   list=XtVaCreateManagedWidget("metalist", listWidgetClass, form, NULL);
-  update=XtVaCreateManagedWidget("update", commandWidgetClass, form, NULL);
+  I_L(update=XtVaCreateManagedWidget("update", commandWidgetClass,
+				     form, NULL));
   I_L(close=XtVaCreateManagedWidget("closecommand", commandWidgetClass,
 				    form, NULL));
 
@@ -299,9 +294,9 @@ static int get_meta_list(char **list, char *errbuf, int n_errbuf)
   if(!server_list) return -1;
 
   server_list_iterate(*server_list,pserver)
-    my_snprintf(line, sizeof(line), "%-35s %-5s %-7s %-9s %2s   %s",
+    my_snprintf(line, sizeof(line), "%-35s %-5s %-11s %-11s %2s   %s",
 		pserver->name, pserver->port, pserver->version,
-		pserver->status, pserver->players, pserver->metastring);
+		_(pserver->status), pserver->players, pserver->metastring);
     if(*list) free(*list);
     *list=mystrdup(line);
     list++;
