@@ -40,6 +40,7 @@
 #include "mem.h"
 #include "packets.h"
 #include "player.h"
+#include "rand.h"
 #include "support.h"
 
 #include "chatline.h"
@@ -66,6 +67,7 @@ extern GC fill_bg_gc;
 static Widget races_dialog_shell=NULL;
 static Widget races_form, races_toggles_form, races_label;
 static Widget *races_toggles=NULL;
+static int *races_toggles_to_nations=NULL;
 static Widget races_leader_form, races_leader;
 static Widget races_leader_pick_popupmenu, races_leader_pick_menubutton;
 static Widget races_sex_toggles[2], races_sex_form, races_sex_label;
@@ -129,6 +131,8 @@ static int races_sex_buttons_get_current(void);
 static int races_style_buttons_get_current(void);
 static void races_sex_buttons_set_current(int i);
 
+static int races_indirect_compare(const void *first, const void *second);
+
 static void races_toggles_callback(Widget w, XtPointer client_data, 
 				   XtPointer call_data);
 static void races_leader_pick_callback(Widget w, XtPointer client_data,
@@ -164,6 +168,7 @@ int unit_to_use_to_pillage;
 int caravan_city_id;
 int caravan_unit_id;
 
+int diplomat_dialog_open = 0;
 int diplomat_id;
 int diplomat_target_id;
 
@@ -386,7 +391,7 @@ static void diplomat_bribe_yes_callback(Widget w, XtPointer client_data,
   req.target_id=diplomat_target_id;
 
   send_packet_diplomat_action(&aconnection, &req);
-  
+
   destroy_message_dialog(w);
 }
 
@@ -450,7 +455,8 @@ static void diplomat_sabotage_callback(Widget w, XtPointer client_data,
 				       XtPointer call_data)
 {
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      find_city_by_id(diplomat_target_id)) { 
     struct packet_diplomat_action req;
@@ -463,6 +469,7 @@ static void diplomat_sabotage_callback(Widget w, XtPointer client_data,
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -471,9 +478,9 @@ static void diplomat_sabotage_callback(Widget w, XtPointer client_data,
 static void diplomat_embassy_callback(Widget w, XtPointer client_data, 
 				      XtPointer call_data)
 {
- 
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      (find_city_by_id(diplomat_target_id))) { 
     struct packet_diplomat_action req;
@@ -485,16 +492,18 @@ static void diplomat_embassy_callback(Widget w, XtPointer client_data,
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
+
 /****************************************************************
 ...
 *****************************************************************/
 static void diplomat_investigate_callback(Widget w, XtPointer client_data, 
 					  XtPointer call_data)
 {
- 
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      (find_city_by_id(diplomat_target_id))) { 
     struct packet_diplomat_action req;
@@ -506,22 +515,23 @@ static void diplomat_investigate_callback(Widget w, XtPointer client_data,
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
+
 /****************************************************************
 ...
 *****************************************************************/
 static void spy_sabotage_unit_callback(Widget w, XtPointer client_data, 
 				       XtPointer call_data)
 {
-
   struct packet_diplomat_action req;
-  
+
   req.action_type=SPY_SABOTAGE_UNIT;
   req.diplomat_id=diplomat_id;
   req.target_id=diplomat_target_id;
-  
+
   send_packet_diplomat_action(&aconnection, &req);
-  
+
   destroy_message_dialog(w);
 }
 
@@ -531,20 +541,21 @@ static void spy_sabotage_unit_callback(Widget w, XtPointer client_data,
 static void spy_poison_callback(Widget w, XtPointer client_data, 
 				XtPointer call_data)
 {
-  
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      (find_city_by_id(diplomat_target_id))) { 
     struct packet_diplomat_action req;
-    
+
     req.action_type=SPY_POISON;
     req.diplomat_id=diplomat_id;
     req.target_id=diplomat_target_id;
-    
+
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -554,19 +565,21 @@ static void diplomat_steal_callback(Widget w, XtPointer client_data,
 				    XtPointer call_data)
 {
   destroy_message_dialog(w);
-  
+  diplomat_dialog_open=0;
+
   if(find_unit_by_id(diplomat_id) && 
      find_city_by_id(diplomat_target_id)) { 
     struct packet_diplomat_action req;
-    
+
     req.action_type=DIPLOMAT_STEAL;
     req.diplomat_id=diplomat_id;
     req.target_id=diplomat_target_id;
     req.value=0;
-    
+
     send_packet_diplomat_action(&aconnection, &req);
   }
 
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -575,11 +588,12 @@ static void diplomat_steal_callback(Widget w, XtPointer client_data,
 static void spy_close_tech_callback(Widget w, XtPointer client_data, 
 				    XtPointer call_data)
 {
-
   if(spy_tech_shell_is_modal)
-     XtSetSensitive(main_form, TRUE);
-   XtDestroyWidget(spy_tech_shell);
-   spy_tech_shell=0;
+    XtSetSensitive(main_form, TRUE);
+  XtDestroyWidget(spy_tech_shell);
+  spy_tech_shell=0;
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -588,11 +602,12 @@ static void spy_close_tech_callback(Widget w, XtPointer client_data,
 static void spy_close_sabotage_callback(Widget w, XtPointer client_data, 
 					XtPointer call_data)
 {
-
   if(spy_sabotage_shell_is_modal)
-     XtSetSensitive(main_form, TRUE);
-   XtDestroyWidget(spy_sabotage_shell);
-   spy_sabotage_shell=0;
+    XtSetSensitive(main_form, TRUE);
+  XtDestroyWidget(spy_sabotage_shell);
+  spy_sabotage_shell=0;
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -640,6 +655,7 @@ static void spy_steal_callback(Widget w, XtPointer client_data,
   
   if(!steal_advance){
     freelog(LOG_NORMAL, "Bug in spy steal tech code");
+    process_diplomat_arrival(NULL, 0);
     return;
   }
   
@@ -654,6 +670,8 @@ static void spy_steal_callback(Widget w, XtPointer client_data,
 
     send_packet_diplomat_action(&aconnection, &req);
   }
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -667,6 +685,7 @@ static void spy_sabotage_callback(Widget w, XtPointer client_data,
   
   if(!sabotage_improvement){
     freelog(LOG_NORMAL, "Bug in spy sabotage code");
+    process_diplomat_arrival(NULL, 0);
     return;
   }
   
@@ -681,6 +700,8 @@ static void spy_sabotage_callback(Widget w, XtPointer client_data,
 
     send_packet_diplomat_action(&aconnection, &req);
   }
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -830,7 +851,7 @@ static int create_improvements_list(struct player *pplayer,
 
   for(i=0; i<B_LAST; i++) 
     if(i != B_PALACE && pcity->improvements[i] && !is_wonder(i)) {
-      improvements_can_sabotage[j] = get_imp_name_ex(pcity, i);
+      improvements_can_sabotage[j] = get_impr_name_ex(pcity, i);
       improvement_type[j++] = i;
     }  
 
@@ -871,6 +892,7 @@ has happened to the city during latency.  Therefore we must initialize
 pvictim to NULL and account for !pvictim in create_advances_list. -- Syela */
   
   destroy_message_dialog(w);
+  diplomat_dialog_open=0;
 
   if(!spy_tech_shell){
     Position x, y;
@@ -890,21 +912,39 @@ pvictim to NULL and account for !pvictim in create_advances_list. -- Syela */
 }
 
 /****************************************************************
-...
+ Requests up-to-date list of improvements, the return of
+ which will trigger the popup_sabotage_dialog() function.
 *****************************************************************/
-static void spy_sabotage_popup(Widget w, XtPointer client_data, 
-			       XtPointer call_data)
+static void spy_request_sabotage_list(Widget w, XtPointer client_data,
+				      XtPointer call_data)
 {
-  struct city *pvcity = find_city_by_id(diplomat_target_id);
-  
   destroy_message_dialog(w);
+  diplomat_dialog_open=0;
 
+  if(find_unit_by_id(diplomat_id) &&
+     (find_city_by_id(diplomat_target_id))) {
+    struct packet_diplomat_action req;
+
+    req.action_type = SPY_GET_SABOTAGE_LIST;
+    req.diplomat_id = diplomat_id;
+    req.target_id = diplomat_target_id;
+
+    send_packet_diplomat_action(&aconnection, &req);
+  }
+}
+
+/****************************************************************
+ Pops-up the Spy sabotage dialog, upon return of list of
+ available improvements requested by the above function.
+*****************************************************************/
+void popup_sabotage_dialog(struct city *pcity)
+{  
   if(!spy_sabotage_shell){
     Position x, y;
     Dimension width, height;
     spy_sabotage_shell_is_modal=1;
 
-    create_improvements_list(game.player_ptr, pvcity, spy_sabotage_shell_is_modal);
+    create_improvements_list(game.player_ptr, pcity, spy_sabotage_shell_is_modal);
     
     XtVaGetValues(toplevel, XtNwidth, &width, XtNheight, &height, NULL);
     
@@ -931,6 +971,8 @@ static void diplomat_incite_yes_callback(Widget w, XtPointer client_data,
   send_packet_diplomat_action(&aconnection, &req);
   
   destroy_message_dialog(w);
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 /****************************************************************
@@ -940,6 +982,8 @@ static void diplomat_incite_no_callback(Widget w, XtPointer client_data,
 					XtPointer call_data)
 {
   destroy_message_dialog(w);
+
+  process_diplomat_arrival(NULL, 0);
 }
 
 
@@ -953,6 +997,7 @@ static void diplomat_incite_callback(Widget w, XtPointer client_data,
   struct packet_generic_integer packet;
 
   destroy_message_dialog(w);
+  diplomat_dialog_open=0;
 
   if(find_unit_by_id(diplomat_id) && 
      (pcity=find_city_by_id(diplomat_target_id))) { 
@@ -996,8 +1041,10 @@ void popup_incite_dialog(struct city *pcity)
 static void diplomat_cancel_callback(Widget w, XtPointer a, XtPointer b)
 {
   destroy_message_dialog(w);
-}
+  diplomat_dialog_open=0;
 
+  process_diplomat_arrival(NULL, 0);
+}
 
 
 /****************************************************************
@@ -1008,17 +1055,20 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
   struct city *pcity;
   struct unit *ptunit;
   Widget shl;
+  char buf[128];
 
   diplomat_id=punit->id;
-  
+
   if((pcity=map_get_city(dest_x, dest_y))){
-    
-    /* Spy/Diplomat acting against a city */ 
-    
+    /* Spy/Diplomat acting against a city */
+
     diplomat_target_id=pcity->id;
+    my_snprintf(buf, sizeof(buf),
+		_("Your %s has arrived at %s.\nWhat is your command?"),
+		unit_name(punit->type), pcity->name);
+
     if(!unit_flag(punit->type, F_SPY)){
-      shl=popup_message_dialog(toplevel, "diplomatdialog", 
-		       _("Sir, the diplomat is waiting for your command"),
+      shl=popup_message_dialog(toplevel, "diplomatdialog", buf,
 			       diplomat_embassy_callback, 0, 1,
 			       diplomat_investigate_callback, 0, 1,
 			       diplomat_sabotage_callback, 0, 1,
@@ -1038,12 +1088,11 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_x, dest_y))
 	XtSetSensitive(XtNameToWidget(shl, "*button4"), FALSE);
     }else{
-      shl=popup_message_dialog(toplevel, "spydialog", 
-			       _("Sir, the spy is waiting for your command"),
+      shl=popup_message_dialog(toplevel, "spydialog", buf,
 			       diplomat_embassy_callback, 0,  1,
 			       diplomat_investigate_callback, 0, 1,
 			       spy_poison_callback,0, 1,
-			       spy_sabotage_popup, 0, 1,
+			       spy_request_sabotage_list, 0, 1,
 			       spy_steal_popup, 0, 1,
 			       diplomat_incite_callback, 0, 1,
 			       diplomat_cancel_callback, 0, 0,
@@ -1062,6 +1111,8 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
       if(!diplomat_can_do_action(punit, DIPLOMAT_INCITE, dest_x, dest_y))
 	XtSetSensitive(XtNameToWidget(shl, "*button5"), FALSE);
     }
+
+    diplomat_dialog_open=1;
   }else{ 
     if((ptunit=unit_list_get(&map_get_tile(dest_x, dest_y)->units, 0))){
       /* Spy/Diplomat acting against a unit */
@@ -1087,6 +1138,14 @@ void popup_diplomat_dialog(struct unit *punit, int dest_x, int dest_y)
   }
 }
 
+/****************************************************************
+...
+*****************************************************************/
+int diplomat_dialog_is_open(void)
+{
+  return diplomat_dialog_open;
+}
+
 
 /****************************************************************
 ...
@@ -1180,6 +1239,14 @@ void popup_caravan_dialog(struct unit *punit,
     XtSetSensitive(XtNameToWidget(caravan_dialog, "*button1"), FALSE);
 }
 
+/****************************************************************
+...
+*****************************************************************/
+int caravan_dialog_is_open(void)
+{
+  return BOOL_VAL(caravan_dialog);
+}
+
 
 /****************************************************************
 ...
@@ -1450,8 +1517,8 @@ Widget popup_message_dialog(Widget parent, char *dialogname, char *text, ...)
 
   i=0;
   va_start(args, text);
-  
-  while((fcb=va_arg(args, void *))) {
+
+  while((fcb=((void(*)(Widget, XtPointer, XtPointer))(va_arg(args, void *))))) {
     client_data=va_arg(args, XtPointer);
     fixed_width=va_arg(args, int);
     my_snprintf(button_name, sizeof(button_name), "button%d", i++);
@@ -1558,7 +1625,7 @@ void popup_unit_select_dialog(struct tile *ptile)
 
     unit_select_ids[i]=punit->id;
 
-    pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+    pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
     
     my_snprintf(buffer, sizeof(buffer), "%s(%s)\n%s", 
 	    punittemp->name, 
@@ -1638,8 +1705,8 @@ void unit_select_all_callback(Widget w, XtPointer client_data,
   XtDestroyWidget(unit_select_dialog_shell);
   
   for(i=0; i<unit_select_no; i++) {
-    struct unit *punit=unit_list_find(&game.player_ptr->units, 
-				      unit_select_ids[i]);
+    struct unit *punit = player_find_unit_by_id(game.player_ptr,
+						unit_select_ids[i]);
     if(punit) {
       request_new_unit_activity(punit, ACTIVITY_IDLE);
       set_unit_focus(punit);
@@ -1661,8 +1728,8 @@ void unit_select_callback(Widget w, XtPointer client_data,
   for(i=0; i<unit_select_no; i++) {
 
     if(unit_select_commands[i]==w) {
-      struct unit *punit=unit_list_find(&game.player_ptr->units, 
-					unit_select_ids[i]);
+      struct unit *punit = player_find_unit_by_id(game.player_ptr,
+						  unit_select_ids[i]);
       if(punit) {
 	request_new_unit_activity(punit, ACTIVITY_IDLE);
 	set_unit_focus(punit);
@@ -1692,6 +1759,7 @@ void popup_races_dialog(void)
   XtVaSetValues(races_dialog_shell, XtNx, x, XtNy, y, NULL);
 
   XtPopup(races_dialog_shell, XtGrabNone);
+  XtSetKeyboardFocus(toplevel, races_dialog_shell);
 }
 
 /****************************************************************
@@ -1711,13 +1779,12 @@ void popdown_races_dialog(void)
 *****************************************************************/
 void create_races_dialog(void)
 {
-  int per_row = 4;
+  int per_row = 5;
   int i, j, len, maxracelen, index;
   char maxracename[MAX_LEN_NAME];
   char namebuf[64];
   int space;
   XtWidgetGeometry geom;
-  XtTranslations textfieldtranslations;
 
   maxracelen = 0;
   for(i=0; i<game.playable_nation_count; i++) {
@@ -1747,6 +1814,8 @@ void create_races_dialog(void)
 
   free(races_toggles);
   races_toggles = fc_calloc(game.playable_nation_count,sizeof(Widget));
+  free(races_toggles_to_nations);
+  races_toggles_to_nations = fc_calloc(game.playable_nation_count,sizeof(int));
 
   for( i = 0; i < ((game.playable_nation_count-1)/per_row)+1; i++) {
     index = i * per_row;
@@ -1944,53 +2013,52 @@ void create_races_dialog(void)
 
   races_ok_command =
     I_L(XtVaCreateManagedWidget("racesokcommand",
-			    commandWidgetClass,
-			    races_action_form,
-			    NULL));
+				commandWidgetClass,
+				races_action_form,
+				NULL));
 
-  /* if(has_capability("dconn_in_sel_nat", aconnection.capability)) */
-  {
-    races_disconnect_command =
-      I_L(XtVaCreateManagedWidget("racesdisconnectcommand",
-			      commandWidgetClass,
-			      races_action_form,
-			      XtNfromHoriz, races_ok_command,
-			      NULL));
+  races_disconnect_command =
+    I_L(XtVaCreateManagedWidget("racesdisconnectcommand",
+				commandWidgetClass,
+				races_action_form,
+				XtNfromHoriz, races_ok_command,
+				NULL));
 
-    races_quit_command =
-      I_L(XtVaCreateManagedWidget("racesquitcommand",
-			      commandWidgetClass,
-			      races_action_form,
-			      XtNfromHoriz, races_disconnect_command,
-			      NULL));
+  races_quit_command =
+    I_L(XtVaCreateManagedWidget("racesquitcommand",
+				commandWidgetClass,
+				races_action_form,
+				XtNfromHoriz, races_disconnect_command,
+				NULL));
 
-    XtAddCallback(races_disconnect_command, XtNcallback,
-		  races_disconnect_command_callback, NULL);
-    XtAddCallback(races_quit_command, XtNcallback,
-		  races_quit_command_callback, NULL);
-  }
+  XtAddCallback(races_disconnect_command, XtNcallback,
+		races_disconnect_command_callback, NULL);
+  XtAddCallback(races_quit_command, XtNcallback,
+		races_quit_command_callback, NULL);
 
 
   for(i=0; i<game.playable_nation_count; i++) {
     XtAddCallback(races_toggles[i], XtNcallback, 
-		  races_toggles_callback, (XtPointer) 0);
+		  races_toggles_callback, (XtPointer)i);
   }
 
 
   XtAddCallback(races_ok_command, XtNcallback,
 		races_ok_command_callback, NULL);
 
-  textfieldtranslations = 
-    XtParseTranslationTable("<Key>Return: key-dialog-races-return()");
-  XtOverrideTranslations(races_leader, textfieldtranslations);
-
 
   XtSetKeyboardFocus(races_form, races_leader);
 
   XtRealizeWidget(races_dialog_shell);
 
   for(i=0; i<game.playable_nation_count; i++) {
-    XtVaSetValues(races_toggles[i], XtNlabel, (XtArgVal)get_nation_name(i), NULL);
+    races_toggles_to_nations[i] = i;
+  }
+  qsort(races_toggles_to_nations, i, sizeof(int), races_indirect_compare);
+  for(i=0; i<game.playable_nation_count; i++) {
+    XtVaSetValues (races_toggles[i],
+		   XtNlabel, (XtArgVal)get_nation_name(races_toggles_to_nations[i]),
+		   NULL);
   }
 
   for(i=0; i<b_s_num; i++) {
@@ -1998,102 +2066,86 @@ void create_races_dialog(void)
 		  (XtArgVal)city_styles[city_style_idx[i]].name, NULL);
   }
 
-  x_simulate_button_click(races_toggles[0]);
+  x_simulate_button_click(races_toggles[myrand(game.playable_nation_count)]);
 }
 
 /****************************************************************
 ...
 *****************************************************************/
-void racesdlg_key_return(Widget w)
+void racesdlg_key_ok(Widget w)
 {
   Widget ok = XtNameToWidget(XtParent(XtParent(w)), "*racesokcommand");
   if (ok)
     x_simulate_button_click(ok);
 }
 
-
 /**************************************************************************
 ...
 **************************************************************************/
 void races_toggles_set_sensitive(int bits1, int bits2)
 {
-  int i, selected, mybits;
+  int i, race, selected;
+  Boolean sens;
 
-  mybits=bits1;
-
-  for(i=0; i<game.playable_nation_count && i<32; i++) {
-    if(mybits&1)
-      XtSetSensitive(races_toggles[i], FALSE);
-    else
-      XtSetSensitive(races_toggles[i], TRUE);
-    mybits>>=1;
-  }
-
-  mybits=bits2;
-
-  for(i=32; i<game.playable_nation_count; i++) {
-    if(mybits&1)
-      XtSetSensitive(races_toggles[i], FALSE);
-    else
-      XtSetSensitive(races_toggles[i], TRUE);
-    mybits>>=1;
+  for(i=0; i<game.playable_nation_count; i++) {
+    race = races_toggles_to_nations[i];
+    sens = !( ( ((race<32)?bits1:bits2) >> ((race<32)?race:race-32) )&1 );
+    XtSetSensitive(races_toggles[i], sens);
   }
 
   if((selected=races_buttons_get_current())==-1)
-     return;
+    return;
+  race = races_toggles_to_nations[selected];
 
-  if( (bits1 & (1<<selected)) || 
-      ( selected>32 && (bits2 & (1<<(selected-32))) ) )
-     XawToggleUnsetCurrent(races_toggles[0]);
+  if( (bits1 & (1<<race)) || 
+      (race>32 && (bits2 & (1<<(race-32))) ) )
+    XawToggleUnsetCurrent(races_toggles[0]);
 }
 
-
 /**************************************************************************
 ...
 **************************************************************************/
 void races_toggles_callback(Widget w, XtPointer client_data, 
 			    XtPointer call_data)
 {
-  int i, j;
+  int index, race;
+  int j;
   int leader_count;
   char **leaders;
   Widget entry;
 
-  for(i=0; i<game.playable_nation_count; i++) {
-    if(w==races_toggles[i]) {
-      leaders = get_nation_leader_names(i, &leader_count);
+  index = (int)client_data;
+  race = races_toggles_to_nations[index];
 
-      if(races_leader_pick_popupmenu)
-	XtDestroyWidget(races_leader_pick_popupmenu);
+  leaders = get_nation_leader_names(race, &leader_count);
 
-      races_leader_pick_popupmenu =
-	XtVaCreatePopupShell("menu",
-			     simpleMenuWidgetClass,
-			     races_leader_pick_menubutton,
-			     NULL);
+  if(races_leader_pick_popupmenu)
+    XtDestroyWidget(races_leader_pick_popupmenu);
 
-      for(j=0; j<leader_count; j++) {
-	entry =
-	  XtVaCreateManagedWidget(leaders[j],
-				  smeBSBObjectClass,
-				  races_leader_pick_popupmenu,
-				  NULL);
-	XtAddCallback(entry,
-		      XtNcallback,
-		      races_leader_pick_callback,
-		      (XtPointer)((MAX_NUM_NATIONS * i) + j));
-      }
+  races_leader_pick_popupmenu =
+    XtVaCreatePopupShell("menu",
+			 simpleMenuWidgetClass,
+			 races_leader_pick_menubutton,
+			 NULL);
 
-      races_leader_set_values(i, 0);
-
-      x_simulate_button_click
-      (
-       races_style_toggles[city_style_ridx[get_nation_city_style(i)]]
-      );
-
-      return;
-    }
+  for(j=0; j<leader_count; j++) {
+    entry =
+      XtVaCreateManagedWidget(leaders[j],
+			      smeBSBObjectClass,
+			      races_leader_pick_popupmenu,
+			      NULL);
+    XtAddCallback(entry,
+		  XtNcallback,
+		  races_leader_pick_callback,
+		  (XtPointer)((MAX_NUM_NATIONS * race) + j));
   }
+
+  races_leader_set_values(race, myrand(leader_count));
+
+  x_simulate_button_click
+  (
+   races_style_toggles[city_style_ridx[get_nation_city_style(race)]]
+  );
 }
 
 /**************************************************************************
@@ -2205,18 +2257,37 @@ void races_sex_buttons_set_current(int i)
   XawToggleSetCurrent(races_sex_toggles[0], dp);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+int races_indirect_compare(const void *first, const void *second)
+{
+  int first_index;
+  int second_index;
+  char *first_string;
+  char *second_string;
+
+  first_index = *((const int *)first);
+  second_index = *((const int *)second);
+
+  first_string = get_nation_name(first_index);
+  second_string = get_nation_name(second_index);
+
+  return mystrcasecmp(first_string, second_string);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
 void races_ok_command_callback(Widget w, XtPointer client_data, 
 			       XtPointer call_data)
 {
-  int selected, selected_sex, selected_style;
+  int selected_index, selected_sex, selected_style;
   XtPointer dp;
 
   struct packet_alloc_nation packet;
 
-  if((selected=races_buttons_get_current())==-1) {
+  if((selected_index=races_buttons_get_current())==-1) {
     append_output_window(_("You must select a nation."));
     return;
   }
@@ -2234,7 +2305,7 @@ void races_ok_command_callback(Widget w, XtPointer client_data,
   XtVaGetValues(races_leader, XtNstring, &dp, NULL);
 
   /* perform a minimum of sanity test on the name */
-  packet.nation_no=selected;
+  packet.nation_no=races_toggles_to_nations[selected_index];
   packet.city_style = get_nation_city_style(packet.nation_no);
   packet.is_male = selected_sex? 0: 1;     /* first button is male */
   packet.city_style = city_style_idx[selected_style];
@@ -2325,56 +2396,3 @@ void taxrates_callback(Widget w, XtPointer client_data, XtPointer call_data)
   send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_RATES);
 
 }
-
-/**************************************************************************
-  Add punit to queue of caravan arrivals, and popup a window for the
-  next arrival in the queue, if there is not already a popup, and
-  re-checking that a popup is appropriate.
-  If punit is NULL, just do for the next arrival in the queue.
-**************************************************************************/
-void process_caravan_arrival(struct unit *punit)
-{
-  static struct genlist arrival_queue;
-  static int is_init_arrival_queue = 0;
-  int *p_id;
-
-  /* arrival_queue is a list of individually malloc-ed ints with
-     punit.id values, for units which have arrived. */
-
-  if (!is_init_arrival_queue) {
-    genlist_init(&arrival_queue);
-    is_init_arrival_queue = 1;
-  }
-
-  if (punit) {
-    p_id = fc_malloc(sizeof(int));
-    *p_id = punit->id;
-    genlist_insert(&arrival_queue, p_id, -1);
-  }
-
-  /* There can only be one dialog at a time: */
-  if (caravan_dialog) {
-    return;
-  }
-  
-  while (genlist_size(&arrival_queue)) {
-    int id;
-    
-    p_id = genlist_get(&arrival_queue, 0);
-    genlist_unlink(&arrival_queue, p_id);
-    id = *p_id;
-    free(p_id);
-    punit = unit_list_find(&game.player_ptr->units, id);
-
-    if (punit && (unit_can_help_build_wonder_here(punit)
-		  || unit_can_est_traderoute_here(punit))
-	&& (!game.player_ptr->ai.control || ai_popup_windows)) {
-      struct city *pcity_dest = map_get_city(punit->x, punit->y);
-      struct city *pcity_homecity = find_city_by_id(punit->homecity);
-      if (pcity_dest && pcity_homecity) {
-	popup_caravan_dialog(punit, pcity_homecity, pcity_dest);
-	return;
-      }
-    }
-  }
-}
