@@ -15,6 +15,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -47,6 +48,7 @@
 #include "climisc.h"
 #include "clinet.h"
 #include "colors.h"
+#include "control.h"
 #include "dialogs.h"
 #include "graphics.h"
 #include "gui_stuff.h"		/* I_SW() */
@@ -66,7 +68,9 @@ AppResources appResources;
 
 extern String fallback_resources[];
 
-extern int num_units_below;
+
+/* ids of the units icons in information display: (or 0) */
+static int unit_ids[MAX_NUM_UNITS_BELOW];  
 
 
 static void setup_widgets(void);
@@ -166,6 +170,9 @@ Pixmap gray50,gray25;
 /* overall font GC                                    */
 GC font_gc;
 XFontStruct *main_font_struct;
+/* productions font GC                                */
+GC prod_font_gc;
+XFontStruct *prod_font_struct;
 
 
 Widget toplevel, main_form, menu_form, below_menu_form, left_column_form;
@@ -177,7 +184,8 @@ Widget map_vertical_scrollbar, map_horizontal_scrollbar;
 Widget inputline_text, outputwindow_text;
 Widget econ_label[10];
 Widget turn_done_button;
-Widget info_command, bulb_label, sun_label, government_label, timeout_label;
+Widget info_command;
+Widget bulb_label, sun_label, flake_label, government_label, timeout_label;
 Widget unit_info_label;
 Widget unit_pix_canvas;
 Widget unit_below_canvas[MAX_NUM_UNITS_BELOW];
@@ -187,7 +195,7 @@ Widget more_arrow_label;
 Window root_window;
 
 /* this pixmap acts as a backing store for the map_canvas widget */
-Pixmap map_canvas_store;
+Pixmap map_canvas_store = 0;
 int map_canvas_store_twidth, map_canvas_store_theight;
 
 /* this pixmap acts as a backing store for the overview_canvas widget */
@@ -265,8 +273,6 @@ void ui_main(int argc, char *argv[])
 {
   int i;
   Pixmap icon_pixmap; 
-  XtTranslations TextFieldTranslations;
-  Dimension w,h;
 
   parse_options(argc, argv);
 
@@ -353,6 +359,18 @@ void ui_main(int argc, char *argv[])
 		       GCForeground|GCBackground|GCFont|GCGraphicsExposures, 
 		       &values);
 
+    prod_font_struct=XLoadQueryFont(display, city_productions_font_name);
+    if(prod_font_struct==0) {
+      freelog(LOG_FATAL, _("Failed loading font: %s"), city_productions_font_name);
+      exit(1);
+    }
+    values.foreground = colors_standard[COLOR_STD_WHITE];
+    values.background = colors_standard[COLOR_STD_BLACK];
+    values.font = prod_font_struct->fid;
+    prod_font_gc= XCreateGC(display, root_window,
+			    GCForeground|GCBackground|GCFont|GCGraphicsExposures,
+			    &values);
+
     values.foreground = 0;
     values.background = 0;
     fill_bg_gc= XCreateGC(display, root_window, 
@@ -387,10 +405,6 @@ void ui_main(int argc, char *argv[])
 
   XtSetKeyboardFocus(bottom_form, inputline_text);
   XtSetKeyboardFocus(below_menu_form, map_canvas);
-  
-  TextFieldTranslations = XtParseTranslationTable  /*BLAH!*/
-		("<Key>Return: key-chatline-return()");
-  XtOverrideTranslations(inputline_text, TextFieldTranslations);
 
   InitializeActions(app_context);
 
@@ -420,13 +434,7 @@ void ui_main(int argc, char *argv[])
   x_interval_id=XtAppAddTimeOut(app_context, 500,
 				(XtTimerCallbackProc)timer_callback, NULL);
 
-  XtVaGetValues(map_canvas, XtNheight, &h, XtNwidth, &w, NULL);
-  map_canvas_store_twidth=w/NORMAL_TILE_WIDTH;
-  map_canvas_store_theight=h/NORMAL_TILE_HEIGHT;
-  map_canvas_store=XCreatePixmap(display, XtWindow(map_canvas), 
-				 map_canvas_store_twidth*NORMAL_TILE_WIDTH,
-				 map_canvas_store_theight*NORMAL_TILE_HEIGHT,
-				 display_depth);
+  map_canvas_resize();
 
   overview_canvas_store=0;
   set_overview_dimensions(80, 50);
@@ -445,8 +453,8 @@ void ui_main(int argc, char *argv[])
     unit_below_pixmap[i]=XCreatePixmap(display, XtWindow(overview_canvas), 
 				       NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT, 
 				       display_depth);  
-  
-  set_bulb_sol_government(0, 0, 0);
+
+  set_indicator_icons(0, 0, 0, 0);
 
   wm_delete_window = XInternAtom(XtDisplay(toplevel), "WM_DELETE_WINDOW", 0);
   XSetWMProtocols(display, XtWindow(toplevel), &wm_delete_window, 1);
@@ -461,12 +469,34 @@ void ui_main(int argc, char *argv[])
 }
 
 
+/**************************************************************************
+  Callack for when user clicks one of the unit icons on left hand side
+  (units on same square as current unit).  Use unit_ids[] data and change
+  focus to clicked unit.
+**************************************************************************/
+static void unit_icon_callback(Widget w, XtPointer client_data,
+			       XtPointer call_data) 
+{
+  struct unit *punit;
+  int i = (size_t)client_data;
+
+  assert(i>=0 && i<num_units_below);
+  if (unit_ids[i] == 0) /* no unit displayed at this place */
+    return;
+  punit=find_unit_by_id(unit_ids[i]);
+  if(punit) { /* should always be true at this point */
+     request_new_unit_activity(punit, ACTIVITY_IDLE);
+     set_unit_focus(punit);
+  }
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
 void setup_widgets(void)
 {
   long i;
+  int econ_label_count=10, econ_label_space=1;
 
   main_form = XtVaCreateManagedWidget("mainform", formWidgetClass, 
 				      toplevel, 
@@ -525,9 +555,8 @@ void setup_widgets(void)
 				       NULL);   
 
 
-
   /* Don't put the citizens in here yet because not loaded yet */
-  for(i=0;i<10;i++)  {
+  for(i=0;i<econ_label_count;i++)  {
     econ_label[i] = XtVaCreateManagedWidget("econlabels",
 					    commandWidgetClass,
 					    left_column_form,
@@ -535,25 +564,25 @@ void setup_widgets(void)
 					    XtNheight, SMALL_TILE_HEIGHT,
 					    i?XtNfromHoriz:NULL, 
 					    i?econ_label[i-1]:NULL,
-					    XtNhorizDistance, 1,
+					    XtNhorizDistance, econ_label_space,
 					    NULL);  
   }
-
-  turn_done_button = I_L(XtVaCreateManagedWidget("turndonebutton", 
-						 commandWidgetClass,
-						 left_column_form,
-						 NULL));
   
   bulb_label = XtVaCreateManagedWidget("bulblabel", 
 				       labelWidgetClass,
 				       left_column_form,
 				       NULL);
-  
+
   sun_label = XtVaCreateManagedWidget("sunlabel", 
 				      labelWidgetClass, 
 				      left_column_form,
 				      NULL);
 
+  flake_label = XtVaCreateManagedWidget("flakelabel", 
+					labelWidgetClass, 
+					left_column_form,
+					NULL);
+
   government_label = XtVaCreateManagedWidget("governmentlabel", 
 					    labelWidgetClass, 
 					    left_column_form,
@@ -564,7 +593,15 @@ void setup_widgets(void)
 					  left_column_form,
 					  NULL);
 
-  
+
+  turn_done_button =
+    I_LW(XtVaCreateManagedWidget("turndonebutton", 
+				 commandWidgetClass,
+				 left_column_form,
+				 XtNwidth, econ_label_count*
+						(SMALL_TILE_WIDTH+econ_label_space),
+				 NULL));
+
   
   unit_info_label = XtVaCreateManagedWidget("unitinfolabel", 
 					    labelWidgetClass, 
@@ -588,6 +625,8 @@ void setup_widgets(void)
 						   XtNwidth, NORMAL_TILE_WIDTH,
 						   XtNheight, NORMAL_TILE_HEIGHT,
 						   NULL);
+    XtAddCallback(unit_below_canvas[i], XtNcallback, unit_icon_callback,
+		  (XtPointer)i);  
   }
 
   more_arrow_label =
@@ -691,6 +730,14 @@ void main_show_info_popup(XEvent *event)
   }
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void sound_bell(void)
+{
+  XBell(display, 100);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -702,9 +749,6 @@ void enable_turn_done_button(void)
   update_turn_done_button(1);
   XtSetSensitive(turn_done_button, 
                  !game.player_ptr->ai.control||ai_manual_turn_done);
-
-  if(sound_bell_at_new_turn)
-    XBell(display, 100);
 }
 
 /**************************************************************************
@@ -733,6 +777,7 @@ void add_net_input(int sock)
 void remove_net_input(void)
 {
   XtRemoveInput(x_input_id);
+  XUndefineCursor(display, XtWindow(map_canvas));
 }
 
 /**************************************************************************
@@ -777,10 +822,57 @@ void timer_callback(caddr_t client_data, XtIntervalId *id)
     
     if(flip) {
       update_timeout_label();
-    if(seconds_to_turndone)
+      if(seconds_to_turndone > 0)
 	seconds_to_turndone--;
+      else
+	seconds_to_turndone = 0;
     }
     
     flip=!flip;
   }
 }
+
+/**************************************************************************
+  Set one of the unit icons in information area based on punit.
+  Use punit==NULL to clear icon.
+  Index 'idx' is -1 for "active unit", or 0 to (num_units_below-1) for
+  units below.  Also updates unit_ids[idx] for idx>=0.
+**************************************************************************/
+void set_unit_icon(int idx, struct unit *punit)
+{
+  Widget w;
+  
+  assert(idx>=-1 && idx<num_units_below);
+  if (idx == -1) {
+    w = unit_pix_canvas;
+  } else {
+    w = unit_below_canvas[idx];
+    unit_ids[idx] = punit ? punit->id : 0;
+  }
+  
+  if (flags_are_transparent || punit==NULL) {
+    XawPixcommClear(w);
+  }
+  if (punit) {
+    put_unit_pixmap(punit, XawPixcommPixmap(w), 0, 0);
+    xaw_expose_now(w);
+  }
+}
+
+/**************************************************************************
+  Set the "more arrow" for the unit icons to on(1) or off(0).
+  Maintains a static record of current state to avoid unnecessary redraws.
+  Note initial state should match initial gui setup (off).
+**************************************************************************/
+void set_unit_icons_more_arrow(int onoff)
+{
+  static int showing = 0;
+  if (onoff==1 && !showing) {
+    xaw_set_bitmap(more_arrow_label, sprites.right_arrow->pixmap);
+    showing = 1;
+  }
+  else if(onoff==0 && showing) {
+    xaw_set_bitmap(more_arrow_label, None);
+    showing = 0;
+  }
+}
