@@ -124,7 +124,7 @@ static void popit(int xin, int yin, int xtile, int ytile)
   struct tile *ptile=map_get_tile(xtile, ytile);
 
   
-  if(ptile->known>=TILE_KNOWN) {
+  if(ptile->known>=TILE_KNOWN_FOGGED) {
     Widget p=XtCreatePopupShell("popupinfo", simpleMenuWidgetClass,
 				map_canvas, NULL, 0);
     my_snprintf(s, sizeof(s), _("Terrain: %s"),
@@ -149,13 +149,13 @@ static void popit(int xin, int yin, int xtile, int ytile)
       XtCreateManagedWidget(s, smeBSBObjectClass, p, NULL, 0);
     }
 
-    if((punit=player_find_visible_unit(game.player_ptr, ptile)) && !pcity) {
+    if((punit=find_visible_unit(ptile)) && !pcity) {
       char cn[64];
       struct unit_type *ptype=get_unit_type(punit->type);
       cn[0]='\0';
       if(punit->owner==game.player_idx) {
 	struct city *pcity;
-	pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+	pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
 	if(pcity)
 	  my_snprintf(cn, sizeof(cn), "/%s", pcity->name);
       }
@@ -403,7 +403,6 @@ void mapctrl_btn_overviewcanvas(XEvent *event)
 void focus_to_next_unit(void)
 {
   advance_unit_focus();
-  /* set_unit_focus(punit_focus); */  /* done in advance_unit_focus */
 }
 
 /**************************************************************************
