@@ -36,6 +36,7 @@
 
 
 #include "civclient.h"
+#include "climisc.h"
 #include "colors.h"
 #include "control.h" /* set_unit_focus_no_center and get_unit_in_focus */
 #include "graphics.h"
@@ -46,12 +47,29 @@
 
 #include "mapview.h"
 
+/*
+The bottom row of the map was sometimes hidden.
+
+As of now the top left corner is always aligned with the tiles. This is what
+causes the problem in the first place. The ideal solution would be to align the
+window with the bottom left tiles if you tried to center the window on a tile
+closer than (screen_tiles_height/2 -1) to the south pole.
+
+But, for now, I just grepped for occurences where the ysize (or the values
+derived from it) were used, and those places that had relevance to drawing the
+map, and I added 1 (using the EXTRA_BOTTOM_ROW constant).
+
+-Thue
+*/
+#define EXTRA_BOTTOM_ROW 1
+
 
 extern Display	*display;
-extern GC civ_gc, font_gc;
+extern GC civ_gc, font_gc, prod_font_gc;
 extern GC fill_bg_gc;
 extern GC fill_tile_gc;
 extern XFontStruct *main_font_struct;
+extern XFontStruct *prod_font_struct;
 extern Window root_window;
 
 extern Widget map_canvas, map_form, info_command, turn_done_button;
@@ -59,10 +77,8 @@ extern Widget map_vertical_scrollbar, map_horizontal_scrollbar;
 extern Widget overview_canvas, main_form, left_column_form;
 extern Widget menu_form, below_menu_form, bottom_form;
 extern Widget econ_label[10];
-extern Widget bulb_label, sun_label, government_label, timeout_label;
+extern Widget bulb_label, sun_label, flake_label, government_label, timeout_label;
 extern Widget unit_info_label;
-extern Widget unit_pix_canvas, unit_below_canvas[MAX_NUM_UNITS_BELOW];
-extern Widget more_arrow_label;
 extern int display_depth;
 extern Pixmap single_tile_pixmap;
 extern Pixmap gray50,gray25;
@@ -87,16 +103,13 @@ int map_view_x0, map_view_y0;
 /* used by map_canvas expose func */ 
 int force_full_repaint;
 
-/* adjusted depending on tile size: */
-int num_units_below = MAX_NUM_UNITS_BELOW;
-
 extern int goto_state;
 extern int paradrop_state;
 extern int nuke_state;
 
 static void pixmap_put_overlay_tile(Pixmap pixmap, int x, int y,
  				    struct Sprite *ssprite);
-static void show_city_names(void);
+static void show_city_descriptions(void);
 
 /* the intro picture is held in this pixmap, which is scaled to
    the screen size */
@@ -114,18 +127,19 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
   struct unit *losing_unit = (hp0 == 0 ? punit0 : punit1);
   int i;
 
-  set_unit_focus_no_center(punit0);
-  set_unit_focus_no_center(punit1);
-  
   if (!do_combat_animation) {
     punit0->hp = hp0;
     punit1->hp = hp1;
+
+    set_units_in_combat(NULL, NULL);
     refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
     refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
 
     return;
   }
 
+  set_units_in_combat(punit0, punit1);
+
   do {
     anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
 
@@ -163,42 +177,11 @@ void decrease_unit_hp_smooth(struct unit *punit0, int hp0,
     usleep_since_timer_start(anim_timer, 20000);
   }
 
+  set_units_in_combat(NULL, NULL);
   refresh_tile_mapcanvas(punit0->x, punit0->y, 1);
   refresh_tile_mapcanvas(punit1->x, punit1->y, 1);
 }
 
-
-/**************************************************************************
-...
-**************************************************************************/
-void blink_active_unit(void)
-{
-  static int is_shown;
-  struct unit *punit;
-  
-  if((punit=get_unit_in_focus())) {
-    struct tile *ptile;
-    ptile=map_get_tile(punit->x, punit->y);
-
-    if(is_shown) {
-      struct unit_list units;
-      units=ptile->units;
-      unit_list_init(&ptile->units);
-      refresh_tile_mapcanvas(punit->x, punit->y, 1);
-      ptile->units=units;
-    }
-    else {
-      /* make sure that the blinking unit is always on the top */      
-      unit_list_unlink(&ptile->units, punit);
-      unit_list_insert(&ptile->units, punit);
-      refresh_tile_mapcanvas(punit->x, punit->y, 1);
-    }
-      
-    is_shown=!is_shown;
-  }
-}
-
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -261,7 +244,10 @@ void update_timeout_label(void)
 {
   char buffer[512];
 
-  my_snprintf(buffer, sizeof(buffer), "%d", seconds_to_turndone);
+  if (game.timeout <= 0)
+    sz_strlcpy(buffer, Q_("?timeout:off"));
+  else
+    format_duration(buffer, sizeof(buffer), seconds_to_turndone);
   xaw_set_label(timeout_label, buffer);
 }
 
@@ -285,12 +271,11 @@ void update_info_label(void)
 	  game.player_ptr->economic.luxury,
 	  game.player_ptr->economic.science);
   xaw_set_label(info_command, buffer);
-  if (game.heating>7) game.heating=7;
-  set_bulb_sol_government(8*game.player_ptr->research.researched/
-			 (research_time(game.player_ptr)+1),
-			 game.heating, 
-			 game.player_ptr->government);
 
+  set_indicator_icons(client_research_sprite(),
+		      client_warming_sprite(),
+		      client_cooling_sprite(),
+		      game.player_ptr->government);
 
   d=0;
   for(;d<(game.player_ptr->economic.luxury)/10;d++)
@@ -307,14 +292,21 @@ void update_info_label(void)
 
 
 /**************************************************************************
-...
+  Update the information label which gives info on the current unit and the
+  square under the current unit, for specified unit.  Note that in practice
+  punit is almost always (or maybe strictly always?) the focus unit.
+  Clears label if punit is NULL.
+  Also updates the cursor for the map_canvas (this is related because the
+  info label includes a "select destination" prompt etc).
+  Also calls update_unit_pix_label() to update the icons for units on this
+  square.
 **************************************************************************/
 void update_unit_info_label(struct unit *punit)
 {
   if(punit) {
     char buffer[512];
     struct city *pcity;
-    pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+    pcity=player_find_city_by_id(game.player_ptr, punit->homecity);
     my_snprintf(buffer, sizeof(buffer), "%s %s\n%s\n%s\n%s", 
 	    get_unit_type(punit->type)->name,
 	    (punit->veteran) ? _("(veteran)") : "",
@@ -343,92 +335,6 @@ void update_unit_info_label(struct unit *punit)
   update_unit_pix_label(punit);
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-void update_unit_pix_label(struct unit *punit)
-{
-  int i;
-  /* what initialises these statics? */
-  static enum unit_activity uactivity = ACTIVITY_UNKNOWN;
-  static int utemplate = U_LAST;
-  static int unit_ids[MAX_NUM_UNITS_BELOW];
-  static int showing_arrow=0;
-  struct genlist_iterator myiter;
-  
-  if(punit) {
-    if(punit->type!=utemplate || punit->activity!=uactivity) {
-      if (flags_are_transparent)
-        XawPixcommClear(unit_pix_canvas); /* STG */
-      put_unit_pixmap(punit, XawPixcommPixmap(unit_pix_canvas), 0, 0);
-      xaw_expose_now(unit_pix_canvas);
-      utemplate=punit->type;
-      uactivity=punit->activity;
-    }
-    genlist_iterator_init(&myiter, 
-			  &(map_get_tile(punit->x, punit->y)->units.list), 0);
-    
-    for(i=0; i<num_units_below && ITERATOR_PTR(myiter); i++) {
-      int id;
-      id=ITERATOR_PTR(myiter) ? ((struct unit *)ITERATOR_PTR(myiter))->id : 0;
-      if(id==punit->id) {
-	ITERATOR_NEXT(myiter);
-	i--;
-	continue;
-      }
-      
-      /* IS THIS INTENTIONAL?? - mjd */
-      if(1 || unit_ids[i]!=id) {
-	if(id) {
-	  if (flags_are_transparent)
-	    XawPixcommClear(unit_below_canvas[i]); /* STG */
-	  put_unit_pixmap((struct unit *)ITERATOR_PTR(myiter),
-			  XawPixcommPixmap(unit_below_canvas[i]),
-			  0, 0);
-	  xaw_expose_now(unit_below_canvas[i]);
-	}
-	else
-	  XawPixcommClear(unit_below_canvas[i]);
-	  
-	unit_ids[i]=id;
-      }
-      ITERATOR_NEXT(myiter);
-    }
-
-    
-    for(; i<num_units_below; i++) {
-      XawPixcommClear(unit_below_canvas[i]);
-      unit_ids[i]=0;
-    }
-
-    
-    if(ITERATOR_PTR(myiter)) {
-      if(!showing_arrow) {
-	xaw_set_bitmap(more_arrow_label, sprites.right_arrow->pixmap);
-	showing_arrow=1;
-      }
-    }
-    else {
-      if(showing_arrow) {
-	xaw_set_bitmap(more_arrow_label, None);
-	showing_arrow=0;
-      }
-    }
-  }
-  else {
-    XawPixcommClear(unit_pix_canvas);
-    utemplate=U_LAST;
-    uactivity=ACTIVITY_UNKNOWN;
-    for(i=0; i<num_units_below; i++) {
-      XawPixcommClear(unit_below_canvas[i]);
-      unit_ids[i]=0;
-    }
-    xaw_set_bitmap(more_arrow_label, None);
-    showing_arrow=0;
-  }
-}
-
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -450,25 +356,25 @@ Pixmap get_citizen_pixmap(int frame)
 /**************************************************************************
 ...
 **************************************************************************/
-void set_bulb_sol_government(int bulb, int sol, int government)
+void set_indicator_icons(int bulb, int sol, int flake, int gov)
 {
   struct Sprite *gov_sprite;
-  
+
   bulb = CLIP(0, bulb, NUM_TILES_PROGRESS-1);
-  sol  = CLIP(0, sol, NUM_TILES_PROGRESS-1);
-  
+  sol = CLIP(0, sol, NUM_TILES_PROGRESS-1);
+  flake = CLIP(0, flake, NUM_TILES_PROGRESS-1);
+
   xaw_set_bitmap(bulb_label, sprites.bulb[bulb]->pixmap);
   xaw_set_bitmap(sun_label, sprites.warming[sol]->pixmap);
+  xaw_set_bitmap(flake_label, sprites.cooling[flake]->pixmap);
 
   if (game.government_count==0) {
     /* not sure what to do here */
     gov_sprite = sprites.citizen[7]; 
   } else {
-    gov_sprite = get_government(government)->sprite;
+    gov_sprite = get_government(gov)->sprite;
   }
-
-  if (gov_sprite)
-    xaw_set_bitmap(government_label, gov_sprite->pixmap);
+  xaw_set_bitmap(government_label, gov_sprite->pixmap);
 }
 
 /**************************************************************************
@@ -525,59 +431,83 @@ int tile_visible_mapcanvas(int x, int y)
 **************************************************************************/
 int tile_visible_and_not_on_border_mapcanvas(int x, int y)
 {
-  return (y>=map_view_y0+1 && y<map_view_y0+map_canvas_store_theight-1 &&
-	  ((x>=map_view_x0+1 && x<map_view_x0+map_canvas_store_twidth-1) ||
-	   (x+map.xsize>=map_view_x0+1 && 
-	    x+map.xsize<map_view_x0+map_canvas_store_twidth-1)));
+  return ((y>=map_view_y0+2 || (y >= map_view_y0 && map_view_y0 == 0))
+	  && (y<map_view_y0+map_canvas_store_theight-2 ||
+	      (y<map_view_y0+map_canvas_store_theight &&
+	       map_view_y0 + map_canvas_store_theight-EXTRA_BOTTOM_ROW == map.ysize))
+	  && ((x>=map_view_x0+2 && x<map_view_x0+map_canvas_store_twidth-2) ||
+	      (x+map.xsize>=map_view_x0+2
+	       && x+map.xsize<map_view_x0+map_canvas_store_twidth-2)));
 }
 
 /**************************************************************************
-...
+Animates punit's "smooth" move from (x0,y0) to (x0+dx,y0+dy).
+Note: Works only for adjacent-square moves.
+(Tiles need not be square.)
 **************************************************************************/
 void move_unit_map_canvas(struct unit *punit, int x0, int y0, int dx, int dy)
 {
-  int i;
+  static struct timer *anim_timer = NULL; 
   int dest_x, dest_y;
 
-  dest_x=map_adjust_x(x0+dx);
-  dest_y=map_adjust_y(y0+dy);
+  /* only works for adjacent-square moves */
+  if ((dx < -1) || (dx > 1) || (dy < -1) || (dy > 1) ||
+      ((dx == 0) && (dy == 0))) {
+    return;
+  }
 
-  if(player_can_see_unit(game.player_ptr, punit) && (
-     tile_visible_mapcanvas(x0, y0) || 
-     tile_visible_mapcanvas(dest_x, dest_y))) {
-    int x, y;
+  dest_x = map_adjust_x(x0+dx);
+  dest_y = map_adjust_y(y0+dy);
 
-    put_unit_pixmap(punit, single_tile_pixmap, 0, 0);
+  if (player_can_see_unit(game.player_ptr, punit) &&
+      (tile_visible_mapcanvas(x0, y0) ||
+       tile_visible_mapcanvas(dest_x, dest_y))) {
+    int i, steps;
+    int start_x, start_y;
+    int this_x, this_y;
 
-    if(x0>=map_view_x0)
-      x=(x0-map_view_x0)*NORMAL_TILE_WIDTH;
-    else
-      x=(map.xsize-map_view_x0+x0)*NORMAL_TILE_WIDTH;
-    
-    y=(y0-map_view_y0)*NORMAL_TILE_HEIGHT;
+    if (smooth_move_unit_steps < 2) {
+      steps = 2;
+    } else if (smooth_move_unit_steps >
+	       MIN(NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT)) {
+      steps = MIN(NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
+    } else {
+      steps = smooth_move_unit_steps;
+    }
 
-    for(i=0; i<NORMAL_TILE_WIDTH; i++) {
-      if(dy>0)
-	XCopyArea(display, map_canvas_store, XtWindow(map_canvas),
-		  civ_gc, x, y, NORMAL_TILE_WIDTH, 1, x, y);
-      else if(dy<0)
-	XCopyArea(display, map_canvas_store, XtWindow(map_canvas),
-		  civ_gc, x, y+NORMAL_TILE_HEIGHT-1, NORMAL_TILE_WIDTH, 1,
-		  x, y+NORMAL_TILE_HEIGHT-1);
-      
-      if(dx>0)
-	XCopyArea(display, map_canvas_store, XtWindow(map_canvas),
-		  civ_gc, x, y, 1, NORMAL_TILE_HEIGHT, x, y);
-      else if(dx<0)
-	XCopyArea(display, map_canvas_store, XtWindow(map_canvas),
-		  civ_gc, x+NORMAL_TILE_WIDTH-1, y, 1, NORMAL_TILE_HEIGHT,
-		  x+NORMAL_TILE_WIDTH-1, y);
+    if(x0 >= map_view_x0) {
+      start_x = (x0 - map_view_x0) * NORMAL_TILE_WIDTH;
+    } else {
+      start_x = (map.xsize - map_view_x0 + x0) * NORMAL_TILE_WIDTH;
+    }
+    start_y = (y0 - map_view_y0) * NORMAL_TILE_HEIGHT;
+
+    this_x = start_x;
+    this_y = start_y;
+
+    for (i = 1; i <= steps; i++) {
+      anim_timer = renew_timer_start(anim_timer, TIMER_USER, TIMER_ACTIVE);
+
+      XCopyArea(display, map_canvas_store, XtWindow(map_canvas), civ_gc,
+		this_x, this_y, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		this_x, this_y);
+
+      this_x = start_x + (dx * ((i * NORMAL_TILE_WIDTH) / steps));
+      this_y = start_y + (dy * ((i * NORMAL_TILE_HEIGHT) / steps));
+
+      XCopyArea(display, map_canvas_store, single_tile_pixmap, civ_gc,
+		this_x, this_y, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		0, 0);
+      put_unit_pixmap(punit, single_tile_pixmap, 0, 0);
+
+      XCopyArea(display, single_tile_pixmap, XtWindow(map_canvas), civ_gc,
+		0, 0, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		this_x, this_y);
 
-      x+=dx; y+=dy;
-      
-      XCopyArea(display, single_tile_pixmap, XtWindow(map_canvas),
-		civ_gc, 0, 0, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT, x, y);
       XSync(display, 0);
+      if (i < steps) {
+	usleep_since_timer_start(anim_timer, 10000);
+      }
     }
   }
 }
@@ -600,8 +530,9 @@ void center_tile_mapcanvas(int x, int y)
 
   new_map_view_x0=map_adjust_x(x-map_canvas_store_twidth/2);
   new_map_view_y0=map_adjust_y(y-map_canvas_store_theight/2);
-  if(new_map_view_y0>map.ysize-map_canvas_store_theight)
-     new_map_view_y0=map_adjust_y(map.ysize-map_canvas_store_theight);
+  if(new_map_view_y0>map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight)
+     new_map_view_y0=
+       map_adjust_y(map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight);
 
   map_view_x0=new_map_view_x0;
   map_view_y0=new_map_view_y0;
@@ -641,28 +572,8 @@ void overview_canvas_expose(Widget w, XEvent *event, Region exposed,
 **************************************************************************/
 static void set_overview_tile_foreground_color(int x, int y)
 {
-  struct tile *ptile=map_get_tile(x, y);
-  struct unit *punit;
-  struct city *pcity;
-  
-  if(!ptile->known)
-    XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_BLACK]);
-  else if((punit=player_find_visible_unit(game.player_ptr, ptile))) {
-    if(punit->owner==game.player_idx)
-      XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_YELLOW]);
-    else
-      XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_RED]);
-  }
-  else if((pcity=map_get_city(x, y))) {
-    if(pcity->owner==game.player_idx)
-      XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_WHITE]);
-    else
-      XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_CYAN]);
-  }
-  else if(ptile->terrain==T_OCEAN)
-    XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_OCEAN]);
-  else
-    XSetForeground(display, fill_bg_gc, colors_standard[COLOR_STD_GROUND]);
+  XSetForeground(display, fill_bg_gc,
+		 colors_standard[overview_tile_color(x, y)]);
 }
 
 
@@ -689,10 +600,11 @@ void refresh_overview_canvas(void)
 **************************************************************************/
 void overview_update_tile(int x, int y)
 {
-  int pos=x+map.xsize/2-(map_view_x0+map_canvas_store_twidth/2);
+  int pos = x + map.xsize/2 - (map_view_x0 + map_canvas_store_twidth/2);
   
-  if(pos>=map.xsize)
-    pos-=map.xsize;
+  pos %= map.xsize;
+  if (pos < 0)
+    pos += map.xsize;
   
   set_overview_tile_foreground_color(x, y);
   XFillRectangle(display, overview_canvas_store, fill_bg_gc, x*2, y*2, 
@@ -751,14 +663,13 @@ void refresh_overview_viewrect(void)
 void map_canvas_expose(Widget w, XEvent *event, Region exposed, 
 		       void *client_data)
 {
-  Dimension height, width;
+  Dimension width, height;
   int tile_width, tile_height;
 
-  XtVaGetValues(w, XtNheight, &height, XtNwidth, &width, NULL);
-
+  XtVaGetValues(w, XtNwidth, &width, XtNheight, &height, NULL);
   tile_width=(width+NORMAL_TILE_WIDTH-1)/NORMAL_TILE_WIDTH;
   tile_height=(height+NORMAL_TILE_HEIGHT-1)/NORMAL_TILE_HEIGHT;
-  
+
   if(get_client_state()!=CLIENT_GAME_RUNNING_STATE) {
     if(!intro_gfx_sprite)  load_intro_gfx();
     if(height!=scaled_intro_pixmap_height || width!=scaled_intro_pixmap_width) {
@@ -772,60 +683,71 @@ void map_canvas_expose(Widget w, XEvent *event, Region exposed,
       scaled_intro_pixmap_width=width;
       scaled_intro_pixmap_height=height;
     }
-    
+
     if(scaled_intro_pixmap)
        XCopyArea(display, scaled_intro_pixmap, XtWindow(map_canvas),
 		 civ_gc,
 		 event->xexpose.x, event->xexpose.y,
 		 event->xexpose.width, event->xexpose.height,
 		 event->xexpose.x, event->xexpose.y);
+
+    if(map_canvas_store_twidth !=tile_width ||
+       map_canvas_store_theight!=tile_height) { /* resized? */
+      map_canvas_resize();
+    }
     return;
   }
   if(scaled_intro_pixmap) {
     XFreePixmap(display, scaled_intro_pixmap);
     scaled_intro_pixmap=0; scaled_intro_pixmap_height=0;
   }
-  
-  if(map.xsize) { /* do we have a map at all */
-    int tile_width=(width+NORMAL_TILE_WIDTH-1)/NORMAL_TILE_WIDTH;
-    int tile_height=(height+NORMAL_TILE_HEIGHT-1)/NORMAL_TILE_HEIGHT;
 
+  if(map.xsize) { /* do we have a map at all */
     if(map_canvas_store_twidth !=tile_width ||
        map_canvas_store_theight!=tile_height) { /* resized? */
-      
-      XFreePixmap(display, map_canvas_store);
-      
-      map_canvas_store_twidth=tile_width;
-      map_canvas_store_theight=tile_height;
+      map_canvas_resize();
 
-      
-      map_canvas_store=XCreatePixmap(display, XtWindow(map_canvas), 
-				     tile_width*NORMAL_TILE_WIDTH,
-				     tile_height*NORMAL_TILE_HEIGHT,
- 				     display_depth);
-				     
       XFillRectangle(display, map_canvas_store, fill_bg_gc, 0, 0, 
 		     NORMAL_TILE_WIDTH*map_canvas_store_twidth,
 		     NORMAL_TILE_HEIGHT*map_canvas_store_theight);
-      
+
       update_map_canvas(0, 0, map_canvas_store_twidth,
 			map_canvas_store_theight, 1);
-      
+
       update_map_canvas_scrollbars();
       refresh_overview_viewrect();
-
-    }
-    else {
+    } else {
       XCopyArea(display, map_canvas_store, XtWindow(map_canvas),
 		civ_gc,
 		event->xexpose.x, event->xexpose.y,
 		event->xexpose.width, event->xexpose.height,
 		event->xexpose.x, event->xexpose.y);
+      show_city_descriptions();
     }
   }
   refresh_overview_canvas();
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void map_canvas_resize(void)
+{
+  Dimension width, height;
+
+  if (map_canvas_store)
+    XFreePixmap(display, map_canvas_store);
+
+  XtVaGetValues(map_canvas, XtNwidth, &width, XtNheight, &height, NULL);
+  map_canvas_store_twidth=((width-1)/NORMAL_TILE_WIDTH)+1;
+  map_canvas_store_theight=((height-1)/NORMAL_TILE_HEIGHT)+1;
+
+  map_canvas_store=XCreatePixmap(display, XtWindow(map_canvas),
+				 map_canvas_store_twidth*NORMAL_TILE_WIDTH,
+				 map_canvas_store_theight*NORMAL_TILE_HEIGHT,
+				 display_depth);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -850,7 +772,7 @@ void update_map_canvas(int tile_x, int tile_y, int width, int height,
 	      tile_y*NORMAL_TILE_HEIGHT);
 
     if(width==map_canvas_store_twidth && height==map_canvas_store_theight) {
-      show_city_names();
+      show_city_descriptions();
     }
     
   }
@@ -871,92 +793,104 @@ void update_map_canvas_scrollbars(void)
 {
   float shown_h=(float)map_canvas_store_twidth/(float)map.xsize;
   float top_h=(float)map_view_x0/(float)map.xsize;
-  float shown_v=(float)map_canvas_store_theight/(float)map.ysize;
-  float top_v=(float)map_view_y0/(float)map.ysize;
+  float shown_v=(float)map_canvas_store_theight/((float)map.ysize+EXTRA_BOTTOM_ROW);
+  float top_v=(float)map_view_y0/((float)map.ysize+EXTRA_BOTTOM_ROW);
 
   XawScrollbarSetThumb(map_horizontal_scrollbar, top_h, shown_h);
   XawScrollbarSetThumb(map_vertical_scrollbar, top_v, shown_v);
 }
 
+/**************************************************************************
+Update display of descriptions associated with cities on the main map.
+**************************************************************************/
+void update_city_descriptions(void)
+{
+  update_map_canvas(0, 0, map_canvas_store_twidth,
+		    map_canvas_store_theight, 1);
+}
+
+/**************************************************************************
+Draw at x = center of string, y = top of string.
+**************************************************************************/
+static void draw_shadowed_string(XFontStruct * font, GC font_gc,
+					  int x, int y, const char *string)
+{
+  Window wndw=XtWindow(map_canvas);
+  int len=strlen(string);
+  int wth=XTextWidth(font, string, len);
+  int xs=x-wth/2;
+  int ys=y+font->ascent;
+
+  XSetForeground(display, font_gc, colors_standard[COLOR_STD_BLACK]);
+  XDrawString(display, wndw, font_gc, xs+1, ys+1, string, len);
+
+  XSetForeground(display, font_gc, colors_standard[COLOR_STD_WHITE]);
+  XDrawString(display, wndw, font_gc, xs, ys, string, len);
+}
 
 /**************************************************************************
 ...
 **************************************************************************/
-static void show_city_names(void)
+static void show_city_descriptions(void)
 {
   int x, y;
 
+  if (!draw_city_names && !draw_city_productions)
+    return;
+
   for(y=0; y<map_canvas_store_theight; ++y) { 
     int ry=map_view_y0+y;
     for(x=0; x<map_canvas_store_twidth; ++x) { 
       int rx=(map_view_x0+x)%map.xsize;
       struct city *pcity;
       if((pcity=map_get_city(rx, ry))) {
-	Window wndw=XtWindow(map_canvas);
-	int w=XTextWidth(main_font_struct, pcity->name, strlen(pcity->name));
-	int xs=x*NORMAL_TILE_WIDTH+NORMAL_TILE_WIDTH/2-w/2;
-	int ys=(y+1)*NORMAL_TILE_HEIGHT+main_font_struct->ascent;
-	int len=strlen(pcity->name);
 
-	XSetForeground(display, font_gc, colors_standard[COLOR_STD_BLACK]);
-	XDrawString(display, wndw, font_gc, xs+1, ys+1, pcity->name, len);
+	if (draw_city_names) {
+	  draw_shadowed_string(main_font_struct, font_gc,
+			       x*NORMAL_TILE_WIDTH+NORMAL_TILE_WIDTH/2,
+			       (y+1)*NORMAL_TILE_HEIGHT,
+			       pcity->name);
+	}
+
+	if (draw_city_productions && (pcity->owner==game.player_idx)) {
+	  int turns, show_turns = 1;
+	  struct unit_type *punit_type;
+	  struct impr_type *pimpr_type;
+	  char *name;
+	  char buffer[512];
+
+	  turns = city_turns_to_build(pcity, pcity->currently_building,
+				      pcity->is_building_unit);
+
+	  if (pcity->is_building_unit) {
+	    punit_type = get_unit_type(pcity->currently_building);
+	    name = punit_type->name;
+	  } else {
+	    pimpr_type = get_improvement_type(pcity->currently_building);
+	    name = pimpr_type->name;
+	    show_turns = (pcity->currently_building != B_CAPITAL);
+	  }
+	  if (show_turns) {
+	    my_snprintf(buffer, sizeof(buffer), "%s %d", name, turns);
+	  } else {
+	    sz_strlcpy(buffer, name);
+	  }
+
+	  draw_shadowed_string(prod_font_struct, prod_font_gc,
+			       x*NORMAL_TILE_WIDTH+NORMAL_TILE_WIDTH/2,
+			       (y+1)*NORMAL_TILE_HEIGHT +
+			         (draw_city_names ?
+				   main_font_struct->ascent +
+				     main_font_struct->descent :
+				   0),
+			       buffer);
+	}
 
-	XSetForeground(display, font_gc, colors_standard[COLOR_STD_WHITE]);
-	XDrawString(display, wndw, font_gc, xs, ys, pcity->name, len);
       }
     }
   }
 }
 
-
-#ifdef UNUSED
-/**************************************************************************
-...
-**************************************************************************/
-void put_city_pixmap(struct city *pcity, Pixmap pm, int xtile, int ytile)
-{
-  struct Sprite *mysprite;
-
-  if(use_solid_color_behind_units) {
-    XSetForeground(display, fill_bg_gc,
-		   colors_standard[player_color(city_owner(pcity))]);
-    XFillRectangle(display, pm, fill_bg_gc, 
-		   xtile*NORMAL_TILE_WIDTH, ytile*NORMAL_TILE_HEIGHT, 
-		   NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT);
-  }
-  else if(!flags_are_transparent) {	/* observe transparency here, too! */
-    mysprite=get_city_nation_flag_sprite(pcity);
-    XCopyArea(display, mysprite->pixmap, pm, civ_gc, 0, 0,
-	      mysprite->width, mysprite->height, 
-	      xtile*NORMAL_TILE_WIDTH, ytile*NORMAL_TILE_HEIGHT);
-  }
-  else {
-    pixmap_put_overlay_tile(pm,xtile,ytile, get_city_nation_flag_sprite(pcity));
-  }
-
-  pixmap_put_overlay_tile(pm, xtile, ytile,
-			  (city_got_citywalls(pcity)
-			   ? sprites.tx.city_walls
-			   : sprites.tx.city));
-
-  if(genlist_size(&((map_get_tile(pcity->x, pcity->y))->units.list)) > 0)
-    pixmap_put_overlay_tile(pm, xtile, ytile, sprites.city.occupied);
-
-  if(pcity->size>=10) {
-    pixmap_put_overlay_tile(pm, xtile, ytile,
-			    sprites.city.size_tens[pcity->size/10]);
-  }
-  pixmap_put_overlay_tile(pm, xtile, ytile, sprites.city.size[pcity->size%10]);
-
-  if(map_get_special(pcity->x, pcity->y) & S_POLLUTION)
-    pixmap_put_overlay_tile(pm, xtile, ytile, sprites.tx.pollution);
-
-  if(city_unhappy(pcity))
-    pixmap_put_overlay_tile(pm, xtile, ytile, sprites.city.disorder);
-  
-}
-#endif
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -1040,30 +974,33 @@ void put_unit_pixmap_city_overlays(struct unit *punit, Pixmap pm)
     pixmap_put_overlay_tile(pm, 0, 1, sprites.upkeep.unhappy[unhappy-1]);
 }
 
-
 /**************************************************************************
 ...
 **************************************************************************/
 void put_nuke_mushroom_pixmaps(int abs_x0, int abs_y0)
 {
   int x, y;
-  
-  for(y=0; y<3; y++)
+
+  for(y=0; y<3; y++) {
     for(x=0; x<3; x++) {
+      int map_x = map_canvas_adjust_x(x-1+abs_x0)*NORMAL_TILE_WIDTH;
+      int map_y = map_canvas_adjust_y(y-1+abs_y0)*NORMAL_TILE_HEIGHT;
       struct Sprite *mysprite = sprites.explode.nuke[y][x];
-      XCopyArea(display, mysprite->pixmap, XtWindow(map_canvas), 
-		civ_gc, 
-		0, 0, 
-		NORMAL_TILE_WIDTH,
-		NORMAL_TILE_HEIGHT,
-		map_canvas_adjust_x(x-1+abs_x0)*NORMAL_TILE_WIDTH, 
-		map_canvas_adjust_y(y-1+abs_y0)*NORMAL_TILE_HEIGHT);
+
+      XCopyArea(display, map_canvas_store, single_tile_pixmap, civ_gc,
+		map_x, map_y, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		0, 0);
+      pixmap_put_overlay_tile(single_tile_pixmap, 0, 0, mysprite);
+      XCopyArea(display, single_tile_pixmap, XtWindow(map_canvas), civ_gc,
+		0, 0, NORMAL_TILE_WIDTH, NORMAL_TILE_HEIGHT,
+		map_x, map_y);
     }
+  }
 
   XSync(display, 0);
   sleep(1);
 
-  update_map_canvas(map_canvas_adjust_x(abs_x0-1), 
+  update_map_canvas(map_canvas_adjust_x(abs_x0-1),
                     map_canvas_adjust_y(abs_y0-1),
 		    3, 3, 1);
 }
@@ -1125,7 +1062,7 @@ void pixmap_put_tile(Pixmap pm, int x, int y, int abs_x0, int abs_y0,
               sprites[0]->width, sprites[0]->height, x1,y1);
     } else
     {
-      /* normally when use_solid_color_behind_units */
+      /* normally when solid_color_behind_units */
       struct city *pcity;
       struct player *pplayer=NULL;
 
@@ -1135,7 +1072,7 @@ void pixmap_put_tile(Pixmap pm, int x, int y, int abs_x0, int abs_y0,
         struct tile *ptile;
         struct unit *punit;
         ptile=map_get_tile(abs_x0, abs_y0);
-        if((punit=player_find_visible_unit(game.player_ptr, ptile)))
+        if ((punit=find_visible_unit(ptile)))
           pplayer = &game.players[punit->owner];
 
       } else
@@ -1287,10 +1224,12 @@ void scrollbar_jump_callback(Widget w, XtPointer client_data,
   if(w==map_horizontal_scrollbar)
     map_view_x0=percent*map.xsize;
   else {
-    map_view_y0=percent*map.ysize;
+    map_view_y0=percent*(map.ysize+EXTRA_BOTTOM_ROW);
     map_view_y0=(map_view_y0<0) ? 0 : map_view_y0;
-    map_view_y0=(map_view_y0>map.ysize-map_canvas_store_theight) ? 
-       map.ysize-map_canvas_store_theight : map_view_y0;
+    map_view_y0=
+      (map_view_y0>map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight) ? 
+	map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight :
+	map_view_y0;
   }
 
   update_map_canvas(0, 0, map_canvas_store_twidth, map_canvas_store_theight, 1);
@@ -1319,7 +1258,7 @@ void scrollbar_scroll_callback(Widget w, XtPointer client_data,
       map_view_x0--;
   }
   else {
-    if(position>0 && map_view_y0<map.ysize-map_canvas_store_theight)
+    if(position>0 && map_view_y0<map.ysize+EXTRA_BOTTOM_ROW-map_canvas_store_theight)
       map_view_y0++;
     else if(position<0 && map_view_y0>0)
       map_view_y0--;
