@@ -132,6 +132,9 @@ static struct MenuEntry kingdom_menu_entries[]={
 
 static struct MenuEntry view_menu_entries[]={
     { { N_("Map Grid"), 0             }, "ctl-g", MENU_VIEW_SHOW_MAP_GRID, 0 },
+    { { N_("City Names"), 0           }, "ctl-n", MENU_VIEW_SHOW_CITY_NAMES, 0 },
+    { { N_("City Productions"), 0     }, "ctl-p", MENU_VIEW_SHOW_CITY_PRODUCTIONS, 0 },
+    { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Center View"), 0          },     "c", MENU_VIEW_CENTER_VIEW, 0 },
     { { 0,                            },       0, MENU_END_OF_LIST, 0 }
 };
@@ -152,6 +155,7 @@ static struct MenuEntry order_menu_entries[]={
     { { N_("Build Airbase"), 0        },     "e", MENU_ORDER_AIRBASE, 0 },
     { { N_("Clean Pollution"),
 	N_("Paradrop"), 0             },     "p", MENU_ORDER_POLLUTION, 0 },
+    { { N_("Clean Nuclear Fallout"), 0},     "n", MENU_ORDER_FALLOUT, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Fortify"), 0              },     "f", MENU_ORDER_FORTIFY, 0 },
     { { N_("Sentry"), 0               },     "s", MENU_ORDER_SENTRY, 0 },
@@ -192,22 +196,22 @@ static struct MenuEntry reports_menu_entries[]={
 };
 
 static struct MenuEntry help_menu_entries[]={
-    { { N_("Help Languages"), 0       },      "", MENU_HELP_LANGUAGES, 0 },
-    { { N_("Help Connecting"), 0      },      "", MENU_HELP_CONNECTING, 0 },
-    { { N_("Help Controls"), 0        },      "", MENU_HELP_CONTROLS, 0 },
-    { { N_("Help Chatline"), 0        },      "", MENU_HELP_CHATLINE, 0 },
-    { { N_("Help Playing"), 0         },      "", MENU_HELP_PLAYING, 0 },
+    { { N_("Languages"), 0            },      "", MENU_HELP_LANGUAGES, 0 },
+    { { N_("Connecting"), 0           },      "", MENU_HELP_CONNECTING, 0 },
+    { { N_("Controls"), 0             },      "", MENU_HELP_CONTROLS, 0 },
+    { { N_("Chatline"), 0             },      "", MENU_HELP_CHATLINE, 0 },
+    { { N_("Playing"), 0              },      "", MENU_HELP_PLAYING, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
-    { { N_("Help Improvements"), 0    },      "", MENU_HELP_IMPROVEMENTS, 0 },
-    { { N_("Help Units"), 0           },      "", MENU_HELP_UNITS, 0 },
-    { { N_("Help Combat"), 0          },      "", MENU_HELP_COMBAT, 0 },
-    { { N_("Help ZOC"), 0             },      "", MENU_HELP_ZOC, 0 },
-    { { N_("Help Technology"), 0      },      "", MENU_HELP_TECH, 0 },
-    { { N_("Help Terrain"), 0         },      "", MENU_HELP_TERRAIN, 0 },
-    { { N_("Help Wonders"), 0         },      "", MENU_HELP_WONDERS, 0 },
-    { { N_("Help Government"), 0      },      "", MENU_HELP_GOVERNMENT, 0 },
-    { { N_("Help Happiness"), 0       },      "", MENU_HELP_HAPPINESS, 0 },
-    { { N_("Help Space Race"), 0      },      "", MENU_HELP_SPACE_RACE, 0 },
+    { { N_("Improvements"), 0         },      "", MENU_HELP_IMPROVEMENTS, 0 },
+    { { N_("Units"), 0                },      "", MENU_HELP_UNITS, 0 },
+    { { N_("Combat"), 0               },      "", MENU_HELP_COMBAT, 0 },
+    { { N_("ZOC"), 0                  },      "", MENU_HELP_ZOC, 0 },
+    { { N_("Technology"), 0           },      "", MENU_HELP_TECH, 0 },
+    { { N_("Terrain"), 0              },      "", MENU_HELP_TERRAIN, 0 },
+    { { N_("Wonders"), 0              },      "", MENU_HELP_WONDERS, 0 },
+    { { N_("Government"), 0           },      "", MENU_HELP_GOVERNMENT, 0 },
+    { { N_("Happiness"), 0            },      "", MENU_HELP_HAPPINESS, 0 },
+    { { N_("Space Race"), 0           },      "", MENU_HELP_SPACE_RACE, 0 },
     { { 0                             },      "", MENU_SEPARATOR_LINE, 0 },
     { { N_("Copying"), 0              },      "", MENU_HELP_COPYING, 0 },
     { { N_("About"), 0                },      "", MENU_HELP_ABOUT, 0 },
@@ -301,9 +305,11 @@ void update_menus(void)
 			   can_unit_do_activity(punit, ACTIVITY_AIRBASE));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_POLLUTION, 
 			   can_unit_do_activity(punit, ACTIVITY_POLLUTION) ||
-			   can_unit_paradropped(punit));
+			   can_unit_paradrop(punit));
+      menu_entry_sensitive(MENU_ORDER, MENU_ORDER_FALLOUT, 
+			   can_unit_do_activity(punit, ACTIVITY_FALLOUT));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_FORTIFY, 
-			   can_unit_do_activity(punit, ACTIVITY_FORTIFY));
+			   can_unit_do_activity(punit, ACTIVITY_FORTIFYING));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_SENTRY, 
 			   can_unit_do_activity(punit, ACTIVITY_SENTRY));
       menu_entry_sensitive(MENU_ORDER, MENU_ORDER_PILLAGE, 
@@ -474,24 +480,24 @@ static void view_menu_callback(Widget w, XtPointer client_data,
   case MENU_VIEW_SHOW_MAP_GRID:
     key_map_grid_toggle();
     break;
+  case MENU_VIEW_SHOW_CITY_NAMES:
+    key_city_names_toggle();
+    break;
+  case MENU_VIEW_SHOW_CITY_PRODUCTIONS:
+    key_city_productions_toggle();
+    break;
   case MENU_VIEW_CENTER_VIEW:
     request_center_focus_unit();
     break;
   }
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
 static void orders_menu_callback(Widget w, XtPointer client_data,
 				 XtPointer garbage)
 {
-
-
-
-
-
   struct unit *punit;
   size_t pane_num = (size_t)client_data;
 
@@ -525,6 +531,9 @@ static void orders_menu_callback(Widget w, XtPointer client_data,
 	key_unit_paradrop();
     }
     break;
+  case MENU_ORDER_FALLOUT:
+    key_unit_fallout();
+    break;
   case MENU_ORDER_FORTIFY:
     key_unit_fortify();
     break;
@@ -583,7 +592,6 @@ static void orders_menu_callback(Widget w, XtPointer client_data,
   }
 }
 
-
 /****************************************************************
 ...
 *****************************************************************/
