@@ -202,10 +202,10 @@ void create_science_dialog(int make_modal)
   Widget science_form;
   Widget  close_command;
   static char *tech_list_names_ptrs[A_LAST+1];
-  static char tech_list_names[A_LAST+1][200];
   int j, flag, num_list;
   size_t i;
   Dimension width;
+  char title_text[512], rate_text[128];
   char current_text[512];
   char goal_text[512];
   char *report_title;
@@ -231,11 +231,11 @@ void create_science_dialog(int make_modal)
   
   for(i=A_FIRST, j=0; i<game.num_tech_types; i++)
     if(get_invention(game.player_ptr, i)==TECH_KNOWN) {
-      sz_strlcpy(tech_list_names[j], advances[i].name);
-      tech_list_names_ptrs[j]=tech_list_names[j];
+      tech_list_names_ptrs[j]=advances[i].name;
       j++;
     }
   tech_list_names_ptrs[j]=0;
+  qsort(tech_list_names_ptrs, j, sizeof(char *), string_ptr_compare);
   num_list = j;
   /* printf("science list num: %d\n", num_list); */
   
@@ -251,6 +251,12 @@ void create_science_dialog(int make_modal)
 					 NULL);   
 
   report_title=get_report_title(_("Science Advisor"));
+  sz_strlcpy(title_text, report_title);
+  free(report_title);
+  my_snprintf(rate_text, sizeof(rate_text), _("\n(%d turns/advance)"),
+	      tech_turns_to_advance(game.player_ptr));
+  sz_strlcat(title_text, rate_text);
+  report_title=create_centered_string(title_text);
   science_label = XtVaCreateManagedWidget("sciencelabel", 
 					  labelWidgetClass, 
 					  science_form,
@@ -479,12 +485,18 @@ void science_dialog_update(void)
   if(science_dialog_shell) {
     char text[512];
     static char *tech_list_names_ptrs[A_LAST+1];
-    static char tech_list_names[A_LAST+1][200];
     int j, flag;
     size_t i;
+    char title_text[512], rate_text[128];
     char *report_title;
     
     report_title=get_report_title(_("Science Advisor"));
+    sz_strlcpy(title_text, report_title);
+    free(report_title);
+    my_snprintf(rate_text, sizeof(rate_text), _("\n(%d turns/advance)"),
+		tech_turns_to_advance(game.player_ptr));
+    sz_strlcat(title_text, rate_text);
+    report_title=create_centered_string(title_text);
     xaw_set_label(science_label, report_title);
     free(report_title);
 
@@ -511,11 +523,11 @@ void science_dialog_update(void)
 
     for(i=A_FIRST, j=0; i<game.num_tech_types; i++)
       if(get_invention(game.player_ptr, i)==TECH_KNOWN) {
-	sz_strlcpy(tech_list_names[j], advances[i].name);
-	tech_list_names_ptrs[j]=tech_list_names[j];
+	tech_list_names_ptrs[j]=advances[i].name;
 	j++;
       }
     tech_list_names_ptrs[j]=0;
+    qsort(tech_list_names_ptrs, j, sizeof(char *), string_ptr_compare);
 
     XawListChange(science_list, tech_list_names_ptrs, 0/*j*/, 0, 1);
 
