@@ -88,8 +88,8 @@ struct worklist_dialog {
   char *worklist_names_ptrs[MAX_LEN_WORKLIST+1];
   char worklist_names[MAX_LEN_WORKLIST][200];
   int worklist_ids[MAX_LEN_WORKLIST];
-  char *worklist_avail_names_ptrs[B_LAST+1+U_LAST+1+1];
-  char worklist_avail_names[B_LAST+1+U_LAST+1][200];
+  char *worklist_avail_names_ptrs[B_LAST+1+U_LAST+1+MAX_NUM_WORKLISTS+1+1];
+  char worklist_avail_names[B_LAST+1+U_LAST+1+MAX_NUM_WORKLISTS+1][200];
   int worklist_avail_ids[B_LAST+1+U_LAST+1+MAX_NUM_WORKLISTS+1];
   int worklist_avail_num_improvements;
   int worklist_avail_num_targets;
@@ -530,7 +530,8 @@ Widget popup_worklist(struct worklist *pwl, struct city *pcity,
 
   /* Fill in the list of available build targets,
      and set correct label in show-advanced toggle. */
-  worklist_show_advanced_callback(pdialog->toggle_show_advanced, pdialog, NULL);
+  worklist_show_advanced_callback(pdialog->toggle_show_advanced,
+				  (XtPointer)pdialog, NULL);
 
   /* force refresh of viewport so the scrollbar is added.
    * Buggy sun athena requires this */
@@ -566,7 +567,7 @@ void worklist_id_to_name(char buf[], int id, int is_unit,
 	    get_unit_name(id), get_unit_type(id)->build_cost);
   else if (pcity)
     sprintf(buf, "%s (%d)",
-	    get_imp_name_ex(pcity, id), get_improvement_type(id)->build_cost);
+	    get_impr_name_ex(pcity, id), get_improvement_type(id)->build_cost);
   else
     sprintf(buf, "%s (%d)",
 	    get_improvement_name(id), get_improvement_type(id)->build_cost);
