@@ -212,6 +212,7 @@ void boot_help_texts(void)
     freelog(LOG_NORMAL, "Did not read help texts");
     return;
   }
+  /* after following call filename may be clobbered; use sf->filename instead */
   if (!section_file_load(sf, filename)) {
     /* this is now unlikely to happen */
     freelog(LOG_NORMAL, "failed reading help-texts");
@@ -350,12 +351,14 @@ void boot_help_texts(void)
 	strcat(long_buffer, "\n\n");
       }
     }
+    free(paras);
     wordwrap_string(long_buffer, 68);
     pitem->text=mystrdup(long_buffer);
     genlist_insert(&help_nodes, pitem, -1);
   }
 
-  section_file_check_unused(sf, filename);
+  free(sec);
+  section_file_check_unused(sf, sf->filename);
   section_file_free(sf);
   booted = 1;
   freelog(LOG_VERBOSE, "Booted help texts ok");
@@ -527,7 +530,7 @@ static void helptext_cathedral_techs(char *buf)
 *****************************************************************/
 void helptext_improvement(char *buf, int which, const char *user_text)
 {
-  struct improvement_type *imp = &improvement_types[which];
+  struct impr_type *imp = &improvement_types[which];
   
   assert(buf&&user_text);
   buf[0] = '\0';
@@ -591,7 +594,7 @@ void helptext_improvement(char *buf, int which, const char *user_text)
 void helptext_wonder(char *buf, int which,
 			    const char *user_text)
 {
-  struct improvement_type *imp = &improvement_types[which];
+  struct impr_type *imp = &improvement_types[which];
   
   assert(buf&&user_text);
   buf[0] = '\0';
@@ -635,12 +638,12 @@ void helptext_unit(char *buf, int i, const char *user_text)
   
   buf[0] = '\0';
   if (utype->transport_capacity>0) {
-    if (unit_flag(i, F_SUBMARINE)) {
-      sprintf(buf+strlen(buf), _("* Can carry and refuel %d missile units.\n"),
-	      utype->transport_capacity);
-    } else if (unit_flag(i, F_CARRIER)) {
+    if (unit_flag(i, F_CARRIER)) {
       sprintf(buf+strlen(buf), _("* Can carry and refuel %d air units.\n"),
 	      utype->transport_capacity);
+    } else if (unit_flag(i, F_MISSILE_CARRIER)) {
+      sprintf(buf+strlen(buf), _("* Can carry and refuel %d missile units.\n"),
+	      utype->transport_capacity);
     } else {
       sprintf(buf+strlen(buf), _("* Can carry %d ground units across water.\n"),
 	      utype->transport_capacity);
@@ -666,6 +669,14 @@ void helptext_unit(char *buf, int i, const char *user_text)
   if (unit_flag(i, F_FIGHTER)) {
     sprintf(buf+strlen(buf), _("* Can attack enemy air units.\n"));
   }
+  if (unit_flag(i, F_PARTIAL_INVIS)) {
+    sprintf(buf+strlen(buf), _("* Is invisible except when next to an"
+			       " enemy unit or city.\n"));
+  }
+  if (unit_flag(i, F_NO_LAND_ATTACK)) {
+    sprintf(buf+strlen(buf), _("* Can only attack units on ocean squares"
+			       " (no land attacks).\n"));
+  }
   if (unit_flag(i, F_MARINES)) {
     sprintf(buf+strlen(buf), _("* Can attack from aboard sea units: against"
 			       " enemy cities and onto land squares."));
@@ -703,6 +714,10 @@ void helptext_unit(char *buf, int i, const char *user_text)
     sprintf(buf+strlen(buf),
 	    _("* Ignores terrain effects (treats all squares as roads).\n"));
   }
+  if (unit_flag(i, F_IGTIRED)) {
+    sprintf(buf+strlen(buf),
+	    _("* Attacks with full strength even if less than one movement left.\n"));
+  }
   if (unit_flag(i, F_IGZOC)) {
     sprintf(buf+strlen(buf), _("* Ignores zones of control.\n"));
   }
@@ -728,10 +743,11 @@ void helptext_unit(char *buf, int i, const char *user_text)
     } else if (utype->fuel>=4) {
       sprintf(buf+strlen(buf), _("%dth "), utype->fuel);
     }
+    /* FIXME: should use something like get_units_with_flag_string() */
     sprintf(buf+strlen(buf), _("turn in a city, or on a Carrier"));
     if (unit_flag(i, F_MISSILE) &&
-	num_role_units(F_SUBMARINE)>0 &&
-	get_unit_type(get_role_unit(F_SUBMARINE,0))->transport_capacity) {
+	num_role_units(F_MISSILE_CARRIER)>0 &&
+	get_unit_type(get_role_unit(F_MISSILE_CARRIER,0))->transport_capacity) {
       sprintf(buf+strlen(buf), _(" or Submarine"));
     }
     sprintf(buf+strlen(buf), _(", or will run out of fuel and be lost.\n"));
