@@ -25,6 +25,7 @@
 #include "fcintl.h"
 #include "game.h"
 #include "government.h"
+#include "idex.h"
 #include "log.h"
 #include "map.h"
 #include "mem.h"
@@ -64,6 +65,44 @@ extern int last_turn_gold_amount;
 extern int did_advance_tech_this_turn;
 extern char name[512];
 
+
+/**************************************************************************
+...
+**************************************************************************/
+static void unpackage_unit(struct unit *punit, struct packet_unit_info *packet)
+{
+  punit->id = packet->id;
+  punit->owner = packet->owner;
+  punit->x = packet->x;
+  punit->y = packet->y;
+  punit->homecity = packet->homecity;
+  punit->veteran = packet->veteran;
+  punit->type = packet->type;
+  punit->moves_left = packet->movesleft;
+  punit->hp = packet->hp;
+  punit->activity = packet->activity;
+  punit->activity_count = packet->activity_count;
+  punit->unhappiness = packet->unhappiness;
+  punit->upkeep = packet->upkeep;
+  punit->upkeep_food = packet->upkeep_food;
+  punit->upkeep_gold = packet->upkeep_gold;
+  punit->ai.control = packet->ai;
+  punit->fuel = packet->fuel;
+  punit->goto_dest_x = packet->goto_dest_x;
+  punit->goto_dest_y = packet->goto_dest_y;
+  punit->activity_target = packet->activity_target;
+  punit->paradropped = packet->paradropped;
+  punit->connecting = packet->connecting;
+  /* not in packet, only in unit struct */
+  punit->focus_status = FOCUS_AVAIL;
+  punit->bribe_cost = 0;	/* done by handle_incite_cost() */
+  punit->foul = 0;		/* never used in client/ */
+  punit->ord_map = 0;		/* never used in client/ */
+  punit->ord_city = 0;		/* never used in client/ */
+  punit->moved = 0;		/* never used in client/ */
+  punit->transported_by = 0;	/* never used in client/ */
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -97,7 +136,6 @@ void handle_join_game_reply(struct packet_join_game_reply *packet)
 void handle_remove_city(struct packet_generic_integer *packet)
 {
   struct city *pcity=find_city_by_id(packet->value);
-  
   if(pcity!=NULL) client_remove_city(pcity);
 }
 
@@ -119,21 +157,30 @@ void handle_nuke_tile(struct packet_nuke_tile *packet)
 }
 
 /**************************************************************************
-this piece of code below is about as bad as it can get
+...
 **************************************************************************/
 void handle_unit_combat(struct packet_unit_combat *packet)
 {
-  struct unit *punit0=game_find_unit_by_id(packet->attacker_unit_id);
-  struct unit *punit1=game_find_unit_by_id(packet->defender_unit_id);
+  int show_combat = FALSE;
+  struct unit *punit0 = find_unit_by_id(packet->attacker_unit_id);
+  struct unit *punit1 = find_unit_by_id(packet->defender_unit_id);
 
-  if(punit0 && tile_visible_mapcanvas(punit0->x, punit0->y) &&
-     punit1 && tile_visible_mapcanvas(punit1->x, punit1->y)) {
-    struct unit *pfocus;
-    pfocus=get_unit_in_focus();
-  
-    decrease_unit_hp_smooth(punit0, packet->attacker_hp,
-			    punit1, packet->defender_hp);
-    set_unit_focus(pfocus);
+  if (punit0 && punit1) {
+    if (tile_visible_mapcanvas(punit0->x, punit0->y) &&
+	tile_visible_mapcanvas(punit1->x, punit1->y)) {
+      show_combat = TRUE;
+    } else if (auto_center_on_combat) {
+      if (punit0->owner == game.player_idx)
+	center_tile_mapcanvas(punit0->x, punit0->y);
+      else
+	center_tile_mapcanvas(punit1->x, punit1->y);
+      show_combat = TRUE;
+    }
+
+    if (show_combat) {
+      decrease_unit_hp_smooth(punit0, packet->attacker_hp,
+			      punit1, packet->defender_hp);
+    }
   }
 }
 
@@ -176,16 +223,33 @@ void handle_city_info(struct packet_city_info *packet)
   int i, x, y, city_is_new;
   struct city *pcity;
 
-  pcity=city_list_find_id(&game.players[packet->owner].cities, packet->id);
+  pcity=find_city_by_id(packet->id);
+
+  if (pcity && (pcity->owner != packet->owner)) {
+    /* With current server, this shouldn't happen: when city changes
+     * owner it is re-created with a new id.  Unclear what to do here;
+     * try to cope...
+     */
+    freelog(LOG_NORMAL,
+	    "Got existing city id (%d %s) with wrong owner (%d %s, old %d %s)",
+	    packet->id, pcity->name, packet->owner,
+	    get_player(packet->owner)->name, pcity->owner,
+	    get_player(pcity->owner)->name);
+    client_remove_city(pcity);
+    pcity = 0;
+  }
 
   if(!pcity) {
     city_is_new=1;
     pcity=fc_malloc(sizeof(struct city));
+    pcity->id=packet->id;
+    idex_register_city(pcity);
   }
-  else
+  else {
     city_is_new=0;
+    assert(pcity->id == packet->id);
+  }
   
-  pcity->id=packet->id;
   pcity->owner=packet->owner;
   pcity->x=packet->x;
   pcity->y=packet->y;
@@ -230,7 +294,13 @@ void handle_city_info(struct packet_city_info *packet)
   pcity->did_sell=packet->did_sell;
   pcity->was_happy=packet->was_happy;
   pcity->airlift=packet->airlift;
-  
+
+  pcity->turn_last_built=packet->turn_last_built;
+  pcity->turn_changed_target=packet->turn_changed_target;
+  pcity->changed_from_id=packet->changed_from_id;
+  pcity->changed_from_is_unit=packet->changed_from_is_unit;
+  pcity->before_change_shields=packet->before_change_shields;
+
   i=0;
   for(y=0; y<CITY_MAP_SIZE; y++) {
     for(x=0; x<CITY_MAP_SIZE; x++) {
@@ -247,6 +317,8 @@ void handle_city_info(struct packet_city_info *packet)
   
   if(city_is_new) {
     unit_list_init(&pcity->units_supported);
+    unit_list_init(&pcity->info_units_supported);
+    unit_list_init(&pcity->info_units_present);
     city_list_insert(&game.players[packet->owner].cities, pcity);
     map_set_city(pcity->x, pcity->y, pcity);
     if(pcity->owner==game.player_idx)
@@ -316,9 +388,9 @@ void handle_new_year(struct packet_new_year *ppacket)
   update_info_label();
 
   player_set_unit_focus_status(game.player_ptr);
-  
   update_unit_focus();
-    
+  auto_center_on_focus_unit();
+
   update_unit_info_label(get_unit_in_focus());
 
   seconds_to_turndone=game.timeout;
@@ -332,9 +404,19 @@ void handle_new_year(struct packet_new_year *ppacket)
   meswin_update_delay_off();
   update_meswin_dialog();
 
-  if(game.player_ptr->ai.control && !ai_manual_turn_done) user_ended_turn();
+  update_city_descriptions();
+
+  if(game.player_ptr->ai.control && !ai_manual_turn_done)
+    user_ended_turn();
+
+  if(sound_bell_at_new_turn &&
+     (!game.player_ptr->ai.control || ai_manual_turn_done))
+    sound_bell();
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 void handle_before_new_year(void)
 {
   clear_notify_window();
@@ -358,7 +440,8 @@ void handle_chat_msg(struct packet_generic_message *packet)
     append_output_window(packet->message);
   if (where & MW_MESSAGES)
     add_notify_window(packet);
-  if (where & MW_POPUP)
+  if ((where & MW_POPUP) &&
+      (!game.player_ptr->ai.control || ai_popup_windows))
     popup_notify_goto_dialog("Popup Request", packet->message, 
 			     packet->x, packet->y);
 }
@@ -409,12 +492,51 @@ void handle_unit_info(struct packet_unit_info *packet)
   struct unit *punit;
   int repaint_unit;
   int repaint_city;		/* regards unit's homecity */
-  
-  repaint_unit=0;
-  repaint_city=0;
-  punit=unit_list_find(&game.players[packet->owner].units, packet->id);
-  
+
+  /* Special case for a diplomat/spy investigating a city:
+     The investigator needs to know the supported and present
+     units of a city, whether or not they are fogged. So, we
+     send a list of them all before sending the city info. */
+  if ((packet->packet_use == UNIT_INFO_CITY_SUPPORTED) ||
+      (packet->packet_use == UNIT_INFO_CITY_PRESENT)) {
+    static int last_serial_num = 0;
+    /* fetch city -- abort if not found */
+    pcity = find_city_by_id(packet->info_city_id);
+    if (!pcity) {
+      return;
+    }
+    /* new serial number -- clear everything */
+    if (last_serial_num != packet->serial_num) {
+      last_serial_num = packet->serial_num;
+      unit_list_iterate(pcity->info_units_supported, psunit) {
+	free(psunit);
+      } unit_list_iterate_end;
+      unit_list_unlink_all(&(pcity->info_units_supported));
+      unit_list_iterate(pcity->info_units_present, ppunit) {
+	free(ppunit);
+      } unit_list_iterate_end;
+      unit_list_unlink_all(&(pcity->info_units_present));
+    }
+    /* okay, append a unit struct to the proper list */
+    if (packet->packet_use == UNIT_INFO_CITY_SUPPORTED) {
+      punit = fc_malloc(sizeof(struct unit));
+      unpackage_unit(punit, packet);
+      unit_list_insert(&(pcity->info_units_supported), punit);
+    } else if (packet->packet_use == UNIT_INFO_CITY_PRESENT) {
+      punit = fc_malloc(sizeof(struct unit));
+      unpackage_unit(punit, packet);
+      unit_list_insert(&(pcity->info_units_present), punit);
+    }
+    /* done with special case */
+    return;
+  }
+
+  repaint_unit = 0;
+  repaint_city = 0;
+  punit = player_find_unit_by_id(&game.players[packet->owner], packet->id);
+
   if(punit) {
+    int dest_x,dest_y;
     if((punit->activity!=packet->activity)         /* change activity */
        || (punit->activity_target!=packet->activity_target)) { /*   or act's target */
       repaint_unit=1;
@@ -430,8 +552,9 @@ void handle_unit_info(struct packet_unit_info *packet)
       if(punit->owner==game.player_idx) 
         refresh_unit_city_dialogs(punit);
       /*      refresh_tile_mapcanvas(punit->x, punit->y, 1);
-      update_unit_pix_label(punit);
-      update_unit_focus(); */
+       *      update_unit_pix_label(punit);
+       *      update_unit_focus();
+       */
 
       /* These two lines force the menus to be updated as appropriate when
 	 the unit activity changes. */
@@ -476,18 +599,14 @@ void handle_unit_info(struct packet_unit_info *packet)
       struct city *pcity;
       pcity=map_get_city(punit->x, punit->y);
       
-      if(tile_is_known(packet->x, packet->y)) {
+      if(tile_is_known(packet->x, packet->y) == TILE_KNOWN) {
 	do_move_unit(punit, packet);
 	update_unit_focus();
       }
       else {
-	unit_list_unlink(&game.players[packet->owner].units, punit);
-	unit_list_unlink(&map_get_tile(punit->x, punit->y)->units, punit);
-	if(punit->homecity && (pcity=find_city_by_id(punit->homecity))) {
-	  unit_list_unlink(&pcity->units_supported, punit);
-	}
-	refresh_tile_mapcanvas(punit->x, punit->y, 1);
-	free(punit);
+	do_move_unit(punit, packet); /* nice to see where a unit is going */
+	client_remove_unit(punit->id);
+	refresh_tile_mapcanvas(packet->x, packet->y, 1);
         return;
       }
       if(pcity)  {
@@ -547,41 +666,26 @@ void handle_unit_info(struct packet_unit_info *packet)
     punit->goto_dest_y=packet->goto_dest_y;
     punit->paradropped=packet->paradropped;
     punit->connecting=packet->connecting;
-  }
   
+    dest_x = packet->x;
+    dest_y = packet->y;
+    /*fog of war*/
+    if (!(tile_is_known(punit->x,punit->y) == TILE_KNOWN)) {
+      client_remove_unit(packet->id);
+      refresh_tile_mapcanvas(dest_x, dest_y, 1);
+    }
+  }
+
   else {      /* create new unit */
     punit=fc_malloc(sizeof(struct unit));
-    
-    punit->id=packet->id;
-    punit->owner=packet->owner;
-    punit->x=packet->x;
-    punit->y=packet->y;
-    punit->veteran=packet->veteran;
-    punit->homecity=packet->homecity;
-    punit->type=packet->type;
-    punit->moves_left=packet->movesleft;
-    punit->hp=packet->hp;
-    punit->unhappiness=packet->unhappiness;
-    punit->activity=packet->activity;
-    punit->upkeep=packet->upkeep;
-    punit->upkeep_food=packet->upkeep_food;
-    punit->upkeep_gold=packet->upkeep_gold;
-    punit->fuel=packet->fuel;
-    punit->goto_dest_x=packet->goto_dest_x;
-    punit->goto_dest_y=packet->goto_dest_y;
-    punit->activity_target=packet->activity_target;
-    punit->ai.control=packet->ai;
-    punit->paradropped=packet->paradropped;
-    punit->connecting=packet->connecting;
-    
+    unpackage_unit(punit, packet);
+    idex_register_unit(punit);
+
     punit->activity_count=0;	/* never used in client/ or common/  --dwp */
-    punit->bribe_cost=0;	/* done by handle_incite_cost() */
-    
-    punit->focus_status=FOCUS_AVAIL;
-    
+
     unit_list_insert(&game.players[packet->owner].units, punit);
     unit_list_insert(&map_get_tile(punit->x, punit->y)->units, punit);
-    
+
     if((pcity=find_city_by_id(punit->homecity)))
       unit_list_insert(&pcity->units_supported, punit);
 
@@ -589,13 +693,11 @@ void handle_unit_info(struct packet_unit_info *packet)
 	   get_nation_name(get_player(punit->owner)->nation),
 	   unit_name(punit->type), punit->x, punit->y, punit->id,
 	   punit->homecity, (pcity ? pcity->name : _("(unknown)")));
-    
-    /* this is ugly - prevent unit from being drawn if it's moved into
-     * screen by a transporter - only works for ground_units.. yak */
-    if(!is_ground_unit(punit) || map_get_terrain(punit->x, punit->y)!=T_OCEAN)
-      repaint_unit=1;
-    else
+
+    if (packet->carried)
       repaint_unit=0;
+    else
+      repaint_unit=1;
   }
 
   if(punit && punit==get_unit_in_focus())
@@ -608,7 +710,11 @@ void handle_unit_info(struct packet_unit_info *packet)
   if(repaint_unit)
     refresh_tile_mapcanvas(punit->x, punit->y, 1);
 
-  update_unit_focus(); 
+  if(packet->select_it && (punit->owner==game.player_idx)) {
+    set_unit_focus_and_select(punit);
+  } else {
+    update_unit_focus(); 
+  }
 }
 
 /**************************************************************************
@@ -645,6 +751,8 @@ void handle_game_info(struct packet_game_info *pinfo)
   game.nplayers=pinfo->nplayers;
   game.globalwarming=pinfo->globalwarming;
   game.heating=pinfo->heating;
+  game.nuclearwinter=pinfo->nuclearwinter;
+  game.cooling=pinfo->cooling;
   if(get_client_state()!=CLIENT_GAME_RUNNING_STATE) {
     game.player_idx=pinfo->player_idx;
     game.player_ptr=&game.players[game.player_idx];
@@ -661,10 +769,38 @@ void handle_game_info(struct packet_game_info *pinfo)
   game.techpenalty=pinfo->techpenalty;
   game.foodbox=pinfo->foodbox;
   game.civstyle=pinfo->civstyle;
+/* when removing "game_ruleset" capability,
+remove this *entire* piece of code (to REMOVE TO HERE, below) */
+if (!has_capability("game_ruleset", aconnection.capability)) {
+if (game.civstyle == 1) {
+game.rgame.min_city_center_food = 0;
+game.rgame.min_city_center_shield = 0;
+game.rgame.min_city_center_trade = 0;
+game.rgame.min_dist_bw_cities = 1;
+game.rgame.init_vis_radius_sq = 2;
+game.rgame.hut_overflight = OVERFLIGHT_NOTHING;
+game.rgame.pillage_select = 0;
+} else {
+game.rgame.min_city_center_food = 1;
+game.rgame.min_city_center_shield = 1;
+game.rgame.min_city_center_trade = 0;
+game.rgame.min_dist_bw_cities = 2;
+game.rgame.init_vis_radius_sq = 5;
+game.rgame.hut_overflight = OVERFLIGHT_FRIGHTEN;
+game.rgame.pillage_select = 1;
+}
+game.rgame.nuke_contamination = CONTAMINATION_POLLUTION;
+}
+/* REMOVE TO HERE */
 
   boot_help = (get_client_state() == CLIENT_GAME_RUNNING_STATE
 	       && game.spacerace != pinfo->spacerace);
   game.spacerace=pinfo->spacerace;
+  if (game.timeout) {
+    if (pinfo->seconds_to_turndone)
+      seconds_to_turndone = pinfo->seconds_to_turndone;
+  } else
+    seconds_to_turndone = 0;
   if (boot_help) {
     boot_help_texts();		/* reboot, after setting game.spacerace */
   }
@@ -694,6 +830,16 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->embassy=pinfo->embassy;
   pplayer->city_style=pinfo->city_style;
 
+  for (i = 0; i < MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS; i++) {
+    pplayer->diplstates[i].type =
+      pinfo->diplstates[i].type;
+    pplayer->diplstates[i].turns_left =
+      pinfo->diplstates[i].turns_left;
+    pplayer->diplstates[i].has_reason_to_cancel =
+      pinfo->diplstates[i].has_reason_to_cancel;
+  }
+  pplayer->reputation = pinfo->reputation;
+
   for (i = 0; i < MAX_NUM_WORKLISTS; i++)
     copy_worklist(&pplayer->worklists[i], &pinfo->worklists[i]);
 
@@ -958,57 +1104,95 @@ void handle_spaceship_info(struct packet_spaceship_info *p)
 }
 
 /**************************************************************************
-...
+This was once very ugly...
 **************************************************************************/
 void handle_tile_info(struct packet_tile_info *packet)
 {
-  int old_known;
+  struct tile *ptile = map_get_tile(packet->x, packet->y);
+  enum tile_terrain_type old_terrain = ptile->terrain;
+  enum known_type old_known = ptile->known;
+  int tile_changed = 0;
 
-  struct tile *ptile=map_get_tile(packet->x, packet->y);
+  if (ptile->terrain != packet->type) { /*terrain*/
+    tile_changed = 1;
+    ptile->terrain = packet->type;
+  }
+  if (ptile->special != packet->special) { /*add-on*/
+    tile_changed = 1;
+    ptile->special = packet->special;
+  }
+  ptile->known = packet->known;
 
-  old_known=ptile->known;
+  /* fog of war remove units */
+  if (ptile->known <= TILE_KNOWN_FOGGED && old_known == TILE_KNOWN) {
+    unit_list_iterate(ptile->units, punit) {
+      client_remove_unit(punit->id);
+    }
+    unit_list_iterate_end;
+  }
 
-  ptile->terrain=packet->type;
-  ptile->special=packet->special;
-
-  ptile->known=packet->known;
-
-  if(packet->known >= TILE_KNOWN && old_known < TILE_KNOWN
-     && ptile->terrain != T_OCEAN)
+  /* update continents */
+  if ((packet->known >= TILE_KNOWN_FOGGED &&
+       old_known == TILE_UNKNOWN && ptile->terrain != T_OCEAN) ||
+      ((old_terrain == T_OCEAN) && (ptile->terrain != T_OCEAN))) {
+    /* new knowledge or new land -- update can handle incrementally */
     climap_update_continents(packet->x, packet->y);
-  
-  if(get_client_state()==CLIENT_GAME_RUNNING_STATE && packet->known>=TILE_KNOWN) {
-    refresh_tile_mapcanvas(packet->x, packet->y, 1); 
-    if(old_known<TILE_KNOWN) { 
-      int known;
-      
-      known=tile_is_known(packet->x-1, packet->y);
-      if(known>=TILE_KNOWN && known!=ptile->known)
-	refresh_tile_mapcanvas(packet->x-1, packet->y, 1); 
+  } else if (old_known >= TILE_KNOWN_FOGGED &&
+	     ((old_terrain != T_OCEAN) && (ptile->terrain == T_OCEAN))) {
+    /* land changed into ocean -- rebuild continents map from scratch */
+    int x, y;
+    for (y = 0; y < map.ysize; y++)
+      for (x = 0; x < map.xsize; x++)
+	map_set_continent(x, y, 0);
+    climap_init_continents();
+    for (y = 0; y < map.ysize; y++)
+      for (x = 0; x < map.xsize; x++)
+	if ((tile_is_known(x, y) >= TILE_KNOWN_FOGGED) &&
+	    (map_get_terrain(x, y) != T_OCEAN))
+	  climap_update_continents(x, y);
+  }
 
-      known=tile_is_known(packet->x+1, packet->y);
-      if(known>=TILE_KNOWN && known!=ptile->known)
-	refresh_tile_mapcanvas(packet->x+1, packet->y, 1); 
+  /* refresh tiles */
+  if(get_client_state()==CLIENT_GAME_RUNNING_STATE) {
+    int x = packet->x, y = packet->y;
 
-      known=tile_is_known(packet->x, packet->y-1);
-      if(known>=TILE_KNOWN && known!=ptile->known)
-	refresh_tile_mapcanvas(packet->x, packet->y-1, 1); 
+    /* the tile itself */
+    if (tile_changed || old_known!=ptile->known)
+      refresh_tile_mapcanvas(x, y, 1);
 
-      known=tile_is_known(packet->x, packet->y+1);
-      if(known>=TILE_KNOWN && known!=ptile->known)
-	refresh_tile_mapcanvas(packet->x, packet->y+1, 1); 
-    }
-    else { /* happens so seldom(new roads etc) so refresh'em all */
-      refresh_tile_mapcanvas(packet->x-1, packet->y, 1); 
-      refresh_tile_mapcanvas(packet->x+1, packet->y, 1); 
-      refresh_tile_mapcanvas(packet->x, packet->y-1, 1); 
-      refresh_tile_mapcanvas(packet->x, packet->y+1, 1); 
-      /* jjm@codewell.com 30dec1998a */
-      refresh_tile_mapcanvas(packet->x-1, packet->y+1, 1); 
-      refresh_tile_mapcanvas(packet->x+1, packet->y+1, 1); 
-      refresh_tile_mapcanvas(packet->x+1, packet->y-1, 1); 
-      refresh_tile_mapcanvas(packet->x-1, packet->y-1, 1); 
+    /* if the terrain or the specials of the tile
+       have changed it affects the adjacent tiles */
+    if (tile_changed) {
+      if (tile_is_known(x-1, y) >= TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x-1, y, 1);
+      if (tile_is_known(x+1, y) >= TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x+1, y, 1);
+      if (tile_is_known(x, y-1) >= TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x, y-1, 1);
+      if (tile_is_known(x, y+1) >= TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x, y+1, 1);
+      if (tile_is_known(x-1, y+1) >= TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x-1, y+1, 1);
+      if (tile_is_known(x+1, y+1) >= TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x+1, y+1, 1);
+      if (tile_is_known(x+1, y-1) >= TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x+1, y-1, 1);
+      if (tile_is_known(x-1, y-1) >= TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x-1, y-1, 1);
+      return;
     }
+
+    /* the "furry edges" on tiles adjacent to an TILE_UNKNOWN tile are removed here */
+    if (old_known == TILE_UNKNOWN && packet->known >= TILE_KNOWN_FOGGED) {     
+      if(tile_is_known(x-1, y)>=TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x-1, y, 1);
+      if(tile_is_known(x+1, y)>=TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x+1, y, 1);
+      if(tile_is_known(x, y-1)>=TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x, y-1, 1);
+      if(tile_is_known(x, y+1)>=TILE_KNOWN_FOGGED)
+	refresh_tile_mapcanvas(x, y+1, 1);
+    } 
   }
 }
 
@@ -1053,6 +1237,7 @@ void handle_ruleset_control(struct packet_ruleset_control *packet)
   
   for(i=0; i<game.government_count; i++) {
     free(get_government(i)->ruler_titles);
+    free(get_government(i)->helptext);
   }
   free(governments);
       
@@ -1081,6 +1266,7 @@ void handle_ruleset_control(struct packet_ruleset_control *packet)
   governments = fc_calloc(game.government_count, sizeof(struct government));
   for(i=0; i<game.government_count; i++) {
     get_government(i)->ruler_titles = NULL;
+    get_government(i)->helptext = NULL;
   }
 
   free_nations(game.nation_count);
@@ -1168,7 +1354,7 @@ void handle_ruleset_tech(struct packet_ruleset_tech *p)
 **************************************************************************/
 void handle_ruleset_building(struct packet_ruleset_building *p)
 {
-  struct improvement_type *b;
+  struct impr_type *b;
 
   if(p->id < 0 || p->id >= B_LAST) {
     freelog(LOG_NORMAL,
@@ -1179,15 +1365,148 @@ void handle_ruleset_building(struct packet_ruleset_building *p)
   b = &improvement_types[p->id];
 
   sz_strlcpy(b->name, p->name);
-  b->is_wonder        = p->is_wonder;
-  b->tech_requirement = p->tech_requirement;
-  b->build_cost       = p->build_cost;      
-  b->shield_upkeep    = p->shield_upkeep;   
-  b->obsolete_by      = p->obsolete_by;     
-  b->variant          = p->variant;         
- 
+  b->tech_req = p->tech_req;
+  b->bldg_req = p->bldg_req;
+  free(b->terr_gate);
+  b->terr_gate = p->terr_gate;		/* pointer assignment */
+  free(b->spec_gate);
+  b->spec_gate = p->spec_gate;		/* pointer assignment */
+  b->equiv_range = p->equiv_range;
+  free(b->equiv_dupl);
+  b->equiv_dupl = p->equiv_dupl;	/* pointer assignment */
+  free(b->equiv_repl);
+  b->equiv_repl = p->equiv_repl;	/* pointer assignment */
+  b->obsolete_by = p->obsolete_by;
+  b->is_wonder = p->is_wonder;
+  b->build_cost = p->build_cost;
+  b->upkeep = p->upkeep;
+  b->sabotage = p->sabotage;
+  free(b->effect);
+  b->effect = p->effect;		/* pointer assignment */
+  b->variant = p->variant;	/* FIXME: remove when gen-impr obsoletes */
   free(b->helptext);
-  b->helptext = p->helptext;	/* pointer assignment */
+  b->helptext = p->helptext;		/* pointer assignment */
+
+#ifdef DEBUG
+  if(p->id == B_LAST-1) {
+    int id;
+    for (id = 0; id < B_LAST; id++) {
+      int inx;
+      b = &improvement_types[id];
+      freelog(LOG_DEBUG, "Impr: %s...",
+	      b->name);
+      freelog(LOG_DEBUG, "  tech_req    %2d/%s",
+	      b->tech_req,
+	      (b->tech_req == A_LAST) ?
+	      "Never" :
+	      advances[b->tech_req].name);
+      freelog(LOG_DEBUG, "  bldg_req    %2d/%s",
+	      b->bldg_req,
+	      (b->bldg_req == B_LAST) ?
+	      "None" :
+	      improvement_types[b->bldg_req].name);
+      freelog(LOG_DEBUG, "  terr_gate...");
+      for (inx = 0; b->terr_gate[inx] != T_LAST; inx++) {
+	freelog(LOG_DEBUG, "    %2d/%s",
+		b->terr_gate[inx], get_terrain_name(b->terr_gate[inx]));
+      }
+      freelog(LOG_DEBUG, "  spec_gate...");
+      for (inx = 0; b->spec_gate[inx] != S_NO_SPECIAL; inx++) {
+	freelog(LOG_DEBUG, "    %2d/%s",
+		b->spec_gate[inx], get_special_name(b->spec_gate[inx]));
+      }
+      freelog(LOG_DEBUG, "  equiv_range %2d/%s",
+	      b->equiv_range, effect_range_name(b->equiv_range));
+      freelog(LOG_DEBUG, "  equiv_dupl...");
+      for (inx = 0; b->equiv_dupl[inx] != B_LAST; inx++) {
+	freelog(LOG_DEBUG, "    %2d/%s",
+		b->equiv_dupl[inx], improvement_types[b->equiv_dupl[inx]].name);
+      }
+      freelog(LOG_DEBUG, "  equiv_repl...");
+      for (inx = 0; b->equiv_repl[inx] != B_LAST; inx++) {
+	freelog(LOG_DEBUG, "    %2d/%s",
+		b->equiv_repl[inx], improvement_types[b->equiv_repl[inx]].name);
+      }
+      freelog(LOG_DEBUG, "  obsolete_by %2d/%s",
+	      b->obsolete_by, advances[b->obsolete_by].name);
+      freelog(LOG_DEBUG, "  is_wonder   %2d", b->is_wonder);
+      freelog(LOG_DEBUG, "  build_cost %3d", b->build_cost);
+      freelog(LOG_DEBUG, "  upkeep      %2d", b->upkeep);
+      freelog(LOG_DEBUG, "  sabotage   %3d", b->sabotage);
+      freelog(LOG_DEBUG, "  effect...");
+      for (inx = 0; b->effect[inx].type != EFT_LAST; inx++) {
+	char buf[1024], *ptr;
+	ptr = buf;
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " %d/%s",
+		    b->effect[inx].type,
+		    effect_type_name(b->effect[inx].type));
+	ptr = strchr(ptr, '\0');
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " range=%d/%s",
+		    b->effect[inx].range,
+		    effect_range_name(b->effect[inx].range));
+	ptr = strchr(ptr, '\0');
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " amount=%d",
+		    b->effect[inx].amount);
+	ptr = strchr(ptr, '\0');
+	freelog(LOG_DEBUG, "   %2d. %s", inx, buf);
+	ptr = buf;
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " cond_bldg=%d/%s",
+		    b->effect[inx].cond_bldg,
+		    (b->effect[inx].cond_bldg == B_LAST) ?
+		    "Uncond." :
+		    improvement_types[b->effect[inx].cond_bldg].name);
+	ptr = strchr(ptr, '\0');
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " cond_gov=%d/%s",
+		    b->effect[inx].cond_gov,
+		    (b->effect[inx].cond_gov == game.government_count) ?
+		    "Uncond." :
+		    get_government_name(b->effect[inx].cond_gov));
+	ptr = strchr(ptr, '\0');
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " cond_adv=%d/%s",
+		    b->effect[inx].cond_adv,
+		    (b->effect[inx].cond_adv == A_NONE) ?
+		    "Uncond." :
+		    (b->effect[inx].cond_adv == A_LAST) ?
+		    "Never" :
+		    advances[b->effect[inx].cond_adv].name);
+	ptr = strchr(ptr, '\0');
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " cond_eff=%d/%s",
+		    b->effect[inx].cond_eff,
+		    (b->effect[inx].cond_eff == EFT_LAST) ?
+		    "Uncond." :
+		    effect_type_name(b->effect[inx].cond_eff));
+	ptr = strchr(ptr, '\0');
+	freelog(LOG_DEBUG, "       %s", buf);
+	ptr = buf;
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " aff_unit=%d/%s",
+		    b->effect[inx].aff_unit,
+		    (b->effect[inx].aff_unit == UCL_LAST) ?
+		    "All" :
+		    unit_class_name(b->effect[inx].aff_unit));
+	ptr = strchr(ptr, '\0');
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " aff_terr=%d/%s",
+		    b->effect[inx].aff_terr,
+		    (b->effect[inx].aff_terr == T_LAST) ?
+		    "None" :
+		    (b->effect[inx].aff_terr == T_UNKNOWN) ?
+		    "All" :
+		    get_terrain_name(b->effect[inx].aff_terr));
+	ptr = strchr(ptr, '\0');
+	my_snprintf(ptr, sizeof(buf)-(ptr-buf), " aff_spec=%04X/%s",
+		    b->effect[inx].aff_spec,
+		    (b->effect[inx].aff_spec == 0) ?
+		    "None" :
+		    (b->effect[inx].aff_spec == S_ALL) ?
+		    "All" :
+		    get_special_name(b->effect[inx].aff_spec));
+	ptr = strchr(ptr, '\0');
+	freelog(LOG_DEBUG, "       %s", buf);
+      }
+      freelog(LOG_DEBUG, "  variant     %2d", b->variant);	/* FIXME: remove when gen-impr obsoletes */
+      freelog(LOG_DEBUG, "  helptext    %s", b->helptext);
+    }
+  }
+#endif
 }
 
 /**************************************************************************
@@ -1259,7 +1578,6 @@ void handle_ruleset_government(struct packet_ruleset_government *p)
   gov->ruler_titles = fc_calloc(gov->num_ruler_titles,
 				sizeof(struct ruler_title));
   
-  free(gov->helptext);
   gov->helptext = p->helptext;	/* pointer assignment */
   
   tilespec_setup_government(p->id);
@@ -1349,11 +1667,13 @@ void handle_ruleset_terrain_control(struct terrain_misc *p)
   terrain_control.may_irrigate = p->may_irrigate;
   terrain_control.may_mine = p->may_mine;
   terrain_control.may_transform = p->may_transform;
+  terrain_control.ocean_reclaim_requirement = p->ocean_reclaim_requirement;
+  terrain_control.land_channel_requirement = p->land_channel_requirement;
   terrain_control.river_move_mode = p->river_move_mode;
   terrain_control.river_defense_bonus = p->river_defense_bonus;
   terrain_control.river_trade_incr = p->river_trade_incr;
-  terrain_control.river_help_text =
-    p->river_help_text ? mystrdup(p->river_help_text) : NULL;
+  free(terrain_control.river_help_text);
+  terrain_control.river_help_text = p->river_help_text; /* malloc'ed string */
   terrain_control.fortress_defense_bonus = p->fortress_defense_bonus;
   terrain_control.road_superhighway_trade_bonus = p->road_superhighway_trade_bonus;
   terrain_control.rail_food_bonus = p->rail_food_bonus;
@@ -1363,6 +1683,9 @@ void handle_ruleset_terrain_control(struct terrain_misc *p)
   terrain_control.pollution_food_penalty = p->pollution_food_penalty;
   terrain_control.pollution_shield_penalty = p->pollution_shield_penalty;
   terrain_control.pollution_trade_penalty = p->pollution_trade_penalty;
+  terrain_control.fallout_food_penalty = p->fallout_food_penalty;
+  terrain_control.fallout_shield_penalty = p->fallout_shield_penalty;
+  terrain_control.fallout_trade_penalty = p->fallout_trade_penalty;
 }
 
 /**************************************************************************
@@ -1420,6 +1743,21 @@ void handle_ruleset_city(struct packet_ruleset_city *packet)
   tilespec_setup_city_tiles(id);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+void handle_ruleset_game(struct packet_ruleset_game *packet)
+{
+  game.rgame.min_city_center_food = packet->min_city_center_food;
+  game.rgame.min_city_center_shield = packet->min_city_center_shield;
+  game.rgame.min_city_center_trade = packet->min_city_center_trade;
+  game.rgame.min_dist_bw_cities = packet->min_dist_bw_cities;
+  game.rgame.init_vis_radius_sq = packet->init_vis_radius_sq;
+  game.rgame.hut_overflight = packet->hut_overflight;
+  game.rgame.pillage_select = packet->pillage_select;
+  game.rgame.nuke_contamination = packet->nuke_contamination;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -1428,15 +1766,17 @@ void handle_incite_cost(struct packet_generic_values *packet)
   struct city *pcity=find_city_by_id(packet->id);
   struct unit *punit=find_unit_by_id(packet->id);
 
-  if(pcity)  {
+  if(pcity) {
     pcity->incite_revolt_cost = packet->value1;
-    popup_incite_dialog(pcity);
+    if(!game.player_ptr->ai.control || ai_popup_windows)
+      popup_incite_dialog(pcity);
     return;
   }
 
   if(punit) {
     punit->bribe_cost = packet->value1;
-    popup_bribe_dialog(punit);
+    if(!game.player_ptr->ai.control || ai_popup_windows)
+      popup_bribe_dialog(punit);
   }
 }
 
@@ -1458,10 +1798,51 @@ void handle_city_name_suggestion(struct packet_city_name_suggestion *packet)
 {
   struct unit *punit;
   
-  punit = unit_list_find(&game.player_ptr->units, packet->id);
+  punit = player_find_unit_by_id(game.player_ptr, packet->id);
   if (punit) {
     popup_newcity_dialog(punit, packet->name);
     return;
   }
   /* maybe unit died; ignore */
 }
+
+/**************************************************************************
+...
+**************************************************************************/
+void handle_diplomat_action(struct packet_diplomat_action *packet)
+{
+  struct unit *pdiplomat=player_find_unit_by_id(game.player_ptr, packet->diplomat_id);
+
+  if (!pdiplomat) {
+    freelog(LOG_NORMAL, "Received bad diplomat id %d in handle_diplomat_action()",
+	    packet->diplomat_id);
+    return;
+  }
+
+  switch(packet->action_type) {
+  case DIPLOMAT_CLIENT_POPUP_DIALOG:
+    process_diplomat_arrival(pdiplomat, packet->target_id);
+    break;
+  default:
+    freelog(LOG_NORMAL, "Received bad action %d in handle_diplomat_action()",
+	    packet->action_type);
+    break;
+  }
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void handle_sabotage_list(struct packet_sabotage_list *packet)
+{
+  struct unit *punit = player_find_unit_by_id(game.player_ptr, packet->diplomat_id);
+  struct city *pcity = find_city_by_id(packet->city_id);
+
+  if (punit && pcity) {
+    int i;
+    for(i=0; i<B_LAST; i++)
+      pcity->improvements[i] = (packet->improvements[i]=='1') ? 1 : 0;
+
+    popup_sabotage_dialog(pcity);
+  }
+}
