@@ -30,17 +30,18 @@
 #include "fcintl.h"
 #include "game.h" /* for fill_xxx */
 #include "government.h"
+#include "hash.h"
 #include "log.h"
 #include "map.h"
 #include "mem.h"
 #include "nation.h"
 #include "player.h"
 #include "registry.h"
+#include "sbuffer.h"
 #include "shared.h"
 #include "support.h"
 #include "unit.h"
 
-#include "colors_g.h"
 #include "control.h" /* for fill_xxx */
 #include "graphics_g.h"
 #include "options.h" /* for fill_xxx */
@@ -58,6 +59,7 @@ int SMALL_TILE_WIDTH;
 int SMALL_TILE_HEIGHT;
 
 char *city_names_font;
+char *city_productions_font_name;
 
 int flags_are_transparent=1;
 
@@ -67,28 +69,15 @@ int num_tiles_explode_unit=0;
 static int num_spec_files = 0;
 static char **spec_filenames;	/* full pathnames */
 
-/* The list of sprites (as used to be in graphics.c)
-   Plus an athing to keep track of currently allocated size.
-   (reallocs are ok, because the pointers we keep are
-   the things stored by tile_sprites, not pointers into
-   the tile_sprites array itself).
+static struct hash_table *sprite_hash = 0;
+/*
+   This hash table sprite_hash maps tilespec tags to tile Sprite pointers.
+   This is kept permanently after setup, for doing lookups on ruleset
+   information (including on reconnections etc).
 */
-static struct Sprite **tile_sprites;
-static struct athing atile_sprites;   /* .ptr is tile_sprites */
-
-/* Use an internal section_file to store the tiles index numbers
-   corresponding to each tag.  Use section_file purely to get
-   hashing functionality.  (Should really have separate hashing
-   stuff, and use that, but this works for now.)
-
-   This is kept permanently after setup, for doing lookups
-   on ruleset information (including on reconnections etc).
-
-   The entries are simply the tags from the tile spec files,
-   with integer values corresponding to index in tile_sprites.
-*/
-static struct section_file tag_sf;
 
+static struct sbuffer *sprite_key_sb;
+/* used to allocate keys for sprite_hash */
 
 #define TILESPEC_CAPSTR "+tilespec2 duplicates_ok"
 /*
@@ -106,40 +95,16 @@ static struct section_file tag_sf;
 */
 
 
-/***************************************************************************
-  Crop sprite for source and store in global tile_sprites.
-  Reallocates tile_sprites as required, using an athing to
-  keep track.  (Note this is not as bad as it sounds: we
-  are only reallocing the pointers to sprites, not the sprites
-  themselves.)
-  Returns index to this sprite in tile_sprites.
-***************************************************************************/
-static int crop_sprite_to_tiles(struct Sprite *source,
-				int x, int y, int dx, int dy)
-{
-  assert(source);
-  
-  if (tile_sprites==NULL) {
-    ath_init(&atile_sprites, sizeof(struct Sprite*));
-  }
-  ath_minnum(&atile_sprites, atile_sprites.n+1);
-  tile_sprites = atile_sprites.ptr;
-  tile_sprites[atile_sprites.n-1] = crop_sprite(source, x, y, dx, dy);
-  return atile_sprites.n-1;
-}
+/*
+  If focus_unit_hidden is true, then no units at
+  the location of the foc unit are ever drawn.
+*/
+static int focus_unit_hidden = 0;
 
-/**********************************************************************
-  Reallocs tile_sprites exactly to current size;
-  Do this at the end to avoid "permanent" waste.
-***********************************************************************/
-static void final_realloc_tile_sprites(void)
-{
-  atile_sprites.ptr = fc_realloc(atile_sprites.ptr,
-				 atile_sprites.size*atile_sprites.n);
-  atile_sprites.n_alloc = atile_sprites.n;
-  tile_sprites = atile_sprites.ptr;
-}
-  
+/*
+  If no_backdrop is true, then no color/flag is drawn behind the city/unit.
+*/
+static int no_backdrop = 0;
 
 /**********************************************************************
   Gets full filename for tilespec file, based on input name.
@@ -187,7 +152,7 @@ static char *tilespec_fullname(const char *tileset_name)
 /**********************************************************************
   Checks options in filename match what we require and support.
   Die if not.
-  which should be "tilespec" or "spec".
+  'which' should be "tilespec" or "spec".
 ***********************************************************************/
 static void check_tilespec_capabilities(struct section_file *file,
 					const char *which,
@@ -272,6 +237,11 @@ void tilespec_read_toplevel(const char *tileset_name)
   c = secfile_lookup_str_default(file, "10x20", "tilespec.city_names_font");
   city_names_font = mystrdup(c);
 
+  c =
+      secfile_lookup_str_default(file, "8x16",
+				 "tilespec.city_productions_font");
+  city_productions_font_name = mystrdup(c);
+
   flags_are_transparent =
     secfile_lookup_int(file, "tilespec.flags_are_transparent"); 
 
@@ -304,21 +274,19 @@ void tilespec_read_toplevel(const char *tileset_name)
 
 /**********************************************************************
   Load one specfile and specified xpm file; splits xpm into tiles,
-  and save indices in tag_sf.
+  and save sprites in sprite_hash.
 ***********************************************************************/
 static void tilespec_load_one(const char *spec_filename)
 {
   struct section_file the_file, *file = &the_file;
-  struct Sprite *big_sprite = NULL;
+  struct Sprite *big_sprite = NULL, *small_sprite;
   char *gfx_filename,*gfx_current_fileext;
   char **gridnames, **tags, **gfx_fileexts;
   int num_grids, num_tags;
   int i, j, k;
   int x_top_left, y_top_left, dx, dy;
   int row, column;
-  int idx, x1, y1;
-
-  freelog(LOG_DEBUG, "loading spec %s", spec_filename);
+  int x1, y1;
 
   freelog(LOG_DEBUG, "loading spec %s", spec_filename);
   if (!section_file_load(file, spec_filename)) {
@@ -379,10 +347,13 @@ static void tilespec_load_one(const char *spec_filename)
 
       x1 = x_top_left + column * dx;
       y1 = y_top_left + row * dy;
-      idx = crop_sprite_to_tiles(big_sprite, x1, y1, dx, dy);
+      small_sprite = crop_sprite(big_sprite, x1, y1, dx, dy);
 
       for(k=0; k<num_tags; k++) {
-	secfile_insert_int(&tag_sf, idx, "%s", tags[k]);
+	/* don't free old sprite, since could be multiple tags
+	   pointing to it */
+	hash_replace(sprite_hash, sbuf_strdup(sprite_key_sb, tags[k]),
+		     small_sprite);
       }
       free(tags);
     }
@@ -409,9 +380,9 @@ static char *nsew_str(int idx)
   return c;
 }
      
-/* Note very safe, but convenient: */
+/* Not very safe, but convenient: */
 #define SET_SPRITE(field, tag) \
-    sprites.field = tile_sprites[secfile_lookup_int(&tag_sf, "%s", (tag))]
+    sprites.field = hash_lookup_data(sprite_hash, tag);
 
 /**********************************************************************
   Initialize 'sprites' structure based on hardwired tags which
@@ -423,7 +394,7 @@ static void tilespec_lookup_sprite_tags(void)
   char dir_char[] = "nsew";
   int i, j;
   
-  assert(secfilehash_hashash(&tag_sf));
+  assert(sprite_hash);
 
   SET_SPRITE(treaty_thumb[0], "treaty.disagree_thumb_down");
   SET_SPRITE(treaty_thumb[1], "treaty.agree_thumb_up");
@@ -433,6 +404,8 @@ static void tilespec_lookup_sprite_tags(void)
     SET_SPRITE(bulb[i], buffer);
     my_snprintf(buffer, sizeof(buffer), "s.warming_sun_%d", i);
     SET_SPRITE(warming[i], buffer);
+    my_snprintf(buffer, sizeof(buffer), "s.cooling_flake_%d", i);
+    SET_SPRITE(cooling[i], buffer);
   }
 
   SET_SPRITE(right_arrow, "s.right_arrow");
@@ -480,9 +453,12 @@ static void tilespec_lookup_sprite_tags(void)
   }
 
   num_tiles_explode_unit = 0;
-  while (section_file_lookup(&tag_sf, "explode.unit_%d", num_tiles_explode_unit))
-    num_tiles_explode_unit++;
-
+  do {
+    my_snprintf(buffer, sizeof(buffer), "explode.unit_%d",
+		num_tiles_explode_unit++);
+  } while (hash_key_exists(sprite_hash, buffer));
+  num_tiles_explode_unit--;
+    
   if (num_tiles_explode_unit==0) {
     sprites.explode.unit = NULL;
   } else {
@@ -498,7 +474,9 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(unit.auto_attack,  "unit.auto_attack");
   SET_SPRITE(unit.auto_settler, "unit.auto_settler");
   SET_SPRITE(unit.auto_explore, "unit.auto_explore");
-  SET_SPRITE(unit.fortify,	"unit.fortify");     
+  SET_SPRITE(unit.fallout,	"unit.fallout");
+  SET_SPRITE(unit.fortified,	"unit.fortified");     
+  SET_SPRITE(unit.fortifying,	"unit.fortifying");     
   SET_SPRITE(unit.fortress,     "unit.fortress");
   SET_SPRITE(unit.airbase,      "unit.airbase");
   SET_SPRITE(unit.go_to,	"unit.goto");     
@@ -542,6 +520,7 @@ static void tilespec_lookup_sprite_tags(void)
   
   SET_SPRITE(user.attention, "user.attention");
 
+  SET_SPRITE(tx.fallout,    "tx.fallout");
   SET_SPRITE(tx.farmland,   "tx.farmland");
   SET_SPRITE(tx.irrigation, "tx.irrigation");
   SET_SPRITE(tx.mine,       "tx.mine");
@@ -550,6 +529,7 @@ static void tilespec_lookup_sprite_tags(void)
   SET_SPRITE(tx.village,    "tx.village");
   SET_SPRITE(tx.fortress,   "tx.fortress");
   SET_SPRITE(tx.airbase,    "tx.airbase");
+  SET_SPRITE(tx.fog,        "tx.fog");
 
   for(i=0; i<NUM_DIRECTION_NSEW; i++) {
     my_snprintf(buffer, sizeof(buffer), "tx.s_river_%s", nsew_str(i));
@@ -578,7 +558,7 @@ static void tilespec_lookup_sprite_tags(void)
 /**********************************************************************
   Load the tiles; requires tilespec_read_toplevel() called previously.
   Leads to tile_sprites being allocated and filled with pointers
-  to sprites.   Also sets up and populates tag_sf, and calls func
+  to sprites.   Also sets up and populates sprite_hash, and calls func
   to initialize 'sprites' structure.
 ***********************************************************************/
 void tilespec_load_tiles(void)
@@ -586,15 +566,12 @@ void tilespec_load_tiles(void)
   int i;
   
   assert(num_spec_files>0);
-  section_file_init(&tag_sf);
+  sprite_hash = hash_new(hash_fval_string, hash_fcmp_string);
+  sprite_key_sb = sbuf_new();
 
   for(i=0; i<num_spec_files; i++) {
     tilespec_load_one(spec_filenames[i]);
   }
-  final_realloc_tile_sprites();
-
-  secfilehash_build(&tag_sf, 1);
-  /* section_file_save(&tag_sf, "_debug_tilespec"); */
 
   tilespec_lookup_sprite_tags();
 }
@@ -607,29 +584,24 @@ static struct Sprite* lookup_sprite_tag_alt(const char *tag, const char *alt,
 					    int required, const char *what,
 					    const char *name)
 {
+  struct Sprite *sp;
   int loglevel = required ? LOG_NORMAL : LOG_DEBUG;
-  int i;
   
-  /* (should get tag_sf before connection) */
-  if (!secfilehash_hashash(&tag_sf)) {
-    freelog(LOG_FATAL, "attempt to lookup for %s %s before tag_sf setup",
+  /* (should get sprite_hash before connection) */
+  if (!sprite_hash) {
+    freelog(LOG_FATAL, "attempt to lookup for %s %s before sprite_hash setup",
 	    what, name);
     exit(1);
   }
 
-  /* real values are always non-negative: */
-  i = secfile_lookup_int_default(&tag_sf, -1, "%s", tag);
-  if (i != -1) {
-    assert(i>=0 && i<atile_sprites.n);
-    return tile_sprites[i];
-  }
+  sp = hash_lookup_data(sprite_hash, tag);
+  if (sp) return sp;
   
-  i = secfile_lookup_int_default(&tag_sf, -1, "%s", alt);
-  if (i != -1) {
-    assert(i>=0 && i<atile_sprites.n);
+  sp = hash_lookup_data(sprite_hash, alt);
+  if (sp) {
     freelog(loglevel, "Using alternate graphic %s (instead of %s) for %s %s",
 	    alt, tag, what, name);
-    return tile_sprites[i];
+    return sp;
   }
   
   freelog(loglevel, "Don't have graphics tags %s or %s for %s %s",
@@ -713,11 +685,11 @@ void tilespec_setup_government(int id)
 ***********************************************************************/
 void tilespec_setup_nation_flag(int id)
 {
-  struct nation_type *this = get_nation_by_idx(id);
+  struct nation_type *this_nation = get_nation_by_idx(id);
 
-  this->flag_sprite = lookup_sprite_tag_alt(this->flag_graphic_str, 
-					    this->flag_graphic_alt,
-					    1, "nation", this->name);
+  this_nation->flag_sprite = lookup_sprite_tag_alt(this_nation->flag_graphic_str, 
+					    this_nation->flag_graphic_alt,
+					    1, "nation", this_nation->name);
 
   /* should probably do something if NULL, eg generic default? */
 }
@@ -779,13 +751,16 @@ static struct Sprite *get_city_occupied_sprite(struct city *pcity)
 static int fill_city_sprite_array(struct Sprite **sprs, struct city *pcity)
 {
   struct Sprite **save_sprs=sprs;
-
-  if(!use_solid_color_behind_units) {
-    /* will be the first sprite if flags_are_transparent == FALSE */
-    *sprs++ = get_city_nation_flag_sprite(pcity);
-  } else *sprs++ = NULL;
+  struct tile *ptile = map_get_tile(pcity->x, pcity->y);
   
-  if(genlist_size(&((map_get_tile(pcity->x, pcity->y))->units.list)) > 0)
+  if(!no_backdrop) {
+    if(!solid_color_behind_units) {
+      /* will be the first sprite if flags_are_transparent == FALSE */
+      *sprs++ = get_city_nation_flag_sprite(pcity);
+    } else *sprs++ = NULL;
+  }
+  
+  if(genlist_size(&(ptile->units.list)) > 0)
     *sprs++ = get_city_occupied_sprite(pcity);
 
   *sprs++ = get_city_sprite(pcity);
@@ -793,17 +768,25 @@ static int fill_city_sprite_array(struct Sprite **sprs, struct city *pcity)
   if(city_got_citywalls(pcity))
     *sprs++ = get_city_wall_sprite(pcity);
 
+  if(map_get_special(pcity->x, pcity->y) & S_POLLUTION)
+    *sprs++ = sprites.tx.pollution;
+  if(map_get_special(pcity->x, pcity->y) & S_FALLOUT)
+    *sprs++ = sprites.tx.fallout;
+
+  if(city_unhappy(pcity))
+    *sprs++ = sprites.city.disorder;
+
+  if(ptile->known==TILE_KNOWN_FOGGED)
+    *sprs++ = sprites.tx.fog;
+
+  /* Put the size sprites last, so that they are not obscured
+   * (and because they can be hard to read if fogged).
+   */
   if(pcity->size>=10)
     *sprs++ = sprites.city.size_tens[pcity->size/10];
 
   *sprs++ = sprites.city.size[pcity->size%10];
 
-  if(map_get_special(pcity->x, pcity->y) & S_POLLUTION)
-    *sprs++ = sprites.tx.pollution;
-
-  if(city_unhappy(pcity))
-    *sprs++ = sprites.city.disorder;
-
   return sprs - save_sprs;
 }
 
@@ -815,14 +798,15 @@ int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit)
   struct Sprite **save_sprs=sprs;
   int ihp;
 
-  if(!use_solid_color_behind_units) {
-    /* will be the first sprite if flags_are_transparent == FALSE */
-    *sprs++ = get_unit_nation_flag_sprite(punit);
-  }
-	else {
-    /* Two NULLs means unit */
-	  *sprs++ = NULL;
-    *sprs++ = NULL;
+  if(!no_backdrop) {
+    if(!solid_color_behind_units) {
+      /* will be the first sprite if flags_are_transparent == FALSE */
+      *sprs++ = get_unit_nation_flag_sprite(punit);
+    } else {
+      /* Two NULLs means unit */
+      *sprs++ = NULL;
+      *sprs++ = NULL;
+    }
   }
 
   *sprs++ = get_unit_type(punit->type)->sprite;
@@ -836,6 +820,9 @@ int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit)
     case ACTIVITY_POLLUTION:
       s = sprites.unit.pollution;
       break;
+    case ACTIVITY_FALLOUT:
+      s = sprites.unit.fallout;
+      break;
     case ACTIVITY_PILLAGE:
       s = sprites.unit.pillage;
       break;
@@ -849,8 +836,11 @@ int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit)
     case ACTIVITY_EXPLORE:
       s = sprites.unit.auto_explore;
       break;
-    case ACTIVITY_FORTIFY:
-      s = sprites.unit.fortify;
+    case ACTIVITY_FORTIFIED:
+      s = sprites.unit.fortified;
+      break;
+    case ACTIVITY_FORTIFYING:
+      s = sprites.unit.fortifying;
       break;
     case ACTIVITY_FORTRESS:
       s = sprites.unit.fortress;
@@ -908,41 +898,43 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
   int tileno;
   struct tile *ptile;
   struct Sprite *mysprite;
-  struct unit *punit;
   struct city *pcity;
+  struct unit *pfocus;
+  struct unit *punit;
   int den_y=map.ysize*.24;
 
   struct Sprite **save_sprs=sprs;
 
   ptile=map_get_tile(abs_x0, abs_y0);
-  punit=get_unit_in_focus();
-  
-  if(abs_y0>=map.ysize || ptile->known<TILE_KNOWN) {
+
+  if(abs_y0>=map.ysize || ptile->known == TILE_UNKNOWN) {
     return 0;
   }
 
-  if(!flags_are_transparent || use_solid_color_behind_units) {
-    /* non-transparent flags -> just draw city or unit. */
-    if((pcity=map_get_city(abs_x0, abs_y0))
-       && (citymode || !(punit=get_unit_in_focus())
-       || punit->x!=abs_x0 || punit->y!=abs_y0
-       || (unit_list_size(&ptile->units)==0))) {
+  pcity=map_get_city(abs_x0, abs_y0);
+  pfocus=get_unit_in_focus();
 
-      /* above, unit_list_size==0 happens when focus unit is blinking --dwp */ 
-      sprs += fill_city_sprite_array(sprs,pcity);
-      return sprs - save_sprs;
-    }
+  if(!flags_are_transparent || solid_color_behind_units) {
+    /* non-transparent flags -> just draw city or unit */
 
-    if((punit=player_find_visible_unit(game.player_ptr, ptile))) {
+    if((punit=find_visible_unit(ptile))) {
       if(!citymode || punit->owner!=game.player_idx) {
-        sprs += fill_unit_sprite_array(sprs,punit);
-        if(unit_list_size(&ptile->units)>1)
-          *sprs++ = sprites.unit.stack;
-        return sprs - save_sprs;
+	if(!focus_unit_hidden || !pfocus ||
+	   punit->x!=pfocus->x || punit->y!=pfocus->y) {
+	  sprs+=fill_unit_sprite_array(sprs,punit);
+	  if(unit_list_size(&ptile->units)>1)  
+	    *sprs++ = sprites.unit.stack;
+	  return sprs - save_sprs;
+	}
       }
     }
+
+    if(pcity) {
+      sprs+=fill_city_sprite_array(sprs,pcity);
+      return sprs - save_sprs;
+    }
   }
-    
+
   ttype=map_get_terrain(abs_x0, abs_y0);
   ttype_east=map_get_terrain(abs_x0+1, abs_y0);
   ttype_west=map_get_terrain(abs_x0-1, abs_y0);
@@ -1135,6 +1127,8 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
   if(tspecial & S_FORTRESS) *sprs++ = sprites.tx.fortress;
   if(tspecial & S_AIRBASE) *sprs++ = sprites.tx.airbase;
   if(tspecial & S_POLLUTION) *sprs++ = sprites.tx.pollution;
+  if(tspecial & S_FALLOUT) *sprs++ = sprites.tx.fallout;
+  if(ptile->known==TILE_KNOWN_FOGGED) *sprs++ = sprites.tx.fog;
 
   if(!citymode) {
     tileno = INDEX_NSEW((tile_is_known(abs_x0, abs_y0-1)==TILE_UNKNOWN),
@@ -1145,17 +1139,23 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
       *sprs++ = sprites.tx.darkness[tileno];
   }
 
-  if(flags_are_transparent) {  /* transparent flags -> draw city or unit last */
-    if((pcity=map_get_city(abs_x0, abs_y0))) {
+  if(flags_are_transparent) {
+    /* transparent flags -> draw city or unit last */
+
+    if(pcity) {
       sprs+=fill_city_sprite_array(sprs,pcity);
     }
 
-    if((punit=player_find_visible_unit(game.player_ptr, ptile))) {
-      if(pcity && punit!=get_unit_in_focus()) return sprs - save_sprs;
+    if((punit=find_visible_unit(ptile))) {
       if(!citymode || punit->owner!=game.player_idx) {
-        sprs+=fill_unit_sprite_array(sprs,punit);
-        if(unit_list_size(&ptile->units)>1)  
-          *sprs++ = sprites.unit.stack;
+	if(!focus_unit_hidden || !pfocus ||
+	   punit->x!=pfocus->x || punit->y!=pfocus->y) {
+	  no_backdrop=BOOL_VAL(pcity);
+	  sprs+=fill_unit_sprite_array(sprs,punit);
+	  no_backdrop=FALSE;
+	  if(unit_list_size(&ptile->units)>1)  
+	    *sprs++ = sprites.unit.stack;
+	}
       }
     }
   }
@@ -1169,15 +1169,17 @@ int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int cit
 ***********************************************************************/
 static void tilespec_setup_style_tile(int style, char *graphics)
 {
+  struct Sprite *sp;
+  char buffer[128];
   int j;
-  int sprite_idx;
 
   city_styles[style].tiles_num = 0;
 
   for(j=0; j<32 && city_styles[style].tiles_num < MAX_CITY_TILES; j++) {
-    sprite_idx = secfile_lookup_int_default( &tag_sf, -1, "%s_%d", graphics, j);
-    if( sprite_idx != -1 ) {
-      sprites.city.tile[style][city_styles[style].tiles_num] = tile_sprites[sprite_idx];
+    my_snprintf(buffer, sizeof(buffer), "%s_%d", graphics, j);
+    sp = hash_lookup_data(sprite_hash, buffer);
+    if (sp) {
+      sprites.city.tile[style][city_styles[style].tiles_num] = sp;
       city_styles[style].tresh[city_styles[style].tiles_num] = j;
       city_styles[style].tiles_num++;
       freelog(LOG_DEBUG, "Found tile %s_%d", graphics, j);
@@ -1188,18 +1190,22 @@ static void tilespec_setup_style_tile(int style, char *graphics)
     return;
 
   /* the wall tile */
-  sprite_idx = secfile_lookup_int_default( &tag_sf, -1, "%s_wall", graphics);
-  if( sprite_idx != -1 )
-    sprites.city.tile[style][city_styles[style].tiles_num] = tile_sprites[sprite_idx];
-  else
+  my_snprintf(buffer, sizeof(buffer), "%s_wall", graphics);
+  sp = hash_lookup_data(sprite_hash, buffer);
+  if (sp) {
+    sprites.city.tile[style][city_styles[style].tiles_num] = sp;
+  } else {
     freelog(LOG_NORMAL, "Warning: no wall tile for graphic %s", graphics);
+  }
 
   /* occupied tile */
-  sprite_idx = secfile_lookup_int_default( &tag_sf, -1, "%s_occupied", graphics);
-  if( sprite_idx != -1 )
-    sprites.city.tile[style][city_styles[style].tiles_num+1] = tile_sprites[sprite_idx];
-  else
+  my_snprintf(buffer, sizeof(buffer), "%s_occupied", graphics);
+  sp = hash_lookup_data(sprite_hash, buffer);
+  if (sp) {
+    sprites.city.tile[style][city_styles[style].tiles_num+1] = sp;
+  } else {
     freelog(LOG_NORMAL, "Warning: no occupied tile for graphic %s", graphics);
+  }
 }
 
 /**********************************************************************
@@ -1225,11 +1231,11 @@ void tilespec_setup_city_tiles(int style)
             city_styles[style].graphic_alt);
 
     sprites.city.tile[style][0] =
-      tile_sprites[secfile_lookup_int( &tag_sf, "cd.city")];
+      hash_lookup_data(sprite_hash, "cd.city");
     sprites.city.tile[style][1] =
-      tile_sprites[secfile_lookup_int( &tag_sf, "cd.city_wall")];
+      hash_lookup_data(sprite_hash, "cd.city_wall");
     sprites.city.tile[style][2] =
-      tile_sprites[secfile_lookup_int( &tag_sf, "cd.occupied")];
+      hash_lookup_data(sprite_hash, "cd.occupied");
     city_styles[style].tiles_num = 1;
     city_styles[style].tresh[0] = 0;
   }
@@ -1267,11 +1273,51 @@ void tilespec_free_city_tiles(int count)
   fixed number of nations.  Now base on player number instead,
   since still limited to less than 14.  Could possibly improve
   to allow players to choose their preferred color etc.
-  Return is really "enum color_std".
   A hack added to avoid returning more that COLOR_STD_RACE13.
   But really there should be more colors available -- jk.
 ***********************************************************************/
-int player_color(struct player *pplayer)
+enum color_std player_color(struct player *pplayer)
 {
-  return COLOR_STD_RACE0 + (pplayer->player_no % MAX_NUM_PLAYERS);
+  return COLOR_STD_RACE0 +
+    (pplayer->player_no %
+     (COLOR_STD_RACE13 - COLOR_STD_RACE0 + 1));
+}
+
+/**********************************************************************
+  Return color for overview map tile.
+***********************************************************************/
+enum color_std overview_tile_color(int x, int y)
+{
+  enum color_std color;
+  struct tile *ptile=map_get_tile(x, y);
+  struct unit *punit;
+  struct city *pcity;
+
+  if(!ptile->known) {
+    color=COLOR_STD_BLACK;
+  } else if((pcity=map_get_city(x, y))) {
+    if(pcity->owner==game.player_idx)
+      color=COLOR_STD_WHITE;
+    else
+      color=COLOR_STD_CYAN;
+  } else if ((punit=find_visible_unit(ptile))) {
+    if(punit->owner==game.player_idx)
+      color=COLOR_STD_YELLOW;
+    else
+      color=COLOR_STD_RED;
+  } else if(ptile->terrain==T_OCEAN) {
+    color=COLOR_STD_OCEAN;
+  } else {
+    color=COLOR_STD_GROUND;
+  }
+
+  return color;
+}
+
+/**********************************************************************
+  Set focus_unit_hidden (q.v.) variable to given value.
+***********************************************************************/
+void set_focus_unit_hidden_state(int hide)
+{
+  focus_unit_hidden = hide;
 }
