@@ -21,6 +21,8 @@
 
 #include "map.h"		/* NUM_DIRECTION_NSEW */
 
+#include "colors_g.h"
+
 struct Sprite;			/* opaque; gui-dep */
 struct unit;
 struct player;
@@ -41,7 +43,10 @@ void tilespec_free_city_tiles(int count);
 int fill_tile_sprite_array(struct Sprite **sprs, int abs_x0, int abs_y0, int citymode);
 int fill_unit_sprite_array(struct Sprite **sprs, struct unit *punit);
 
-int player_color(struct player *pplayer);
+enum color_std player_color(struct player *pplayer);
+enum color_std overview_tile_color(int x, int y);
+
+void set_focus_unit_hidden_state(int hide);
 
 /* This the way directional indices are now encoded: */
 
@@ -65,6 +70,7 @@ struct named_sprites {
   struct Sprite
     *bulb[NUM_TILES_PROGRESS],
     *warming[NUM_TILES_PROGRESS],
+    *cooling[NUM_TILES_PROGRESS],
     *citizen[NUM_TILES_CITIZEN],   /* internal code... */
     *treaty_thumb[2],     /* 0=disagree, 1=agree */
     *right_arrow;
@@ -93,7 +99,9 @@ struct named_sprites {
       *auto_attack,
       *auto_settler,
       *auto_explore,
-      *fortify,
+      *fallout,
+      *fortified,
+      *fortifying,
       *fortress,
       *airbase,
       *go_to,			/* goto is a C keyword :-) */
@@ -137,6 +145,8 @@ struct named_sprites {
       *village,
       *fortress,
       *airbase,
+      *fallout,
+      *fog,
       *spec_river[NUM_DIRECTION_NSEW],
       *coast_cape[NUM_DIRECTION_NSEW],	      /* first unused */
       *darkness[NUM_DIRECTION_NSEW],         /* first unused */
@@ -188,6 +198,10 @@ extern int SMALL_TILE_HEIGHT;
 
 extern char *city_names_font;
 
+/* name of font to use to draw city productions on main map */
+
+extern char *city_productions_font_name;
+
 extern int flags_are_transparent;
 
 extern int num_tiles_explode_unit;
