@@ -70,9 +70,27 @@ const char * const our_capability = our_capability_internal;
  * are not directly related to the capability strings discussed here.)
  */
 
-#define CAPABILITY "+1.10"
+#define CAPABILITY "+1.11 diplomat_investigate_fix production_change_fix" \
+" game_ruleset nuclear_fallout land_channel_requirement"
 
-/* "+1.10" is protocol for 1.10.0 stable release
+/* "+1.11" is protocol for 1.11.0 stable release.
+
+   "diplomat_investigate_fix" extends the protocol so that diplomat
+   investigation of a city reveals the correct supported and present
+   unit lists.
+
+   "production_change_fix" extends the protocol so that city production
+   changes are correctly accounted for, allowing for recovery of penalty
+   when changing back to original class.
+
+   "game_ruleset" is use of game.ruleset instead of civstyle option.
+
+   "nuclear_fallout" is protocol extension for having nuclear fallout
+   be distinct from industrial pollution.
+
+   "land_channel_requirement" extends the protocol for the requirement
+   of a minimum number of ocean tiles adjacent to a land tile wished to
+   be changed to ocean.
 */
 
 void init_our_capability(void)
