@@ -14,6 +14,7 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -62,12 +63,100 @@ The improvement_types array is now setup in:
    client/packhand.c (for the client)
 *****************************************************************/
 
-struct improvement_type improvement_types[B_LAST];
+struct impr_type improvement_types[B_LAST];
 
 char **misc_city_names; 
 
 struct citystyle *city_styles = NULL;
 
+
+/* Names of effect ranges.
+ * (These must correspond to enum effect_range_id in city.h.)
+ */
+static char *effect_range_names[] = {
+  "None",
+  "Building",
+  "City",
+  "Island",
+  "Player",
+  "World"
+};
+
+/* Names of effect types.
+ * (These must correspond to enum effect_type_id in city.h.)
+ */
+static char *effect_type_names[] = {
+  "Adv_Parasite",
+  "Airlift",
+  "Any_Government",
+  "Barb_Attack",
+  "Barb_Defend",
+  "Capital_City",
+  "Capital_Exists",
+  "Enable_Nuke",
+  "Enable_Space",
+  "Enemy_Peaceful",
+  "Food_Add_Tile",
+  "Food_Bonus",
+  "Food_Inc_Tile",
+  "Food_Per_Tile",
+  "Give_Imm_Adv",
+  "Growth_Food",
+  "Have_Embassies",
+  "Improve_Rep",
+  "Luxury_Bonus",
+  "Luxury_Pct",
+  "Make_Content",
+  "Make_Content_Mil",
+  "Make_Content_Pct",
+  "Make_Happy",
+  "May_Declare_War",
+  "No_Anarchy",
+  "No_Sink_Deep",
+  "Nuke_Proof",
+  "Pollu_Adj",
+  "Pollu_Adj_Pop",
+  "Pollu_Adj_Prod",
+  "Pollu_Decrease",
+  "Prod_Add_Tile",
+  "Prod_Bonus",
+  "Prod_Inc_Tile",
+  "Prod_Per_Tile",
+  "Prod_To_Gold",
+  "Reduce_Corrupt",
+  "Reduce_Waste",
+  "Reveal_Cities",
+  "Reveal_Map",
+  "Revolt_Dist",
+  "Science_Bonus",
+  "Science_Pct",
+  "Size_Unlimit",
+  "Slow_Nuke_Winter",
+  "Slow_Global_Warm",
+  "Space_Part",
+  "Spy_Resistant",
+  "Tax_Bonus",
+  "Tax_Pct",
+  "Trade_Add_Tile",
+  "Trade_Bonus",
+  "Trade_Inc_Tile",
+  "Trade_Per_Tile",
+  "Trade_Route_Pct",
+  "Unit_Defend",
+  "Unit_Move",
+  "Unit_No_Lose_Pop",
+  "Unit_Recover",
+  "Unit_Repair",
+  "Unit_Vet_Combat",
+  "Unit_Veteran",
+  "Upgrade_Units",
+  "Upgrade_One_Step",
+  "Upgrade_One_Leap",
+  "Upgrade_All_Step",
+  "Upgrade_All_Leap",
+  "Upkeep_Free"
+};
+
 /**************************************************************************
   Set the worker on the citymap.  Also sets the worked field in the map.
 **************************************************************************/
@@ -121,10 +210,76 @@ int map_to_city_y(struct city *pcity, int y)
 	return y-pcity->y+2;
 }
 
+/**************************************************************************
+  Convert effect range names to enum; case insensitive;
+  returns EFR_LAST if can't match.
+**************************************************************************/
+Eff_Range_id effect_range_from_str(char *str)
+{
+  Eff_Range_id ret_id;
+
+  assert(sizeof(effect_range_names)/sizeof(effect_range_names[0])==EFR_LAST);
+
+  for (ret_id = 0; ret_id < EFR_LAST; ret_id++) {
+    if (0 == mystrcasecmp(effect_range_names[ret_id], str)) {
+      return ret_id;
+    }
+  }
+
+  return EFR_LAST;
+}
+
+/**************************************************************************
+  Return effect range name; NULL if bad id.
+**************************************************************************/
+char *effect_range_name(Eff_Range_id id)
+{
+  assert(sizeof(effect_range_names)/sizeof(effect_range_names[0])==EFR_LAST);
+
+  if (id < EFR_LAST) {
+    return effect_range_names[id];
+  } else {
+    return NULL;
+  }
+}
+
+/**************************************************************************
+  Convert effect type names to enum; case insensitive;
+  returns EFT_LAST if can't match.
+**************************************************************************/
+Eff_Type_id effect_type_from_str(char *str)
+{
+  Eff_Type_id ret_id;
+
+  assert(sizeof(effect_type_names)/sizeof(effect_type_names[0])==EFT_LAST);
+
+  for (ret_id = 0; ret_id < EFT_LAST; ret_id++) {
+    if (0 == mystrcasecmp(effect_type_names[ret_id], str)) {
+      return ret_id;
+    }
+  }
+
+  return EFT_LAST;
+}
+
+/**************************************************************************
+  Return effect type name; NULL if bad id.
+**************************************************************************/
+char *effect_type_name(Eff_Type_id id)
+{
+  assert(sizeof(effect_type_names)/sizeof(effect_type_names[0])==EFT_LAST);
+
+  if (id < EFT_LAST) {
+    return effect_type_names[id];
+  } else {
+    return NULL;
+  }
+}
+
 /****************************************************************
 ...
 *****************************************************************/
-int wonder_replacement(struct city *pcity, enum improvement_type_id id)
+int wonder_replacement(struct city *pcity, Impr_Type_id id)
 {
   if(is_wonder(id)) return 0;
   switch (id) {
@@ -171,7 +326,7 @@ int wonder_replacement(struct city *pcity, enum improvement_type_id id)
 /****************************************************************
 ...
 *****************************************************************/
-char *get_imp_name_ex(struct city *pcity, enum improvement_type_id id)
+char *get_impr_name_ex(struct city *pcity, Impr_Type_id id)
 {
   static char buffer[256];
   char *state = Q_("?wonder:w");
@@ -194,7 +349,7 @@ char *get_imp_name_ex(struct city *pcity, enum improvement_type_id id)
 ...
 *****************************************************************/
 
-char *get_improvement_name(enum improvement_type_id id)
+char *get_improvement_name(Impr_Type_id id)
 {
   return get_improvement_type(id)->name; 
 }
@@ -203,7 +358,7 @@ char *get_improvement_name(enum improvement_type_id id)
 ...
 **************************************************************************/
 
-int improvement_value(enum improvement_type_id id)
+int improvement_value(Impr_Type_id id)
 {
   return (improvement_types[id].build_cost);
 }
@@ -212,7 +367,7 @@ int improvement_value(enum improvement_type_id id)
 ...
 **************************************************************************/
 
-int is_wonder(enum improvement_type_id id)
+int is_wonder(Impr_Type_id id)
 {
   return (improvement_types[id].is_wonder);
 }
@@ -222,11 +377,11 @@ Returns 1 if the improvement_type "exists" in this game, 0 otherwise.
 An improvement_type doesn't exist if one of:
 - id is out of range;
 - the improvement_type has been flagged as removed by setting its
-  tech_requirement to A_LAST;
+  tech_req to A_LAST;
 - it is a space part, and the spacerace is not enabled.
 Arguably this should be called improvement_type_exists, but that's too long.
 **************************************************************************/
-int improvement_exists(enum improvement_type_id id)
+int improvement_exists(Impr_Type_id id)
 {
   if (id<0 || id>=B_LAST)
     return 0;
@@ -235,14 +390,14 @@ int improvement_exists(enum improvement_type_id id)
       && !game.spacerace)
     return 0;
 
-  return (improvement_types[id].tech_requirement!=A_LAST);
+  return (improvement_types[id].tech_req!=A_LAST);
 }
 
 /**************************************************************************
 Does a linear search of improvement_types[].name
 Returns B_LAST if none match.
 **************************************************************************/
-enum improvement_type_id find_improvement_by_name(char *s)
+Impr_Type_id find_improvement_by_name(char *s)
 {
   int i;
 
@@ -254,9 +409,9 @@ enum improvement_type_id find_improvement_by_name(char *s)
 }
 
 /**************************************************************************
-...
+FIXME: remove when gen-impr obsoletes
 **************************************************************************/
-int improvement_variant(enum improvement_type_id id)
+int improvement_variant(Impr_Type_id id)
 {
   return improvement_types[id].variant;
 }
@@ -264,7 +419,7 @@ int improvement_variant(enum improvement_type_id id)
 /**************************************************************************
 ...
 **************************************************************************/
-int improvement_obsolete(struct player *pplayer, enum improvement_type_id id) 
+int improvement_obsolete(struct player *pplayer, Impr_Type_id id) 
 {
   if (improvement_types[id].obsolete_by==A_NONE) 
     return 0;
@@ -303,18 +458,28 @@ int city_buy_cost(struct city *pcity)
 ...
 **************************************************************************/
 
-int wonder_obsolete(enum improvement_type_id id)
+int wonder_obsolete(Impr_Type_id id)
 { 
   if (improvement_types[id].obsolete_by==A_NONE)
     return 0;
   return (game.global_advances[improvement_types[id].obsolete_by]);
 }
 
+/**************************************************************************
+Barbarians don't get enough knowledges to be counted as normal players.
+**************************************************************************/
+
+int is_wonder_useful(Impr_Type_id id)
+{
+  if ((id == B_GREAT) && (get_num_human_and_ai_players () < 3)) return 0;
+  return 1;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
 
-struct improvement_type *get_improvement_type(enum improvement_type_id id)
+struct impr_type *get_improvement_type(Impr_Type_id id)
 {
   return &improvement_types[id];
 }
@@ -332,7 +497,7 @@ struct player *city_owner(struct city *pcity)
 ...
 *****************************************************************/
 
-int could_build_improvement(struct city *pcity, enum improvement_type_id id)
+int could_build_improvement(struct city *pcity, Impr_Type_id id)
 { /* modularized so the AI can choose the tech it wants -- Syela */
   if (!improvement_exists(id))
     return 0;
@@ -355,7 +520,7 @@ int could_build_improvement(struct city *pcity, enum improvement_type_id id)
     return 0;
   if ((id==B_HARBOUR || id==B_COASTAL || id == B_OFFSHORE || id == B_PORT) && !is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN))
     return 0;
-  if ((id == B_HYDRO || id == B_HOOVER)
+  if (id == B_HYDRO
       && !(map_get_terrain(pcity->x, pcity->y) == T_RIVER)
       && !(map_get_special(pcity->x, pcity->y) & S_RIVER)
       && !(map_get_terrain(pcity->x, pcity->y) == T_MOUNTAINS)
@@ -392,15 +557,15 @@ int could_build_improvement(struct city *pcity, enum improvement_type_id id)
   Whether player could build this improvement, assuming they had
   the tech req, and assuming a city with the right pre-reqs etc.
 *****************************************************************/
-int could_player_eventually_build_improvement(struct player *p, enum improvement_type_id id)
+int could_player_eventually_build_improvement(struct player *p,
+					      Impr_Type_id id)
 {
   if (!improvement_exists(id))
     return 0;
 
-  /* You can't build an improvement if it's tech requirement is
-     Never. */
-  if (improvement_types[id].tech_requirement == A_LAST)
-    return 0;
+  /* You can't build an improvement if it's tech requirement is Never. */
+  /*   if (improvement_types[id].tech_req == A_LAST) return 0; */
+  /* This is what "exists" means, done by improvement_exists() above --dwp */
   
   if (id == B_SSTRUCTURAL || id == B_SCOMP || id == B_SMODULE) {
     if (!game.global_wonders[B_APOLLO]) {
@@ -424,13 +589,13 @@ int could_player_eventually_build_improvement(struct player *p, enum improvement
   return 1;
 }
 
-int could_player_build_improvement(struct player *p, enum improvement_type_id id)
+int could_player_build_improvement(struct player *p, Impr_Type_id id)
 {
   if (!could_player_eventually_build_improvement(p, id))
     return 0;
 
   /* Make sure we have the tech /now/.*/
-  if (get_invention(p, improvement_types[id].tech_requirement) == TECH_KNOWN)
+  if (get_invention(p, improvement_types[id].tech_req) == TECH_KNOWN)
     return 1;
   return 0;
 }
@@ -441,7 +606,7 @@ int could_player_build_improvement(struct player *p, enum improvement_type_id id
 Can this improvement get built in this city, by the player
 who owns the city?
 *****************************************************************/
-int can_build_improvement(struct city *pcity, enum improvement_type_id id)
+int can_build_improvement(struct city *pcity, Impr_Type_id id)
 {
   struct player *p=city_owner(pcity);
   if (!improvement_exists(id))
@@ -454,7 +619,7 @@ int can_build_improvement(struct city *pcity, enum improvement_type_id id)
 /****************************************************************
 Will this city ever be able to build this improvement?
 *****************************************************************/
-int can_eventually_build_improvement(struct city *pcity, enum improvement_type_id id)
+int can_eventually_build_improvement(struct city *pcity, Impr_Type_id id)
 {
   /* Can the _player_ ever build the improvement? */
   if (!could_player_eventually_build_improvement(city_owner(pcity), id))
@@ -482,9 +647,8 @@ int can_eventually_build_improvement(struct city *pcity, enum improvement_type_i
       !is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN))
     return 0;
   
-  /* A hydro dam and the Hoover dam wonder have some fun terrain 
-     requirements. */
-  if ((id == B_HYDRO || id == B_HOOVER)
+  /* A hydro dam has some fun terrain requirements. */
+  if ((id == B_HYDRO)
       && !(map_get_terrain(pcity->x, pcity->y) == T_RIVER)
       && !(map_get_special(pcity->x, pcity->y) & S_RIVER)
       && !(map_get_terrain(pcity->x, pcity->y) == T_MOUNTAINS)
@@ -505,7 +669,7 @@ int can_eventually_build_improvement(struct city *pcity, enum improvement_type_i
 Can a player build this improvement somewhere?  Ignores the
 fact that player may not have a city with appropriate prereqs.
 *****************************************************************/
-int can_player_build_improvement(struct player *p, enum improvement_type_id id)
+int can_player_build_improvement(struct player *p, Impr_Type_id id)
 {
   if (!improvement_exists(id))
     return 0;
@@ -556,8 +720,9 @@ int can_build_unit(struct city *pcity, Unit_Type_id id)
 {  
   if (!can_build_unit_direct(pcity, id))
     return 0;
-  if (can_build_unit_direct(pcity, unit_types[id].obsoleted_by))
-    return 0;
+  while(unit_type_exists((id = unit_types[id].obsoleted_by)))
+    if (can_player_build_unit_direct(city_owner(pcity), id))
+	return 0;
   return 1;
 }
 
@@ -588,8 +753,9 @@ int can_player_build_unit(struct player *p, Unit_Type_id id)
 {  
   if (!can_player_build_unit_direct(p, id))
     return 0;
-  if (can_player_build_unit_direct(p, unit_types[id].obsoleted_by))
-    return 0;
+  while(unit_type_exists((id = unit_types[id].obsoleted_by)))
+    if (can_player_build_unit_direct(p, id))
+	return 0;
   return 1;
 }
 
@@ -602,8 +768,9 @@ int can_player_eventually_build_unit(struct player *p, Unit_Type_id id)
 {
   if (!unit_type_exists(id))
     return 0;
-  if (can_player_build_unit_direct(p, unit_types[id].obsoleted_by))
-    return 0;
+  while(unit_type_exists((id = unit_types[id].obsoleted_by)))
+    if (can_player_build_unit_direct(p, id))
+	return 0;
   return 1;
 }
 
@@ -627,7 +794,7 @@ int city_population(struct city *pcity)
 ...
 **************************************************************************/
 
-int city_got_building(struct city *pcity,  enum improvement_type_id id) 
+int city_got_building(struct city *pcity,  Impr_Type_id id) 
 {
   if (!improvement_exists(id))
     return 0;
@@ -644,10 +811,10 @@ int improvement_upkeep(struct city *pcity, int i)
     return 0;
   if (is_wonder(i))
     return 0;
-  if (improvement_types[i].shield_upkeep == 1 &&
+  if (improvement_types[i].upkeep == 1 &&
       city_affected_by_wonder(pcity, B_ASMITHS)) 
     return 0;
-  return (improvement_types[i].shield_upkeep);
+  return (improvement_types[i].upkeep);
 }
 
 /**************************************************************************
@@ -659,9 +826,9 @@ static int improvement_upkeep_asmiths(struct city *pcity, int i, int asmiths)
     return 0;
   if (is_wonder(i))
     return 0;
-  if (asmiths && improvement_types[i].shield_upkeep == 1) 
+  if (asmiths && improvement_types[i].upkeep == 1) 
     return 0;
-  return (improvement_types[i].shield_upkeep);
+  return (improvement_types[i].upkeep);
 }
 
 /**************************************************************************
@@ -685,9 +852,9 @@ int get_shields_tile(int x, int y, struct city *pcity)
   else
     s=get_tile_type(tile_t)->shield;
 
-  if (x == 2 &&  y == 2 && !s) /* Always give a single shield on city square */
-    s=1;
-  
+  if (s<game.rgame.min_city_center_shield && x==2 && y==2)
+    s=game.rgame.min_city_center_shield;
+
   if (spec_t & S_MINE) {
     s += (get_tile_type(tile_t))->mining_shield_incr;
   }
@@ -704,6 +871,8 @@ int get_shields_tile(int x, int y, struct city *pcity)
     s--;
   if (spec_t & S_POLLUTION)
     s-=(s*terrain_control.pollution_shield_penalty)/100; /* The shields here is icky */
+  if (spec_t & S_FALLOUT)
+    s-=(s*terrain_control.fallout_shield_penalty)/100;
   return s;
 }
 
@@ -724,8 +893,11 @@ int get_trade_tile(int x, int y, struct city *pcity)
     t=get_tile_type(tile_t)->trade_special_2;
   else
     t=get_tile_type(tile_t)->trade;
-    
-  if (spec_t & S_RIVER) {
+
+  if (t<game.rgame.min_city_center_trade && x==2 && y==2)
+    t=game.rgame.min_city_center_trade;
+
+  if ((spec_t & S_RIVER) && (tile_t != T_OCEAN)) {
     t += terrain_control.river_trade_incr;
   }
   if (spec_t & S_ROAD) {
@@ -754,6 +926,8 @@ int get_trade_tile(int x, int y, struct city *pcity)
       t--;
     if (spec_t & S_POLLUTION)
       t-=(t*terrain_control.pollution_trade_penalty)/100; /* The trade here is dirty */
+    if (spec_t & S_FALLOUT)
+      t-=(t*terrain_control.fallout_trade_penalty)/100;
   }
   return t;
 }
@@ -787,6 +961,9 @@ int get_food_tile(int x, int y, struct city *pcity)
   else
     f=get_tile_type(tile_t)->food;
 
+  if (f<game.rgame.min_city_center_food && x==2 && y==2)
+    f=game.rgame.min_city_center_food;
+
   if ((spec_t & S_IRRIGATION) || city_auto_water) {
     f += type->irrigation_food_incr;
     if (((spec_t & S_FARMLAND) ||
@@ -803,9 +980,6 @@ int get_food_tile(int x, int y, struct city *pcity)
   if (spec_t & S_RAILROAD)
     f+=(f*terrain_control.rail_food_bonus)/100;
 
-  if (game.civstyle==2 && tile_t==T_MOUNTAINS && x==2 && y==2)
-    f++;
-
   if (f)
     f += (celeb ? g->celeb_food_bonus : g->food_bonus);
   if (before_penalty && f > before_penalty) 
@@ -813,10 +987,31 @@ int get_food_tile(int x, int y, struct city *pcity)
 
   if (spec_t & S_POLLUTION)
     f-=(f*terrain_control.pollution_food_penalty)/100; /* The food here is yucky */
+  if (spec_t & S_FALLOUT)
+    f-=(f*terrain_control.fallout_food_penalty)/100;
 
   return f;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+int city_can_be_built_here(int x, int y)
+{
+  int dx, dy;
+
+  if(map_get_terrain(x, y)==T_OCEAN)
+    return 0;
+
+  /* game.rgame.min_dist_bw_cities minimum is 1, which means adjacent is okay */
+  for (dx = -game.rgame.min_dist_bw_cities+1; dx < game.rgame.min_dist_bw_cities; dx++)
+    for (dy = -game.rgame.min_dist_bw_cities+1; dy < game.rgame.min_dist_bw_cities; dy++)
+      if (map_get_city(x + dx, y + dy))
+	return 0;
+
+  return 1;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -894,7 +1089,7 @@ int city_gold_surplus(struct city *pcity)
  (The improvement_type_id should be an improvement, not a wonder.)
  Note also: city_got_citywalls(), and server/citytools:city_got_barracks()
 **************************************************************************/
-int city_got_effect(struct city *pcity, enum improvement_type_id id)
+int city_got_effect(struct city *pcity, Impr_Type_id id)
 {
   return city_got_building(pcity, id) || wonder_replacement(pcity, id);
 }
@@ -914,7 +1109,7 @@ int city_got_citywalls(struct city *pcity)
 /**************************************************************************
 ...
 **************************************************************************/
-int city_affected_by_wonder(struct city *pcity, enum improvement_type_id id) /*FIX*/
+int city_affected_by_wonder(struct city *pcity, Impr_Type_id id) /*FIX*/
 {
   struct city *tmp;
   if (!improvement_exists(id))
@@ -1018,77 +1213,7 @@ int city_rapture_grow(struct city *pcity)
 	  government_has_flag(g, G_RAPTURE_CITY_GROWTH));
 }
 
-/**************************************************************************/
-
-/* The find_city_by_id() code has returned from its trip to server land and
- * lives in common once again.  There are two ways find_city_by_id() works. 
- * If citycachesize!=0, it uses a fast (and potentially large, 256K) lookup
- * table to find the city.  If citycachesize==0, it uses a slow linear
- * searching method.  citycachesize is set by calling initialize_city_cache(),
- * which code that wants to use the fast method should do.  Currently the
- * server uses the cache, and the client doesn't.  */
-
-static struct city **citycache = NULL;
-static int citycachesize = 0;
-
-/**************************************************************************
-  Initialize the cache.  This enables the fast find_city_by_id() code.  Once
-  enabled, the cache has to be kept consistent by calling add_city_to_cache()
-  and remove_city_from_cache() as appropriate.
-**************************************************************************/
-void initialize_city_cache(void)
-{
-  int i;
-
-  if(citycache) free(citycache);
-  citycachesize=128;
-  citycache=fc_malloc(sizeof(*citycache) * citycachesize);
-  for(i=0;i<citycachesize;i++) citycache[i]=NULL;
-}
-
-/**************************************************************************
-  Double the size of the cache.  add_city_to_cache() calls this when needed.
-**************************************************************************/
-static void reallocate_cache(void)
-{
-  int i;
-
-  freelog(LOG_DEBUG,"Increasing max city id index from %d to %d",
-       citycachesize,citycachesize*2);
-  citycachesize*=2;
-  citycache=fc_realloc(citycache,sizeof(*citycache)*citycachesize);
-  for(i=citycachesize/2;i<citycachesize;i++)  citycache[i]=NULL;
-}
-
-/**************************************************************************
-  Add the city pointer to the lookup table.  If the calling program isn't
-  using the cache, it's still safe to call this function.
-**************************************************************************/
-void add_city_to_cache(struct city *pcity)
-{
-  if(!citycachesize) return;		/* Not using the cache, return */
-  if(pcity->id < citycachesize)  {
-    citycache[pcity->id]=pcity;
-  } else {
-    reallocate_cache();
-    add_city_to_cache(pcity);
-  }
-}
-
-/**************************************************************************
-  Remove the city pointer from the lookup table.  If the calling program isn't
-  using the cache, it's still safe to call this function.
-**************************************************************************/
-void remove_city_from_cache(int id)
-{
-  if(!citycachesize) return;		/* Not using the cache, return */
-  if(id >= citycachesize) {
-    freelog(LOG_FATAL, "Tried to delete bogus city id, %d",id);
-    exit(0);
-  }
-  citycache[id]=NULL;
-}
-
+#ifdef UNUSED
 /**************************************************************************
   The slow method for find_city_by_id(), searches though the city lists in
   linear time.  Used in the client, which doesn't need to lookup city IDs that
@@ -1105,23 +1230,7 @@ static struct city *find_city_by_id_list(int city_id)
 
   return 0;
 }
-
-/**************************************************************************
-  Often used function to get a city pointer from a city ID.  This uses either
-  a fast lookup table, or a slow list searching method.
-**************************************************************************/
-struct city *find_city_by_id(int id)
-{
-  /* This is sometimes called with id=unit.ai.charge, which is either
-   * a unit id or a city id; if its a unit id then that id won't be used
-   * for a city (?), so that is ok, except that it is possible that it
-   * might exceed citycachesize. --dwp
-   */
-  if(!citycachesize) return find_city_by_id_list(id);
-  if(id<0 || id>=citycachesize) return NULL;
-  return citycache[id];
-}
-
+#endif
 
 /**************************************************************************
 ...
@@ -1250,38 +1359,128 @@ int get_style_by_name(char *style_name)
     return -1;
 }
 
+/**************************************************************************
+ Compute and optionally apply the change-production penalty for the given
+ production change (to target,is_unit) in the given city (pcity).
+ Always returns the number of shields which would be in the stock if
+ the penalty had been applied.
+
+ If we switch the "class" of the target sometime after a city has produced
+ (i.e., not on the turn immediately after), then there's a shield loss.
+ But only on the first switch that turn.  Also, if ever change back to
+ original improvement class of this turn, restore lost production.
+**************************************************************************/
+int city_change_production_penalty(struct city *pcity,
+				   int target, int is_unit, int apply_it)
+{
+  int shield_stock_after_adjustment = pcity->shield_stock;
+
+  freelog(LOG_DEBUG, "compute change-production penalty: %s -> %s",
+	  pcity->is_building_unit ?
+	    unit_types[pcity->currently_building].name :
+	    improvement_types[pcity->currently_building].name,
+	  is_unit ?
+	    unit_types[target].name :
+	    improvement_types[target].name
+	  );
+  freelog(LOG_DEBUG,
+	  "before: (stock=%d last/test=%d/%d changed=%d year=%d orig: %d %s)",
+	  pcity->shield_stock,
+	  pcity->turn_last_built, game_next_year(pcity->turn_last_built),
+	  pcity->turn_changed_target, game.year,
+	  pcity->before_change_shields,
+	  pcity->changed_from_is_unit ?
+	    unit_types[pcity->changed_from_id].name :
+	    improvement_types[pcity->changed_from_id].name
+	  );
+
+  /* If already building this thing, nothing to do... */
+  if ((pcity->is_building_unit == is_unit) &&
+      (pcity->currently_building == target)) {
+    freelog(LOG_DEBUG, "...did not change production; do nothing");
+    return shield_stock_after_adjustment;
+  }
+
+  /* We're changing production... */
+  if (((pcity->is_building_unit != is_unit) ||
+       (!is_unit &&
+	(is_wonder(pcity->currently_building) != is_wonder(target)))) &&
+      (game_next_year(pcity->turn_last_built) < game.year)) {
+    /* We've changed "class", and it's not the turn immed after completion. */
+    freelog(LOG_DEBUG, "...changed prod class in latter year");
+    if (pcity->turn_changed_target != game.year) {
+      /* This is the first change this turn... */
+      freelog(LOG_DEBUG, "   ...first change this year; penalty");
+      /* Compute new shield_stock after production penalty. */
+      shield_stock_after_adjustment /= 2;
+      /* Only if really applying the penalty... */
+      if (apply_it) {
+	/* Apply the penalty. */
+	pcity->shield_stock = shield_stock_after_adjustment;
+	/* Only set turn_changed_target when we actually cop the penalty -
+	   otherwise you can change to something of the same class first,
+	   and suffer no penalty for any later changes in the same turn. */
+	pcity->turn_changed_target = game.year;
+      }
+    } else if ((pcity->changed_from_is_unit == is_unit) &&
+	       (is_unit ||
+		(is_wonder(pcity->changed_from_id) == is_wonder(target)))) {
+      /* This is a change back to the initial production class... */
+      freelog(LOG_DEBUG, "   ...revert to initial production class");
+      /* Compute new shield_stock after reverting penalty. */
+      shield_stock_after_adjustment = pcity->before_change_shields;
+      /* Only if really applying the penalty... */
+      if (apply_it) {
+	/* Undo penalty. */
+	pcity->shield_stock = shield_stock_after_adjustment;
+	/* Pretend we havn't changed this turn. */
+	pcity->turn_changed_target = GAME_START_YEAR;
+      }
+#ifdef DEBUG
+    } else {
+      freelog(LOG_DEBUG, "   ...latter change in year; no penalty");
+#endif
+    }
+#ifdef DEBUG
+  } else {
+    freelog(LOG_DEBUG, "...same class or first year; no penalty");
+#endif
+  }
+
+  freelog(LOG_DEBUG,
+	  "--> %-3d (stock=%d last/test=%d/%d changed=%d year=%d orig: %d %s)",
+	  shield_stock_after_adjustment,
+	  pcity->shield_stock,
+	  pcity->turn_last_built, game_next_year(pcity->turn_last_built),
+	  pcity->turn_changed_target, game.year,
+	  pcity->before_change_shields,
+	  pcity->changed_from_is_unit ?
+	    unit_types[pcity->changed_from_id].name :
+	    improvement_types[pcity->changed_from_id].name
+	  );
+
+  return shield_stock_after_adjustment;
+}
+
 /**************************************************************************
  Calculates the turns which are needed to build the requested
- type in the city. GUI Independent.
+ improvement in the city.  GUI Independent.
 **************************************************************************/
 int city_turns_to_build(struct city *pcity, int id, int id_is_unit)
 {
   int city_shield_surplus = pcity->shield_surplus;
+  int city_shield_stock =
+    city_change_production_penalty(pcity, id, id_is_unit, FALSE);
+  int improvement_cost = id_is_unit ?
+    get_unit_type(id)->build_cost : get_improvement_type(id)->build_cost;
 
-  if (city_shield_surplus > 0)
-  {
-    int rounds, cost;
-    int shield_stock = pcity->shield_stock;
-
-    if (id_is_unit)
-    {
-      if (!pcity->is_building_unit)
-	shield_stock /= 2;
-      cost = get_unit_type(id)->build_cost;
-    }
-    else
-    {
-      if (pcity->is_building_unit ||
-	  (is_wonder(pcity->currently_building) != is_wonder(id)))
-	shield_stock /= 2;
-      cost = get_improvement_type(id)->build_cost;
-    }
-
-    rounds = (cost - shield_stock + city_shield_surplus - 1) /
+  if (city_shield_stock >= improvement_cost) {
+    return 1;
+  } else if (city_shield_surplus > 0) {
+    return
+      (improvement_cost - city_shield_stock + city_shield_surplus - 1) /
       city_shield_surplus;
-
-    return (rounds > 0) ? rounds : 1;
+  } else {
+    return 999;
   }
-
-  return 999;
 }
