@@ -21,6 +21,7 @@
 #include "city.h"
 #include "fcintl.h"
 #include "government.h"
+#include "idex.h"
 #include "log.h"
 #include "map.h"
 #include "mem.h"
@@ -636,24 +637,28 @@ struct city *game_find_city_by_name(char *name)
 
 
 /**************************************************************************
-...
+  Often used function to get a city pointer from a city ID.
+  City may be any city in the game.  This now always uses fast idex
+  method, instead of looking through all cities of all players.
 **************************************************************************/
-struct unit *game_find_unit_by_id(int unit_id)
+struct city *find_city_by_id(int id)
 {
-  int i;
-  struct unit *punit;
-
-  for(i=0; i<game.nplayers; i++)
-    if((punit=unit_list_find(&game.players[i].units, unit_id)))  
-      return punit;
-
-  return 0;
+  return idex_lookup_city(id);
 }
 
 
+/**************************************************************************
+  Find unit out of all units in game: now uses fast idex method,
+  instead of looking through all units of all players.
+**************************************************************************/
+struct unit *find_unit_by_id(int id)
+{
+  return idex_lookup_unit(id);
+}
+
 
 /**************************************************************************
-...
+If in the server use wipe_unit().
 **************************************************************************/
 void game_remove_unit(int unit_id)
 {
@@ -661,13 +666,13 @@ void game_remove_unit(int unit_id)
 
   freelog(LOG_DEBUG, "game_remove_unit %d", unit_id);
   
-  if((punit=game_find_unit_by_id(unit_id))) {
+  if((punit=find_unit_by_id(unit_id))) {
     struct city *pcity;
 
     freelog(LOG_DEBUG, "removing unit %d, %s %s (%d %d) hcity %d",
 	   unit_id, get_nation_name(get_player(punit->owner)->nation),
 	   unit_name(punit->type), punit->x, punit->y, punit->homecity);
-    
+
     pcity=player_find_city_by_id(get_player(punit->owner), punit->homecity);
     if(pcity)
       unit_list_unlink(&pcity->units_supported, punit);
@@ -680,6 +685,8 @@ void game_remove_unit(int unit_id)
     unit_list_unlink(&map_get_tile(punit->x, punit->y)->units, punit);
     unit_list_unlink(&game.players[punit->owner].units, punit);
     
+    idex_unregister_unit(punit);
+    
     free(punit);
     if(is_server)
       dealloc_id(unit_id);
@@ -702,6 +709,8 @@ void game_remove_city(struct city *pcity)
   }
   city_list_unlink(&game.players[pcity->owner].cities, pcity);
   map_set_city(pcity->x, pcity->y, NULL);
+  idex_unregister_city(pcity);
+  free(pcity->worklist);
   free(pcity);
 }
 
@@ -714,6 +723,8 @@ void game_init(void)
   game.is_new_game = 1;
   game.globalwarming=0;
   game.warminglevel=8;
+  game.nuclearwinter=0;
+  game.coolinglevel=8;
   game.gold        = GAME_DEFAULT_GOLD;
   game.tech        = GAME_DEFAULT_TECHLEVEL;
   game.skill_level = GAME_DEFAULT_SKILL_LEVEL;
@@ -730,23 +741,35 @@ void game_init(void)
   game.freecost    = GAME_DEFAULT_FREECOST;
   game.conquercost = GAME_DEFAULT_CONQUERCOST;
   game.settlers    = GAME_DEFAULT_SETTLERS;
+  game.explorer    = GAME_DEFAULT_EXPLORER;
+  game.dispersion  = GAME_DEFAULT_DISPERSION;
   game.cityfactor  = GAME_DEFAULT_CITYFACTOR;
   game.civilwarsize= GAME_DEFAULT_CIVILWARSIZE;
-  game.explorer    = GAME_DEFAULT_EXPLORER;
   game.unhappysize = GAME_DEFAULT_UNHAPPYSIZE;
   game.foodbox     = GAME_DEFAULT_FOODBOX;
   game.aqueductloss= GAME_DEFAULT_AQUEDUCTLOSS;
+  game.killcitizen = GAME_DEFAULT_KILLCITIZEN;
   game.scorelog    = GAME_DEFAULT_SCORELOG;
   game.techpenalty = GAME_DEFAULT_TECHPENALTY;
   game.civstyle    = GAME_DEFAULT_CIVSTYLE;
   game.razechance  = GAME_DEFAULT_RAZECHANCE;
   game.spacerace   = GAME_DEFAULT_SPACERACE;
+  game.fogofwar    = GAME_DEFAULT_FOGOFWAR;
+  game.fogofwar_old= game.fogofwar;
+  game.auto_ai_toggle = GAME_DEFAULT_AUTO_AI_TOGGLE;
   game.barbarianrate  = GAME_DEFAULT_BARBARIANRATE;
   game.onsetbarbarian = GAME_DEFAULT_ONSETBARBARIAN;
+  game.nbarbarians = 0;
   game.occupychance= GAME_DEFAULT_OCCUPYCHANCE;
   game.heating     = 0;
+  game.cooling     = 0;
   sz_strlcpy(game.save_name, "civgame");
   game.save_nturns=10;
+#ifdef HAVE_LIBZ
+  game.save_compress_level = GAME_DEFAULT_COMPRESS_LEVEL;
+#else
+  game.save_compress_level = GAME_NO_COMPRESS_LEVEL;
+#endif
   game.randseed=GAME_DEFAULT_RANDSEED;
 
   sz_strlcpy(game.ruleset.techs,       GAME_DEFAULT_RULESET);
@@ -756,6 +779,7 @@ void game_init(void)
   sz_strlcpy(game.ruleset.governments, GAME_DEFAULT_RULESET);
   sz_strlcpy(game.ruleset.nations,     GAME_DEFAULT_RULESET);
   sz_strlcpy(game.ruleset.cities,      GAME_DEFAULT_RULESET);
+  sz_strlcpy(game.ruleset.game,        GAME_DEFAULT_RULESET);
   game.firepower_factor = 1;
   game.num_unit_types = 0;
   game.num_tech_types = 0;
@@ -768,6 +792,7 @@ void game_init(void)
   sz_strlcpy(game.demography, GAME_DEFAULT_DEMOGRAPHY);
 
   map_init();
+  idex_init();
   
   for(i=0; i<MAX_NUM_PLAYERS+MAX_NUM_BARBARIANS; i++)
     player_init(&game.players[i]);
@@ -777,6 +802,7 @@ void game_init(void)
     game.global_wonders[i]=0;
   game.player_idx=0;
   game.player_ptr=&game.players[0];
+  terrain_control.river_help_text = NULL;
 }
 
 void initialize_globals(void)
@@ -822,7 +848,7 @@ int game_next_year(int year)
   spaceshipparts= 0;
   if (game.spacerace) {
     for(i=0; parts[i] < B_LAST; i++) {
-      int t = improvement_types[parts[i]].tech_requirement;
+      int t = improvement_types[parts[i]].tech_req;
       if(tech_exists(t) && game.global_advances[t]) 
 	spaceshipparts++;
     }
@@ -876,7 +902,7 @@ void game_remove_all_players(void)
 void game_remove_player(int plrno)
 {
   struct player *pplayer=&game.players[plrno];
-  
+
   unit_list_iterate(pplayer->units, punit) 
     game_remove_unit(punit->id);
   unit_list_iterate_end;
@@ -884,6 +910,8 @@ void game_remove_player(int plrno)
   city_list_iterate(pplayer->cities, pcity) 
     game_remove_city(pcity);
   city_list_iterate_end;
+
+  if (is_barbarian(pplayer)) game.nbarbarians--;
 }
 
 /***************************************************************
@@ -892,32 +920,18 @@ void game_remove_player(int plrno)
 void game_renumber_players(int plrno)
 {
   int i;
-  
+
   for(i=plrno; i<game.nplayers-1; ++i) {
     game.players[i]=game.players[i+1];
     game.players[i].player_no=i;
-    unit_list_iterate(game.players[i].units, punit)
-      punit->owner=i;
-    unit_list_iterate_end;
-
-    city_list_iterate(game.players[i].cities, pcity) {
-      pcity->owner=i;
-      pcity->original= (pcity->original<plrno) ? pcity->original : pcity->original-1;
-    }
-    city_list_iterate_end;
   }
-  
+
   if(game.player_idx>plrno) {
     game.player_idx--;
     game.player_ptr=&game.players[game.player_idx];
   }
-  
+
   game.nplayers--;
-  
-  for(i=0; i<game.nplayers-1; ++i) {
-    game.players[i].embassy=WIPEBIT(game.players[i].embassy, plrno);
-  }
-  
 }
 
 /**************************************************************************
@@ -929,6 +943,14 @@ struct player *get_player(int player_id)
     return &game.players[player_id];
 }
 
+/**************************************************************************
+This function is used by is_wonder_useful to estimate if it is worthwhile
+to build the great library.
+**************************************************************************/
+int get_num_human_and_ai_players(void)
+{
+  return game.nplayers-game.nbarbarians;
+}
 
 /***************************************************************
   For various data, copy eg .name to .name_orig and put
@@ -942,35 +964,35 @@ void translate_data_names(void)
   int i, j;
   
   for (i=0; i<game.num_tech_types; i++) {
-    struct advance *this = &advances[i];
-    sz_strlcpy(this->name_orig, this->name);
-    sz_strlcpy(this->name, Q_(this->name_orig));
+    struct advance *tthis = &advances[i];
+    sz_strlcpy(tthis->name_orig, tthis->name);
+    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
   }
   for (i=0; i<game.num_unit_types; i++) {
-    struct unit_type *this = &unit_types[i];
-    sz_strlcpy(this->name_orig, this->name);
-    sz_strlcpy(this->name, Q_(this->name_orig));
+    struct unit_type *tthis = &unit_types[i];
+    sz_strlcpy(tthis->name_orig, tthis->name);
+    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
   }
   for (i=0; i<B_LAST; i++) {
-    struct improvement_type *this = &improvement_types[i];
-    sz_strlcpy(this->name_orig, this->name);
-    sz_strlcpy(this->name, Q_(this->name_orig));
+    struct impr_type *tthis = &improvement_types[i];
+    sz_strlcpy(tthis->name_orig, tthis->name);
+    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
   }
   for (i=T_FIRST; i<T_COUNT; i++) {
-    struct tile_type *this = &tile_types[i];
-    sz_strlcpy(this->terrain_name_orig, this->terrain_name);
-    sz_strlcpy(this->terrain_name, Q_(this->terrain_name_orig));
-    sz_strlcpy(this->special_1_name_orig, this->special_1_name);
-    sz_strlcpy(this->special_1_name, Q_(this->special_1_name_orig));
-    sz_strlcpy(this->special_2_name_orig, this->special_2_name);
-    sz_strlcpy(this->special_2_name, Q_(this->special_2_name_orig));
+    struct tile_type *tthis = &tile_types[i];
+    sz_strlcpy(tthis->terrain_name_orig, tthis->terrain_name);
+    sz_strlcpy(tthis->terrain_name, Q_(tthis->terrain_name_orig));
+    sz_strlcpy(tthis->special_1_name_orig, tthis->special_1_name);
+    sz_strlcpy(tthis->special_1_name, Q_(tthis->special_1_name_orig));
+    sz_strlcpy(tthis->special_2_name_orig, tthis->special_2_name);
+    sz_strlcpy(tthis->special_2_name, Q_(tthis->special_2_name_orig));
   }
   for (i=0; i<game.government_count; i++) {
-    struct government *this = &governments[i];
-    sz_strlcpy(this->name_orig, this->name);
-    sz_strlcpy(this->name, Q_(this->name_orig));
-    for(j=0; j<this->num_ruler_titles; j++) {
-      struct ruler_title *that = &this->ruler_titles[j];
+    struct government *tthis = &governments[i];
+    sz_strlcpy(tthis->name_orig, tthis->name);
+    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
+    for(j=0; j<tthis->num_ruler_titles; j++) {
+      struct ruler_title *that = &tthis->ruler_titles[j];
       sz_strlcpy(that->male_title_orig, that->male_title);
       sz_strlcpy(that->male_title, Q_(that->male_title_orig));
       sz_strlcpy(that->female_title_orig, that->female_title);
@@ -978,15 +1000,15 @@ void translate_data_names(void)
     }
   }
   for (i=0; i<game.nation_count; i++) {
-    struct nation_type *this = get_nation_by_idx(i);
-    sz_strlcpy(this->name_orig, this->name);
-    sz_strlcpy(this->name, Q_(this->name_orig));
-    sz_strlcpy(this->name_plural_orig, this->name_plural);
-    sz_strlcpy(this->name_plural, Q_(this->name_plural_orig));
+    struct nation_type *tthis = get_nation_by_idx(i);
+    sz_strlcpy(tthis->name_orig, tthis->name);
+    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
+    sz_strlcpy(tthis->name_plural_orig, tthis->name_plural);
+    sz_strlcpy(tthis->name_plural, Q_(tthis->name_plural_orig));
   }
   for (i=0; i<game.styles_count; i++) {
-    struct citystyle *this = &city_styles[i];
-    sz_strlcpy(this->name_orig, this->name);
-    sz_strlcpy(this->name, Q_(this->name_orig));
+    struct citystyle *tthis = &city_styles[i];
+    sz_strlcpy(tthis->name_orig, tthis->name);
+    sz_strlcpy(tthis->name, Q_(tthis->name_orig));
   }
 }
