@@ -39,30 +39,94 @@ static struct tile void_tile;
 struct terrain_misc terrain_control;
 struct tile_type tile_types[T_LAST];
 
+/* used to compute neighboring tiles */
+const int DIR_DX[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
+const int DIR_DY[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
+
+/* Names of specials.
+ * (These must correspond to enum tile_special_type in terrain.h.)
+ */
+static char *tile_special_type_names[] =
+{
+  N_("Special1"),
+  N_("Road"),
+  N_("Irrigation"),
+  N_("Railroad"),
+  N_("Mine"),
+  N_("Pollution"),
+  N_("Hut"),
+  N_("Fortress"),
+  N_("Special2"),
+  N_("River"),
+  N_("Farmland"),
+  N_("Airbase"),
+  N_("Fallout")
+};
+
 /***************************************************************
   Return a (static) string with terrain name;
   eg: "Hills"
   eg: "Hills (Coals)"
-  eg: "Polluted Hills (Coals)"
+  eg: "Hills (Coals) [Pollution]"
 ***************************************************************/
 char *map_get_tile_info_text(int x, int y)
 {
   static char s[64];
   struct tile *ptile=map_get_tile(x, y);
-  
-  my_snprintf(s, sizeof(s), "%s%s",
-	      (ptile->special&S_POLLUTION ? _("Polluted ") : ""),
-	      tile_types[ptile->terrain].terrain_name);
+  int first;
+
+  sz_strlcpy(s, tile_types[ptile->terrain].terrain_name);
   if(ptile->special&S_RIVER) {
-    cat_snprintf(s, sizeof(s), "/%s", _("River"));
+    sz_strlcat(s, "/");
+    sz_strlcat(s, _(get_special_name(S_RIVER)));
   }
-  if(ptile->special&S_SPECIAL_1) {
-    cat_snprintf(s, sizeof(s), "(%s)",
-		 tile_types[ptile->terrain].special_1_name);
-  } else if(ptile->special&S_SPECIAL_2) {
-    cat_snprintf(s, sizeof(s), "(%s)",
-		 tile_types[ptile->terrain].special_2_name);
+
+  first = 1;
+  if (ptile->special & S_SPECIAL_1) {
+    if (first) {
+      first = 0;
+      sz_strlcat(s, " (");
+    } else {
+      sz_strlcat(s, "/");
+    }
+    sz_strlcat(s, tile_types[ptile->terrain].special_1_name);
   }
+  if (ptile->special & S_SPECIAL_2) {
+    if (first) {
+      first = 0;
+      sz_strlcat(s, " (");
+    } else {
+      sz_strlcat(s, "/");
+    }
+    sz_strlcat(s, tile_types[ptile->terrain].special_2_name);
+  }
+  if (!first) {
+    sz_strlcat(s, ")");
+  }
+
+  first = 1;
+  if (ptile->special & S_POLLUTION) {
+    if (first) {
+      first = 0;
+      sz_strlcat(s, " [");
+    } else {
+      sz_strlcat(s, "/");
+    }
+    sz_strlcat(s, _(get_special_name(S_POLLUTION)));
+  }
+  if (ptile->special & S_FALLOUT) {
+    if (first) {
+      first = 0;
+      sz_strlcat(s, " [");
+    } else {
+      sz_strlcat(s, "/");
+    }
+    sz_strlcat(s, _(get_special_name(S_FALLOUT)));
+  }
+  if (!first) {
+    sz_strlcat(s, "]");
+  }
+
   return s;
 }
 
@@ -144,14 +208,63 @@ struct tile_type *get_tile_type(enum tile_terrain_type type)
 enum tile_terrain_type get_terrain_by_name(char * name)
 {
   enum tile_terrain_type tt;
-  for (tt = T_FIRST; tt < T_COUNT; tt++)
-    {
-      if (0 == strcmp (tile_types[tt].terrain_name, name))
-	{
-	  break;
-	}
+  for (tt = T_FIRST; tt < T_COUNT; tt++) {
+    if (0 == strcmp (tile_types[tt].terrain_name, name)) {
+      break;
     }
-  return (tt);
+  }
+  return tt;
+}
+
+/***************************************************************
+...
+***************************************************************/
+char *get_terrain_name(enum tile_terrain_type type)
+{
+  if (type < T_COUNT) {
+    return get_tile_type(type)->terrain_name;
+  } else {
+    return NULL;
+  }
+}
+
+/***************************************************************
+...
+***************************************************************/
+enum tile_special_type get_special_by_name(char * name)
+{
+  int inx;
+  enum tile_special_type st = 1;
+
+  for (inx = 0;
+       inx < sizeof(tile_special_type_names)/sizeof(tile_special_type_names[0]);
+       inx++) {
+    if (0 == strcmp(name, tile_special_type_names[inx])) {
+      return st;
+    }
+    st <<= 1;
+  }
+
+  return S_NO_SPECIAL;
+}
+
+/***************************************************************
+...
+***************************************************************/
+char *get_special_name(enum tile_special_type type)
+{
+  int inx;
+
+  for (inx = 0;
+       inx < sizeof(tile_special_type_names)/sizeof(tile_special_type_names[0]);
+       inx++) {
+    if (type & 0x1) {
+      return tile_special_type_names[inx];
+    }
+    type >>= 1;
+  }
+
+  return NULL;
 }
 
 /***************************************************************
@@ -214,15 +327,15 @@ int is_terrain_near_tile(int x, int y, enum tile_terrain_type t)
 ***************************************************************/
 int isnt_terrain_near_tile(int x, int y, enum tile_terrain_type t)
 {
-  if (map_get_terrain(x-1, y+1)!=t) return 1;
-  if (map_get_terrain(x+1, y-1)!=t) return 1;
-  if (map_get_terrain(x-1, y-1)!=t) return 1;
-  if (map_get_terrain(x+1, y+1)!=t) return 1;
-  if (map_get_terrain(x, y+1)!=t)   return 1;
-  if (map_get_terrain(x-1, y)!=t)   return 1;
-  if (map_get_terrain(x+1, y)!=t)   return 1;
-  if (map_get_terrain(x, y-1)!=t)   return 1;
-  return 0;
+  if (map_get_terrain(x-1, y+1)==t) return 0;
+  if (map_get_terrain(x+1, y-1)==t) return 0;
+  if (map_get_terrain(x-1, y-1)==t) return 0;
+  if (map_get_terrain(x+1, y+1)==t) return 0;
+  if (map_get_terrain(x, y+1)==t)   return 0;
+  if (map_get_terrain(x-1, y)==t)   return 0;
+  if (map_get_terrain(x+1, y)==t)   return 0;
+  if (map_get_terrain(x, y-1)==t)   return 0;
+  return 1;
 }
 
 /***************************************************************
@@ -350,10 +463,6 @@ int is_starter_close(int x, int y, int nr, int dist)
   if (map_get_tile(x, y)->special&S_HUT)
     return 1;
   
-  /* don't start on a river: */
-  if (map_get_tile(x, y)->special&S_RIVER)
-    return 1;
-  
   /* don't want them starting on the poles: */
   if (map_get_continent(x, y)<=2)
     return 1;
@@ -659,6 +768,29 @@ int map_transform_time(int x, int y)
   return tile_types[map_get_terrain(x, y)].transform_time;
 }
 
+/***************************************************************
+...
+***************************************************************/
+static void clear_infrastructure(int x, int y)
+{
+  map_clear_special(x, y, S_ROAD);
+  map_clear_special(x, y, S_IRRIGATION);
+  map_clear_special(x, y, S_RAILROAD);
+  map_clear_special(x, y, S_MINE);
+  map_clear_special(x, y, S_FORTRESS);
+  map_clear_special(x, y, S_FARMLAND);
+  map_clear_special(x, y, S_AIRBASE);
+}
+
+/***************************************************************
+...
+***************************************************************/
+static void clear_dirtiness(int x, int y)
+{
+  map_clear_special(x, y, S_POLLUTION);
+  map_clear_special(x, y, S_FALLOUT);
+}
+
 /***************************************************************
 ...
 ***************************************************************/
@@ -678,6 +810,12 @@ void map_irrigate_tile(int x, int y)
   }
   else if(result!=T_LAST) {
     map_set_terrain(x, y, result);
+    if (result==T_OCEAN) {
+      clear_infrastructure(x, y);
+      clear_dirtiness(x, y);
+      map_clear_special(x, y, S_RIVER);	/* FIXME: When rest of code can handle
+					   rivers in oceans, don't clear this! */
+    }
     reset_move_costs(x, y);
   }
   map_clear_special(x, y, S_MINE);
@@ -697,6 +835,12 @@ void map_mine_tile(int x, int y)
     map_set_special(x, y, S_MINE);
   else if(result!=T_LAST) {
     map_set_terrain(x, y, result);
+    if (result==T_OCEAN) {
+      clear_infrastructure(x, y);
+      clear_dirtiness(x, y);
+      map_clear_special(x, y, S_RIVER);	/* FIXME: When rest of code can handle
+					   rivers in oceans, don't clear this! */
+    }
     reset_move_costs(x, y);
   }
   map_clear_special(x,y, S_FARMLAND);
@@ -715,6 +859,12 @@ void map_transform_tile(int x, int y)
   
   if (result != T_LAST) {
     map_set_terrain(x, y, result);
+    if (result==T_OCEAN) {
+      clear_infrastructure(x, y);
+      clear_dirtiness(x, y);
+      map_clear_special(x, y, S_RIVER);	/* FIXME: When rest of code can handle
+					   rivers in oceans, don't clear this! */
+    }
     reset_move_costs(x, y);
   }
 
@@ -730,37 +880,6 @@ void map_transform_tile(int x, int y)
   }
 }
 
-/***************************************************************
-  (x,y) are map coords, assumed to be already adjusted;
-  we calculate elements of xx[3], yy[3] for adjusted
-  coordinates of adjacent tiles, offsets [0]=-1, [1]=0, [2]=+1
-  Here we adjust yy[] values to stay inside [0,map.ysize-1]
-***************************************************************/
-void map_calc_adjacent_xy(int x, int y, int *xx, int *yy)
-{
-  if((xx[2]=x+1)==map.xsize) xx[2]=0;
-  if((xx[0]=x-1)==-1) xx[0]=map.xsize-1;
-  xx[1] = x;
-  if ((yy[0]=y-1)==-1) yy[0] = 0;
-  if ((yy[2]=y+1)==map.ysize) yy[2]=y;
-  yy[1] = y;
-}
-
-/***************************************************************
-  Like map_calc_adjacent_xy(), except don't adjust the yy[] values.
-  Note that if y is out of range, then map_get_tile(x,y) returns
-  &void_tile, hence the _void in the name of this function.
-***************************************************************/
-void map_calc_adjacent_xy_void(int x, int y, int *xx, int *yy)
-{
-  if((xx[2]=x+1)==map.xsize) xx[2]=0;
-  if((xx[0]=x-1)==-1) xx[0]=map.xsize-1;
-  xx[1] = x;
-  yy[0] = y - 1;
-  yy[1] = y;
-  yy[2] = y + 1;
-}
-
 /***************************************************************
   The basic cost to move punit from tile t1 to tile t2.
   That is, tile_move_cost(), with pre-calculated tile pointers;
@@ -854,26 +973,23 @@ static void debug_log_move_costs(char *str, int x, int y, struct tile *tile0)
 ***************************************************************/
 void reset_move_costs(int x, int y)
 {
-  int k, x1, y1, xx[3], yy[3];
-  int ii[8] = { 0, 1, 2, 0, 2, 0, 1, 2 };
-  int jj[8] = { 0, 0, 0, 1, 1, 2, 2, 2 };
+  int dir, x1, y1;
   int maxcost = 72; /* should be big enough without being TOO big */
   struct tile *tile0, *tile1;
  
   x = map_adjust_x(x);
   tile0 = map_get_tile(x, y);
-  map_calc_adjacent_xy_void(x, y, xx, yy);
   debug_log_move_costs("Resetting move costs for", x, y, tile0);
 
-  for (k = 0; k < 8; k++) {
-    x1 = xx[ii[k]];
-    y1 = yy[jj[k]];
+  for (dir = 0; dir < 8; dir++) {
+    x1 = map_adjust_x(x + DIR_DX[dir]);
+    y1 = y + DIR_DY[dir];
     tile1 = map_get_tile(x1, y1);
-    tile0->move_cost[k] = tile_move_cost_ai(tile0, tile1, x, y,
-					    x1, y1, maxcost);
+    tile0->move_cost[dir] = tile_move_cost_ai(tile0, tile1, x, y,
+					      x1, y1, maxcost);
     /* reverse: not at all obfuscated now --dwp */
     /* this might muck with void_tile, but who cares? */
-    tile1->move_cost[7 - k] = tile_move_cost_ai(tile1, tile0, x1, y1,
+    tile1->move_cost[7 - dir] = tile_move_cost_ai(tile1, tile0, x1, y1,
 						x, y, maxcost);
   }
   debug_log_move_costs("Reset move costs for", x, y, tile0);
@@ -886,25 +1002,22 @@ void reset_move_costs(int x, int y)
 ***************************************************************/
 void initialize_move_costs(void)
 {
-  int x, y, x1, y1, k, xx[3], yy[3];
-  int ii[8] = { 0, 1, 2, 0, 2, 0, 1, 2 };
-  int jj[8] = { 0, 0, 0, 1, 1, 2, 2, 2 };
+  int x, y, x1, y1, dir;
   int maxcost = 72; /* should be big enough without being TOO big */
   struct tile *tile0, *tile1;
 
-  for (k = 0; k < 8; k++) {
-    void_tile.move_cost[k] = maxcost;
+  for (dir = 0; dir < 8; dir++) {
+    void_tile.move_cost[dir] = maxcost;
   }
 
   for (x = 0; x < map.xsize; x++) {
     for (y = 0; y < map.ysize; y++) {
       tile0 = map_get_tile(x, y);
-      map_calc_adjacent_xy_void(x, y, xx, yy);
-      for (k = 0; k < 8; k++) {
-	x1 = xx[ii[k]];
-	y1 = yy[jj[k]];
+      for (dir = 0; dir < 8; dir++) {
+        x1 = map_adjust_x(x + DIR_DX[dir]);
+        y1 = y + DIR_DY[dir];
         tile1 = map_get_tile(x1, y1);
-        tile0->move_cost[k] = tile_move_cost_ai(tile0, tile1, x, y,
+        tile0->move_cost[dir] = tile_move_cost_ai(tile0, tile1, x, y,
 						x1, y1, maxcost);
       }
     }
@@ -912,22 +1025,13 @@ void initialize_move_costs(void)
 }
 
 /***************************************************************
-  The cost to move punit from x1,y1 to x2,y2
-  The tiles are assumed to be adjacent
+  The cost to move punit from where it is to tile x,y.
+  It is assumed the move is a valid one, e.g. the tiles are adjacent.
 ***************************************************************/
-int tile_move_cost(struct unit *punit, int x1, int y1, int x2, int y2)
+int map_move_cost(struct unit *punit, int x, int y)
 {
-  return tile_move_cost_ptrs(punit, map_get_tile(x1,y1),
-			     map_get_tile(x2,y2), x1, y1, x2, y2);
-}
-
-/***************************************************************
-  The cost to move punit from where it is to tile x1,y1.
-  It is assumed the move is a valid one, e.g. the tiles are adjacent
-***************************************************************/
-int map_move_cost(struct unit *punit, int x1, int y1)
-{
-  return tile_move_cost(punit, punit->x, punit->y, x1, y1);
+  return tile_move_cost_ptrs(punit, map_get_tile(punit->x,punit->y),
+			     map_get_tile(x, y), punit->x, punit->y, x, y);
 }
 
 /***************************************************************
@@ -953,6 +1057,7 @@ void tile_init(struct tile *ptile)
   ptile->terrain=T_UNKNOWN;
   ptile->special=S_NO_SPECIAL;
   ptile->known=0;
+  ptile->sent=0;
   ptile->city=NULL;
   unit_list_init(&ptile->units);
   ptile->worked = NULL; /* pointer to city working tile */
@@ -1062,16 +1167,7 @@ void map_set_city(int x, int y, struct city *pcity)
 
 
 /***************************************************************
-...
-***************************************************************/
-int map_get_known(int x, int y, struct player *pplayer)
-{
-  return ((map.tiles+map_adjust_x(x)+
-	   map_adjust_y(y)*map.xsize)->known)&(1u<<pplayer->player_no);
-}
-
-/***************************************************************
-...
+Only for use on the client side
 ***************************************************************/
 enum known_type tile_is_known(int x, int y)
 {
@@ -1079,38 +1175,6 @@ enum known_type tile_is_known(int x, int y)
 	   map_adjust_y(y)*map.xsize)->known);
 }
 
-
-/***************************************************************
-...
-***************************************************************/
-void map_set_known(int x, int y, struct player *pplayer)
-{
-  (map.tiles+map_adjust_x(x)+
-   map_adjust_y(y)*map.xsize)->known|=(1u<<pplayer->player_no);
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_clear_known(int x, int y, struct player *pplayer)
-{
-  (map.tiles+map_adjust_x(x)+
-   map_adjust_y(y)*map.xsize)->known&=~(1u<<pplayer->player_no);
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_know_all(struct player *pplayer)
-{
-  int x, y;
-  for (x = 0; x < map.xsize; ++x) {
-    for (y = 0; y < map.ysize; ++y) {
-      map_set_known(x, y, pplayer);
-    }
-  }
-}
-
 /***************************************************************
   Are (x1,y1) and (x2,y2) really the same when adjusted?
   This function might be necessary ALOT of places...
@@ -1120,3 +1184,27 @@ int same_pos(int x1, int y1, int x2, int y2)
   return (map_adjust_x(x1) == map_adjust_x(x2)
 	  && map_adjust_y(y1) == map_adjust_y(y2)); 
 }
+
+/***************************************************************
+If we just adjust we might light a tile twice; That would be a
+Mayor problem with fog of war
+***************************************************************/
+int is_real_tile(int x, int y)
+{
+  return y >= 0 && y < map.ysize;
+}
+
+/**************************************************************************
+Normalizes the map position. Returns TRUE if it is valid, false otherwise.
+**************************************************************************/
+int normalize_map_pos(int *x, int *y) {
+  if (*y < 0 || *y >= map.ysize)
+    return FALSE;
+
+  if (*x < 0)
+    *x += map.xsize;
+  else if (*x >= map.xsize)
+    *x -= map.xsize;
+
+  return TRUE;
+}
