@@ -89,6 +89,7 @@ enum packet_type {
   PACKET_INCITE_INQ,
   PACKET_INCITE_COST,
   PACKET_UNIT_UPGRADE,
+  PACKET_PLAYER_CANCEL_PACT,
   PACKET_RULESET_TECH,
   PACKET_RULESET_UNIT,
   PACKET_RULESET_BUILDING,
@@ -106,7 +107,11 @@ enum packet_type {
   PACKET_RULESET_NATION,
   PACKET_UNIT_PARADROP_TO,
   PACKET_RULESET_CITY,
-  PACKET_UNIT_CONNECT
+  PACKET_UNIT_CONNECT,
+  PACKET_SABOTAGE_LIST,
+  PACKET_ADVANCE_FOCUS,
+  PACKET_RULESET_GAME,
+  PACKET_LAST  /* leave this last */
 };
 
 enum report_type {
@@ -128,8 +133,14 @@ enum spaceship_action_type {
   SSHIP_ACT_PLACE_SOLAR_PANELS
 };
 
+enum unit_info_use {
+  UNIT_INFO_IDENTITY,
+  UNIT_INFO_CITY_SUPPORTED,
+  UNIT_INFO_CITY_PRESENT
+};
+
 /*********************************************************
-  diplomat action!
+  diplomacy action!
 *********************************************************/
 struct packet_diplomacy_info {
   int plrno0, plrno1;
@@ -289,10 +300,17 @@ struct packet_unit_info {
   int activity_target;
   int paradropped;
   int connecting;
+  /* in packet only, not in unit struct */
+  int carried;
+  int select_it;
+  int packet_use;	/* see enum unit_info_use */
+  int info_city_id;	/* for UNIT_INFO_CITY_SUPPORTED
+			   and UNIT_INFO_CITY_PRESENT uses */
+  int serial_num;	/* a 16-bit unsigned number, never zero
+			   (not used by UNIT_INFO_IDENTITY) */
 };
 
 
-
 /*********************************************************
 ...
 *********************************************************/
@@ -319,6 +337,12 @@ struct packet_city_info {
   int is_building_unit;
   int currently_building;
 
+  int turn_last_built;
+  int turn_changed_target;
+  int changed_from_id;
+  int changed_from_is_unit;
+  int before_change_shields;
+
   struct worklist worklist;
 
   char improvements[B_LAST+1];
@@ -405,6 +429,8 @@ struct packet_player_info {
   int nation;
   int turn_done, nturns_idle;
   int is_alive;
+  int reputation;
+  struct player_diplstate diplstates[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
   int gold, tax, science, luxury;
   int researched;
   int researchpoints;
@@ -542,12 +568,20 @@ struct packet_ruleset_tech {
 struct packet_ruleset_building {
   int id;			/* index for improvement_types[] */
   char name[MAX_LEN_NAME];
+  Tech_Type_id tech_req;
+  Impr_Type_id bldg_req;
+  enum tile_terrain_type *terr_gate;
+  enum tile_special_type *spec_gate;
+  Eff_Range_id equiv_range;
+  Impr_Type_id *equiv_dupl;
+  Impr_Type_id *equiv_repl;
+  Tech_Type_id obsolete_by;
   int is_wonder;
-  int tech_requirement;
   int build_cost;
-  int shield_upkeep;
-  int obsolete_by;
-  int variant;
+  int upkeep;
+  int sabotage;
+  struct impr_effect *effect;
+  int variant;		/* FIXME: remove when gen-impr obsoletes */
   char *helptext;		/* same as for packet_ruleset_unit, above */
 };
 
@@ -684,6 +718,17 @@ struct packet_ruleset_city {
   int replaced_by;
 };
 
+struct packet_ruleset_game {
+  int min_city_center_food;
+  int min_city_center_shield;
+  int min_city_center_trade;
+  int min_dist_bw_cities;
+  int init_vis_radius_sq;
+  int hut_overflight;
+  int pillage_select;
+  int nuke_contamination;
+};
+
 /*********************************************************
 ...
 *********************************************************/
@@ -700,6 +745,8 @@ struct packet_game_info {
   int player_idx;
   int globalwarming;
   int heating;
+  int nuclearwinter;
+  int cooling;
   int cityfactor;
   int unhappysize;
   int diplcost,freecost,conquercost;
@@ -708,6 +755,8 @@ struct packet_game_info {
   int foodbox;
   int techpenalty;
   int spacerace;
+  /* the following values are computed each time packet_game_info is sent */
+  int seconds_to_turndone;
 };
 
 /*********************************************************
@@ -752,6 +801,15 @@ struct socket_packet_buffer {
   unsigned char data[10*MAX_LEN_PACKET];
 };
 
+struct packet_sabotage_list
+{
+  int diplomat_id;
+  int city_id;
+  char improvements[B_LAST+1];
+};
+
+
+typedef	void	CLOSE_FUN	(struct connection *pc);
 
 
 struct connection {
@@ -940,6 +998,11 @@ int send_packet_ruleset_city(struct connection *pc,
 struct packet_ruleset_city *
 receive_packet_ruleset_city(struct connection *pc);
 
+int send_packet_ruleset_game(struct connection *pc,
+                             struct packet_ruleset_game *packet);
+struct packet_ruleset_game *
+receive_packet_ruleset_game(struct connection *pc);
+
 int send_packet_before_end_year(struct connection *pc);
 
 int send_packet_generic_values(struct connection *pc, int type,
@@ -962,8 +1025,14 @@ int send_packet_city_name_suggestion(struct connection *pc,
 struct packet_city_name_suggestion *
 receive_packet_city_name_suggestion(struct connection *pc);
 
+int send_packet_sabotage_list(struct connection *pc,
+			      struct packet_sabotage_list *packet);
+struct packet_sabotage_list *
+receive_packet_sabotage_list(struct connection *pc);
+
 void *get_packet_from_connection(struct connection *pc, int *ptype);
 void remove_packet_from_buffer(struct socket_packet_buffer *buffer);
+void close_socket_set_callback(CLOSE_FUN *fun);
 void flush_connection_send_buffer(struct connection *pc);
 void connection_do_buffer(struct connection *pc);
 void connection_do_unbuffer(struct connection *pc);
