@@ -27,6 +27,12 @@
   - Whitespace lines are ignored, as are lines where the first
   non-whitespace character is ';' (comment lines).
   Optionally '#' can also be used for comments.
+
+  - A line of the form:
+       *include "filename"
+  includes the named file at that point.  (The '*' must be the
+  first character on the line.) The file is found by looking in
+  FREECIV_PATH.  Non-infinite recursive includes are allowed.
   
   - A line with "[name]" labels the start of a section with
   that name; one of these must be the first non-comment line in
@@ -135,9 +141,7 @@
   - For convenience, store the key (the full section+entry name)
     in the hash table (some memory overhead).
   - The number of entries is fixed when the hash table is built.
-  - Use closed hashing with simple collision resolution.
-  - Should have implemented abstract hash table and used it here,
-    rather than tie hashing closely to section_file...
+  - Now uses hash.c
 **************************************************************************/
 
 #include <stdio.h>
@@ -149,7 +153,9 @@
 
 #include "astring.h"
 #include "genlist.h"
+#include "hash.h"
 #include "inputfile.h"
+#include "ioz.h"
 #include "log.h"
 #include "mem.h"
 #include "sbuffer.h"
@@ -160,8 +166,6 @@
 
 #define MAX_LEN_BUFFER 1024
 
-#define DO_HASH 1
-#define HASH_DEBUG 1		/* 0,1,2 */
 #define SAVE_TABLES 1		/* set to 0 for old-style savefiles */
 #define SECF_DEBUG_ENTRIES 0	/* LOG_DEBUG each entry value */
 
@@ -205,27 +209,15 @@ struct section {
        TYPED_LIST_ITERATE_REV(struct section, seclist, psection)
 #define section_list_iterate_rev_end  LIST_ITERATE_REV_END
 
-
-struct hash_entry {
-  struct entry *data;
-  char *key_val;		/* including section prefix */
-  int hash_val;			/* not really necessary, but may reduce
-				   key comparisons? */
-};
-
+/* The hash table and some extra data: */
 struct hash_data {
-  struct hash_entry *table;
-  int num_buckets;
+  struct hash_table *htbl;
   int num_entries_hashbuild;
-  int num_collisions;
   int allow_duplicates;
   int num_duplicates;
 };
 
-static int hashfunc(char *name, int num_buckets);
 static void secfilehash_check(struct section_file *file);
-static struct hash_entry *secfilehash_lookup(struct section_file *file,
-					     char *key, int *hash_return);
 static void secfilehash_insert(struct section_file *file,
 			       char *key, struct entry *data);
 
@@ -354,7 +346,9 @@ static struct entry *new_entry(struct sbuffer *sb, const char *name,
 /**************************************************************************
 ...
 **************************************************************************/
-int section_file_load(struct section_file *sf, const char *filename)
+static int section_file_load_dup(struct section_file *sf,
+				 const char *filename,
+				 int allow_duplicates)
 {
   struct inputfile *inf;
   struct section *psection = NULL;
@@ -369,7 +363,7 @@ int section_file_load(struct section_file *sf, const char *filename)
   struct athing columns_tab;	        /* astrings for column headings */
   struct astring *columns = NULL;	/* -> columns_tab.ptr */
 
-  inf = inf_open(filename);
+  inf = inf_open(filename, datafilename);
   if (!inf) {
     freelog(LOG_NORMAL, "Could not open file \"%s\"", filename);
     return 0;
@@ -393,10 +387,27 @@ int section_file_load(struct section_file *sf, const char *filename)
       if (table_state) {
 	inf_die(inf, "new section during table");
       }
-      psection = sbuf_malloc(sb, sizeof(struct section));
-      psection->name = sbuf_strdup(sb, tok);
-      entry_list_init(&psection->entries);
-      section_list_insert_back(sf->sections, psection);
+      /* Check if we already have a section with this name.
+	 (Could ignore this and have a duplicate sections internally,
+	 but then secfile_get_secnames_prefix would return duplicates.)
+	 Duplicate section in input are likely to be useful for includes.
+	 This is slow if there are lots of sections; [cs]hould have a
+	 hash on section names.
+      */
+      psection = NULL;
+      section_list_iterate(*sf->sections, asection) {
+	if (strcmp(asection->name, tok)==0) {
+	  psection = asection;
+	  break;
+	}
+      }
+      section_list_iterate_end;
+      if (psection==NULL) {
+	psection = sbuf_malloc(sb, sizeof(struct section));
+	psection->name = sbuf_strdup(sb, tok);
+	entry_list_init(&psection->entries);
+	section_list_insert_back(sf->sections, psection);
+      }
       inf_token_required(inf, INF_TOK_EOL);
       continue;
     }
@@ -506,13 +517,27 @@ int section_file_load(struct section_file *sf, const char *filename)
   }
   ath_free(&columns_tab);
   
-  if(DO_HASH) {
-    secfilehash_build(sf, 0);
-  }
+  secfilehash_build(sf, allow_duplicates);
     
   return 1;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+int section_file_load(struct section_file *sf, const char *filename)
+{
+  return section_file_load_dup(sf, filename, 1);
+}
+
+/**************************************************************************
+  Load a section_file, but disallow (die on) duplicate entries.
+**************************************************************************/
+int section_file_load_nodup(struct section_file *sf, const char *filename)
+{
+  return section_file_load_dup(sf, filename, 0);
+}
+
 /**************************************************************************
  Save the previously filled in section_file to disk.
  
@@ -526,19 +551,25 @@ int section_file_load(struct section_file *sf, const char *filename)
  This should be followed by the other column values for u0,
  and then subsequent u1, u2, etc, in strict order with no omissions,
  and with all of the columns for all uN in the same order as for u0.
+
+ If compression_level is non-zero, then compress using zlib.  (Should
+ only supply non-zero compression_level if already know that HAVE_LIBZ.)
+ Below simply specifies FZ_ZLIB method, since fz_fopen() automatically
+ changes to FZ_PLAIN method when level==0.
 **************************************************************************/
-int section_file_save(struct section_file *my_section_file, const char *filename)
+int section_file_save(struct section_file *my_section_file, const char *filename,
+		      int compression_level)
 {
-  FILE *fs;
+  fz_FILE *fs;
 
   struct genlist_iterator ent_iter, save_iter, col_iter;
   struct entry *pentry, *col_pentry;
 
-  if(!(fs=fopen(filename, "w")))
+  if(!(fs=fz_fopen(filename, "w", FZ_ZLIB, compression_level)))
     return 0;
 
   section_list_iterate(*my_section_file->sections, psection) {
-    fprintf(fs, "\n[%s]\n", psection->name);
+    fz_fprintf(fs, "\n[%s]\n", psection->name);
 
     /* Following doesn't use entry_list_iterate() because we want to do
      * tricky things with the iterators...
@@ -577,7 +608,7 @@ int section_file_save(struct section_file *my_section_file, const char *filename
 	first[offset-2] = '\0';
 	sz_strlcpy(base, first);
 	first[offset-2] = '0';
-	fprintf(fs, "%s={", base);
+	fz_fprintf(fs, "%s={", base);
 
 	/* Save an iterator at this first entry, which we can later use
 	 * to repeatedly iterate over column names:
@@ -590,10 +621,10 @@ int section_file_save(struct section_file *my_section_file, const char *filename
 	for( ; (col_pentry = ITERATOR_PTR(col_iter)); ITERATOR_NEXT(col_iter)) {
 	  if(strncmp(col_pentry->name, first, offset) != 0)
 	    break;
-	  fprintf(fs, "%c\"%s\"", (ncol==0?' ':','), col_pentry->name+offset);
+	  fz_fprintf(fs, "%c\"%s\"", (ncol==0?' ':','), col_pentry->name+offset);
 	  ncol++;
 	}
-	fprintf(fs, "\n");
+	fz_fprintf(fs, "\n");
 
 	/* Iterate over rows and columns, incrementing ent_iter as we go,
 	 * and writing values to the table.  Have a separate iterator
@@ -615,25 +646,25 @@ int section_file_save(struct section_file *my_section_file, const char *filename
 	    if(icol != 0) {
 	      freelog(LOG_NORMAL, "unexpected exit from tabular at %s (%s)",
 		      pentry->name, filename);
-	      fprintf(fs, "\n");
+	      fz_fprintf(fs, "\n");
 	    }
-	    fprintf(fs, "}\n");
+	    fz_fprintf(fs, "}\n");
 	    break;
 	  }
 
 	  if(icol>0)
-	    fprintf(fs, ",");
+	    fz_fprintf(fs, ",");
 	  if(pentry->svalue) 
-	    fprintf(fs, "\"%s\"", moutstr(pentry->svalue));
+	    fz_fprintf(fs, "\"%s\"", moutstr(pentry->svalue));
 	  else
-	    fprintf(fs, "%d", pentry->ivalue);
+	    fz_fprintf(fs, "%d", pentry->ivalue);
 	  
 	  ITERATOR_NEXT(ent_iter);
 	  ITERATOR_NEXT(col_iter);
 	  
 	  icol++;
 	  if(icol==ncol) {
-	    fprintf(fs, "\n");
+	    fz_fprintf(fs, "\n");
 	    irow++;
 	    icol = 0;
 	    col_iter = save_iter;
@@ -644,17 +675,25 @@ int section_file_save(struct section_file *my_section_file, const char *filename
       if(!pentry) break;
 
       if(pentry->svalue)
-	fprintf(fs, "%s=\"%s\"\n", pentry->name, moutstr(pentry->svalue));
+	fz_fprintf(fs, "%s=\"%s\"\n", pentry->name, moutstr(pentry->svalue));
       else
-	fprintf(fs, "%s=%d\n", pentry->name, pentry->ivalue);
+	fz_fprintf(fs, "%s=%d\n", pentry->name, pentry->ivalue);
     }
   }
   section_list_iterate_end;
   
   moutstr(NULL);		/* free internal buffer */
 
-  if(ferror(fs) || fclose(fs) == EOF)
+  if (fz_ferror(fs)) {
+    freelog(LOG_NORMAL, "Error before closing %s: %s", filename,
+	    fz_strerror(fs));
+    fz_fclose(fs);
     return 0;
+  }
+  if (fz_fclose(fs) != 0) {
+    freelog(LOG_NORMAL, "Error closing %s", filename);
+    return 0;
+  }
 
   return 1;
 }
@@ -872,7 +911,6 @@ section_file_lookup_internal(struct section_file *my_section_file,
   char ent_name[MAX_LEN_BUFFER];
   char mod_fullpath[2*MAX_LEN_BUFFER];
   int len;
-  struct hash_entry *hentry;
   struct entry *result;
 
   /* freelog(LOG_DEBUG, "looking up: %s", fullpath); */
@@ -887,14 +925,11 @@ section_file_lookup_internal(struct section_file *my_section_file,
   }
   
   if (secfilehash_hashash(my_section_file)) {
-    hentry = secfilehash_lookup(my_section_file, fullpath, NULL);
-    if (hentry->data) {
-      result = hentry->data;
+    result = hash_lookup_data(my_section_file->hashd->htbl, fullpath);
+    if (result) {
       result->used++;
-      return result;
-    } else {
-      return 0;
     }
+    return result;
   }
 
   if(!(pdelim=strchr(fullpath, '.'))) /* i dont like strtok */
@@ -984,37 +1019,12 @@ section_file_insert_internal(struct section_file *my_section_file,
 }
 
 
-/**************************************************************************
-  Anyone know a good general string->hash function?
-**************************************************************************/
-/* plenty of primes, not too small, in random order: */
-static unsigned int coeff[] = { 113, 59, 23, 73, 31, 79, 97, 103, 67, 109,
-				71, 89, 53, 37, 101, 41, 29, 43, 13, 61,
-				107, 47, 83, 17 };
-#define NCOEFF (sizeof(coeff)/sizeof(coeff[0]))
-static int hashfunc(char *name, int num_buckets)
-{
-  unsigned int result=0;
-  int i;
-
-  /* if(name[0]=='p') name+=6; */  /* no longer needed --dwp */
-  for(i=0; *name; i++, name++) {
-    if (i==NCOEFF) {
-      i = 0;
-    }
-    result *= (unsigned int)5;
-    result += coeff[i] * (unsigned int)(*name);
-  }
-  return (result % (unsigned int)num_buckets);
-}
-
 /**************************************************************************
  Return 0 if the section_file has not been setup for hashing.
 **************************************************************************/
 int secfilehash_hashash(struct section_file *file)
 {
-  return (file->hashd!=NULL && file->hashd->table!=NULL
-	  && file->hashd->num_buckets!=0);
+  return (file->hashd!=NULL && hash_num_buckets(file->hashd->htbl)!=0);
 }
 
 /**************************************************************************
@@ -1035,71 +1045,32 @@ static void secfilehash_check(struct section_file *file)
 }
 
 /**************************************************************************
-  Return pointer to entry in hash table where key resides, or
-  where it should go if its to be a new key.
-  Insert the hash value in parameter (*hash_return) unless hash_return==NULL
-**************************************************************************/
-static struct hash_entry *secfilehash_lookup(struct section_file *file,
-					     char *key, int *hash_return)
-{
-  int hash_val, i;
-  struct hash_entry *hentry;
-
-  secfilehash_check(file);
-  i = hash_val = hashfunc(key, file->hashd->num_buckets);
-  if (hash_return) {
-    *hash_return = hash_val;
-  }
-  do {
-    hentry = &(file->hashd->table[i]);
-    if (hentry->key_val == NULL) {	/* empty position in table */
-      return hentry;
-    }
-    if (hentry->hash_val==hash_val &&
-	strcmp(hentry->key_val, key)==0) { /* match */
-      return hentry;
-    }
-    file->hashd->num_collisions++;
-    if (HASH_DEBUG>=2) {
-      freelog(LOG_DEBUG, "Hash collision for \"%s\", %d\n   with \"%s\", %d",
-	   key, hash_val, hentry->key_val, hentry->hash_val);
-    }
-    i++;
-    if (i==file->hashd->num_buckets) {
-      i=0;
-    }
-  } while (i!=hash_val);	/* catch loop all the way round  */
-  freelog(LOG_FATAL, "Full hash table?? (sectionfile %s)",
-	  secfile_filename(file));
-  exit(1);
-}
-
-
-/**************************************************************************
- Insert a entry into the hash table.
- Note the hash table key_val is malloced here.
+ Insert a entry into the hash table.  The key is malloced here (using sbuf;
+ malloc somewhere required by hash implementation).
 **************************************************************************/
 static void secfilehash_insert(struct section_file *file,
 			       char *key, struct entry *data)
 {
-  struct hash_entry *hentry;
-  int hash_val;
+  struct entry *hentry;
 
-  hentry = secfilehash_lookup(file, key, &hash_val);
-  if (hentry->key_val != NULL) {
+  key = sbuf_strdup(file->sb, key);
+  hentry = hash_replace(file->hashd->htbl, key, data);
+  if (hentry) {
     if (file->hashd->allow_duplicates) {
+      hentry->used = 1;
       file->hashd->num_duplicates++;
-      /* Fall-through, to replace entry; could 'return' here and
-	 then first entry would be used and following ones ignored. */
+      /* Subsequent entries replace earlier ones; could do differently so
+	 that first entry would be used and following ones ignored.
+	 Need to mark the replaced one as used or else it will show
+	 up when we iterate the sections and entries (since hash
+	 lookup will never find it to mark it as used).
+      */
     } else {
       freelog(LOG_FATAL, "Tried to insert same value twice: %s (sectionfile %s)",
 	      key, secfile_filename(file));
       exit(1);
     }
   }
-  hentry->data = data;
-  hentry->key_val = sbuf_strdup(file->sb, key);
-  hentry->hash_val = hash_val;
 }
 
 /**************************************************************************
@@ -1113,35 +1084,15 @@ void secfilehash_build(struct section_file *file, int allow_duplicates)
 {
   struct hash_data *hashd;
   char buf[256];
-  int i;
-  unsigned int uent, pow2;
 
   hashd = file->hashd = fc_malloc(sizeof(struct hash_data));
-
-  /* Power of two which is larger than num_entries, then
-     extra factor of 2 for breathing space: */
-  uent = file->num_entries;
-  pow2 = 2;
-  while (uent) {
-    uent >>= 1;
-    pow2 <<= 1;
-  };
-  if (pow2<128) pow2 = 128;
+  hashd->htbl = hash_new_nentries(hash_fval_string, hash_fcmp_string,
+				  file->num_entries);
   
-  hashd->num_buckets = pow2;
   hashd->num_entries_hashbuild = file->num_entries;
-  hashd->num_collisions = 0;
   hashd->allow_duplicates = allow_duplicates;
   hashd->num_duplicates = 0;
   
-  hashd->table = fc_malloc(hashd->num_buckets*sizeof(struct hash_entry));
-
-  for(i=0; i<hashd->num_buckets; i++) {
-    hashd->table[i].data = NULL;
-    hashd->table[i].key_val = NULL;
-    hashd->table[i].hash_val = -1;
-  }
-
   section_list_iterate(*file->sections, psection) {
     entry_list_iterate(psection->entries, pentry) {
       my_snprintf(buf, sizeof(buf), "%s.%s", psection->name, pentry->name);
@@ -1151,14 +1102,9 @@ void secfilehash_build(struct section_file *file, int allow_duplicates)
   }
   section_list_iterate_end;
   
-  if (HASH_DEBUG>=1) {
-    freelog(LOG_DEBUG, "Hash collisions during build: "
-	    "%d (%d entries, %d buckets)",
-	    hashd->num_collisions, file->num_entries, hashd->num_buckets );
-    if (hashd->allow_duplicates) {
-      freelog(LOG_DEBUG, "Hash duplicates during build: %d",
-	      hashd->num_duplicates);
-    }
+  if (hashd->allow_duplicates) {
+    freelog(LOG_DEBUG, "Hash duplicates during build: %d",
+	    hashd->num_duplicates);
   }
 }
 
@@ -1169,7 +1115,7 @@ void secfilehash_build(struct section_file *file, int allow_duplicates)
 void secfilehash_free(struct section_file *file)
 {
   secfilehash_check(file);
-  free(file->hashd->table);
+  hash_free(file->hashd->htbl);
   free(file->hashd);
 }
 
