@@ -30,8 +30,10 @@ struct section_file {
 void section_file_init(struct section_file *file);
 int section_file_load(struct section_file *my_section_file,
 		      const char *filename);
+int section_file_load_nodup(struct section_file *my_section_file,
+			    const char *filename);
 int section_file_save(struct section_file *my_section_file,
-		      const char *filename);
+		      const char *filename, int compression_level);
 void section_file_free(struct section_file *file);
 void section_file_check_unused(struct section_file *file,
 			       const char *filename);
