@@ -14,10 +14,10 @@
 #include <config.h>
 #endif
 
+#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>		/* free */
 #include <string.h>
-#include <assert.h>
 
 #include "astring.h"
 #include "fcintl.h"
@@ -50,9 +50,9 @@ static char *move_type_names[] = {
 static char *flag_names[] = {
   "Caravan", "Missile", "IgZOC", "NonMil", "IgTer", "Carrier",
   "OneAttack", "Pikemen", "Horse", "IgWall", "FieldUnit", "AEGIS",
-  "Fighter", "Marines", "Submarine", "Settlers", "Diplomat",
+  "Fighter", "Marines", "Partial_Invis", "Settlers", "Diplomat",
   "Trireme", "Nuclear", "Spy", "Transform", "Paratroopers",
-  "Airbase", "Cities"
+  "Airbase", "Cities", "IgTired", "Missile_Carrier", "No_Land_Attack"
 };
 static char *role_names[] = {
   "FirstBuild", "Explorer", "Hut", "HutTech", "Partisan",
@@ -61,6 +61,14 @@ static char *role_names[] = {
   "BarbarianBuild", "BarbarianBuildTech", "BarbarianLeader",
   "BarbarianSea", "BarbarianSeaTech"
 };
+static char *unit_class_names[] = {
+  "Air",
+  "Helicopter",
+  "Land",
+  "Missile",
+  "Nuclear",
+  "Sea"
+};
 
 /***************************************************************
 ...
@@ -83,21 +91,6 @@ char *get_unit_name(Unit_Type_id id)
   return buffer;
 }
 
-/***************************************************************
-...
-***************************************************************/
-struct unit *find_unit_by_id(int id)
-{
-  int i;
-
-  for(i=0; i<game.nplayers; i++) {
-    struct unit *punit;
-    if((punit=unit_list_find(&game.players[i].units, id)))
-      return punit;
-  }
-  return 0;
-}
-
 /***************************************************************
 ...
 ***************************************************************/
@@ -162,45 +155,58 @@ int is_diplomat_action_available(struct unit *pdiplomat,
 				 int destx, int desty)
 {
   struct city *pcity=map_get_city(destx, desty);
+  int playerid = pdiplomat->owner;
 
   if(pcity) {  
-    if(pcity->owner!=pdiplomat->owner) {
+    if(pcity->owner!=pdiplomat->owner &&
+       real_map_distance(pdiplomat->x, pdiplomat->y, pcity->x, pcity->y) <= 1) {
       if(action==DIPLOMAT_SABOTAGE)
-        return 1;
+        return players_at_war(playerid, pcity->owner);
       if(action==DIPLOMAT_EMBASSY &&
+	 !is_barbarian(&game.players[pcity->owner]) &&
 	 !player_has_embassy(&game.players[pdiplomat->owner], 
 			     &game.players[pcity->owner]))
 	return 1;
       if(action==SPY_POISON &&
 	 pcity->size>1 &&
 	 unit_flag(pdiplomat->type, F_SPY))
-        return 1;
+        return players_at_war(playerid, pcity->owner);
       if(action==DIPLOMAT_INVESTIGATE)
         return 1;
-      if(action==DIPLOMAT_STEAL)
+      if(action==DIPLOMAT_STEAL && !is_barbarian(&game.players[pcity->owner]))
         return 1;
       if(action==DIPLOMAT_INCITE)
-        return 1;
+        return !players_allied(pcity->owner, pdiplomat->owner);
       if(action==DIPLOMAT_ANY_ACTION)
         return 1;
+      if (action==SPY_GET_SABOTAGE_LIST && unit_flag(pdiplomat->type, F_SPY))
+	return players_at_war(playerid, pcity->owner);
+    }
+  } else { /* Action against a unit at a tile */
+    /* If it is made possible to do action against allied units
+       handle_unit_move_request() should be changed so that pdefender
+       is also set to allied units */
+    struct tile *ptile = map_get_tile(destx, desty);
+    struct unit *punit;
+
+    if ((action==SPY_SABOTAGE_UNIT || action==DIPLOMAT_ANY_ACTION) &&
+	unit_list_size(&ptile->units)==1 &&
+	unit_flag(pdiplomat->type, F_SPY)) {
+      punit = unit_list_get(&ptile->units, 0);
+      return players_at_war(playerid, punit->owner);
+    }
+
+    if ((action==DIPLOMAT_BRIBE || action==DIPLOMAT_ANY_ACTION) &&
+	unit_list_size(&ptile->units)==1) {
+      punit = unit_list_get(&ptile->units, 0);
+      return !players_allied(punit->owner, pdiplomat->owner);
     }
-  } else {
-    struct tile *ptile=map_get_tile(destx, desty);
-    if((action==SPY_SABOTAGE_UNIT || action==DIPLOMAT_ANY_ACTION) &&
-       unit_list_size(&ptile->units)==1 &&
-       unit_list_get(&ptile->units, 0)->owner!=pdiplomat->owner &&
-       unit_flag(pdiplomat->type, F_SPY))
-      return 1;
-    if((action==DIPLOMAT_BRIBE || action==DIPLOMAT_ANY_ACTION) &&
-       unit_list_size(&ptile->units)==1 &&
-       unit_list_get(&ptile->units, 0)->owner!=pdiplomat->owner)
-      return 1;
   }
   return 0;
 }
 
 /**************************************************************************
-...
+FIXME: Maybe we should allow airlifts between allies
 **************************************************************************/
 int unit_can_airlift_to(struct unit *punit, struct city *pcity)
 {
@@ -270,152 +276,25 @@ int unit_can_defend_here(struct unit *punit)
 }
 
 /**************************************************************************
-...
+Returns the number of free spaces for ground units. Can be 0 or negative.
 **************************************************************************/
-int is_transporter_with_free_space(struct player *pplayer, int x, int y)
+int ground_unit_transporter_capacity(int x, int y, int playerid)
 {
-  int none_transporters=0, total_capacity=0;
+  int availability = 0;
+  struct tile *ptile = map_get_tile(x, y);
 
   unit_list_iterate(map_get_tile(x, y)->units, punit) {
-    if(is_ground_units_transport(punit))
-      total_capacity+=get_transporter_capacity(punit);
-    else if (is_ground_unit(punit))
-      none_transporters++;
-  }
-  unit_list_iterate_end;
-  
-  return(total_capacity>none_transporters ? total_capacity-none_transporters : 0);
-}
-
-/**************************************************************************
-The above one isn't quite what I want - Kris
-Specifically: only pplayer's transports count; only returns true/false;
-and returns true if (capacity == passengers).  --dwp
-**************************************************************************/
-int is_enough_transporter_space (struct player *pplayer, int x, int y)
-{
-  int none_transporters=0, total_capacity=0;
-  
-  unit_list_iterate(map_get_tile(x, y)->units, punit) {
-    if(is_ground_units_transport(punit)
-       && punit->owner == pplayer->player_no)
-      total_capacity+=get_transporter_capacity(punit);
-    else if (is_ground_unit(punit))
-      none_transporters++;
-  }
-  unit_list_iterate_end;
-  
-  return(total_capacity>=none_transporters ? 1 : 0);
-}
-
-
-/**************************************************************************
- can't use the unit_list_iterate macro here
-**************************************************************************/
-void transporter_cargo_to_unitlist(struct unit *ptran, struct unit_list *list)
-{
-  struct genlist_iterator myiter;
-  struct unit_list *srclist;
-  int cargo;
-  
-  unit_list_init(list);
-    
-  srclist=&map_get_tile(ptran->x, ptran->y)->units;
-  
-  genlist_iterator_init(&myiter, &srclist->list, 0);
-  
-  cargo=get_transporter_capacity(ptran);
-  
-  for(; cargo && ITERATOR_PTR(myiter);) {
-    struct unit *punit=(struct unit *)ITERATOR_PTR(myiter);
-    ITERATOR_NEXT(myiter);
-    if(((is_ground_unit(punit) && is_ground_units_transport(ptran))
-	|| (is_air_unit(punit) && unit_flag(ptran->type, F_CARRIER))
-	|| (unit_flag(punit->type, F_MISSILE)
-	    && unit_flag(ptran->type, F_SUBMARINE)))
-       && (!map_get_city(punit->x, punit->y)
-	   || punit->activity==ACTIVITY_SENTRY)) {
-      unit_list_unlink(srclist, punit);
-      unit_list_insert(list, punit);
-      cargo--;
-    }
-  }
-}
-
-/**************************************************************************
- Get the _minimum_ cargo list, taking into account cities, other boats.
- Eg, for when unit is disbanded, or supporting city captured.
- Note: for Carriers and Subs the minimum list is defined as empty:
- the planes/missiles will still be in the air and have a chance to
- get to safety.
- Note: this modifies the map units list, so iterators beware.
-**************************************************************************/
-void transporter_min_cargo_to_unitlist(struct unit *ptran,
-				       struct unit_list *list)
-{
-  int x = ptran->x;
-  int y = ptran->y;
-  int this_capacity = get_transporter_capacity(ptran);
-  struct unit_list *srclist;
-  int tile_capacity, tile_ncargo, nlost;
-  
-  unit_list_init(list);
-
-  if (!is_sailing_unit(ptran)
-      || !is_ground_units_transport(ptran)
-      || (map_get_terrain(x, y) != T_OCEAN) /* includes cities */ ) {
-    return;
-  }
-
-  srclist = &map_get_tile(x,y)->units;
-
-  /* this iteration is non-destructive; just counting: */
-  tile_capacity = 0;
-  tile_ncargo = 0;
-  unit_list_iterate((*srclist), punit) {
-    if (is_sailing_unit(punit)
-        && !is_ground_units_transport(ptran)) {
-      tile_capacity += get_transporter_capacity(punit);
-    } else if (is_ground_unit(punit)) {
-      tile_ncargo++;
+    if (punit->owner == playerid) {
+      if (is_ground_units_transport(punit)
+	  && !(is_ground_unit(punit) && ptile->terrain == T_OCEAN))
+	availability += get_transporter_capacity(punit);
+      else if (is_ground_unit(punit))
+	availability--;
     }
   }
   unit_list_iterate_end;
 
-  tile_capacity -= this_capacity;
-  nlost = tile_ncargo - tile_capacity;
-
-  if (nlost <= 0) {
-    return;
-  }
-
-  /* actually, unit_list_iterate _is_ safe for this */
-  unit_list_iterate((*srclist), punit) {
-    if(is_ground_unit(punit)) {
-      unit_list_unlink(srclist, punit);
-      unit_list_insert(list, punit);
-      nlost--;
-      if (nlost==0) break;
-    }
-  }
-  unit_list_iterate_end;
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void move_unit_list_to_tile(struct unit_list *units, int x, int y)
-{
-  struct genlist_iterator myiter;
-  
-  genlist_iterator_init(&myiter, &units->list, 0);
-
-  for(; ITERATOR_PTR(myiter); ITERATOR_NEXT(myiter)) {
-    struct unit *punit=(struct unit *)ITERATOR_PTR(myiter);
-    punit->x=x;
-    punit->y=y;
-    unit_list_insert_back(&map_get_tile(x, y)->units, punit);
-  }
+  return availability;
 }
 
 /**************************************************************************
@@ -431,9 +310,19 @@ int get_transporter_capacity(struct unit *punit)
 **************************************************************************/
 int is_ground_units_transport(struct unit *punit)
 {
-return (get_transporter_capacity(punit)
-	&& !unit_flag(punit->type, F_SUBMARINE)
-	&& !unit_flag(punit->type, F_CARRIER));
+  return (get_transporter_capacity(punit)
+	  && !unit_flag(punit->type, F_MISSILE_CARRIER)
+	  && !unit_flag(punit->type, F_CARRIER));
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+int is_air_units_transport(struct unit *punit)
+{
+  return (get_transporter_capacity(punit)
+	  && (unit_flag(punit->type, F_MISSILE_CARRIER)
+	      || unit_flag(punit->type, F_CARRIER)));
 }
 
 /**************************************************************************
@@ -502,9 +391,17 @@ int is_water_unit(Unit_Type_id id)
 /**************************************************************************
 ...
 **************************************************************************/
-int is_military_unit(struct unit *this_unit)
+int is_military_unit(struct unit *punit)
 {
-  return (unit_flag(this_unit->type, F_NONMIL) == 0);
+  return (unit_flag(punit->type, F_NONMIL) == 0);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+int is_diplomat_unit(struct unit *punit)
+{
+  return (unit_flag(punit->type, F_DIPLOMAT));
 }
 
 /**************************************************************************
@@ -544,7 +441,7 @@ int utype_gold_cost(struct unit_type *ut, struct government *g)
 **************************************************************************/
 int is_ground_threat(struct player *pplayer, struct unit *punit)
 {
-  return ((pplayer->player_no != punit->owner)
+  return (players_at_war(pplayer->player_no, punit->owner)
 	  && (unit_flag(punit->type, F_DIPLOMAT)
 	      || (is_ground_unit(punit)
 		  && is_military_unit(punit))));
@@ -582,14 +479,19 @@ int is_field_unit(struct unit *punit)
 /**************************************************************************
   Is the unit one that is invisible on the map, which is currently limited
   to subs and missiles in subs.
+  FIXME: this should be made more general: does not handle cargo units
+  on an invisible transport, or planes on invisible carrier.
 **************************************************************************/
 int is_hiding_unit(struct unit *punit)
 {
-  if(unit_flag(punit->type, F_SUBMARINE)) return 1;
+  if(unit_flag(punit->type, F_PARTIAL_INVIS)) return 1;
   if(unit_flag(punit->type, F_MISSILE)) {
     if(map_get_terrain(punit->x, punit->y)==T_OCEAN) {
       unit_list_iterate(map_get_tile(punit->x, punit->y)->units, punit2) {
-	if(unit_flag(punit2->type, F_SUBMARINE)) return 1;
+	if(unit_flag(punit2->type, F_PARTIAL_INVIS)
+	   && unit_flag(punit2->type, F_MISSILE_CARRIER)) {
+	  return 1;
+	}
       } unit_list_iterate_end;
     }
   }
@@ -632,6 +534,21 @@ char *unit_name(Unit_Type_id id)
   return (unit_types[id].name);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+char *unit_class_name(Unit_Class_id id)
+{
+  if ((id >= 0) && (id < UCL_LAST)) {
+    return unit_class_names[id];
+  } else {
+    return "";
+  }
+}
+
+/**************************************************************************
+...
+**************************************************************************/
 struct unit_type *get_unit_type(Unit_Type_id id)
 {
   return &unit_types[id];
@@ -684,48 +601,25 @@ char *get_units_with_flag_string(int flag)
   return NULL;
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-void raise_unit_top(struct unit *punit)
-{
-  struct tile *ptile;
-
-  ptile=map_get_tile(punit->x, punit->y);
-
-  unit_list_unlink(&ptile->units, punit);
-  unit_list_insert(&ptile->units, punit);
-
-  unit_list_get(&ptile->units, 0);
-}
-
 /**************************************************************************
 ...
 **************************************************************************/
 int can_unit_build_city(struct unit *punit)
 {
-  int x, y;
-
   if(!unit_flag(punit->type, F_CITIES))
     return 0;
-    
-  if(map_get_terrain(punit->x, punit->y)==T_OCEAN)
-    return 0;
 
   if(!punit->moves_left)
     return 0;
 
-  if(game.civstyle==1) {
-    if(map_get_city(punit->x, punit->y))
-      return 0;
-  } else {
-    for(x=-1; x<=1; x++)
-      for(y=-1; y<=1; y++)
-	if(map_get_city(punit->x + x, punit->y + y))
-	  return 0;
-  }
+  return city_can_be_built_here(punit->x, punit->y);
+}
 
-  return 1;
+/**************************************************************************
+...
+**************************************************************************/
+int kills_citizen_after_attack(struct unit *punit) {
+  return (game.killcitizen >> ((int)unit_types[punit->type].move_type-1)) & 1;
 }
 
 /**************************************************************************
@@ -746,6 +640,8 @@ int can_unit_add_to_city(struct unit *punit)
     return 0;
   if(pcity->size > game.add_to_size_limit)
     return 0;
+  if(pcity->owner != punit->owner)
+    return 0;
 
   if(improvement_exists(B_AQUEDUCT)
      && !city_got_building(pcity, B_AQUEDUCT) 
@@ -774,21 +670,20 @@ int can_unit_change_homecity(struct unit *punit)
 **************************************************************************/
 int can_upgrade_unittype(struct player *pplayer, Unit_Type_id id)
 {
-  if (get_invention(pplayer, 
-		    unit_types[id].tech_requirement)!=TECH_KNOWN)
+  Unit_Type_id best_upgrade = -1;
+
+  if (!can_player_build_unit_direct(pplayer, id))
     return -1;
-  if (unit_types[id].obsoleted_by == -1)
-    return -1;
-  if (get_invention(pplayer,
-		    unit_types[unit_types[id].obsoleted_by].tech_requirement)!=TECH_KNOWN) 
-    return -1;
-  while (unit_types[id].obsoleted_by!=-1 && get_invention(pplayer, 
-		       unit_types[unit_types[id].obsoleted_by].tech_requirement)==TECH_KNOWN) {
-    id = unit_types[id].obsoleted_by;
-  }
-  return id;
+  while (unit_type_exists(id = unit_types[id].obsoleted_by))
+    if (can_player_build_unit_direct(pplayer, id))
+      best_upgrade = id;
+
+  return best_upgrade;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 int unit_upgrade_price(struct player *pplayer, Unit_Type_id from, Unit_Type_id to)
 {
   int total, build;
@@ -840,27 +735,28 @@ int can_unit_do_connect (struct unit *punit, enum unit_activity activity)
 /**************************************************************************
 Return name of activity in static buffer
 **************************************************************************/
-
 char* get_activity_text (int activity)
 {
   char *text;
 
   switch (activity) {
-  case ACTIVITY_IDLE:      text = _("Idle"); break;
-  case ACTIVITY_POLLUTION: text = _("Pollution"); break;
-  case ACTIVITY_ROAD:      text = _("Road"); break;
-  case ACTIVITY_MINE:      text = _("Mine"); break;
-  case ACTIVITY_IRRIGATE:  text = _("Irrigation"); break;
-  case ACTIVITY_FORTIFY:   text = _("Fortify"); break;
-  case ACTIVITY_FORTRESS:  text = _("Fortress"); break;
-  case ACTIVITY_SENTRY:    text = _("Sentry"); break;
-  case ACTIVITY_RAILROAD:  text = _("Railroad"); break;
-  case ACTIVITY_PILLAGE:   text = _("Pillage"); break;
-  case ACTIVITY_GOTO:      text = _("Goto"); break;
-  case ACTIVITY_EXPLORE:   text = _("Explore"); break;
-  case ACTIVITY_TRANSFORM: text = _("Transform"); break;
-  case ACTIVITY_AIRBASE:   text = _("Airbase"); break;
-  default:                 text = _("Unknown"); break;
+  case ACTIVITY_IDLE:		text = _("Idle"); break;
+  case ACTIVITY_POLLUTION:	text = _("Pollution"); break;
+  case ACTIVITY_ROAD:		text = _("Road"); break;
+  case ACTIVITY_MINE:		text = _("Mine"); break;
+  case ACTIVITY_IRRIGATE:	text = _("Irrigation"); break;
+  case ACTIVITY_FORTIFYING:	text = _("Fortifying"); break;
+  case ACTIVITY_FORTIFIED:	text = _("Fortified"); break;
+  case ACTIVITY_FORTRESS:	text = _("Fortress"); break;
+  case ACTIVITY_SENTRY:		text = _("Sentry"); break;
+  case ACTIVITY_RAILROAD:	text = _("Railroad"); break;
+  case ACTIVITY_PILLAGE:	text = _("Pillage"); break;
+  case ACTIVITY_GOTO:		text = _("Goto"); break;
+  case ACTIVITY_EXPLORE:	text = _("Explore"); break;
+  case ACTIVITY_TRANSFORM:	text = _("Transform"); break;
+  case ACTIVITY_AIRBASE:	text = _("Airbase"); break;
+  case ACTIVITY_FALLOUT:	text = _("Fallout"); break;
+  default:			text = _("Unknown"); break;
   }
 
   return text;
@@ -872,7 +768,7 @@ That is if the unit is in a friendly city or on an Airbase
 special, have enough movepoints left and have not paradropped
 before in this turn.
 **************************************************************************/
-int can_unit_paradropped(struct unit *punit)
+int can_unit_paradrop(struct unit *punit)
 {
   struct city *pcity;
   struct unit_type *utype;
@@ -899,6 +795,66 @@ int can_unit_paradropped(struct unit *punit)
   return 1;
 }
 
+/**************************************************************************
+This function returns true if the tile at the given location can be
+"reclaimed" from ocean into land.  This is the case only when there are
+a sufficient number of adjacent tiles that are not ocean.
+**************************************************************************/
+static int can_reclaim_ocean(int x, int y)
+{
+  int i, j, landtiles;
+
+  if (terrain_control.ocean_reclaim_requirement >= 9)
+    return 0;
+  if (terrain_control.ocean_reclaim_requirement <= 0)
+    return 1;
+
+  landtiles = 0;
+  for (i = -1; i <= 1; i++) {
+    for (j = -1; j <= 1; j++) {
+      if (i || j) {
+	if (map_get_tile(x+i, y+j)->terrain != T_OCEAN) {
+	  landtiles++;
+	}
+	if (landtiles >= terrain_control.ocean_reclaim_requirement)
+	  return 1;
+      }
+    }
+  }
+
+  return 0;
+}
+
+/**************************************************************************
+This function returns true if the tile at the given location can be
+"channeled" from land into ocean.  This is the case only when there are
+a sufficient number of adjacent tiles that are ocean.
+**************************************************************************/
+static int can_channel_land(int x, int y)
+{
+  int i, j, oceantiles;
+
+  if (terrain_control.land_channel_requirement >= 9)
+    return 0;
+  if (terrain_control.land_channel_requirement <= 0)
+    return 1;
+
+  oceantiles = 0;
+  for (i = -1; i <= 1; i++) {
+    for (j = -1; j <= 1; j++) {
+      if (i || j) {
+	if (map_get_tile(x+i, y+j)->terrain == T_OCEAN) {
+	  oceantiles++;
+	}
+	if (oceantiles >= terrain_control.land_channel_requirement)
+	  return 1;
+      }
+    }
+  }
+
+  return 0;
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -908,46 +864,54 @@ int can_unit_do_activity(struct unit *punit, enum unit_activity activity)
 }
 
 /**************************************************************************
-...
+Note that if you make changes here you should also change the code for
+autosettlers in server/settler.c. The code there does not use this function
+as it would be a ajor CPU hog.
 **************************************************************************/
 int can_unit_do_activity_targeted(struct unit *punit,
 				  enum unit_activity activity, int target)
 {
+  struct player *pplayer;
   struct tile *ptile;
   struct tile_type *type;
-  struct player *pplayer;
+
   pplayer = &game.players[punit->owner];
-  
-  ptile=map_get_tile(punit->x, punit->y);
-  type=get_tile_type(ptile->terrain);
-  
-  if(activity==ACTIVITY_IDLE)
+  ptile = map_get_tile(punit->x, punit->y);
+  type = get_tile_type(ptile->terrain);
+
+  switch(activity) {
+  case ACTIVITY_IDLE:
+  case ACTIVITY_GOTO:
     return 1;
 
-  if (activity == ACTIVITY_EXPLORE) /* added 980803 by Syela */
-    return (is_ground_unit(punit) || is_sailing_unit(punit));
-
-  /* can't go directly from non-idle to non-idle activity */
-  if(punit->activity!=ACTIVITY_IDLE && punit->activity != activity) return 0;
-
-  switch(activity)  {
   case ACTIVITY_POLLUTION:
-    return unit_flag(punit->type, F_SETTLERS) && punit->moves_left &&
-           (ptile->special&S_POLLUTION);
+    return unit_flag(punit->type, F_SETTLERS) && (ptile->special&S_POLLUTION);
+
+  case ACTIVITY_FALLOUT:
+    return unit_flag(punit->type, F_SETTLERS) && (ptile->special&S_FALLOUT);
 
   case ACTIVITY_ROAD:
-    return terrain_control.may_road &&
-           unit_flag(punit->type, F_SETTLERS) && punit->moves_left &&
-           !(ptile->special&S_ROAD) && ptile->terrain!=T_OCEAN &&
-	   ((ptile->terrain!=T_RIVER && !(ptile->special&S_RIVER)) || 
-	    player_knows_techs_with_flag(pplayer, TF_BRIDGE));
+    return (terrain_control.may_road &&
+	    unit_flag(punit->type, F_SETTLERS) &&
+	    !(ptile->special&S_ROAD) && type->road_time &&
+	    ((ptile->terrain!=T_RIVER && !(ptile->special&S_RIVER)) || 
+	     player_knows_techs_with_flag(pplayer, TF_BRIDGE)));
 
   case ACTIVITY_MINE:
     /* Don't allow it if someone else is irrigating this tile.
      * *Do* allow it if they're transforming - the mine may survive */
     if (terrain_control.may_mine &&
-	unit_flag(punit->type, F_SETTLERS) && punit->moves_left &&
-	type->mining_result!=T_LAST && !(ptile->special&S_MINE)) {
+	unit_flag(punit->type, F_SETTLERS) &&
+	( (ptile->terrain==type->mining_result && 
+	   !(ptile->special&S_MINE)) ||
+	  (ptile->terrain!=type->mining_result &&
+	   type->mining_result!=T_LAST &&
+	   (ptile->terrain!=T_OCEAN || type->mining_result==T_OCEAN ||
+	    can_reclaim_ocean(punit->x, punit->y)) &&
+	   (ptile->terrain==T_OCEAN || type->mining_result!=T_OCEAN ||
+	    can_channel_land(punit->x, punit->y)) &&
+	   (type->mining_result!=T_OCEAN ||
+	    !(map_get_city(punit->x, punit->y)))) )) {
       unit_list_iterate(ptile->units, tunit) {
 	if(tunit->activity==ACTIVITY_IRRIGATE) return 0;
       }
@@ -959,14 +923,20 @@ int can_unit_do_activity_targeted(struct unit *punit,
     /* Don't allow it if someone else is mining this tile.
      * *Do* allow it if they're transforming - the irrigation may survive */
     if (terrain_control.may_irrigate &&
-	unit_flag(punit->type, F_SETTLERS) && punit->moves_left &&
+	unit_flag(punit->type, F_SETTLERS) &&
 	(!(ptile->special&S_IRRIGATION) ||
 	 (!(ptile->special&S_FARMLAND) &&
 	  player_knows_techs_with_flag(pplayer, TF_FARMLAND))) &&
 	( (ptile->terrain==type->irrigation_result && 
 	   is_water_adjacent_to_tile(punit->x, punit->y)) ||
 	  (ptile->terrain!=type->irrigation_result &&
-	   type->irrigation_result!=T_LAST))) {
+	   type->irrigation_result!=T_LAST &&
+	   (ptile->terrain!=T_OCEAN || type->irrigation_result==T_OCEAN ||
+	    can_reclaim_ocean(punit->x, punit->y)) &&
+	   (ptile->terrain==T_OCEAN || type->irrigation_result!=T_OCEAN ||
+	    can_channel_land(punit->x, punit->y)) &&
+	   (type->irrigation_result!=T_OCEAN ||
+	    !(map_get_city(punit->x, punit->y)))) )) {
       unit_list_iterate(ptile->units, tunit) {
 	if(tunit->activity==ACTIVITY_MINE) return 0;
       }
@@ -974,43 +944,51 @@ int can_unit_do_activity_targeted(struct unit *punit,
       return 1;
     } else return 0;
 
-  case ACTIVITY_FORTIFY:
-    return is_ground_unit(punit) && punit->moves_left &&
-           !unit_flag(punit->type, F_SETTLERS);
+  case ACTIVITY_FORTIFYING:
+    return (is_ground_unit(punit) &&
+	    (punit->activity != ACTIVITY_FORTIFIED) &&
+	    !unit_flag(punit->type, F_SETTLERS));
+
+  case ACTIVITY_FORTIFIED:
+    return 0;
 
   case ACTIVITY_FORTRESS:
-    return unit_flag(punit->type, F_SETTLERS) && punit->moves_left &&
-           player_knows_techs_with_flag(pplayer, TF_FORTRESS) &&
-	   !(ptile->special&S_FORTRESS) && ptile->terrain!=T_OCEAN;
+    return (unit_flag(punit->type, F_SETTLERS) &&
+	    player_knows_techs_with_flag(pplayer, TF_FORTRESS) &&
+	    !(ptile->special&S_FORTRESS) && ptile->terrain!=T_OCEAN);
 
   case ACTIVITY_AIRBASE:
-    return unit_flag(punit->type, F_AIRBASE) && punit->moves_left &&
-           player_knows_techs_with_flag(pplayer, TF_AIRBASE) &&
-           !(ptile->special&S_AIRBASE) && ptile->terrain!=T_OCEAN;
+    return (unit_flag(punit->type, F_AIRBASE) &&
+	    player_knows_techs_with_flag(pplayer, TF_AIRBASE) &&
+	    !(ptile->special&S_AIRBASE) && ptile->terrain!=T_OCEAN);
 
   case ACTIVITY_SENTRY:
     return 1;
 
   case ACTIVITY_RAILROAD:
     /* if the tile has road, the terrain must be ok.. */
-    return terrain_control.may_road &&
-           unit_flag(punit->type, F_SETTLERS) && punit->moves_left &&
-           ((ptile->special&S_ROAD) || (punit->connecting
-	    && (ptile->terrain!=T_OCEAN &&
-		((ptile->terrain!=T_RIVER && !(ptile->special&S_RIVER))
-		 || player_knows_techs_with_flag(pplayer, TF_BRIDGE)))))
-           && !(ptile->special&S_RAILROAD) &&
-	   player_knows_techs_with_flag(pplayer, TF_RAILROAD);
+    return (terrain_control.may_road &&
+	    unit_flag(punit->type, F_SETTLERS) &&
+	    ((ptile->special&S_ROAD) ||
+	     (punit->connecting &&
+	      (type->road_time &&
+	       ((ptile->terrain!=T_RIVER && !(ptile->special&S_RIVER))
+		|| player_knows_techs_with_flag(pplayer, TF_BRIDGE))))) &&
+	    !(ptile->special&S_RAILROAD) &&
+	    player_knows_techs_with_flag(pplayer, TF_RAILROAD));
 
   case ACTIVITY_PILLAGE:
     {
       int pspresent;
       int psworking;
       pspresent = get_tile_infrastructure_set(ptile);
-      if (pspresent && is_ground_unit(punit) && punit->moves_left) {
+      if (pspresent && is_ground_unit(punit)) {
 	psworking = get_unit_tile_pillage_set(punit->x, punit->y);
 	if (target == S_NO_SPECIAL)
 	  return ((pspresent & (~psworking)) != 0);
+	else if ((!game.rgame.pillage_select) &&
+		 (target != get_preferred_pillage(pspresent)))
+	  return 0;
 	else
 	  return ((pspresent & (~psworking) & target) != 0);
       } else {
@@ -1018,11 +996,20 @@ int can_unit_do_activity_targeted(struct unit *punit,
       }
     }
 
+  case ACTIVITY_EXPLORE:
+    return (is_ground_unit(punit) || is_sailing_unit(punit));
+
   case ACTIVITY_TRANSFORM:
-    return terrain_control.may_transform &&
-           (ptile->terrain != T_OCEAN) &&
-           (ptile->terrain != T_RIVER) &&
-	   unit_flag(punit->type, F_TRANSFORM) && punit->moves_left;
+    return (terrain_control.may_transform &&
+	    (type->transform_result!=T_LAST) &&
+	    (ptile->terrain!=type->transform_result) &&
+	    (ptile->terrain!=T_OCEAN || type->transform_result==T_OCEAN ||
+	     can_reclaim_ocean(punit->x, punit->y)) &&
+	    (ptile->terrain==T_OCEAN || type->transform_result!=T_OCEAN ||
+	     can_channel_land(punit->x, punit->y)) &&
+	    (type->transform_result!=T_OCEAN ||
+	     !(map_get_city(punit->x, punit->y))) &&
+	    unit_flag(punit->type, F_TRANSFORM));
 
   default:
     freelog(LOG_NORMAL,"Unknown activity %d\n",activity);
@@ -1086,7 +1073,7 @@ char *unit_description(struct unit *punit)
   struct city *pcity;
   static char buffer[512];
 
-  pcity=city_list_find_id(&game.player_ptr->cities, punit->homecity);
+  pcity = player_find_city_by_id(game.player_ptr, punit->homecity);
 
   my_snprintf(buffer, sizeof(buffer), "%s%s\n%s\n%s", 
 	  get_unit_type(punit->type)->name, 
@@ -1143,12 +1130,14 @@ char *unit_activity_text(struct unit *punit)
      }
      return text;
    case ACTIVITY_POLLUTION:
+   case ACTIVITY_FALLOUT:
    case ACTIVITY_ROAD:
    case ACTIVITY_RAILROAD:
    case ACTIVITY_MINE: 
    case ACTIVITY_IRRIGATE:
    case ACTIVITY_TRANSFORM:
-   case ACTIVITY_FORTIFY:
+   case ACTIVITY_FORTIFYING:
+   case ACTIVITY_FORTIFIED:
    case ACTIVITY_AIRBASE:
    case ACTIVITY_FORTRESS:
    case ACTIVITY_SENTRY:
@@ -1389,6 +1378,25 @@ enum unit_move_type unit_move_type_from_str(char *s)
   return 0;
 }
 
+/**************************************************************************
+  Convert Unit_Class_id names to enum; case insensitive;
+  returns UCL_LAST if can't match.
+**************************************************************************/
+Unit_Class_id unit_class_from_str(char *s)
+{
+  Unit_Class_id i;
+
+  assert(sizeof(unit_class_names)/sizeof(unit_class_names[0])==UCL_LAST);
+
+  for (i = 0; i < UCL_LAST; i++) {
+    if (mystrcasecmp(unit_class_names[i], s)==0) {
+      return i;
+    }
+  }
+
+  return UCL_LAST;
+}
+
 /**************************************************************************
   Convert flag names to enum; case insensitive;
   returns F_LAST if can't match.
@@ -1424,3 +1432,89 @@ enum unit_role_id unit_role_from_str(char *s)
   }
   return L_LAST;
 }
+
+/**************************************************************************
+...
+**************************************************************************/
+struct player *unit_owner(struct unit *punit)
+{
+  return (&game.players[punit->owner]);
+}
+
+/**************************************************************************
+Returns the number of free spaces for missiles. Can be 0 or negative.
+**************************************************************************/
+int missile_carrier_capacity(int x, int y, int playerid)
+{
+  struct tile *ptile = map_get_tile(x, y);
+  int misonly = 0;
+  int airall = 0;
+  int totalcap;
+
+  unit_list_iterate(map_get_tile(x, y)->units, punit) {
+    if (punit->owner == playerid) {
+      if (unit_flag(punit->type, F_CARRIER)
+	  && !(is_ground_unit(punit) && ptile->terrain == T_OCEAN)) {
+	airall += get_transporter_capacity(punit);
+	continue;
+      }
+      if (unit_flag(punit->type, F_MISSILE_CARRIER)
+	  && !(is_ground_unit(punit) && ptile->terrain == T_OCEAN)) {
+	misonly += get_transporter_capacity(punit);
+	continue;
+      }
+      if (is_air_unit(punit)) {
+	if (unit_flag(punit->type, F_MISSILE))
+	  misonly--;
+	else
+	  airall--;
+      }
+    }
+  }
+  unit_list_iterate_end;
+
+  if (airall < 0)
+    airall = 0;
+
+  totalcap = airall + misonly;
+
+  return totalcap;
+}
+
+/**************************************************************************
+Returns the number of free spaces for airunits (includes missiles).
+Can be 0 or negative.
+**************************************************************************/
+int airunit_carrier_capacity(int x, int y, int playerid)
+{
+  struct tile *ptile = map_get_tile(x, y);
+  int misonly = 0;
+  int airall = 0;
+
+  unit_list_iterate(map_get_tile(x, y)->units, punit) {
+    if (punit->owner == playerid) {
+      if (unit_flag(punit->type, F_CARRIER)
+	  && !(is_ground_unit(punit) && ptile->terrain == T_OCEAN)) {
+	airall += get_transporter_capacity(punit);
+	continue;
+      }
+      if (unit_flag(punit->type, F_MISSILE_CARRIER)
+	  && !(is_ground_unit(punit) && ptile->terrain == T_OCEAN)) {
+	misonly += get_transporter_capacity(punit);
+	continue;
+      }
+      if (is_air_unit(punit)) {
+	if (unit_flag(punit->type, F_MISSILE))
+	  misonly--;
+	else
+	  airall--;
+      }
+    }
+  }
+  unit_list_iterate_end;
+
+  if (misonly < 0)
+    airall += misonly;
+
+  return airall;
+}
