@@ -90,7 +90,8 @@ static struct unit *search_best_target(struct player *pplayer,
       
       targets = &(map_get_tile(x, y)->units);
       if (unit_list_size(targets) == 0) continue;
-      if (unit_list_get(targets, 0)->owner == punit->owner) continue;
+      if (!is_enemy_unit_tile(map_get_tile(x, y), pplayer->player_no))
+	continue;
       
       if (debug) freelog(LOG_DEBUG,  "found enemy unit/stack at %d,%d", x, y);
       enemy = get_defender(pplayer, punit, x, y);
@@ -138,7 +139,7 @@ static struct unit *search_best_target(struct player *pplayer,
 		     pcity->name, unit_name(punit->type),
 		     unit_name(enemy->type));
     punit->ai.control=0;
-    handle_unit_activity_request(pplayer,punit,ACTIVITY_IDLE);
+    handle_unit_activity_request(punit, ACTIVITY_IDLE);
     return NULL;
   }
 
@@ -169,7 +170,7 @@ static void auto_attack_with_unit(struct player *pplayer, struct city *pcity,
   punit->goto_dest_x=enemy->x;
   punit->goto_dest_y=enemy->y;
   
-  send_unit_info(0,punit,0);
+  send_unit_info(0,punit);
   do_unit_goto(pplayer,punit,GOTO_MOVE_ANY);
   
   punit = find_unit_by_id(id);
@@ -178,12 +179,12 @@ static void auto_attack_with_unit(struct player *pplayer, struct city *pcity,
     set_unit_activity(punit, ACTIVITY_GOTO);
     punit->goto_dest_x=pcity->x;
     punit->goto_dest_y=pcity->y;
-    send_unit_info(0,punit,0);
+    send_unit_info(0,punit);
     
     do_unit_goto(pplayer,punit,GOTO_MOVE_ANY);
     
     if (unit_list_find(&map_get_tile(pcity->x, pcity->y)->units, id)) {
-      handle_unit_activity_request(pplayer,punit,ACTIVITY_IDLE);
+      handle_unit_activity_request(punit, ACTIVITY_IDLE);
     }
   }
   
