@@ -21,6 +21,7 @@
 #include "events.h"
 #include "fcintl.h"
 #include "government.h"
+#include "idex.h"
 #include "log.h"
 #include "map.h"
 #include "mem.h"
@@ -73,7 +74,7 @@ int can_sell_building(struct city *pcity, int id)
 ...
 **************************************************************************/
 
-struct city *find_city_wonder(enum improvement_type_id id)
+struct city *find_city_wonder(Impr_Type_id id)
 {
   return (find_city_by_id(game.global_wonders[id]));
 }
@@ -172,6 +173,9 @@ int can_place_worker_here(struct city *pcity, int x, int y)
     && !is_worked_here(map_adjust_x(pcity->x+x-2), pcity->y+y-2));
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 int food_weighting(int n)
 {
   static int value[56] = { -1, 57, 38, 25, 19, 15, 12, 10, 9, 8, 7,
@@ -181,8 +185,11 @@ int food_weighting(int n)
   return(value[n]);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 int city_tile_value(struct city *pcity, int x, int y, int foodneed, int prodneed)
-{ /* by Syela, unifies best_tile, best_food_tile, worst_elvis_tile */
+{
   int i, j, k;
   struct player *plr;
 
@@ -197,18 +204,23 @@ int city_tile_value(struct city *pcity, int x, int y, int foodneed, int prodneed
   if (prodneed > 0) j += 9 * (MIN(j, prodneed));
   j *= SHIELD_WEIGHTING * city_shield_bonus(pcity);
   j /= 100;
+
   k = get_trade_tile(x, y, pcity) * pcity->ai.trade_want *
       (city_tax_bonus(pcity) * (plr->economic.tax + plr->economic.luxury) +
        city_science_bonus(pcity) * plr->economic.science) / 10000;
+
   return(i + j + k);
 }  
 
+/**************************************************************************
+...
+**************************************************************************/
 int worst_worker_tile_value(struct city *pcity)
 {
   int x, y;
   int worst = 0, tmp;
   city_map_iterate(x, y) {
-    if ((x !=2 || y != 2) && get_worker_city(pcity, x, y) == C_TILE_WORKER) {
+    if ((x != 2 || y != 2) && get_worker_city(pcity, x, y) == C_TILE_WORKER) {
       tmp = city_tile_value(pcity, x, y, 0, 0);
       if (tmp < worst || !worst) worst = tmp;
     }
@@ -216,37 +228,32 @@ int worst_worker_tile_value(struct city *pcity)
   return(worst);
 }
 
-int better_tile(struct city *pcity, int x, int y, int bx, int by, int foodneed, int prodneed)
-{
-  return (city_tile_value(pcity, x, y, foodneed, prodneed) >
-          city_tile_value(pcity, bx, by, foodneed, prodneed));
-}
 /**************************************************************************
 ...
 **************************************************************************/
-int best_tile(struct city *pcity, int x, int y, int bx, int by)
-{ /* obsoleted but not deleted by Syela */
-  return (4*get_food_tile(x, y, pcity) +
-          12*get_shields_tile(x, y, pcity) +
-          8*get_trade_tile(x, y, pcity) >
-          4*get_food_tile(bx, by, pcity) +
-          12*get_shields_tile(bx, by, pcity) +
-          8*get_trade_tile(bx, by, pcity) 
-          );
+int best_worker_tile_value(struct city *pcity)
+{
+  int x, y;
+  int best = 0, tmp;
+  city_map_iterate(x, y) {
+    if ((x == 2 || y == 2) ||
+	(get_worker_city(pcity, x, y) == C_TILE_WORKER) ||
+	can_place_worker_here(pcity, x, y)) {
+      tmp = city_tile_value(pcity, x, y, 0, 0);
+      if (tmp < best || !best) best = tmp;
+    }
+  }
+  return(best);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-int best_food_tile(struct city *pcity, int x, int y, int bx, int by)
-{ /* obsoleted but not deleted by Syela */
-  return (100*get_food_tile(x, y, pcity) +
-          12*get_shields_tile(x, y, pcity) +
-          8*get_trade_tile(x, y, pcity) >
-          100*get_food_tile(bx, by, pcity) +
-          12*get_shields_tile(bx, by, pcity) +
-          8*get_trade_tile(bx, by, pcity) 
-          );
+int better_tile(struct city *pcity, int x, int y, int bx, int by,
+		int foodneed, int prodneed)
+{
+  return (city_tile_value(pcity, x, y, foodneed, prodneed) >
+          city_tile_value(pcity, bx, by, foodneed, prodneed));
 }
 
 /**************************************************************************
@@ -578,29 +585,26 @@ int wants_to_be_bigger(struct city *pcity)
 }
 
 
-/*
- *  Units in a bought city are transferred to the new owner, units 
- * supported by the city, but held in other cities are updated to
- * reflect those cities as their new homecity.  Units supported 
- * by the bought city, that are not in a city square may be deleted.
- * This depends on the value of kill_outside.  Just in case the
- * supported units are in an unexplored part of the map, the 
- * area around them is lightened.
- *
- * - Kris Bubendorfer <Kris.Bubendorfer@MCS.VUW.AC.NZ>
- *
- * If verbose is true, send extra messages to players detailing what
- * happens to all the units.  --dwp
- *
- * If statement added to see if the owner of pcity is the same as vcity. If so,
- * then the city is disbanded and not bought, and we only need to evaluate the
- * units supported by the city, and not those actually present.
- * - Thue <thue.kristensen@get2net.dk>
- *
- * Interpretation of kill_outside changed to mean the radius outside
- * of which supported units are killed.  If 0, all supported units not
- * in the city are killed.  If -1, no supported units are killed.  --jjm
- */
+/*********************************************************************
+ Units in a bought city are transferred to the new owner, units 
+ supported by the city, but held in other cities are updated to
+ reflect those cities as their new homecity.  Units supported 
+ by the bought city, that are not in a city square may be deleted.
+ This depends on the value of kill_outside.  If the units are in
+ an unexplored square the area is automatically reveiled when the
+ unit if created.
+
+ - Kris Bubendorfer <Kris.Bubendorfer@MCS.VUW.AC.NZ>
+
+ If verbose is true, send extra messages to players detailing what
+ happens to all the units.  --dwp
+
+ Interpretation of kill_outside changed to mean the radius outside
+ of which supported units are killed.  If 0, all supported units not
+ in the city are killed.  If -1, no supported units are killed.  --jjm
+
+ pcity can be NULL.
+***********************************************************************/
 void transfer_city_units(struct player *pplayer, struct player *pvictim, 
 			 struct city *pcity, struct city *vcity, 
 			 int kill_outside, int verbose)
@@ -608,9 +612,10 @@ void transfer_city_units(struct player *pplayer, struct player *pvictim,
   int x = vcity->x;
   int y = vcity->y;
 
-  if (city_owner(pcity)!=city_owner(vcity)) { /* true=>bought, false =>disbanded */
-    /* Transfer units in the city to the new owner */
-    unit_list_iterate(map_get_tile(x, y)->units, vunit)  {
+  /* Transfer enemy units in the city to the new owner */
+  unit_list_iterate(map_get_tile(x, y)->units, vunit)  {
+    /* Dont transfer units already owned by new city-owner --wegge */ 
+    if (unit_owner(vunit) == pvictim && pcity) {
       freelog(LOG_VERBOSE, "Transfered %s in %s from %s to %s",
 	      unit_name(vunit->type), vcity->name, pvictim->name, pplayer->name);
       if (verbose) {
@@ -618,11 +623,12 @@ void transfer_city_units(struct player *pplayer, struct player *pvictim,
 		      unit_name(vunit->type), vcity->name,
 		      pvictim->name, pplayer->name);
       }
+      
       create_unit_full(pplayer, x, y, vunit->type, vunit->veteran,
 		       pcity->id, vunit->moves_left, vunit->hp);
-      wipe_unit(0, vunit);
-    } unit_list_iterate_end;
-  }
+      wipe_unit_safe(vunit, &myiter);
+    }
+  } unit_list_iterate_end;
 
   /* Any remaining units supported by the city are either given new home
    * cities or maybe destroyed */
@@ -653,15 +659,15 @@ void transfer_city_units(struct player *pplayer, struct player *pvictim,
 	create_unit_full(city_owner(new_home_city), vunit->x, vunit->y,
 			 vunit->type, vunit->veteran, new_home_city->id,
 			 vunit->moves_left, vunit->hp);
-      } else {
+      } else if (pcity){
 	create_unit_full(pplayer, vunit->x, vunit->y,
 			 vunit->type, vunit->veteran, pcity->id,
 			 vunit->moves_left, vunit->hp);
       }
-    }else if((kill_outside < 0) ||
-	     ((kill_outside > 0) &&
-	      (real_map_distance(vunit->x, vunit->y, x, y) <= kill_outside))) {
-
+    } else if(((kill_outside < 0) ||
+	       ((kill_outside > 0) &&
+		(real_map_distance(vunit->x, vunit->y, x, y) <= kill_outside)))
+	      && pcity) {
       freelog(LOG_VERBOSE, "Transfered %s at (%d, %d) from %s to %s",
 	      unit_name(vunit->type), vunit->x, vunit->y,
 	      pvictim->name, pplayer->name);
@@ -674,47 +680,47 @@ void transfer_city_units(struct player *pplayer, struct player *pvictim,
       create_unit_full(pplayer, vunit->x, vunit->y, vunit->type, 
 		       vunit->veteran, pcity->id, vunit->moves_left,
 		       vunit->hp);
-      lighten_area(pplayer, vunit->x,vunit->y);
-
     }
-    wipe_unit_spec_safe(0, vunit, NULL, 0);
-  } unit_list_iterate_end;
+
+    wipe_unit_spec_safe(vunit, NULL, 0);
+  }
+  unit_list_iterate_end;
 }
 
+/**********************************************************************
+dist_nearest_city (in ai.c) does not seem to do what I want or expect
+this function finds the closest friendly city to pos x,y.  I'm sure 
+there must be a similar function somewhere, I just can't find it.
 
-/*
- * dist_nearest_city (in ai.c) does not seem to do what I want or expect
- * this function finds the closest friendly city to pos x,y.  I'm sure 
- * there must be a similar function somewhere, I just can't find it.
- * 
- *                                - Kris Bubendorfer 
- */
+                               - Kris Bubendorfer 
 
-struct city *find_closest_owned_city(struct player *pplayer, int x, int y)
+If sea_required, returned city must be adjacent to ocean.
+If pexclcity, do not return it as the closest city.
+Returns NULL if no satisfactory city can be found.
+***********************************************************************/
+struct city *find_closest_owned_city(struct player *pplayer, int x, int y,
+				     int sea_required, struct city *pexclcity)
 {
-  int dist;
-  struct city *rcity = city_list_get(&pplayer->cities, 0);
-  
-  if(rcity){
-    dist = real_map_distance(x, y, rcity->x, rcity->y);
-    
-    city_list_iterate(pplayer->cities, pcity)
-      if (real_map_distance(x, y, pcity->x, pcity->y) < dist){
-	dist = real_map_distance(x, y, pcity->x, pcity->y);
-	rcity = pcity;
-      }      
-    city_list_iterate_end;
-  }  
+  int dist = -1;
+  struct city *rcity = NULL;
+  city_list_iterate(pplayer->cities, pcity)
+    if ((real_map_distance(x, y, pcity->x, pcity->y) < dist || dist == -1) && 
+	(!sea_required || is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN)) &&
+	(!pexclcity || (pexclcity != pcity))) {
+      dist = real_map_distance(x, y, pcity->x, pcity->y);
+      rcity = pcity;
+    }      
+  city_list_iterate_end;
+
   return rcity;
 }
 
-/*
- * This function creates a new player and copies all of it's science
- * research etc.  Players are both thrown into anarchy and gold is
- * split between both players.
- *                                - Kris Bubendorfer 
- */
-
+/**********************************************************************
+This function creates a new player and copies all of it's science
+research etc.  Players are both thrown into anarchy and gold is
+split between both players.
+                               - Kris Bubendorfer 
+***********************************************************************/
 static struct player *split_player(struct player *pplayer)
 {
   int *nations_used, i, num_nations_avail=game.playable_nation_count, pick;
@@ -725,7 +731,7 @@ static struct player *split_player(struct player *pplayer)
   
   /* make a new player */
 
-  player_init(cplayer);
+  server_player_init(cplayer, 1);
   
   /* select a new name and nation for the copied player. */
 
@@ -759,6 +765,10 @@ static struct player *split_player(struct player *pplayer)
   cplayer->government = game.government_when_anarchy;  
   pplayer->revolution = 1;
   cplayer->capital = 1;
+  for(i = 0; i < MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS; i++) {
+    cplayer->diplstates[i].type = DS_NEUTRAL;
+    cplayer->diplstates[i].has_reason_to_cancel = 0;
+  }
 
   /* Split the resources */
   
@@ -940,12 +950,19 @@ void civil_war(struct player *pplayer)
   city_list_iterate(pplayer->cities, pcity) {
     if (!city_got_building(pcity, B_PALACE)) {
       if (i >= j || (i > 0 && myrand(2))) {
-	
-	/* Transfer city and units supported by this city to the new owner */
-      
-	if(!(pnewcity = transfer_city(cplayer,pplayer,pcity))){
+	/* Transfer city and units supported by this city to the new owner
+
+	 We do NOT resolve stack conflicts here, but rather later.
+	 Reason: if we have a transporter from one city which is carrying
+	 a unit from another city, and both cities join the rebellion. We
+	 resolved stack conflicts for each city we would teleport the first
+	 of the units we met since the other would have another owner */
+	if(!(pnewcity = transfer_city(cplayer, pplayer, pcity, -1, 0, 0, 0))){
 	   freelog(LOG_VERBOSE,
 		   "Transfer city returned no city - aborting civil war.");
+	   unit_list_iterate(pplayer->units, punit) 
+	     resolve_unit_stack(punit->x, punit->y, 0);
+	   unit_list_iterate_end;
 	   return;
 	}
 	
@@ -953,32 +970,20 @@ void civil_war(struct player *pplayer)
 		pnewcity->name, cplayer->name);
 	notify_player(pplayer, _("Game: %s declares allegiance to %s."),
 		      pnewcity->name,cplayer->name);
-	map_set_city(pnewcity->x, pnewcity->y, pnewcity);   
-	transfer_city_units(cplayer, pplayer, pnewcity, pcity, -1, 0);
-	remove_city(pcity); /* don't forget this! */
-	map_set_city(pnewcity->x, pnewcity->y, pnewcity);
-
-	reestablish_city_trade_routes(pnewcity);
-	city_check_workers(cplayer,pnewcity);
-	update_map_with_city_workers(pnewcity);
-	city_refresh(pnewcity);
-	initialize_infrastructure_cache(pnewcity);
-	send_city_info(0, pnewcity, 0);
 	i--;
       }
     }
     j--;
   }
   city_list_iterate_end;
-  
-  /* Resolve Stack conflicts */
 
   i = 0;
-
+  
   unit_list_iterate(pplayer->units, punit) 
     resolve_unit_stack(punit->x, punit->y, 0);
   unit_list_iterate_end;
-  
+
+
   notify_player(0,
 		_("Game: The capture of %s's capital and the destruction "
 		  "of the empire's administrative\n"
@@ -987,32 +992,22 @@ void civil_war(struct player *pplayer)
 		  "      and the upstart %s now holds power in %d "
 		  "rebel provinces."),
 		pplayer->name, cplayer->name, city_list_size(&cplayer->cities));
-    
 }  
 
 
-/* 
- * Transfers pcity from dest, srcplayer.  No units are moved.  This
- * is vanila.  It is now used for incite, and I suspect it could
- * be used to trade cities in the diplomatic section too.
- *
- *  - Kris Bubendorfer
- *
- * Note that the old city is not yet destroyed, which means we
- * can't yet transfer (re-establish) the trade routes for
- * the new city.  (Though the new city struct has the same
- * traderoute data as the old city.)  Thus you should call
- *    reestablish_trade_routes(p_new_city);
- * sometime _after_ calling
- *    remove_city(p_old_city);
- * after this function.  --dwp
- *
- */
+/**********************************************************************
+Handles all transactions in relation to transferring a city.
+
+The kill_outside and transfer_unit_verbose arguments are passed to
+transfer_city_units(), which is called in the middle of the function.
+***********************************************************************/
 struct city *transfer_city(struct player *pplayer, struct player *cplayer,
-			   struct city *pcity)
+			   struct city *pcity, int kill_outside,
+			   int transfer_unit_verbose, int resolve_stack, int raze)
 {
   struct city *pnewcity;
-  int i;
+  int *resolve_list=NULL;
+  int i, no_units=0;
 
   if (!pcity)
     return NULL;
@@ -1025,19 +1020,82 @@ struct city *transfer_city(struct player *pplayer, struct player *cplayer,
 
   pnewcity=fc_malloc(sizeof(struct city));
   *pnewcity=*pcity;
+  pnewcity->worklist = fc_malloc(sizeof(struct worklist));
+  *pnewcity->worklist = *pcity->worklist;
 
   pnewcity->id=get_next_id_number();
-  add_city_to_cache(pnewcity);
+  idex_register_city(pnewcity);
 
   for (i = 0; i < B_LAST; i++) {
     if (is_wonder(i) && city_got_building(pnewcity, i))
       game.global_wonders[i] = pnewcity->id;
   }
-  pnewcity->owner=pplayer->player_no;
+  pnewcity->owner = pplayer->player_no;
   unit_list_init(&pnewcity->units_supported);
   city_list_insert(&pplayer->cities, pnewcity);
 
-  give_citymap_from_player_to_player(pnewcity, cplayer, pplayer);
+  give_citymap_from_player_to_player(pcity, cplayer, pplayer);
+  map_unfog_city_area(pnewcity);
+
+  /* transfer_city_units() destroys the city's units_supported
+     list; we save the list so we can resolve units afterwards. */
+  if (resolve_stack) {
+    no_units = unit_list_size(&(pcity->units_supported));
+    if (no_units > 0) {
+      resolve_list = fc_malloc(no_units * 2 * sizeof(int));
+      if (resolve_list) {
+	i = 0;
+	unit_list_iterate(pcity->units_supported, punit) {
+	  resolve_list[i]   = punit->x;
+	  resolve_list[i+1] = punit->y;
+	  i += 2;
+	} unit_list_iterate_end;
+      }
+    }
+  }
+  transfer_city_units(pplayer, cplayer, pnewcity, pcity,
+		      kill_outside, transfer_unit_verbose);
+  remove_city(pcity);
+  map_set_city(pnewcity->x, pnewcity->y, pnewcity);
+  reset_move_costs(pnewcity->x, pnewcity->y);
+
+  if (resolve_stack && (no_units > 0) && resolve_list) {
+    for (i = 0; i < no_units * 2; i += 2)
+      resolve_unit_stack(resolve_list[i], resolve_list[i+1],
+			 transfer_unit_verbose);
+    free(resolve_list);
+  }
+
+  reestablish_city_trade_routes(pnewcity); 
+  city_check_workers(pplayer ,pnewcity);
+  update_map_with_city_workers(pnewcity);
+  city_refresh(pnewcity);
+  initialize_infrastructure_cache(pnewcity);
+  if (raze)
+    raze_city(pnewcity);
+
+  send_city_info(0, pnewcity);
+  
+  if (terrain_control.may_road &&
+      (player_knows_techs_with_flag (pplayer, TF_RAILROAD)) &&
+      (!player_knows_techs_with_flag (cplayer, TF_RAILROAD)) &&
+      (!(map_get_special (pnewcity->x, pnewcity->y) & S_RAILROAD))) {
+    notify_player (pplayer,
+		   _("Game: The people in %s are stunned by your"
+		     " technological insight!\n"
+		     "      Workers spontaneously gather and upgrade"
+		     " the city with railroads."),
+		   pnewcity->name);
+    map_set_special (pnewcity->x, pnewcity->y, S_RAILROAD);
+    send_tile_info (0, pnewcity->x, pnewcity->y);
+  }
+
+  /* We lose visibility as the old city is removed, and if the square isn't
+     shown here the old owner will never get told about the new city */
+  update_dumb_city(cplayer, pnewcity);
+  send_city_info(cplayer, pnewcity);
+
+  maybe_make_first_contact(pnewcity->x, pnewcity->y, pplayer->player_no);
 
   return pnewcity;
 }
