@@ -48,7 +48,7 @@
 
 #include "cityturn.h"
 
-extern signed short int minimap[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
+extern signed int minimap[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
 
 static void set_tax_income(struct city *pcity);
 static void set_food_trade_shields(struct city *pcity);
@@ -73,9 +73,9 @@ static int worklist_change_build_target(struct player *pplayer,
 					struct city *pcity);
 
 static int city_build_stuff(struct player *pplayer, struct city *pcity);
-static int upgrade_improvement(struct city *pcity, int imp);
+static int improvement_upgrades_to(struct city *pcity, int imp);
 static void upgrade_building_prod(struct city *pcity);
-static Unit_Type_id upgrade_unit(struct city *pcity, Unit_Type_id id);
+static Unit_Type_id unit_upgrades_to(struct city *pcity, Unit_Type_id id);
 static void upgrade_unit_prod(struct city *pcity);
 static void obsolete_building_test(struct city *pcity, int b1, int b2);
 static void pay_for_buildings(struct player *pplayer, struct city *pcity);
@@ -84,6 +84,7 @@ static void sanity_check_city(struct city *pcity);
 
 static void disband_city(struct city *pcity);
 
+static void begin_city_turn(struct player *pplayer, struct city *pcity);
 static int update_city_activity(struct player *pplayer, struct city *pcity);
 
 static void worker_loop(struct city *pcity, int *foodneed,
@@ -397,7 +398,7 @@ void global_city_refresh(struct player *pplayer)
   connection_do_buffer(pplayer->conn);
   city_list_iterate(pplayer->cities, pcity)
     city_refresh(pcity);
-    send_city_info(pplayer, pcity, 1);
+    send_city_info(pplayer, pcity);
   city_list_iterate_end;
   connection_do_unbuffer(pplayer->conn);
 }
@@ -408,7 +409,7 @@ under a Republic or Democracy.
 A unit is *not* aggressive if one or more of following is true:
 - zero attack strength
 - inside a city
-- inside a fortress within 3 squares of a friendly city (new)
+- ground unit inside a fortress within 3 squares of a friendly city
 **************************************************************************/
 int unit_being_aggressive(struct unit *punit)
 {
@@ -416,7 +417,8 @@ int unit_being_aggressive(struct unit *punit)
     return 0;
   if (map_get_city(punit->x,punit->y))
     return 0;
-  if (map_get_special(punit->x,punit->y)&S_FORTRESS) {
+  if (is_ground_unit(punit) &&
+      map_get_special(punit->x,punit->y)&S_FORTRESS) {
     city_list_iterate(get_player(punit->owner)->cities, pcity) {
       if (real_map_distance(punit->x, punit->y, pcity->x, pcity->y)<=3)
 	return 0;
@@ -462,8 +464,10 @@ static void city_support(struct city *pcity)
   */
   if (g->martial_law_max > 0) {
     int city_units = 0;
-    unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, this_unit) {
-      if (city_units < g->martial_law_max && is_military_unit(this_unit))
+    unit_list_iterate(map_get_tile(pcity->x, pcity->y)->units, punit) {
+      if (city_units < g->martial_law_max
+	  && is_military_unit(punit)
+	  && punit->owner == pcity->owner)
 	city_units++;
     }
     unit_list_iterate_end;
@@ -482,6 +486,12 @@ static void city_support(struct city *pcity)
     int food_cost = utype_food_cost(ut, g);
     int gold_cost = utype_gold_cost(ut, g);
 
+    /* Save old values so ve can decide if the unit info should be resent */
+    int old_unhappiness = this_unit->unhappiness;
+    int old_upkeep      = this_unit->upkeep;
+    int old_upkeep_food = this_unit->upkeep_food;
+    int old_upkeep_gold = this_unit->upkeep_gold;
+
     /* set current upkeep on unit to zero */
     this_unit->unhappiness = 0;
     this_unit->upkeep      = 0;
@@ -534,8 +544,14 @@ static void city_support(struct city *pcity)
         this_unit->upkeep_gold = gold_cost;
       }
     }
-  }
-  unit_list_iterate_end;
+    
+    /* Send unit info if anything has changed */
+    if (this_unit->unhappiness != old_unhappiness
+	|| this_unit->upkeep != old_upkeep
+	|| this_unit->upkeep_food != old_upkeep_food
+	|| this_unit->upkeep_gold != old_upkeep_gold)
+      send_unit_info(unit_owner(this_unit), this_unit);
+  } unit_list_iterate_end;
 }
 
 
@@ -714,7 +730,53 @@ void auto_arrange_workers(struct city *pcity)
   pcity->ppl_elvis=workers;
 
   city_refresh(pcity);
-  send_city_info(city_owner(pcity), pcity, 1);
+  send_city_info(city_owner(pcity), pcity);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void send_city_turn_notifications(struct player *pplayer, struct city *pcity)
+{
+  int turns_growth, turns_granary;
+
+  if (pcity->food_surplus > 0) {
+    turns_growth = (((pcity->size+1) * game.foodbox) - pcity->food_stock - 1)
+		   / pcity->food_surplus;
+
+    if (!city_got_effect(pcity,B_GRANARY) && !pcity->is_building_unit &&
+	(pcity->currently_building == B_GRANARY) && (pcity->shield_surplus > 0)) {
+      turns_granary = (improvement_value(B_GRANARY) - pcity->shield_stock)
+		      / pcity->shield_surplus;
+      /* if growth and granary completion occur simultaneously, granary
+	 preserves food.  -AJS */
+      if ((turns_growth < 5) && (turns_granary < 5) &&
+	  (turns_growth < turns_granary)) {
+	notify_player_ex(city_owner(pcity), pcity->x, pcity->y,
+			 E_CITY_GRAN_THROTTLE,
+			 _("Game: Suggest throttling growth in %s to use %s "
+			   "(being built) more effectively."), pcity->name,
+			 improvement_types[B_GRANARY].name);
+      }
+    }
+
+    if ((turns_growth <= 0) && !city_celebrating(pcity)) {
+      notify_player_ex(city_owner(pcity), pcity->x, pcity->y,
+		       E_CITY_MAY_SOON_GROW,
+		       _("Game: %s may soon grow to size %i."),
+		       pcity->name, pcity->size + 1);
+    }
+  }
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+void begin_cities_turn(struct player *pplayer)
+{
+  city_list_iterate(pplayer->cities, pcity)
+     begin_city_turn(pplayer, pcity);
+  city_list_iterate_end;
 }
 
 /**************************************************************************
@@ -763,11 +825,11 @@ void city_auto_remove_worker(struct city *pcity)
   } 
   auto_arrange_workers(pcity);
   city_refresh(pcity);
-  send_city_info(&game.players[pcity->owner], pcity, 0);
+  send_city_info(city_owner(pcity), pcity);
 }
 
 /**************************************************************************
-...
+Note: We do not send info about the city to the clients as part of this function
 **************************************************************************/
 static void city_increase_size(struct city *pcity)
 {
@@ -804,7 +866,7 @@ static void city_increase_size(struct city *pcity)
 		       improvement_types[B_SEWER].name);
     } else {
       notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_CITY_AQUEDUCT,
-		       "Game: %s needs %s to grow any further.",
+		       _("Game: %s needs %s to grow any further."),
 		       pcity->name, improvement_types[B_SEWER].name);
     }
     /* Granary can only hold so much */
@@ -856,7 +918,6 @@ static void city_increase_size(struct city *pcity)
       ai_scientists_taxmen(pcity);
 
   connection_do_buffer(city_owner(pcity)->conn);
-  send_city_info(&game.players[pcity->owner], pcity, 0);
   notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_CITY_GROWTH,
                    _("Game: %s grows to size %d."), pcity->name, pcity->size);
   connection_do_unbuffer(city_owner(pcity)->conn);
@@ -898,7 +959,7 @@ static void city_populate(struct city *pcity)
     unit_list_iterate(pcity->units_supported, punit) {
       if (get_unit_type(punit->type)->food_cost > 0) {
 	char *utname = get_unit_type(punit->type)->name;
-	wipe_unit(0, punit);
+	wipe_unit_safe(punit, &myiter);
 	notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_UNIT_LOST,
 			 _("Game: Famine feared in %s, %s lost!"), 
 			 pcity->name, utname);
@@ -956,7 +1017,7 @@ int advisor_choose_build(struct player *pplayer, struct city *pcity)
   the side-effect of removing from the worklist any no-longer-available
   targets as well as the target actually selected, if any.
 **************************************************************************/
-int worklist_change_build_target(struct player *pplayer, struct city *pcity)
+static int worklist_change_build_target(struct player *pplayer, struct city *pcity)
 {
   int success = 0;
   int i;
@@ -979,7 +1040,7 @@ int worklist_change_build_target(struct player *pplayer, struct city *pcity)
     /* Sanity checks */
     if (is_unit &&
 	!can_build_unit(pcity, target)) {
-      int new_target = upgrade_unit(pcity, target);
+      int new_target = unit_upgrades_to(pcity, target);
 
       /* If the city can never build this unit or its descendants, drop it. */
       if (!can_eventually_build_unit(pcity, new_target)) {
@@ -1019,7 +1080,7 @@ int worklist_change_build_target(struct player *pplayer, struct city *pcity)
 	target = new_target;
       }
     } else if (!is_unit && !can_build_improvement(pcity, target)) {
-      int new_target = upgrade_improvement(pcity, target);
+      int new_target = improvement_upgrades_to(pcity, target);
 
       /* If the city can never build this improvement, drop it. */
       if (!can_eventually_build_improvement(pcity, new_target)) {
@@ -1028,7 +1089,7 @@ int worklist_change_build_target(struct player *pplayer, struct city *pcity)
 			 _("Game: %s can't build %s from the worklist.  "
 			   "Purging..."),
 			 pcity->name,
-			 get_imp_name_ex(pcity, target));
+			 get_impr_name_ex(pcity, target));
 
 	/* Purge this worklist item. */
 	worklist_remove(pcity->worklist, i-1);
@@ -1047,14 +1108,14 @@ int worklist_change_build_target(struct player *pplayer, struct city *pcity)
 			 _("Game: %s can't build %s from the worklist; "
 			   "tech not yet available.  Postponing..."),
 			 pcity->name,
-			 get_imp_name_ex(pcity, target));
+			 get_impr_name_ex(pcity, target));
 	continue;
       } else {
 	/* Hey, we can upgrade the improvement!  */
 	notify_player_ex(pplayer, pcity->x, pcity->y, E_WORKLIST,
 			 _("Game: Production of %s is upgraded to %s in %s."),
-			 get_imp_name_ex(pcity, target), 
-			 get_imp_name_ex(pcity, new_target),
+			 get_impr_name_ex(pcity, target), 
+			 get_impr_name_ex(pcity, new_target),
 			 pcity->name);
 	target = new_target;
       }
@@ -1100,9 +1161,9 @@ static void obsolete_building_test(struct city *pcity, int b1, int b2)
   !!! Note:  I hear that the building ruleset code is going to be
   overhauled soon.  If this happens, then this function should be updated
   to follow the new model.  This function will probably look a lot like
-  upgrade_unit().
+  unit_upgrades_to().
 **************************************************************************/
-static int upgrade_improvement(struct city *pcity, int imp)
+static int improvement_upgrades_to(struct city *pcity, int imp)
 {
   if (imp == B_BARRACKS && can_build_improvement(pcity, B_BARRACKS3))
     return B_BARRACKS3;
@@ -1130,15 +1191,17 @@ static void upgrade_building_prod(struct city *pcity)
   id doesn't guarantee that pcity really _can_ build id; just that
   pcity can't build whatever _obsoletes_ id.
 **************************************************************************/
-static Unit_Type_id upgrade_unit(struct city *pcity, Unit_Type_id id)
+static Unit_Type_id unit_upgrades_to(struct city *pcity, Unit_Type_id id)
 {
-  while(can_build_unit_direct(pcity, id) &&
-	can_build_unit_direct(pcity, unit_types[id].obsoleted_by))
-  {
-    id = unit_types[id].obsoleted_by;
-  }
-    
-  return id;
+  Unit_Type_id latest_ok = id;
+
+  if (!can_build_unit_direct(pcity, id))
+    return -1;
+  while(unit_type_exists(id = unit_types[id].obsoleted_by))
+    if (can_build_unit_direct(pcity, id))
+      latest_ok = id;
+
+  return latest_ok;
 }
 
 /**************************************************************************
@@ -1148,8 +1211,8 @@ static void upgrade_unit_prod(struct city *pcity)
 {
   struct player *pplayer=&game.players[pcity->owner];
   int id = pcity->currently_building;
-  int id2 = upgrade_unit(pcity, unit_types[id].obsoleted_by);
-    
+  int id2 = unit_upgrades_to(pcity, unit_types[id].obsoleted_by);
+
   if (can_build_unit_direct(pcity, id2)) {
     pcity->currently_building=id2;
     notify_player_ex(pplayer, pcity->x, pcity->y, E_UNIT_UPGRADED, 
@@ -1168,6 +1231,7 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 {
   struct government *g = get_gov_pplayer(pplayer);
   int space_part;
+  int city_built_city_builder;
 
   while (pcity->shield_surplus<0) {
     unit_list_iterate(pcity->units_supported, punit) {
@@ -1175,7 +1239,7 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 	notify_player_ex(pplayer, pcity->x, pcity->y, E_UNIT_LOST,
 			 _("Game: %s can't upkeep %s, unit disbanded."),
 			 pcity->name, get_unit_type(punit->type)->name);
-	wipe_unit(pplayer, punit);
+	wipe_unit_safe(punit, &myiter);
 	break;
       }
     }
@@ -1192,7 +1256,7 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
     if (!can_build_improvement(pcity, pcity->currently_building)) {
       notify_player_ex(pplayer, pcity->x, pcity->y, E_CITY_CANTBUILD,
 		    _("Game: %s is building %s, which is no longer available."),
-	pcity->name,get_imp_name_ex(pcity, pcity->currently_building));
+	pcity->name,get_impr_name_ex(pcity, pcity->currently_building));
       return 1;
     }
     if (pcity->shield_stock>=improvement_value(pcity->currently_building)) {
@@ -1224,17 +1288,17 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 	notify_player_ex(0, pcity->x, pcity->y, E_WONDER_BUILD,
 		      _("Game: The %s have finished building %s in %s."),
 		      get_nation_name_plural(pplayer->nation),
-		      get_imp_name_ex(pcity, pcity->currently_building),
+		      get_impr_name_ex(pcity, pcity->currently_building),
 		      pcity->name);
         gamelog(GAMELOG_WONDER,"%s build %s in %s",
                 get_nation_name_plural(pplayer->nation),
-                get_imp_name_ex(pcity, pcity->currently_building),
+                get_impr_name_ex(pcity, pcity->currently_building),
                 pcity->name);
 
       } else 
 	gamelog(GAMELOG_IMP, "%s build %s in %s",
                 get_nation_name_plural(pplayer->nation),
-                get_imp_name_ex(pcity, pcity->currently_building),
+                get_impr_name_ex(pcity, pcity->currently_building),
                 pcity->name);
       
       notify_player_ex(pplayer, pcity->x, pcity->y, E_IMP_BUILD,
@@ -1270,12 +1334,14 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
       }
     } 
   } else {
+    city_built_city_builder = 0;
+
     upgrade_unit_prod(pcity);
+
     /* FIXME: F_CITIES should be changed to any unit
      * that 'contains' 1 (or more) pop -- sjolie
      */
     if(pcity->shield_stock>=unit_value(pcity->currently_building)) {
-      pcity->turn_last_built = game.year;
       if (unit_flag(pcity->currently_building, F_CITIES)) {
 	if (pcity->size==1) {
 
@@ -1291,15 +1357,23 @@ static int city_build_stuff(struct player *pplayer, struct city *pcity)
 	  return 1;
 
 	}
-	pcity->size--;
-	city_auto_remove_worker(pcity);
+	city_built_city_builder = 1;
       }
+      
+      pcity->turn_last_built = game.year;
+      /* don't update turn_last_built if we returned above for size==1 */
 
       create_unit(pplayer, pcity->x, pcity->y, pcity->currently_building,
 		  do_make_unit_veteran(pcity, pcity->currently_building), 
 		  pcity->id, -1);
       /* to eliminate micromanagement, we only subtract the unit's cost */
       pcity->shield_stock-=unit_value(pcity->currently_building); 
+
+      if (city_built_city_builder) {
+	pcity->size--;
+	city_auto_remove_worker(pcity);
+      }
+
       notify_player_ex(pplayer, pcity->x, pcity->y, E_UNIT_BUILD,
 		       _("Game: %s is finished building %s."), 
 		       pcity->name, 
@@ -1332,7 +1406,7 @@ static void pay_for_buildings(struct player *pplayer, struct city *pcity)
     if (city_got_building(pcity, i)) {
       if (is_wonder(i)) {
 	if (wonder_obsolete(i)) {
-	  switch (improvement_types[i].shield_upkeep) {
+	  switch (improvement_types[i].upkeep) {
 	  case 1:
 	    pplayer->economic.gold+=3;
 	    break;
@@ -1369,13 +1443,14 @@ void city_check_workers(struct player *pplayer, struct city *pcity)
     
     if(unit_list_size(&ptile->units)>0) {
       struct unit *punit=unit_list_get(&ptile->units, 0);
-      if(pplayer->player_no!=punit->owner) {
-	if(get_worker_city(pcity, x, y)==C_TILE_WORKER) {
+      if (players_at_war(pplayer->player_no, punit->owner)) {
+	if (get_worker_city(pcity, x, y)==C_TILE_WORKER) {
 /*	  pcity->ppl_elvis++; -- this is really not polite -- Syela */
   	  set_worker_city(pcity, x, y, C_TILE_UNAVAILABLE);
           add_adjust_workers(pcity); /* will place the displaced */
           city_refresh(pcity); /* may be unnecessary; can't hurt */
-        } else	set_worker_city(pcity, x, y, C_TILE_UNAVAILABLE);
+        } else
+	  set_worker_city(pcity, x, y, C_TILE_UNAVAILABLE);
 	continue;
       }
     }
@@ -1404,7 +1479,7 @@ static void check_pollution(struct city *pcity)
 	if ( (map_get_terrain(x,y)!=T_OCEAN && map_get_terrain(x,y)<=T_TUNDRA) &&
 	     (!(map_get_special(x,y)&S_POLLUTION)) ) { 
 	  map_set_special(x,y, S_POLLUTION);
-	  send_tile_info(0, x, y, TILE_KNOWN);
+	  send_tile_info(0, x, y);
 	  notify_player_ex(city_owner(pcity), pcity->x, pcity->y, E_POLLUTION,
 			   _("Game: Pollution near %s."), pcity->name);
 	  return;
@@ -1472,44 +1547,42 @@ void city_incite_cost(struct city *pcity)
 }
 
 /**************************************************************************
- called every turn, for every city. 
+ Called every turn, at beginning of turn, for every city.
+**************************************************************************/
+static void begin_city_turn(struct player *pplayer, struct city *pcity)
+{
+  /* remember what this city is building at start of turn,
+     so user can switch production back without penalty */
+  pcity->changed_from_id = pcity->currently_building;
+  pcity->changed_from_is_unit = pcity->is_building_unit;
+  pcity->before_change_shields = pcity->shield_stock;
+  freelog(LOG_DEBUG,
+	  "In %s, building %s.  Beg of Turn shields = %d",
+	  pcity->name,
+	  pcity->changed_from_is_unit ?
+	    unit_types[pcity->changed_from_id].name :
+	    improvement_types[pcity->changed_from_id].name,
+	  pcity->before_change_shields
+	  );
+}
+
+/**************************************************************************
+ Called every turn, at end of turn, for every city.
 **************************************************************************/
 static int update_city_activity(struct player *pplayer, struct city *pcity)
 {
   struct government *g = get_gov_pcity(pcity);
   int got_tech = 0;
-  int turns_growth, turns_granary;
 
   city_check_workers(pplayer, pcity);
   city_refresh(pcity);
 
-  if (!city_got_effect(pcity,B_GRANARY) && !pcity->is_building_unit &&
-    (pcity->currently_building == B_GRANARY) && (pcity->food_surplus > 0)
-    && (pcity->shield_surplus > 0)) {
-    turns_growth = (((pcity->size+1) * game.foodbox) - pcity->food_stock)
-                     / pcity->food_surplus;
-    turns_granary = (improvement_value(B_GRANARY) - pcity->shield_stock)
-                     / pcity->shield_surplus;
-    /* if growth and granary completion occur simultaneously, granary
-       preserves food.  -AJS */
-    if ((turns_growth < 5) && (turns_granary < 5) &&
-        (turns_growth < turns_granary)) {
-       notify_player_ex(city_owner(pcity), pcity->x, pcity->y,
-			E_CITY_GRAN_THROTTLE,
-			_("Game: %s suggest throttling growth to use %s "
-			  "(being built) more effectively."), pcity->name,
-			improvement_types[B_GRANARY].name);
-       }
-    }
-  
-
-/* the AI often has widespread disorder when the Gardens or Oracle
-become obsolete.  This is a quick hack to prevent this.  980805 -- Syela */
+  /* the AI often has widespread disorder when the Gardens or Oracle
+     become obsolete.  This is a quick hack to prevent this.  980805 -- Syela */
   while (pplayer->ai.control && city_unhappy(pcity)) {
     if (!ai_make_elvis(pcity)) break;
   } /* putting this lower in the routine would basically be cheating. -- Syela */
 
-
   /* reporting of celebrations rewritten, copying the treatment of disorder below,
      with the added rapture rounds count.  991219 -- Jing */
   if (city_build_stuff(pplayer, pcity)) {
@@ -1537,7 +1610,7 @@ become obsolete.  This is a quick hack to prevent this.  980805 -- Syela */
     {
       int id=pcity->id;
       city_populate(pcity);
-      if(!city_list_find_id(&pplayer->cities, id))
+      if(!player_find_city_by_id(pplayer, id))
 	return 0;
     }
 
@@ -1573,7 +1646,7 @@ become obsolete.  This is a quick hack to prevent this.  980805 -- Syela */
     check_pollution(pcity);
     city_incite_cost(pcity);
 
-    send_city_info(0, pcity, 0);
+    send_city_info(0, pcity);
     if (pcity->anarchy>2 && government_has_flag(g, G_REVOLUTION_WHEN_UNHAPPY)) {
       notify_player_ex(pplayer, pcity->x, pcity->y, E_ANARCHY,
 		       _("Game: The people have overthrown your %s, "
@@ -1589,24 +1662,14 @@ become obsolete.  This is a quick hack to prevent this.  980805 -- Syela */
 /**************************************************************************
  disband a city into a settler, supported by the closest city -- Massimo
 **************************************************************************/
-
 static void disband_city(struct city *pcity)
 {
   struct player *pplayer = get_player(pcity->owner);
-  int x = pcity->x, y = pcity->y, dist, dist1;
+  int x = pcity->x, y = pcity->y;
   struct city *rcity=NULL;
 
-  /* We cannot use find_closest_owned_city, since it would return pcity */
-  if (city_list_get(&pplayer->cities, 0)) {
-    dist = 9999;
-    city_list_iterate(pplayer->cities, pcity1);
-    dist1 = real_map_distance(x, y, pcity1->x, pcity1->y);
-    if (dist1 && dist1 < dist) {
-      dist = dist1;
-      rcity = pcity1;
-    }      
-    city_list_iterate_end;
-  }
+  /* find closest city other than pcity */
+  rcity = find_closest_owned_city(pplayer, x, y, 0, pcity);
 
   if (!rcity) {
     /* What should we do when we try to disband our only city? */
