@@ -92,7 +92,7 @@
 
 #include "civserver.h"
 
-/* main() belongs at the bottom of files!! -- Syela */
+static void begin_turn(void);
 static void before_end_year(void);
 static int end_turn(void);
 static void send_all_info(struct player *dest);
@@ -110,6 +110,10 @@ static void handle_request_join_game(struct connection *pconn,
 				     struct packet_req_join_game *request);
 static void handle_turn_done(int player_no);
 static void send_select_nation(struct player *pplayer);
+static void enable_fog_of_war_player(struct player *pplayer);
+static void disable_fog_of_war_player(struct player *pplayer);
+static void enable_fog_of_war(void);
+static void disable_fog_of_war(void);
 
 #ifdef GENERATING_MAC
 static void Mac_options(int *argc, char *argv[]);
@@ -157,14 +161,15 @@ int num_nations_avail;
 int main(int argc, char *argv[])
 {
   int h=0, v=0, no_meta=1;
+  int i;
   char *log_filename=NULL;
   char *gamelog_filename=NULL;
   char *load_filename=NULL;
   char *script_filename=NULL;
   char *option=NULL;
-  int i;
   int save_counter=0;
   int loglevel=LOG_NORMAL;
+  struct timer *eot_timer;	/* time server processing at end-of-turn */
 
   init_nls();
   dont_run_as_root(argv[0], "freeciv_server");
@@ -282,7 +287,11 @@ int main(int argc, char *argv[])
 
   init_our_capability();
   game_init();
-  initialize_city_cache();
+
+  /* load a script file */
+
+  if (script_filename)
+    read_init_script(script_filename);
 
   /* load a saved game */
   
@@ -293,7 +302,7 @@ int main(int argc, char *argv[])
     freelog(LOG_NORMAL, _("Loading saved game: %s"), load_filename);
     loadtimer = new_timer_start(TIMER_CPU, TIMER_ACTIVE);
     uloadtimer = new_timer_start(TIMER_USER, TIMER_ACTIVE);
-    if(!section_file_load(&file, load_filename)) { 
+    if(!section_file_load_nodup(&file, load_filename)) { 
       freelog(LOG_FATAL, _("Couldn't load savefile: %s"), load_filename);
       exit(1);
     }
@@ -309,7 +318,7 @@ int main(int argc, char *argv[])
 	    read_timer_seconds_free(loadtimer),
 	    read_timer_seconds_free(uloadtimer));
   }
-  
+
   /* init network */  
   init_connections(); 
   server_open_socket();
@@ -325,9 +334,6 @@ int main(int argc, char *argv[])
   freelog(LOG_NORMAL, _("Now accepting new client connections."));
   server_state=PRE_GAME_STATE;
 
-  if (script_filename)
-    read_init_script(script_filename);
-
   while(server_state==PRE_GAME_STATE)
     sniff_packets();
 
@@ -351,6 +357,15 @@ main_start_players:
       nations_used[i]=i;
   }
 
+  if (game.auto_ai_toggle) {
+    for (i=0;i<game.nplayers;i++) {
+      struct player *pplayer = get_player(i);
+      if (!pplayer->conn && !pplayer->ai.control) {
+	toggle_ai_player_direct(NULL, pplayer);
+      }
+    }
+  }
+
   /* Allow players to select a nation (case new game).
    * AI players may not yet have a nation; these will be selected
    * in generate_ai_players() later
@@ -385,6 +400,12 @@ main_start_players:
     }
   }
 
+  /* When to start this timer the first time round is a bit arbitrary:
+     here start straight after stop sniffing for select_races and
+     before all following server initialization etc.
+  */
+  eot_timer = new_timer_start(TIMER_CPU, TIMER_ACTIVE);
+
   if(!game.randseed) {
     /* We strip the high bit for now because neither game file nor
        server options can handle unsigned ints yet. - Cedric */
@@ -412,11 +433,15 @@ main_start_players:
   send_server_info_to_metaserver(1,0);
 
   if(game.is_new_game) {
-    int i;
+    /* Before the player map is allocated (and initiailized)! */
+    game.fogofwar_old = game.fogofwar;
+
     for(i=0; i<game.nplayers; i++) {
-      init_tech(&game.players[i], game.tech);
-      player_limit_to_government_rates(&game.players[i]);
-      game.players[i].economic.gold=game.gold;
+      struct player *pplayer = &game.players[i];
+      player_map_allocate(pplayer);
+      init_tech(pplayer, game.tech);
+      player_limit_to_government_rates(pplayer);
+      pplayer->economic.gold=game.gold;
     }
     game.max_players=game.nplayers;
 
@@ -442,26 +467,45 @@ main_start_players:
   }
   
   send_all_info(0);
-
-  if(game.is_new_game) 
+  
+  if(game.is_new_game)
     init_new_game();
 
   game.is_new_game = 0;
 
   send_game_state(0, CLIENT_GAME_RUNNING_STATE);
-  
+
   while(server_state==RUN_GAME_STATE) {
+    /* absolute beginning of a turn */
+    freelog(LOG_DEBUG, "Begin turn");
+    begin_turn();
+
     force_end_of_sniff=0;
+
     freelog(LOG_DEBUG, "Shuffleplayers");
     shuffle_players();
     freelog(LOG_DEBUG, "Aistartturn");
     ai_start_turn();
 
+    /* Before sniff (human player activites), report time to now: */
+    freelog(LOG_VERBOSE, "End/start-turn server/ai activities: %g seconds",
+	    read_timer_seconds(eot_timer));
+
     freelog(LOG_DEBUG, "sniffingpackets");
     while(sniff_packets()==1);
+
+    /* After sniff, re-zero the timer: (read-out above on next loop) */
+    clear_timer_start(eot_timer);
     
     for(i=0;i<game.nplayers;i++)
       connection_do_buffer(game.players[i].conn);
+
+    before_end_year();
+    /* This empties the client Messages window; put this before
+       everything else below, since otherwise any messages from
+       the following parts get wiped out before the user gets a
+       chance to see them.  --dwp
+    */
     freelog(LOG_DEBUG, "Season of native unrests");
     summon_barbarians(); /* wild guess really, no idea where to put it, but
                             I want to give them chance to move their units */
@@ -469,7 +513,6 @@ main_start_players:
     auto_settlers(); /* moved this after ai_start_turn for efficiency -- Syela */
     /* moved after sniff_packets for even more efficiency.
        What a guy I am. -- Syela */
-    before_end_year(); /* resetting David P's message window -- Syela */
     /* and now, we must manage our remaining units BEFORE the cities that are
        empty get to refresh and defend themselves.  How totally stupid. */
     ai_start_turn(); /* Misleading name for manage_units -- Syela */
@@ -488,9 +531,10 @@ main_start_players:
     send_year_to_clients(game.year);
     freelog(LOG_DEBUG, "Sendinfotometaserver");
     send_server_info_to_metaserver(0,0);
+
     for(i=0;i<game.nplayers;i++)
       connection_do_unbuffer(game.players[i].conn);
-      
+
     if(++save_counter>=game.save_nturns && game.save_nturns>0) {
       save_counter=0;
       save_game_auto();
@@ -500,7 +544,7 @@ main_start_players:
   }
 
   show_ending();
-  
+  show_map_to_all();
   notify_player(0, _("Game: The game is over..."));
 
   while(server_state==GAME_OVER_STATE) {
@@ -544,6 +588,7 @@ int send_server_info_to_metaserver(int do_send,int reset_timer)
   static struct timer *time_since_last_send = NULL;
   char desc[4096], info[4096];
   int i;
+  int num_nonbarbarians;
 
   if (reset_timer && time_since_last_send)
   {
@@ -562,34 +607,43 @@ int send_server_info_to_metaserver(int do_send,int reset_timer)
     time_since_last_send = new_timer(TIMER_USER, TIMER_ACTIVE);
   }
 
+  for (num_nonbarbarians=0, i=0; i<game.nplayers; ++i) {
+    if (!is_barbarian(&game.players[i])) {
+      ++num_nonbarbarians;
+    }
+  }
+
   /* build description block */
   desc[0]='\0';
   
   cat_snprintf(desc, sizeof(desc), "Freeciv\n");
   cat_snprintf(desc, sizeof(desc), VERSION_STRING"\n");
+  /* note: the following strings are not translated here;
+     we mark them so they may be translated when received by a client */
   switch(server_state) {
   case PRE_GAME_STATE:
-    cat_snprintf(desc, sizeof(desc), "Pregame\n");
+    cat_snprintf(desc, sizeof(desc), N_("Pregame"));
     break;
   case RUN_GAME_STATE:
-    cat_snprintf(desc, sizeof(desc), "Running\n");
+    cat_snprintf(desc, sizeof(desc), N_("Running"));
     break;
   case GAME_OVER_STATE:
-    cat_snprintf(desc, sizeof(desc), "Game over\n");
+    cat_snprintf(desc, sizeof(desc), N_("Game over"));
     break;
   default:
-    cat_snprintf(desc, sizeof(desc), "Waiting\n");
+    cat_snprintf(desc, sizeof(desc), N_("Waiting"));
   }
+  cat_snprintf(desc, sizeof(desc), "\n");
   cat_snprintf(desc, sizeof(desc), "%s\n",metaserver_servername);
   cat_snprintf(desc, sizeof(desc), "%d\n", port);
-  cat_snprintf(desc, sizeof(desc), "%d\n", game.nplayers);
+  cat_snprintf(desc, sizeof(desc), "%d\n", num_nonbarbarians);
   cat_snprintf(desc, sizeof(desc), "%s", metaserver_info_line);
-    
+
   /* now build the info block */
   info[0]='\0';
   cat_snprintf(info, sizeof(info),
 	  "Players:%d  Min players:%d  Max players:%d\n",
-	  game.nplayers, game.min_players, game.max_players);
+	  num_nonbarbarians,  game.min_players, game.max_players);
   cat_snprintf(info, sizeof(info),
 	  "Timeout:%d  Year: %s\n",
 	  game.timeout, textyear(game.year));
@@ -600,8 +654,10 @@ int send_server_info_to_metaserver(int do_send,int reset_timer)
   cat_snprintf(info, sizeof(info),
 	       "----------------------------------------\n");
   for(i=0; i<game.nplayers; ++i) {
-    cat_snprintf(info, sizeof(info), "%2d   %-20s %s\n", 
+    if (!is_barbarian(&game.players[i])) {
+      cat_snprintf(info, sizeof(info), "%2d   %-20s %s\n", 
 		 i, game.players[i].name, game.players[i].addr);
+    }
   }
 
   clear_timer_start(time_since_last_send);
@@ -619,32 +675,11 @@ static void send_all_info(struct player *dest)
   send_player_info(0, dest);
   send_spaceship_info(0, dest);
   send_all_known_tiles(dest);
+  send_all_known_cities(dest);
+  send_all_known_units(dest);
+  send_player_turn_notifications(dest);
 }
 
-#ifdef UNUSED
-/**************************************************************************
-...
-**************************************************************************/
-int find_highest_used_id(void)
-{
-  int i, max_id;
-
-  max_id=0;
-  for(i=0; i<game.nplayers; i++) {
-    city_list_iterate(game.players[i].cities, pcity) {
-      max_id=MAX(max_id, pcity->id);
-    }
-    city_list_iterate_end;
-    unit_list_iterate(game.players[i].units, punit) {
-      max_id=MAX(max_id, punit->id);
-    }
-    unit_list_iterate_end;
-  }
-  return max_id;
-}
-#endif /* UNUSED */
-
-
 /**************************************************************************
 ...
 **************************************************************************/
@@ -657,12 +692,16 @@ static void do_apollo_program(void)
   if ((cityid=game.global_wonders[B_APOLLO]) && 
       (pcity=find_city_by_id(cityid))) {
     pplayer=city_owner(pcity);
-    
-    for(i=0; i<game.nplayers; i++) {
-      city_list_iterate(game.players[i].cities, pcity)
-	
-	light_square(pplayer, pcity->x, pcity->y, 0);
-      city_list_iterate_end;
+    if (game.civstyle == 1) {
+      for(i=0; i<game.nplayers; i++) {
+	city_list_iterate(game.players[i].cities, pcity)
+	  show_area(pplayer, pcity->x, pcity->y, 0);
+	city_list_iterate_end;
+      }
+    } else {
+      map_know_all(pplayer);
+      send_all_known_tiles(pplayer);
+      send_all_known_cities(pplayer);
     }
   }
 }
@@ -670,31 +709,87 @@ static void do_apollo_program(void)
 /**************************************************************************
 ...
 **************************************************************************/
-static void update_pollution(void)
+static void marco_polo_make_contact(void)
 {
-  int x,y,count=0;
-  
-  for (x=0;x<map.xsize;x++) 
-    for (y=0;y<map.ysize;y++) 
-      if (map_get_special(x,y)&S_POLLUTION) {
+  int cityid  = game.global_wonders[B_MARCO];
+  int o;
+  struct city *pcity;
+  if (cityid && (pcity = find_city_by_id(cityid)))
+    for (o = 0; o < game.nplayers; o++)
+      make_contact(pcity->owner, o, pcity->x, pcity->y);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
+static void update_environmental_upset(enum tile_special_type cause,
+				       int *current, int *accum, int *level,
+				       void (*upset_action_fn)(int))
+{
+  int x, y, count;
+
+  count = 0;
+  for (x = 0; x < map.xsize; x++) {
+    for (y = 0; y < map.ysize; y++) {
+      if (map_get_special(x,y) & cause) {
 	count++;
       }
-  game.heating=count;
-  game.globalwarming+=count;
-  if (game.globalwarming<game.warminglevel) 
-    game.globalwarming=0;
-  else {
-    game.globalwarming-=game.warminglevel;
-    if (myrand(200)<=game.globalwarming) {
-      freelog(LOG_NORMAL, "Global warming:%d\n", count);
-      global_warming(map.xsize/10+map.ysize/10+game.globalwarming*5);
-      game.globalwarming=0;
+    }
+  }
+
+  *current = count;
+  *accum += count;
+  if (*accum < *level) {
+    *accum = 0;
+  } else {
+    *accum -= *level;
+    if (myrand(200) <= *accum) {
+      upset_action_fn((map.xsize / 10) + (map.ysize / 10) + ((*accum) * 5));
+      *accum = 0;
+      *level+=4;
       send_all_known_tiles(0);
-      notify_player_ex(0, 0,0, E_WARMING,
-		       _("Game: Global warming has occurred!"));
-      notify_player(0, _("Game: Coastlines have been flooded and vast ranges "
-			 "of grassland have become deserts."));
-      game.warminglevel+=4;
+    }
+  }
+
+  freelog(LOG_DEBUG,
+	  "environmental_upset: cause=%-4d current=%-2d level=%-2d accum=%-2d",
+	  cause, *current, *level, *accum);
+}
+
+/**************************************************************************
+ check for cease-fires running out; update reputation; update cancelling
+ reasons
+**************************************************************************/
+static void update_diplomatics()
+{
+  int p, p2;
+
+  for(p = 0; p < game.nplayers; p++) {
+    for(p2 = 0; p2 < game.nplayers; p2++) {
+      game.players[p].diplstates[p2].has_reason_to_cancel =
+	MAX(game.players[p].diplstates[p2].has_reason_to_cancel - 1, 0);
+
+      if(game.players[p].diplstates[p2].type == DS_CEASEFIRE) {
+	switch(--game.players[p].diplstates[p2].turns_left) {
+	case 1:
+	  notify_player(&game.players[p],
+			_("Game: Concerned citizens point "
+			  "out that the cease-fire with %s will run out soon."),
+			game.players[p2].name);
+	  break;
+	case -1:
+	  notify_player(&game.players[p],
+			_("Game: The cease-fire with %s has "
+			  "run out. You are now neutral towards the %s."),
+			game.players[p2].name,
+			get_nation_name_plural(game.players[p2].nation));
+	  game.players[p].diplstates[p2].type = DS_NEUTRAL;
+	  break;
+	}
+      }
+      game.players[p].reputation =
+	MIN(game.players[p].reputation + GAME_REPUTATION_INCR,
+	    GAME_MAX_REPUTATION);
     }
   }
 }
@@ -747,12 +842,51 @@ static void ai_start_turn(void)
   }
 }
 
+/**************************************************************************
+Handle the beginning of each turn.
+Note: This does not give "time" to any player;
+      it is solely for updating turn-dependent data.
+**************************************************************************/
+static void begin_turn(void)
+{
+  int i;
+
+  /* See if the value of fog of war has changed */
+  if (game.fogofwar != game.fogofwar_old) {
+    if (game.fogofwar == 1) {
+      enable_fog_of_war();
+      game.fogofwar_old = 1;
+    } else {
+      disable_fog_of_war();
+      game.fogofwar_old = 0;
+    }
+  }
+
+  for(i=0; i<game.nplayers; i++)
+    connection_do_buffer(game.players[i].conn);
+
+  for(i=0; i<game.nplayers; i++) {
+    struct player *pplayer = &game.players[i];
+    freelog(LOG_DEBUG, "beginning player turn for #%d (%s)",
+	    i, pplayer->name);
+    begin_player_turn(pplayer);
+  }
+  for (i=0; i<game.nplayers;i++) {
+    send_player_cities(&game.players[i]);
+  }
+
+  for(i=0; i<game.nplayers; i++)
+    connection_do_unbuffer(game.players[i].conn);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
 static int end_turn(void)
 {
   int i;
-  
-  nocity_send = 1;
 
+  nocity_send = 1;
   for(i=0; i<game.nplayers; i++) {
     /* game.nplayers may increase during this loop, due to
        goto-ing units causing a civil war.  Thus shuffled[i]
@@ -771,31 +905,63 @@ static int end_turn(void)
   for (i=0; i<game.nplayers;i++) {
     send_player_cities(&game.players[i]);
   }
-  update_pollution();
+  update_environmental_upset(S_POLLUTION, &game.heating,
+			     &game.globalwarming, &game.warminglevel,
+			     global_warming);
+  update_environmental_upset(S_FALLOUT, &game.cooling,
+			     &game.nuclearwinter, &game.coolinglevel,
+			     nuclear_winter);
+  update_diplomatics();
   do_apollo_program();
+  marco_polo_make_contact();
   make_history_report();
+  for (i=0; i<game.nplayers;i++) {
+    send_player_turn_notifications(&game.players[i]);
+  }
   freelog(LOG_DEBUG, "Turn ended.");
+  game.turn_start = time(NULL);
   return 1;
 }
 
-
 /**************************************************************************
 Unconditionally save the game, with specified filename.
 Always prints a message: either save ok, or failed.
+
+Note that if !HAVE_LIBZ, then game.save_compress_level should never
+become non-zero, so no need to check HAVE_LIBZ explicitly here as well.
 **************************************************************************/
-void save_game(char *filename)
+void save_game(char *orig_filename)
 {
+  char filename[600];
   struct section_file file;
+  struct timer *timer_cpu, *timer_user;
+
+  sz_strlcpy(filename, orig_filename);
   
+  timer_cpu = new_timer_start(TIMER_CPU, TIMER_ACTIVE);
+  timer_user = new_timer_start(TIMER_USER, TIMER_ACTIVE);
+    
   section_file_init(&file);
   game_save(&file);
-  
-  if(!section_file_save(&file, filename))
+
+  if (game.save_compress_level > 0) {
+    /* Append ".gz" to filename if not there: */
+    size_t len = strlen(filename);
+    if (len < 3 || strcmp(filename+len-3, ".gz") != 0) {
+      sz_strlcat(filename, ".gz");
+    }
+  }
+
+  if(!section_file_save(&file, filename, game.save_compress_level))
     con_write(C_FAIL, _("Failed saving game as %s"), filename);
   else
     con_write(C_OK, _("Game saved as %s"), filename);
 
   section_file_free(&file);
+
+  freelog(LOG_VERBOSE, "Save time: %g seconds (%g apparent)",
+	  read_timer_seconds_free(timer_cpu),
+	  read_timer_seconds_free(timer_user));
 }
 
 /**************************************************************************
@@ -828,6 +994,7 @@ void start_game(void)
 
   server_state=SELECT_RACES_STATE; /* loaded ??? */
   force_end_of_sniff=1;
+  game.turn_start = time(NULL);
 }
 
 
@@ -1052,6 +1219,10 @@ void handle_packet_input(struct connection *pconn, char *packet, int type)
   case PACKET_UNIT_PARADROP_TO:
     handle_unit_paradrop_to(pplayer, (struct packet_unit_request *)packet);
     break;
+  case PACKET_PLAYER_CANCEL_PACT:
+    handle_player_cancel_pact(pplayer,
+			      ((struct packet_generic_integer *)packet)->value);
+    break;
   case PACKET_UNIT_CONNECT:
     handle_unit_connect(pplayer, (struct packet_unit_connect *)packet);
     break;
@@ -1103,6 +1274,9 @@ int check_for_full_turn_done(void)
 {
   int i;
 
+  /* fixedlength is only applicable if we have a timeout set */
+  if (game.fixedlength && game.timeout)
+    return 0;
   for(i=0; i<game.nplayers; i++) {
     if (game.turnblock) {
       if (!game.players[i].ai.control && game.players[i].is_alive &&
@@ -1349,7 +1523,7 @@ void accept_new_player(char *name, struct connection *pconn)
 {
   struct player *pplayer = &game.players[game.nplayers];
 
-  player_init(pplayer);
+  server_player_init(pplayer, 0);
   /* sometimes needed if players connect/disconnect to avoid
    * inheriting stale AI status etc
    */
@@ -1358,6 +1532,7 @@ void accept_new_player(char *name, struct connection *pconn)
   sz_strlcpy(pplayer->username, name);
   pplayer->conn = pconn;
   pplayer->is_connected = (pconn!=NULL);
+
   if (pconn) {
     sz_strlcpy(pplayer->addr, pconn->addr); 
     join_game_accept(pplayer, 0);
@@ -1441,6 +1616,15 @@ static void handle_request_join_game(struct connection *pconn,
   if( (pplayer=find_player_by_name(req->name)) || 
       (pplayer=find_player_by_user(req->name))     ) {
     if(!pplayer->conn) {
+      if(is_barbarian(pplayer)) {
+        reject_new_player(_("Sorry, you can't observe a barbarian."), pconn);
+        freelog(LOG_NORMAL,
+		_("No observation of barbarians - %s was rejected."),
+		req->name);
+        close_connection(pconn);
+        return;
+      }
+
       pplayer->conn=pconn;
       pplayer->is_connected=1;
       sz_strlcpy(pplayer->addr, pconn->addr); 
@@ -1452,6 +1636,9 @@ static void handle_request_join_game(struct connection *pconn,
         send_game_state(pplayer, CLIENT_GAME_RUNNING_STATE);
 	send_player_info(NULL,NULL);
       }
+      if (game.auto_ai_toggle && pplayer->ai.control) {
+	toggle_ai_player_direct(NULL, pplayer);
+      }
       return;
     }
 
@@ -1493,7 +1680,6 @@ static void handle_request_join_game(struct connection *pconn,
     reject_new_player(_("Sorry, you can't join.  The game is already running."),
 		      pconn);
     freelog(LOG_NORMAL, _("Game running - %s was rejected."), req->name);
-    lost_connection_to_player(pconn);
     close_connection(pconn);
 
     return;
@@ -1507,34 +1693,47 @@ static void handle_request_join_game(struct connection *pconn,
     return;
   }
 
-  accept_new_player(req->name, pconn);    
+  accept_new_player(req->name, pconn);
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-
 void lost_connection_to_player(struct connection *pconn)
 {
+  struct player *pplayer = NULL;
   int i;
 
-  for(i=0; i<game.nplayers; i++)
-    if(game.players[i].conn==pconn) {
-      game.players[i].conn=NULL;
-      game.players[i].is_connected=0;
-      sz_strlcpy(game.players[i].addr, "---.---.---.---");
-      freelog(LOG_NORMAL, _("Lost connection to %s."), game.players[i].name);
-      send_player_info(&game.players[i], 0);
-      notify_player(0, _("Game: Lost connection to %s."), game.players[i].name);
-
-      if(game.is_new_game && (server_state==PRE_GAME_STATE ||
-			 server_state==SELECT_RACES_STATE))
-	 server_remove_player(&game.players[i]);
-      check_for_full_turn_done();
-      return;
+  /* Find the player: */
+  for(i=0; i<game.nplayers; i++) {
+    if (game.players[i].conn == pconn) {
+      pplayer = &game.players[i];
+      break;
     }
+  }
+  if (pplayer == NULL) {
+    /* This happens eg if the player has not yet joined properly. */
+    freelog(LOG_FATAL, _("Lost connection to <unknown>."));
+    return;
+  }
+  
+  freelog(LOG_NORMAL, _("Lost connection to %s."), pplayer->name);
+  
+  pplayer->conn = NULL;
+  pplayer->is_connected = 0;
+  sz_strlcpy(pplayer->addr, "---.---.---.---");
 
-  freelog(LOG_FATAL, _("Lost connection to <unknown>."));
+  send_player_info(&game.players[i], 0);
+  notify_player(0, _("Game: Lost connection to %s."), pplayer->name);
+
+  if (game.is_new_game && (server_state==PRE_GAME_STATE ||
+			   server_state==SELECT_RACES_STATE)) {
+    server_remove_player(&game.players[i]);
+  }
+  else if (game.auto_ai_toggle && !pplayer->ai.control) {
+    toggle_ai_player_direct(NULL, pplayer);
+  }
+  check_for_full_turn_done();
 }
 
 /**************************************************************************
@@ -1607,6 +1806,13 @@ static void generate_ai_players(void)
              game.playable_nation_count);
   }
 
+  if (game.max_players < game.aifill) {
+    game.aifill = game.max_players;
+    freelog( LOG_NORMAL,
+	     _("Maxplayers smaller than aifill; aifill reduced to %d."),
+             game.max_players);
+  }
+
   for(;game.nplayers < game.aifill;) {
      nation = mark_nation_as_used(nations_avail[myrand(num_nations_avail)]);
      pick_ai_player_name(nation,player_name);
@@ -1637,9 +1843,9 @@ static void generate_ai_players(void)
 /*************************************************************************
  Used in pick_ai_player_name() below; buf has size at least MAX_LEN_NAME;
 *************************************************************************/
-static int good_name(char *try, char *buf) {
-  if (!find_player_by_name(try)) {
-     mystrlcpy(buf, try, MAX_LEN_NAME);
+static int good_name(char *ptry, char *buf) {
+  if (!find_player_by_name(ptry)) {
+     mystrlcpy(buf, ptry, MAX_LEN_NAME);
      return 1;
   }
   return 0;
@@ -1716,6 +1922,79 @@ static void announce_ai_player (struct player *pplayer) {
 
 }
 
+/*************************************************************************
+...
+*************************************************************************/
+static void enable_fog_of_war_player(struct player *pplayer)
+{
+  int x,y;
+  int playerid = pplayer->player_no;
+  struct tile *ptile;
+  for (x = 0; x < map.xsize; x++)
+    for (y = 0; y < map.ysize; y++) {
+      ptile = map_get_tile(x,y);
+      map_change_seen(x, y, playerid, -1);
+      if (map_get_seen(x, y, playerid) == 0)
+	update_player_tile_last_seen(pplayer, x, y);
+    }
+  send_all_known_tiles(pplayer);
+}
+
+/*************************************************************************
+...
+*************************************************************************/
+static void enable_fog_of_war(void)
+{
+  int o;
+  for (o = 0; o < game.nplayers; o++)
+    enable_fog_of_war_player(&game.players[o]);
+}
+
+/*************************************************************************
+...
+*************************************************************************/
+static void disable_fog_of_war_player(struct player *pplayer)
+{
+  int x,y;
+  int playerid = pplayer->player_no;
+  for (x = 0; x < map.xsize; x++) {
+    for (y = 0; y < map.ysize; y++) {
+      map_change_seen(x, y, playerid, +1);
+      if (map_get_known(x, y, pplayer)) {
+	struct city *pcity = map_get_city(x,y);
+	reality_check_city(pplayer, x, y);
+	if (pcity)
+	  update_dumb_city(pplayer, pcity);
+      }
+    }
+  }
+  send_all_known_tiles(pplayer);
+  send_all_known_units(pplayer);
+  send_all_known_cities(pplayer);
+}
+
+/*************************************************************************
+...
+*************************************************************************/
+static void disable_fog_of_war(void)
+{
+  int o;
+  for (o = 0; o < game.nplayers; o++)
+    disable_fog_of_war_player(&game.players[o]);
+}
+
+
+/********************************************************************** 
+The initmap option is used because we don't want to initialize the map
+before the x and y sizes have been determined
+***********************************************************************/
+void server_player_init(struct player *pplayer, int initmap)
+{
+  if (initmap)
+    player_map_allocate(pplayer);
+  player_init(pplayer);
+}
+
 #ifdef GENERATING_MAC
 /*************************************************************************
   generate an option dialog if no options have been passed in
