@@ -20,6 +20,7 @@
 #include <assert.h>
 
 #include "capability.h"
+#include "fcintl.h"
 #include "game.h"
 #include "log.h"
 #include "map.h"
@@ -37,6 +38,7 @@
 #include "plrhand.h"
 #include "ruleset.h"
 #include "unitfunc.h"
+#include "unittools.h"
 
 #include "gamehand.h"
 
@@ -52,6 +54,8 @@ diplchance_percent"
 void init_new_game(void)
 {
   int i, j, x, y;
+  int vx, vy, dx, dy;
+  Unit_Type_id utype;
   int start_pos[MAX_NUM_PLAYERS]; /* indices into map.start_positions[] */
   
   if (!map.fixed_start_positions) {
@@ -67,18 +71,16 @@ void init_new_game(void)
       map.start_positions[i].x=x;
       map.start_positions[i].y=y;
     }
-  }
+    for(i=0; i<game.nplayers; i++) {
+      start_pos[i] = i;
+    } 
+  } else {
   /* In a scenario, choose starting positions by nation.
      If there are too few starts for number of nations, assign
      to nations with specific starts first, then assign rest
      to random from remainder.  (Would be better to label start
      positions by nation etc, but this will do for now. --dwp)
   */
-  if(!map.fixed_start_positions) {
-    for(i=0; i<game.nplayers; i++) {
-      start_pos[i] = i;
-    } 
-  } else {
     const int npos = map.num_start_positions;
     int *pos_used = fc_calloc(npos, sizeof(int));
     int nrem = npos;		/* remaining unused starts */
@@ -111,30 +113,52 @@ void init_new_game(void)
     }
     free(pos_used);
   }
-  
-  for(i=0; i<game.nplayers; i++) {
-    x=map.start_positions[start_pos[i]].x;
-    y=map.start_positions[start_pos[i]].y;
-    /* For scenarios, huts may coincide with player starts;
-       remove any such hut: */
-    if(map_get_special(x,y)&S_HUT) {
-      map_clear_special(x,y,S_HUT);
-      freelog(LOG_VERBOSE, "Removed hut on start position for %s",
-	      game.players[i].name);
+
+  /* Loop over all players, creating their initial units... */
+  for (i = 0; i < game.nplayers; i++) {
+    /* Start positions are warranted to be land. */
+    x = map.start_positions[start_pos[i]].x;
+    y = map.start_positions[start_pos[i]].y;
+    /* Loop over all initial units... */
+    for (j = 0; j < (game.settlers + game.explorer); j++) {
+      /* Determine a place to put the unit within the dispersion area.
+         (Always put first unit on start position.) */
+      if ((game.dispersion <= 0) || (j == 0)) {
+	dx = x;
+	dy = y;
+      } else {
+	do {
+	  dx = x + myrand (2*game.dispersion+1) - game.dispersion;
+	  dy = y + myrand (2*game.dispersion+1) - game.dispersion;
+	  dx = map_adjust_x(dx);
+	} while (!(is_real_tile(dx, dy) &&
+		   map_same_continent(x, y, dx, dy) &&
+		   (map_get_terrain(dx, dy) != T_OCEAN) &&
+		   !is_non_allied_unit_tile(map_get_tile(dx, dy),
+					    game.players[i].player_no)));
+      }
+      /* For scenarios, huts may coincide with player starts;
+	 remove any such hut: */
+      if (map_get_special(dx, dy) & S_HUT) {
+        map_clear_special(dx, dy, S_HUT);
+        freelog(LOG_VERBOSE, "Removed hut on start position for %s",
+		game.players[i].name);
+      }
+      /* Expose visible area. */
+      for (vx = 1; (vx * vx) <= game.rgame.init_vis_radius_sq; vx++) {
+	for (vy = 1; (vy * vy) <= game.rgame.init_vis_radius_sq; vy++) {
+	  if (((vx *vx) + (vy *vy)) <= game.rgame.init_vis_radius_sq) {
+	    show_area(&game.players[i], dx-vx+1, dy-vy+1, 1);
+	    show_area(&game.players[i], dx+vx-1, dy-vy+1, 1);
+	    show_area(&game.players[i], dx-vx+1, dy+vy-1, 1);
+	    show_area(&game.players[i], dx+vx-1, dy+vy-1, 1);
+	  }
+	}
+      }
+      /* Create the unit of an appropriate type. */
+      utype = get_role_unit((j < game.settlers) ? F_CITIES : L_EXPLORER, 0);
+      create_unit(&game.players[i], dx, dy, utype, 0, 0, -1);
     }
-    /* Civ 1 exposes a single square radius of the map to start the game. */
-    /* Civ 2 exposes a "city radius". -AJS */
-    light_square(&game.players[i], x, y, 1);
-    if (game.civstyle==2) {
-      light_square(&game.players[i], x-1, y, 1);
-      light_square(&game.players[i], x+1, y, 1);
-      light_square(&game.players[i], x, y-1, 1);
-      light_square(&game.players[i], x, y+1, 1);
-    }
-    for (j=0;j<game.settlers;j++) 
-      create_unit(&game.players[i], x, y, get_role_unit(F_CITIES,0), 0, 0, -1);
-    for (j=0;j<game.explorer;j++) 
-      create_unit(&game.players[i], x, y, get_role_unit(L_EXPLORER,0), 0, 0, -1);
   }
 }
 
@@ -149,9 +173,11 @@ void send_year_to_clients(int year)
   apacket.year=year;
 
   for(i=0; i<game.nplayers; i++) {
-    game.players[i].turn_done=0;
-    game.players[i].nturns_idle++;
-    send_packet_new_year(game.players[i].conn, &apacket);
+    struct player *pplayer = &game.players[i];
+    pplayer->turn_done=0;
+    pplayer->nturns_idle++;
+    send_packet_new_year(pplayer->conn, &apacket);
+    notify_player(pplayer, _("Year: %s"), textyear(game.year));
   }
 }
 
@@ -195,6 +221,8 @@ void send_game_info(struct player *dest)
   ginfo.nplayers=game.nplayers;
   ginfo.globalwarming=game.globalwarming;
   ginfo.heating=game.heating;
+  ginfo.nuclearwinter=game.nuclearwinter;
+  ginfo.cooling=game.cooling;
   ginfo.techpenalty=game.techpenalty;
   ginfo.foodbox = game.foodbox;
   ginfo.civstyle=game.civstyle;
@@ -203,6 +231,10 @@ void send_game_info(struct player *dest)
     ginfo.global_advances[i]=game.global_advances[i];
   for(i=0; i<B_LAST; i++)
     ginfo.global_wonders[i]=game.global_wonders[i];
+  /* the following values are computed every
+     time a packet_game_info packet is created */
+  if (game.timeout)
+    ginfo.seconds_to_turndone=game.turn_start + game.timeout - time(NULL);
 
   for(o=0; o<game.nplayers; o++)           /* dests */
     if(!dest || &game.players[o]==dest) {
@@ -304,6 +336,8 @@ void game_load(struct section_file *file)
   game.nplayers      = secfile_lookup_int(file, "game.nplayers");
   game.globalwarming = secfile_lookup_int(file, "game.globalwarming");
   game.warminglevel  = secfile_lookup_int(file, "game.warminglevel");
+  game.nuclearwinter = secfile_lookup_int_default(file, 0, "game.nuclearwinter");
+  game.coolinglevel  = secfile_lookup_int_default(file, 8, "game.coolinglevel");
   game.unhappysize   = secfile_lookup_int(file, "game.unhappysize");
 
   if (game.version >= 10100) {
@@ -325,10 +359,13 @@ void game_load(struct section_file *file)
     section_file_lookup(file, "game.farmfood");
   }
   if (game.version >= 10300) {
-    game.civstyle = secfile_lookup_int(file, "game.civstyle");
+    game.civstyle = secfile_lookup_int_default(file, 0, "game.civstyle");
     game.save_nturns = secfile_lookup_int(file, "game.save_nturns");
   }
 
+  game.fogofwar = secfile_lookup_int_default(file, 0, "game.fogofwar");
+  game.fogofwar_old = game.fogofwar;
+
   if ((game.version == 10604 && section_file_lookup(file,"savefile.options"))
       || (game.version > 10604)) 
     savefile_options = secfile_lookup_str(file,"savefile.options");
@@ -360,12 +397,18 @@ void game_load(struct section_file *file)
   }
   game.aqueductloss = secfile_lookup_int_default(file, game.aqueductloss,
 						 "game.aqueductloss");
+  game.killcitizen = secfile_lookup_int_default(file, game.killcitizen,
+						 "game.killcitizen");
   game.turnblock = secfile_lookup_int_default(file,game.turnblock,
-                         "game.turnblock");
+					      "game.turnblock");
+  game.fixedlength = secfile_lookup_int_default(file,game.fixedlength,
+						"game.fixedlength");
   game.barbarianrate = secfile_lookup_int_default(file, game.barbarianrate,
 						  "game.barbarians");
   game.onsetbarbarian = secfile_lookup_int_default(file, game.onsetbarbarian,
 						   "game.onsetbarbs");
+  game.nbarbarians = 0; /* counted in player_load for compatibility with 
+			   1.10.0 */
   game.occupychance = secfile_lookup_int_default(file, game.occupychance,
   						 "game.occupychance");
   
@@ -405,6 +448,15 @@ void game_load(struct section_file *file)
 					"game.ruleset.nations"));
   sz_strlcpy(game.ruleset.cities,
 	     secfile_lookup_str_default(file, "default", "game.ruleset.cities"));
+  if(game.civstyle == 1) {
+    string = "civ1";
+  } else {
+    string = "default";
+    game.civstyle = GAME_DEFAULT_CIVSTYLE;
+  }
+  sz_strlcpy(game.ruleset.game,
+	     secfile_lookup_str_default(file, string,
+					"game.ruleset.game"));
 
   sz_strlcpy(game.demography,
 	     secfile_lookup_str_default(file, GAME_DEFAULT_DEMOGRAPHY,
@@ -413,19 +465,26 @@ void game_load(struct section_file *file)
   game.spacerace = secfile_lookup_int_default(file, game.spacerace,
 					      "game.spacerace");
 
+  game.auto_ai_toggle = secfile_lookup_int_default(file, game.auto_ai_toggle,
+						   "game.auto_ai_toggle");
+
   game.heating=0;
+  game.cooling=0;
   if(tmp_server_state==PRE_GAME_STATE 
      || has_capability("startoptions", savefile_options)) {
     if (game.version >= 10300) {
       game.settlers = secfile_lookup_int(file, "game.settlers");
       game.explorer = secfile_lookup_int(file, "game.explorer");
+      game.dispersion =
+	secfile_lookup_int_default(file, GAME_DEFAULT_DISPERSION, "game.dispersion");
 
       map.riches = secfile_lookup_int(file, "map.riches");
       map.huts = secfile_lookup_int(file, "map.huts");
       map.generator = secfile_lookup_int(file, "map.generator");
       map.seed = secfile_lookup_int(file, "map.seed");
       map.landpercent = secfile_lookup_int(file, "map.landpercent");
-      map.grasssize = secfile_lookup_int_default(file, MAP_DEFAULT_GRASS, "map.grasssize");
+      map.grasssize =
+	secfile_lookup_int_default(file, MAP_DEFAULT_GRASS, "map.grasssize");
       map.swampsize = secfile_lookup_int(file, "map.swampsize");
       map.deserts = secfile_lookup_int(file, "map.deserts");
       map.riverlength = secfile_lookup_int(file, "map.riverlength");
@@ -486,7 +545,17 @@ void game_load(struct section_file *file)
   for(i=0; i<game.nplayers; i++) {
     player_load(&game.players[i], i, file); 
   }
-  
+  /* Since the cities must be placed on the map to put them on the
+     player map we do this afterwards */
+  for(i=0; i<game.nplayers; i++) {
+    player_map_load(&game.players[i], i, file); 
+  }
+  for(i=0; i<game.nplayers; i++) {
+    struct player *pplayer = get_player(i);
+    if (pplayer->ai.control)
+      neutralize_ai_player(pplayer);
+  }  
+
   initialize_globals();
   apply_unit_ordering();
 
@@ -554,10 +623,13 @@ void game_save(struct section_file *file)
   secfile_insert_int(file, game.nplayers, "game.nplayers");
   secfile_insert_int(file, game.globalwarming, "game.globalwarming");
   secfile_insert_int(file, game.warminglevel, "game.warminglevel");
+  secfile_insert_int(file, game.nuclearwinter, "game.nuclearwinter");
+  secfile_insert_int(file, game.coolinglevel, "game.coolinglevel");
   secfile_insert_int(file, game.unhappysize, "game.unhappysize");
   secfile_insert_int(file, game.cityfactor, "game.cityfactor");
   secfile_insert_int(file, game.civilwarsize, "game.civilwarsize");
   secfile_insert_int(file, game.diplcost, "game.diplcost");
+  secfile_insert_int(file, game.fogofwar, "game.fogofwar");
   secfile_insert_int(file, game.freecost, "game.freecost");
   secfile_insert_int(file, game.conquercost, "game.conquercost");
   secfile_insert_int(file, game.foodbox, "game.foodbox");
@@ -568,9 +640,12 @@ void game_save(struct section_file *file)
   secfile_insert_int(file, game.aifill, "game.aifill");
   secfile_insert_int(file, game.scorelog, "game.scorelog");
   secfile_insert_int(file, game.spacerace, "game.spacerace");
+  secfile_insert_int(file, game.auto_ai_toggle, "game.auto_ai_toggle");
   secfile_insert_int(file, game.diplchance, "game.diplchance");
   secfile_insert_int(file, game.aqueductloss, "game.aqueductloss");
+  secfile_insert_int(file, game.killcitizen, "game.killcitizen");
   secfile_insert_int(file, game.turnblock, "game.turnblock");
+  secfile_insert_int(file, game.fixedlength, "game.fixedlength");
   secfile_insert_int(file, game.barbarianrate, "game.barbarians");
   secfile_insert_int(file, game.onsetbarbarian, "game.onsetbarbs");
   secfile_insert_int(file, game.occupychance, "game.occupychance");
@@ -581,6 +656,7 @@ void game_save(struct section_file *file)
   secfile_insert_str(file, game.ruleset.governments, "game.ruleset.governments");
   secfile_insert_str(file, game.ruleset.nations, "game.ruleset.nations");
   secfile_insert_str(file, game.ruleset.cities, "game.ruleset.cities");
+  secfile_insert_str(file, game.ruleset.game, "game.ruleset.game");
   secfile_insert_str(file, game.demography, "game.demography");
 
   if (1) {
@@ -593,6 +669,7 @@ void game_save(struct section_file *file)
     secfile_insert_int(file, map.ysize, "map.height");
     secfile_insert_int(file, game.settlers, "game.settlers");
     secfile_insert_int(file, game.explorer, "game.explorer");
+    secfile_insert_int(file, game.dispersion, "game.dispersion");
     secfile_insert_int(file, map.seed, "map.seed");
     secfile_insert_int(file, map.landpercent, "map.landpercent");
     secfile_insert_int(file, map.riches, "map.riches");
@@ -649,3 +726,5 @@ void game_save(struct section_file *file)
   map_save(file);
 
 }
+
+
