@@ -20,6 +20,7 @@ struct packet_player_request;
 struct section_file;
 
 void server_remove_player(struct player *pplayer);
+void begin_player_turn(struct player *pplayer);
 void update_player_activities(struct player *pplayer);
 
 void handle_player_revolution(struct player *pplayer);
@@ -28,6 +29,10 @@ void handle_player_ai_options(struct player *pplayer,
 void handle_player_rates(struct player *pplayer, 
 			 struct packet_player_request *preq);
 void check_player_government_rates(struct player *pplayer);
+void handle_player_cancel_pact(struct player *pplayer, int other_player);
+void make_contact(int player1, int player2, int x, int y);
+void maybe_make_first_contact(int x, int y, int playerid);
+void neutralize_ai_player(struct player *pplayer);
 
 void send_player_info(struct player *src, struct player *dest);
 
@@ -64,12 +69,15 @@ int choose_goal_tech(struct player *plr);
 void wonders_of_the_world(struct player *pplayer);
 void demographics_report(struct player *pplayer);
 void top_five_cities(struct player *pplayer);
+void send_player_turn_notifications(struct player *pplayer);
 void make_history_report(void);
 void do_dipl_cost(struct player *pplayer);
 void do_free_cost(struct player *pplayer);
 void do_conquer_cost(struct player *pplayer);
 void show_ending(void);
+void show_map_to_all(void);
 void player_load(struct player *plr, int plrno, struct section_file *file);
+void player_map_load(struct player *plr, int plrno, struct section_file *file);
 void player_save(struct player *plr, int plrno, struct section_file *file);
 
 #endif  /* FC__PLRHAND_H */
