@@ -57,7 +57,14 @@
 #include <arpa/inet.h>
 #endif
 
+#ifdef HAVE_LIBREADLINE
+#include <readline/readline.h>
+#include <readline/history.h>
+#endif
+
 #include "log.h"
+#include "mem.h"
+#include "netintf.h"
 #include "packets.h"
 #include "shared.h"
 #include "support.h"
@@ -68,8 +75,6 @@
 
 #include "sernet.h"
 
-#define MY_FD_ZERO(p) memset((void *)(p), 0, sizeof(*(p)))
-
 struct connection connections[MAX_NUM_CONNECTIONS];
 
 #ifdef GENERATING_MAC      /* mac network globals */
@@ -81,8 +86,48 @@ extern int port;
 #endif
 extern int force_end_of_sniff;
 
+#ifdef __VMS
+#include <descrip.h>
+#include <iodef.h>
+#include <stsdef.h>
+#include <starlet.h>
+#include <lib$routines.h>
+#include <efndef.h>
+static unsigned long int tt_chan;
+static char input_char = 0;
+static char got_input = 0;
+void user_interrupt_callback();
+#endif
 
-/*****************************************************************************/
+
+#ifdef HAVE_LIBREADLINE
+/****************************************************************************/
+
+#define HISTORY_FILENAME  ".civserver_history"
+#define HISTORY_LENGTH    100
+
+static char *history_file = NULL;
+
+static int readline_handled_input = 0;
+
+/*****************************************************************************
+...
+*****************************************************************************/
+static void handle_readline_input_callback(char *line)
+{
+  if (line) {
+    if (*line)
+      add_history(line);
+    con_prompt_enter();		/* just got an 'Enter' hit */
+    handle_stdin_input((struct player *)NULL, line);
+  }
+  readline_handled_input = 1;
+}
+
+#endif
+/*****************************************************************************
+...
+*****************************************************************************/
 void close_connection(struct connection *pconn)
 {
   close(pconn->sock);
@@ -90,7 +135,9 @@ void close_connection(struct connection *pconn)
   pconn->access_level=ALLOW_NONE;
 }
 
-/*****************************************************************************/
+/*****************************************************************************
+...
+*****************************************************************************/
 void close_connections_and_socket(void)
 {
   int i;
@@ -101,6 +148,22 @@ void close_connections_and_socket(void)
     }
   }
   close(sock);
+
+#ifdef HAVE_LIBREADLINE
+  if (history_file) {
+    write_history(history_file);
+    history_truncate_file(history_file, HISTORY_LENGTH);
+  }
+#endif
+}
+
+/*****************************************************************************
+...
+*****************************************************************************/
+static void close_socket_callback(struct connection *pc)
+{
+  lost_connection_to_player(pc);
+  close_connection(pc);
 }
 
 /*****************************************************************************
@@ -119,19 +182,47 @@ int sniff_packets(void)
 {
   int i;
   int max_desc;
-  fd_set readfs;
+  fd_set readfs, exceptfs;
   struct timeval tv;
-  static time_t time_at_turn_end;
   static int year;
 #ifdef SOCKET_ZERO_ISNT_STDIN
   char buf[BUF_SIZE+1];
   char *bufptr = buf;
 #endif
-  
+
+  con_prompt_init();
+
+#ifdef HAVE_LIBREADLINE
+  {
+    static int readline_initialized = 0;
+
+    if (!readline_initialized) {
+      char *home_dir = user_home_dir();
+      if (home_dir) {
+	history_file =
+	  fc_malloc(strlen(home_dir) + 1 + strlen(HISTORY_FILENAME) + 1);
+	if (history_file) {
+	  strcpy(history_file, home_dir);
+	  strcat(history_file, "/");
+	  strcat(history_file, HISTORY_FILENAME);
+	  using_history();
+	  read_history(history_file);
+	}
+      }
+
+      rl_initialize();
+      rl_callback_handler_install("> ", handle_readline_input_callback);
+
+      readline_initialized = 1;
+    }
+  }
+#endif
+
   if(year!=game.year) {
-    time_at_turn_end = time(NULL) + game.timeout;
     if (server_state == RUN_GAME_STATE) year=game.year;
   }
+  if (!game.timeout)
+    game.turn_start = time(NULL);
   
   while(1) {
     con_prompt_on();		/* accepting new input */
@@ -142,42 +233,81 @@ int sniff_packets(void)
       return 2;
     }
     
-    tv.tv_sec=1; tv.tv_usec=0;
+    tv.tv_sec=1;
+    tv.tv_usec=0;
     
     MY_FD_ZERO(&readfs);
+    MY_FD_ZERO(&exceptfs);
+#ifndef SOCKET_ZERO_ISNT_STDIN
     FD_SET(0, &readfs);	
+#endif
     FD_SET(sock, &readfs);
+    FD_SET(sock, &exceptfs);
     max_desc=sock;
     
     for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
       if(connections[i].used) {
 	FD_SET(connections[i].sock, &readfs);
+	FD_SET(connections[i].sock, &exceptfs);
+        max_desc=MAX(connections[i].sock, max_desc);
       }
-      max_desc=MAX(connections[i].sock, max_desc);
     }
     con_prompt_off();		/* output doesn't generate a new prompt */
     
-    if(select(max_desc+1, &readfs, NULL, NULL, &tv)==0) { /* timeout */
+    if(select(max_desc+1, &readfs, NULL, &exceptfs, &tv)==0) { /* timeout */
       send_server_info_to_metaserver(0,0);
       if((game.timeout) 
-	&& (time(NULL)>time_at_turn_end)
+	&& (time(NULL)>game.turn_start + game.timeout)
 	&& (server_state == RUN_GAME_STATE)){
 	con_prompt_off();
 	return 0;
       }
+#ifdef __VMS
+      {
+	struct { short numchars; char firstchar; char reserved; int reserved2; } ttchar;
+	unsigned long status;
+	status = sys$qiow(EFN$C_ENF,tt_chan,IO$_SENSEMODE|IO$M_TYPEAHDCNT,0,0,0,
+			  &ttchar,sizeof(ttchar),0,0,0,0);
+	if (!$VMS_STATUS_SUCCESS(status)) lib$stop(status);
+	if (ttchar.numchars)
+	  FD_SET(0, &readfs);
+	else
+	  continue;
+      }
+#else
 #ifdef SOCKET_ZERO_ISNT_STDIN
     if (feof(stdin))
 #endif
       continue;
+#endif
+    }
+    if (!game.timeout)
+      game.turn_start = time(NULL);
+
+    if(FD_ISSET(sock, &exceptfs)) {	     /* handle Ctrl-Z suspend/resume */
+      continue;
     }
-  
     if(FD_ISSET(sock, &readfs)) {	     /* new players connects */
       freelog(LOG_VERBOSE, "got new connection");
       if(server_accept_connection(sock)==-1)
 	freelog(LOG_NORMAL, "failed accepting connection");
     }
+    for(i=0; i<MAX_NUM_CONNECTIONS; i++) {   /* check for freaky players */
+      if(connections[i].used && FD_ISSET(connections[i].sock, &exceptfs)) {
+	freelog(LOG_VERBOSE, "cut freaky player");
+	close_socket_callback(&connections[i]);
+      }
+    }
 #ifndef SOCKET_ZERO_ISNT_STDIN
-    else if(FD_ISSET(0, &readfs)) {    /* input from server operator */
+    if(FD_ISSET(0, &readfs)) {    /* input from server operator */
+#ifdef HAVE_LIBREADLINE
+      rl_callback_read_char();
+      if (readline_handled_input) {
+	readline_handled_input = 0;
+	con_prompt_enter_clear();
+      }
+      continue;
+#else
       int didget;
       char buf[BUF_SIZE+1];
       
@@ -188,9 +318,10 @@ int sniff_packets(void)
       *(buf+didget)='\0';
       con_prompt_enter();	/* will need a new prompt, regardless */
       handle_stdin_input((struct player *)NULL, buf);
+#endif
     }
 #else
-    else if(!feof(stdin)) {    /* input from server operator */
+    if(!feof(stdin)) {    /* input from server operator */
       /* fetch chars until \n or run out of space in buffer */
       while ((*bufptr=fgetc(stdin)) != EOF) {
           if (*bufptr == '\n') *bufptr = '\0';
@@ -202,31 +333,31 @@ int sniff_packets(void)
           }
           if ((bufptr-buf) <= BUF_SIZE) bufptr++; /* prevent overrun */
       }
-  }
+    }
 #endif
     else {                             /* input from a player */
-      for(i=0; i<MAX_NUM_CONNECTIONS; i++)
+      for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
 	if(connections[i].used && FD_ISSET(connections[i].sock, &readfs)) {
 	  if(read_socket_data(connections[i].sock, 
-			      &connections[i].buffer)>0) {
+			      &connections[i].buffer)>=0) {
 	    char *packet;
 	    int type;
 	    while((packet=get_packet_from_connection(&connections[i], &type)))
 	      handle_packet_input(&connections[i], packet, type);
 	  }
 	  else {
-	    lost_connection_to_player(&connections[i]);
-	    close_connection(&connections[i]);
+	    close_socket_callback(&connections[i]);
 	  }
 	}
+      }
     }
     break;
   }
   con_prompt_off();
-  
+
   if((game.timeout) 
-    && (time(NULL)>time_at_turn_end)
-    && (game.timeout)) return 0;
+     && (time(NULL)>game.turn_start + game.timeout))
+    return 0;
   return 1;
 }
   
@@ -240,36 +371,41 @@ int server_accept_connection(int sockfd)
   int new_sock;
   struct sockaddr_in fromend;
   struct hostent *from;
+  int i;
 
-  fromlen = sizeof fromend;
+  fromlen = sizeof(fromend);
 
-  new_sock = accept(sockfd, (struct sockaddr *) &fromend, &fromlen);
-
-  from=gethostbyaddr((char*)&fromend.sin_addr, sizeof(sizeof(fromend.sin_addr)), 
-		     AF_INET);
-
-  if(new_sock!=-1) {
-    int i;
-    for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
-      if(!connections[i].used) {
-	connections[i].used=1;
-	connections[i].sock=new_sock;
-	connections[i].player=NULL;
-	connections[i].buffer.ndata=0;
-	connections[i].first_packet=1;
-	connections[i].byte_swap=0;
-	connections[i].access_level=default_access_level;
-	
-	sz_strlcpy(connections[i].addr, (from ? from->h_name : "unknown"));
-	
-	freelog(LOG_VERBOSE, "connection from %s", connections[i].addr);
-	return 0;
-      }
-    }
-    freelog(LOG_FATAL, "maximum number of connections reached");
+  if ((new_sock=accept(sockfd, (struct sockaddr *) &fromend, &fromlen)) == -1) {
+    freelog(LOG_VERBOSE, "accept failed: %s", mystrerror(errno));
     return -1;
   }
 
+  my_nonblock(new_sock);
+
+  from=gethostbyaddr((char*)&fromend.sin_addr, sizeof(fromend.sin_addr),
+		     AF_INET);
+
+  for(i=0; i<MAX_NUM_CONNECTIONS; i++) {
+    if(!connections[i].used) {
+      connections[i].used=1;
+      connections[i].sock=new_sock;
+      connections[i].player=NULL;
+      connections[i].buffer.ndata=0;
+      connections[i].first_packet=1;
+      connections[i].byte_swap=0;
+      connections[i].access_level=(i?default_access_level:first_access_level);
+
+      if (from)
+	sz_strlcpy(connections[i].addr, from->h_name);
+      else
+	sz_strlcpy(connections[i].addr, inet_ntoa(fromend.sin_addr));
+
+      freelog(LOG_VERBOSE, "connection from %s", connections[i].addr);
+      return 0;
+    }
+  }
+
+  freelog(LOG_FATAL, "maximum number of connections reached");
   return -1;
 }
 
@@ -284,38 +420,39 @@ int server_open_socket(void)
   struct sockaddr_in src;
   int opt;
 
-  src.sin_addr.s_addr = INADDR_ANY;
-  src.sin_family = AF_INET;
-  src.sin_port = htons(port);
-
-
   /* broken pipes are ignored. */
 #ifdef HAVE_SIGPIPE
   signal (SIGPIPE, SIG_IGN);
 #endif
 
-  if((sock = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+  if((sock = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
     freelog(LOG_FATAL, "socket failed: %s", mystrerror(errno));
     exit(1);
   }
 
   opt=1; 
   if(setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, 
-		(char*)&opt, sizeof(opt))) {
-/*		(const char*)&opt, sizeof(opt))) {      gave me warnings -- Syela */
-    freelog(LOG_FATAL, "setsockopt failed: %s", mystrerror(errno));
+		(char*)&opt, sizeof(opt)) == -1) {
+    freelog(LOG_FATAL, "SO_REUSEADDR failed: %s", mystrerror(errno));
   }
 
-  if(bind(sock, (struct sockaddr *) &src, sizeof (src)) < 0) {
+  memset(&src, 0, sizeof(src));
+  src.sin_family = AF_INET;
+  src.sin_addr.s_addr = htonl(INADDR_ANY);
+  src.sin_port = htons(port);
+
+
+  if(bind(sock, (struct sockaddr *) &src, sizeof (src)) == -1) {
     freelog(LOG_FATAL, "bind failed: %s", mystrerror(errno));
     exit(1);
   }
 
-  if(listen(sock, MAX_NUM_CONNECTIONS) < 0) {
+  if(listen(sock, MAX_NUM_CONNECTIONS) == -1) {
     freelog(LOG_FATAL, "listen failed: %s", mystrerror(errno));
     exit(1);
   }
 
+  close_socket_set_callback(close_socket_callback);
   return 0;
 }
 
@@ -330,4 +467,12 @@ void init_connections(void)
     connections[i].used=0;
     connections[i].buffer.ndata=0;
   }
+#ifdef __VMS
+  {
+    unsigned long status;
+    $DESCRIPTOR (tt_desc, "SYS$INPUT");
+    status = sys$assign(&tt_desc,&tt_chan,0,0);
+    if (!$VMS_STATUS_SUCCESS(status)) lib$stop(status);
+  }
+#endif
 }
