@@ -13,9 +13,9 @@
 #ifndef FC__SERNET_H
 #define FC__SERNET_H
 
-#include "packets.h"
+struct connection;
 
-#define MAX_NUM_CONNECTIONS 16
+#define MAX_NUM_CONNECTIONS (MAX_NUM_PLAYERS+MAX_NUM_BARBARIANS)
 #define DEFAULT_SOCK_PORT 5555
 #define BUF_SIZE 512
 
