@@ -14,6 +14,7 @@
 #include <string.h>
 #include <assert.h>
 
+#include "city.h"
 #include "game.h"
 #include "government.h"
 #include "log.h"
@@ -37,8 +38,8 @@
 
 extern struct move_cost_map warmap;
 
-signed short int minimap[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
-static unsigned short int territory[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
+signed int minimap[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
+static unsigned int territory[MAP_MAX_WIDTH][MAP_MAX_HEIGHT];
 /* negative: in_city_radius, 0: unassigned, positive: city_des */
 
 /*************************************************************************/
@@ -49,6 +50,8 @@ static void auto_settlers_player(struct player *pplayer);
 static int is_already_assigned(struct unit *myunit, struct player *pplayer, 
 			       int x, int y);
 
+static int city_exists_within_city_radius(int x, int y);
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -66,9 +69,10 @@ const char *get_a_name(struct player *pplayer)
 **************************************************************************/
 static int ai_do_build_city(struct player *pplayer, struct unit *punit)
 {
-  int x,y;
+  int x, y;
   struct packet_unit_request req;
   struct city *pcity;
+  int i, j;
   req.unit_id=punit->id;
   sz_strlcpy(req.name, get_a_name(pplayer));
   x = punit->x; y = punit->y; /* Trevor Pering points out that punit gets freed */
@@ -76,41 +80,57 @@ static int ai_do_build_city(struct player *pplayer, struct unit *punit)
   pcity=map_get_city(x, y); /* so we need to cache x and y for a very short time */
   if (!pcity)
     return 0;
-  for (y=0;y<5;y++)
-    for (x=0;x<5;x++) {
-      if ((x==0 || x==4) && (y==0 || y==4)) 
-        continue;
-      light_square(pplayer, x+pcity->x-2, y+pcity->y-2, 0);
-    }
+
+  /* initialize infrastructure cache for both this city and other cities
+     nearby. This is neccesary to avoid having settlers want to transform
+     a city into the ocean. */
+  city_map_iterate(i, j) {
+    int x_itr = map_adjust_x(x+i-2);
+    int y_itr = map_adjust_y(y+j-2);
+    struct city *pcity2 = map_get_city(x_itr, y_itr);
+    if (pcity2 && city_owner(pcity2) == pplayer)
+      initialize_infrastructure_cache(pcity2);
+  }
   return 1;
 }
 
-int amortize(int b, int d)
+/**************************************************************************
+amortize(benefit, delay) returns benefit * ((MORT - 1)/MORT)^delay
+(^ = to the power of)
+
+Plus, it has tests to prevent the numbers getting too big.  It takes
+advantage of the fact that (23/24)^12 approximately = 3/5 to chug through
+delay in chunks of 12, and then does the remaining multiplications of (23/24).
+**************************************************************************/
+int amortize(int benefit, int delay)
 {
   int num = MORT - 1;
   int denom;
   int s = 1;
-  assert(d >= 0);
-  if (b < 0) { s = -1; b *= s; }
-  while (d && b) {
+  assert(delay >= 0);
+  if (benefit < 0) { s = -1; benefit *= s; }
+  while (delay && benefit) {
     denom = 1;
-    while (d >= 12 && !(b>>28) && !(denom>>27)) {
-      b *= 3;          /* this is a kluge but it is 99.9% accurate and saves time */
+    while (delay >= 12 && !(benefit>>28) && !(denom>>27)) {
+      benefit *= 3;          /* this is a kluge but it is 99.9% accurate and saves time */
       denom *= 5;      /* as long as MORT remains 24! -- Syela */
-      d -= 12;
+      delay -= 12;
     }
-    while (!(b>>25) && d && !(denom>>25)) {
-      b *= num;
+    while (!(benefit>>25) && delay && !(denom>>25)) {
+      benefit *= num;
       denom *= MORT;
-      d--;
+      delay--;
     }
-    if (denom > 1) {
-      b = (b + (denom/2)) / denom;
+    if (denom > 1) { /* The "+ (denom/2)" makes the rounding correct */
+      benefit = (benefit + (denom/2)) / denom;
     }
   }
-  return(b * s);
+  return(benefit * s);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 void generate_minimap(void)
 {
   int a, i, j;
@@ -125,6 +145,9 @@ void generate_minimap(void)
   }
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 void remove_city_from_minimap(int x, int y)
 {
   int i, j, n, xx;
@@ -145,6 +168,9 @@ void remove_city_from_minimap(int x, int y)
   }
 }    
 
+/**************************************************************************
+...
+**************************************************************************/
 void add_city_to_minimap(int x, int y)
 {
   int i, j, n, xx;
@@ -165,6 +191,9 @@ void add_city_to_minimap(int x, int y)
   }
 }    
 
+/**************************************************************************
+...
+**************************************************************************/
 void locally_zero_minimap(int x, int y)
 {
   int i, j, n, xx;
@@ -180,6 +209,9 @@ void locally_zero_minimap(int x, int y)
   }
 }  
 
+/**************************************************************************
+...
+**************************************************************************/
 int city_desirability(struct player *pplayer, int x, int y)
 { /* this whole funct assumes G_REP^H^H^HDEMOCRACY -- Syela */
   int taken[5][5], food[5][5], shield[5][5], trade[5][5];
@@ -426,7 +458,7 @@ int make_dx(int x1, int x2)
  return 1 if there is already a unit on this square or one destined for it 
  (via goto)
 **************************************************************************/
-int is_already_assigned(struct unit *myunit, struct player *pplayer, int x, int y)
+static int is_already_assigned(struct unit *myunit, struct player *pplayer, int x, int y)
 { 
   x=map_adjust_x(x);
   y=map_adjust_y(y);
@@ -445,40 +477,7 @@ int is_already_assigned(struct unit *myunit, struct player *pplayer, int x, int
   return(map_get_tile(x, y)->assigned & (1<<pplayer->player_no));
 }
 
-#ifdef UNUSED
-int old_is_already_assigned(struct unit *myunit, struct player *pplayer, int x, int y)
-{
-  x=map_adjust_x(x);
-  y=map_adjust_y(y);
-#ifdef RIDICULOUS
-  if (same_pos(myunit->x, myunit->y, x, y))
-    return 0;
-#endif
-  unit_list_iterate(map_get_tile(x, y)->units, punit) {
-    if (myunit==punit) continue;
-    if (punit->owner!=pplayer->player_no)
-      return 1;
-    if (unit_flag(punit->type, F_SETTLERS) && unit_flag(myunit->type, F_SETTLERS))
-      return 1;
-  }
-  unit_list_iterate_end;
-#ifndef RIDICULOUS
-  if (same_pos(myunit->x, myunit->y, x, y))
-    return 0;
-#endif
-  unit_list_iterate(pplayer->units, punit) {
-    if (myunit==punit) continue;
-    if (punit->owner!=pplayer->player_no)
-      return 1;
-    if (same_pos(punit->goto_dest_x, punit->goto_dest_y, x, y) && unit_flag(punit->type, F_SETTLERS) && unit_flag(myunit->type,F_SETTLERS) && punit->activity==ACTIVITY_GOTO)
-      return 1;
-    if (same_pos(punit->goto_dest_x, punit->goto_dest_y, x, y) && !unit_flag(myunit->type, F_SETTLERS) && punit->activity==ACTIVITY_GOTO)
-      return 1;
-  }
-  unit_list_iterate_end;
-  return 0;
-}
-#endif
+/*************************************************************************/
 
 /* all of the benefit and ai_calc routines rewritten by Syela */
 /* to conform with city_tile_value and related calculations elsewhere */
@@ -486,7 +485,11 @@ int old_is_already_assigned(struct unit *myunit, struct player *pplayer, int x,
 /* I don't really want to rewrite them and possibly screw them up. */
 /* The cache should keep the CPU increase linear instead of quadratic. -- Syela */
 
-int ai_calc_pollution(struct city *pcity, struct player *pplayer, int i, int j)
+/**************************************************************************
+...
+**************************************************************************/
+static int ai_calc_pollution(struct city *pcity, struct player *pplayer,
+			     int i, int j, int best)
 {
   int x, y, m;
   x = pcity->x + i - 2; y = pcity->y + j - 2;
@@ -494,9 +497,30 @@ int ai_calc_pollution(struct city *pcity, struct player *pplayer, int i, int j)
   map_clear_special(x, y, S_POLLUTION);
   m = city_tile_value(pcity, i, j, 0, 0);
   map_set_special(x, y, S_POLLUTION);
+  m = (m + best + 50) * 2;
   return(m);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
+static int ai_calc_fallout(struct city *pcity, struct player *pplayer,
+			   int i, int j, int best)
+{
+  int x, y, m;
+  x = pcity->x + i - 2; y = pcity->y + j - 2;
+  if (!(map_get_special(x, y) & S_FALLOUT)) return(-1);
+  map_clear_special(x, y, S_FALLOUT);
+  m = city_tile_value(pcity, i, j, 0, 0);
+  map_set_special(x, y, S_FALLOUT);
+  if (!pplayer->ai.control)
+    m = (m + best + 50) * 2;
+  return(m);
+}
+
+/**************************************************************************
+...
+**************************************************************************/
 static int is_wet(struct player *pplayer, int x, int y)
 {
   enum tile_terrain_type t;
@@ -511,7 +535,11 @@ static int is_wet(struct player *pplayer, int x, int y)
   return 0;
 }
 
-int ai_calc_irrigate(struct city *pcity, struct player *pplayer, int i, int j)
+/**************************************************************************
+...
+**************************************************************************/
+static int ai_calc_irrigate(struct city *pcity, struct player *pplayer,
+			    int i, int j)
 {
   int m, x = pcity->x + i - 2, y = pcity->y + j - 2;
   struct tile *ptile = map_get_tile(x, y);
@@ -521,6 +549,8 @@ int ai_calc_irrigate(struct city *pcity, struct player *pplayer, int i, int j)
 
   if (ptile->terrain != type->irrigation_result &&
       type->irrigation_result != T_LAST) { /* EXPERIMENTAL 980905 -- Syela */
+    if (ptile->city && type->irrigation_result == T_OCEAN)
+      return -1;
     ptile->terrain = type->irrigation_result;
     map_clear_special(x, y, S_MINE);
     m = city_tile_value(pcity, i, j, 0, 0);
@@ -549,7 +579,11 @@ int ai_calc_irrigate(struct city *pcity, struct player *pplayer, int i, int j)
   } else return(-1);
 }
 
-int ai_calc_mine(struct city *pcity, struct player *pplayer, int i, int j)
+/**************************************************************************
+...
+**************************************************************************/
+static int ai_calc_mine(struct city *pcity, struct player *pplayer,
+			int i, int j)
 {
   int m, x = pcity->x + i - 2, y = pcity->y + j - 2;
   struct tile *ptile = map_get_tile(x, y);
@@ -569,6 +603,7 @@ int ai_calc_mine(struct city *pcity, struct player *pplayer, int i, int j)
     return(m);
   } else 
 #endif
+    /* Note that this code means we will never try to mine a city into the ocean */
   if ((ptile->terrain == T_HILLS || ptile->terrain == T_MOUNTAINS) &&
       !(ptile->special&S_IRRIGATION) && !(ptile->special&S_MINE)) {
     map_set_special(x, y, S_MINE);
@@ -578,7 +613,11 @@ int ai_calc_mine(struct city *pcity, struct player *pplayer, int i, int j)
   } else return(-1);
 }
 
-int ai_calc_transform(struct city *pcity, struct player *pplayer, int i, int j)
+/**************************************************************************
+...
+**************************************************************************/
+static int ai_calc_transform(struct city *pcity, struct player *pplayer,
+			     int i, int j)
 {
   int m, x = pcity->x + i - 2, y = pcity->y + j - 2;
   struct tile *ptile = map_get_tile(x, y);
@@ -590,6 +629,9 @@ int ai_calc_transform(struct city *pcity, struct player *pplayer, int i, int j)
   
   if ((t == T_ARCTIC || t == T_DESERT || t == T_JUNGLE || t == T_SWAMP  || 
        t == T_TUNDRA || t == T_MOUNTAINS) && r != T_LAST) {
+    if (r == T_OCEAN && ptile->city)
+      return -1;
+
     ptile->terrain = r;
 
     if (get_tile_type(r)->mining_result != r) 
@@ -607,6 +649,9 @@ int ai_calc_transform(struct city *pcity, struct player *pplayer, int i, int j)
   } else return(-1);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 static int road_bonus(int x, int y, int spc)
 {
   int m = 0, k;
@@ -642,7 +687,11 @@ static int road_bonus(int x, int y, int spc)
   return(m);
 }
 
-int ai_calc_road(struct city *pcity, struct player *pplayer, int i, int j)
+/**************************************************************************
+...
+**************************************************************************/
+static int ai_calc_road(struct city *pcity, struct player *pplayer,
+			int i, int j)
 {
   int x, y, m;
   struct tile *ptile;
@@ -659,7 +708,11 @@ int ai_calc_road(struct city *pcity, struct player *pplayer, int i, int j)
   } else return(-1);
 }
 
-int ai_calc_railroad(struct city *pcity, struct player *pplayer, int i, int j)
+/**************************************************************************
+...
+**************************************************************************/
+static int ai_calc_railroad(struct city *pcity, struct player *pplayer,
+			    int i, int j)
 {
   int x, y, m;
   struct tile *ptile;
@@ -718,10 +771,14 @@ int is_ok_city_spot(int x, int y)
       if (map_distance(x, y, pcity->x, pcity->y)<=8) {
         dx=make_dx(pcity->x, x);
         dy=make_dy(pcity->y, y);
+	/* these are heuristics... */
         if (dx<=5 && dy<5)
           return 0;
         if (dx<5 && dy<=5)
           return 0;
+	/* this is the law... */
+	if (dx<game.rgame.min_dist_bw_cities && dy<game.rgame.min_dist_bw_cities)
+          return 0;
       }
     }
     city_list_iterate_end;
@@ -730,7 +787,7 @@ int is_ok_city_spot(int x, int y)
 }
 
 /*************************************************************************
-  return the city if any that is in range of this square.
+  return true if a city is in range of this square.
 **************************************************************************/
 int in_city_radius(int x, int y)
 {
@@ -749,11 +806,14 @@ int auto_settler_do_goto(struct player *pplayer, struct unit *punit, int x, int
   punit->goto_dest_x=map_adjust_x(x);
   punit->goto_dest_y=map_adjust_y(y);
   set_unit_activity(punit, ACTIVITY_GOTO);
-  send_unit_info(0, punit, 0);
+  send_unit_info(0, punit);
   do_unit_goto(pplayer, punit, GOTO_MOVE_ANY);
   return 1;
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 int find_boat(struct player *pplayer, int *x, int *y, int cap)
 { /* this function uses the current warmap, whatever it may hold */
 /* unit is no longer an arg!  we just trust the map! -- Syela */
@@ -762,7 +822,7 @@ int find_boat(struct player *pplayer, int *x, int *y, int cap)
     if (is_ground_units_transport(aunit)) {
       if (warmap.cost[aunit->x][aunit->y] < best &&
           (warmap.cost[aunit->x][aunit->y] == 0 ||
-          is_transporter_with_free_space(pplayer, aunit->x, aunit->y) >= cap)) {
+          ground_unit_transporter_capacity(aunit->x, aunit->y, pplayer->player_no) >= cap)) {
         id = aunit->id;
         best = warmap.cost[aunit->x][aunit->y];
         *x = aunit->x;
@@ -776,7 +836,7 @@ int find_boat(struct player *pplayer, int *x, int *y, int cap)
     if (pcity->is_building_unit &&
         unit_types[pcity->currently_building].transport_capacity &&
         !unit_flag(pcity->currently_building, F_CARRIER) &&
-	!unit_flag(pcity->currently_building, F_SUBMARINE)) {
+	!unit_flag(pcity->currently_building, F_MISSILE_CARRIER)) {
       if (warmap.cost[pcity->x][pcity->y] < best) {
         id = pcity->id;
         best = warmap.cost[pcity->x][pcity->y];
@@ -789,6 +849,9 @@ int find_boat(struct player *pplayer, int *x, int *y, int cap)
   return(id);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 struct unit *other_passengers(struct unit *punit)
 {
   unit_list_iterate(map_get_tile(punit->x, punit->y)->units, aunit)
@@ -797,10 +860,51 @@ struct unit *other_passengers(struct unit *punit)
   return 0;
 }
 
-/********************************************************************
+/**************************************************************************
+...
+**************************************************************************/
+static void consider_settler_action(struct player *pplayer, enum unit_activity act,
+				    int extra, int newv, int oldv, int in_use,
+				    int d, int *best_newv, int *best_oldv,
+				    int *best_act, int *gx, int *gy, int x, int y)
+{
+  int a, b=0;
+  int consider;
+
+  if (extra >= 0) {
+    consider = (newv >= 0);
+  } else {
+    consider = (newv >= oldv);
+  }
+
+  if (consider) {
+    if (extra >= 0) {
+      newv = MAX(newv, oldv) + extra;
+    }
+    /* give squares which can be improved and are currently in use a bonus */
+    b = MAX((newv - oldv)*((in_use) ? 64:32), MORT);
+    a = amortize(b, d);
+    newv = ((a * b) / (MAX(1, b - a)))/64;
+  } else {
+    newv = 0;
+  }
+
+  if (newv > *best_newv || (newv == *best_newv && oldv > *best_oldv)) {
+    freelog(LOG_DEBUG,
+	    "Replacing (%d, %d) = %d with %s (%d, %d) = %d [d=%d b=%d]",
+	    *gx, *gy, *best_newv, get_activity_text(act), x, y, newv, d, b);
+    *best_newv = newv;
+    *best_oldv = oldv;
+    *best_act = act;
+    *gx = x;
+    *gy = y;
+  }
+}
+
+/**************************************************************************
   find some work for the settler
-*********************************************************************/
-int auto_settler_findwork(struct player *pplayer, struct unit *punit)
+**************************************************************************/
+static int auto_settler_findwork(struct player *pplayer, struct unit *punit)
 {
   struct city *mycity = map_get_city(punit->x, punit->y);
   int nav_known       = (get_invention(pplayer, game.rtech.nav) == TECH_KNOWN);
@@ -817,6 +921,8 @@ int auto_settler_findwork(struct player *pplayer, struct unit *punit)
   int best_oldv = 9999;		/* oldv of best target so far; compared if
 				   newv==best_newv; not initialized to zero,
 				   so that newv=0 activities are not chosen */
+  int in_use;			/* true if the target square is being used
+				   by one of our cities */
   int best_act = 0;		/* ACTIVITY_ of best target so far */
   int food_upkeep;		/* upkeep food value for single settler  */
   int food_cost;		/* estimated food cost to produce settler */
@@ -825,12 +931,15 @@ int auto_settler_findwork(struct player *pplayer, struct unit *punit)
   struct unit *ferryboat;	/* if non-null, boatid boat at unit's x,y */
   
   int x, y, i, j;
-  int a, b=0, d=0;
-  int worst;
-  int save_newv;		/* for debugging only */
-  
+  int b=0, d=0;
+
   struct ai_choice choice;	/* for nav want only */
 
+#ifdef DEBUG
+  /* for debugging only */
+  int save_newv;
+#endif
+
   choice.type = 1;
   choice.want = 0;		/* will change as needed */
   choice.choice = get_role_unit(L_FERRYBOAT, -1);
@@ -857,10 +966,11 @@ int auto_settler_findwork(struct player *pplayer, struct unit *punit)
      AI settlers improving enemy cities.  arguably should include city_spot  */
   generate_warmap(mycity, punit);
   city_list_iterate(pplayer->cities, pcity)
-    worst = worst_worker_tile_value(pcity);
+    freelog(LOG_DEBUG, "%s", pcity->name);
     /* try to work near the city */
     city_map_iterate_outwards(i, j)
       if (get_worker_city(pcity, i, j) == C_TILE_UNAVAILABLE) continue;
+      in_use = (get_worker_city(pcity, i, j) == C_TILE_WORKER);
       x = map_adjust_x(pcity->x + i - 2);
       y = pcity->y + j - 2;	/* No adjust! */
       if ((y >= 0 && y < map.ysize)
@@ -874,130 +984,81 @@ int auto_settler_findwork(struct player *pplayer, struct unit *punit)
 	   was but subroutines are not -- Syela	*/
 	mv_turns = (warmap.cost[x][y]) / mv_rate;
         oldv = city_tile_value(pcity, i, j, 0, 0);
-        if (worst > oldv && (i != 2 || j != 2)) oldv = worst;
-	/* perhaps I should just subtract worst rather than oldv but
-	   I prefer this -- Syela */
 
-/* Christopher Neufeld provided a rather crufty patch to make settlers
-needlessly idle less often.  I didn't want to apply it, but the ideas
-for the -1 return values in ai_calc_*, the !irrigation in calc_mine,
-and the prioritization of useless (b <= 0) activities are his. -- Syela
-*/
+	/* now, consider various activities... */
 
-	newv = pcity->ai.irrigate[i][j];
-        if (newv >= oldv) { /* worth evaluating */
-          b = MAX((newv - oldv)*64, MORT);
-          d = (map_build_irrigation_time(x, y) * 3 + mv_rate - 1) / mv_rate
-	    + mv_turns;
-          a = amortize(b, d);
-          newv = ((a * b) / (MAX(1, b - a)))/64;
-        } else newv = 0;
-        if ((newv > best_newv || (newv == best_newv && oldv > best_oldv))
-	    && ai_fuzzy(pplayer, 1)) {
-	  freelog(LOG_DEBUG,
-		  "Replacing (%d, %d) = %d with (%d, %d) I=%d d=%d, b=%d",
-		  gx, gy, best_newv, x, y, newv, d, b);
-	  best_act = ACTIVITY_IRRIGATE;
-	  best_newv=newv; gx=x; gy=y; best_oldv=oldv;
-        }
+	d = (map_build_irrigation_time(x, y)*3 + mv_rate - 1)/mv_rate +
+	  mv_turns;
+	consider_settler_action(pplayer, ACTIVITY_IRRIGATE, -1,
+				pcity->ai.irrigate[i][j], oldv, in_use, d,
+				&best_newv, &best_oldv, &best_act, &gx, &gy,
+				x, y);
 
-	newv = pcity->ai.transform[i][j];
-        if (newv >= oldv) { /* worth evaluating */
-          b = MAX((newv - oldv)*64, MORT);
-          d = (map_transform_time(x, y) * 3 + mv_rate - 1) / mv_rate
-	    + mv_turns;
-          a = amortize(b, d);
-          newv = ((a * b) / (MAX(1, b - a)))/64;
-        } else newv = 0;
-        if ((newv > best_newv || (newv == best_newv && oldv > best_oldv))
-	    && ai_fuzzy(pplayer, 1)) {
-	  freelog(LOG_DEBUG,
-		  "Replacing (%d, %d) = %d with (%d, %d) I=%d d=%d, b=%d",
-		  gx, gy, best_newv, x, y, newv, d, b);
-	  best_act = ACTIVITY_TRANSFORM;
-	  best_newv=newv; gx=x; gy=y; best_oldv=oldv;
-        }
+	if (unit_flag(punit->type, F_TRANSFORM)) {
+	  d = (map_transform_time(x, y)*3 + mv_rate - 1)/mv_rate +
+	    mv_turns;
+	  consider_settler_action(pplayer, ACTIVITY_TRANSFORM, -1,
+				  pcity->ai.transform[i][j], oldv, in_use, d,
+				  &best_newv, &best_oldv, &best_act, &gx, &gy,
+				  x, y);
+	}
 
-	newv = pcity->ai.mine[i][j];
-        if (newv >= oldv) {
-          b = MAX((newv - oldv)*64, MORT);
-          d = (map_build_mine_time(x, y) * 3 + mv_rate - 1) / mv_rate
-	    + mv_turns;
-          a = amortize(b, d);
-          newv = ((a * b) / (MAX(1, b - a)))/64;
-        } else newv = 0;
-	if ((newv > best_newv || (newv == best_newv && oldv > best_oldv))
-	    && ai_fuzzy(pplayer, 1)) {
-	  freelog(LOG_DEBUG,
-		  "Replacing (%d, %d) = %d with (%d, %d) M=%d d=%d, b=%d",
-		  gx, gy, best_newv, x, y, newv, d, b);
-	  best_act = ACTIVITY_MINE;
-	  best_newv=newv; gx=x; gy=y; best_oldv=oldv;
-        }
+	d = (map_build_mine_time(x, y)*3 + mv_rate - 1)/mv_rate +
+	  mv_turns;
+	consider_settler_action(pplayer, ACTIVITY_MINE, -1,
+				pcity->ai.mine[i][j], oldv, in_use, d,
+				&best_newv, &best_oldv, &best_act, &gx, &gy,
+				x, y);
 
         if (!(map_get_tile(x,y)->special&S_ROAD)) {
-  	  newv = pcity->ai.road[i][j];
-          if (newv >= 0) {    
-            newv = MAX(newv, oldv) + road_bonus(x, y, S_ROAD) * 8;
-	    /* guessing about the weighting based on unit upkeeps;
-	       (* 11) was too high! -- Syela */
-            b = MAX((newv - oldv)*64, MORT);
-            d = (map_build_road_time(x, y) * 3 + 3 + mv_rate - 1) / mv_rate
-	      + mv_turns; /* uniquely weird! */
-            a = amortize(b, d);
-            newv = ((a * b) / (MAX(1, b - a)))/64;
-          } else newv = 0;
-          if ((newv > best_newv || (newv == best_newv && oldv > best_oldv))
-	      && ai_fuzzy(pplayer, 1)) {
-	    freelog(LOG_DEBUG, "Replacing (%d, %d) = %d"
-		    " with (%d, %d) R=%d d=%d, b=%d",
-		    gx, gy, best_newv, x, y, newv, d, b);
-	    best_act = ACTIVITY_ROAD;
-	    best_newv=newv; gx=x; gy=y; best_oldv=oldv;
-          }
+	  d = (map_build_road_time(x, y)*3 + 3 + mv_rate - 1)/mv_rate +
+	    mv_turns;
+	  consider_settler_action(pplayer, ACTIVITY_ROAD,
+				  road_bonus(x, y, S_ROAD) * 8,
+				  pcity->ai.road[i][j], oldv, in_use, d,
+				  &best_newv, &best_oldv, &best_act, &gx, &gy,
+				  x, y);
 
-	  newv = pcity->ai.railroad[i][j];
-          if (newv >= 0) {    
-            newv = MAX(newv, oldv) + road_bonus(x, y, S_RAILROAD) * 4;
-            b = MAX((newv - oldv)*64, MORT);
-            d = (3 * 3 + 3 * map_build_road_time(x,y) + 3 + mv_rate - 1)
-	         / mv_rate + mv_turns;
-            a = amortize(b, d);
-            newv = ((a * b) / (MAX(1, b - a)))/64;
-          } else newv = 0;
-          if ((newv > best_newv || (newv == best_newv && oldv > best_oldv))
-	      && ai_fuzzy(pplayer, 1)) {
-  	    best_act = ACTIVITY_ROAD;
-	    best_newv=newv; gx=x; gy=y; best_oldv=oldv;
-          }
+	  d = (3*3 + 3*map_build_road_time(x,y) + 3 + mv_rate - 1)/mv_rate +
+	    mv_turns;
+	  consider_settler_action(pplayer, ACTIVITY_ROAD,
+				  road_bonus(x, y, S_RAILROAD) * 4,
+				  pcity->ai.railroad[i][j], oldv, in_use, d,
+				  &best_newv, &best_oldv, &best_act, &gx, &gy,
+				  x, y);
         } else {
-	  newv = pcity->ai.railroad[i][j];
-          if (newv >= 0) {    
-            newv = MAX(newv, oldv) + road_bonus(x, y, S_RAILROAD) * 4;
-            b = MAX((newv - oldv)*64, MORT);
-            d = (3 * 3 + mv_rate - 1) / mv_rate + mv_turns;
-            a = amortize(b, d);
-            newv = ((a * b) / (MAX(1, b - a)))/64;
-          } else newv = 0;
-          if ((newv > best_newv || (newv == best_newv && oldv > best_oldv))
-	      && ai_fuzzy(pplayer, 1)) {
-  	    best_act = ACTIVITY_RAILROAD;
-	    best_newv=newv; gx=x; gy=y; best_oldv=oldv;
-          }
+	  d = (3*3 + mv_rate - 1)/mv_rate +
+	    mv_turns;
+	  consider_settler_action(pplayer, ACTIVITY_RAILROAD,
+				  road_bonus(x, y, S_RAILROAD) * 4,
+				  pcity->ai.railroad[i][j], oldv, in_use, d,
+				  &best_newv, &best_oldv, &best_act, &gx, &gy,
+				  x, y);
         } /* end S_ROAD else */
 
-	newv = pcity->ai.detox[i][j];
-        if (newv >= 0) {
-          newv = MAX(newv, oldv) + pplayer->ai.warmth;
-          b = MAX((newv - oldv)*64, MORT);
-          d = (3 * 3 + mv_turns + mv_rate - 1) / mv_rate + mv_turns;
-          a = amortize(b, d);
-          newv = ((a * b) / (MAX(1, b - a)))/64;
-        } else newv = 0;
-        if (newv > best_newv || (newv == best_newv && oldv > best_oldv)) {
-	  best_act = ACTIVITY_POLLUTION;
-	  best_newv=newv; gx=x; gy=y; best_oldv=oldv;
-        }
+	d = (3*3 + mv_rate - 1)/mv_rate +
+	  mv_turns;
+	consider_settler_action(pplayer, ACTIVITY_POLLUTION,
+				pplayer->ai.warmth,
+				pcity->ai.detox[i][j], oldv, in_use, d,
+				&best_newv, &best_oldv, &best_act, &gx, &gy,
+				x, y);
+
+	d = (3*3 + mv_rate - 1)/mv_rate +
+	  mv_turns;
+	consider_settler_action(pplayer, ACTIVITY_FALLOUT,
+				pplayer->ai.warmth,
+				pcity->ai.derad[i][j], oldv, in_use, d,
+				&best_newv, &best_oldv, &best_act, &gx, &gy,
+				x, y);
+
+	freelog(LOG_DEBUG,
+		"(%d %d) I=%+-4d O=%+-4d M=%+-4d R=%+-4d RR=%+-4d P=%+-4d N=%+-4d",
+		i, j,
+		pcity->ai.irrigate[i][j], pcity->ai.transform[i][j],
+		pcity->ai.mine[i][j], pcity->ai.road[i][j],
+		pcity->ai.railroad[i][j], pcity->ai.detox[i][j],
+		pcity->ai.derad[i][j]);
       } /* end if we are a legal destination */
     city_map_iterate_outwards_end;
   city_list_iterate_end;
@@ -1011,12 +1072,18 @@ and the prioritization of useless (b <= 0) activities are his. -- Syela
 
   if (best_newv > 0) {
     freelog(LOG_DEBUG,
-	    "Settler %d@(%d,%d) wants to %d at (%d,%d) with desire %d",
-	    punit->id, punit->x, punit->y, best_act, gx, gy, best_newv);
+	    "Settler %d@(%d,%d) wants to %s at (%d,%d) with desire %d",
+	    punit->id, punit->x, punit->y, get_activity_text(best_act),
+	    gx, gy, best_newv);
   }
+#ifdef DEBUG
   save_newv = best_newv;
-  
-  boatid = find_boat(pplayer, &bx, &by, 1); /* might need 2 for body */
+#endif
+
+  if (pplayer->ai.control)
+    boatid = find_boat(pplayer, &bx, &by, 1); /* might need 2 for body */
+  else
+    boatid = 0;
   ferryboat = unit_list_find(&(map_get_tile(punit->x, punit->y)->units), boatid);
   if (ferryboat)
     really_generate_warmap(mycity, ferryboat, SEA_MOVING);
@@ -1043,7 +1110,9 @@ and the prioritization of useless (b <= 0) activities are his. -- Syela
 	    && map_get_terrain(x, y) != T_OCEAN
 	    && (territory[x][y]&(1<<player_num))
 				/* pretty good, hope it's enough! -- Syela */
-            && (near < 8 || map_get_continent(x, y) != ucont)) {
+            && (near < 8 || map_get_continent(x, y) != ucont)
+	    && city_can_be_built_here(x,y)
+	    && !city_exists_within_city_radius(x,y)) {
 
 	  /* potential target, calculate mv_cost: */
           if (ferryboat) {
@@ -1113,7 +1182,7 @@ improving those tiles, and then immigrating shortly thereafter. -- Syela
           if (map_get_continent(x, y) != ucont && !nav_known && near >= 8) {
             if (newv > choice.want && !punit->id) choice.want = newv;
 	    freelog(LOG_DEBUG,
-		    "%s@(%d, %d) city_des (%d, %d) = %d, newv = %d, d = %d", 
+		    "%s @(%d, %d) city_des (%d, %d) = %d, newv = %d, d = %d", 
 		    (punit->id ? unit_types[punit->type].name : mycity->name), 
 		    punit->x, punit->y, x, y, b, newv, d);
           } else if (newv > best_newv) {
@@ -1134,14 +1203,12 @@ improving those tiles, and then immigrating shortly thereafter. -- Syela
   if (choice.want > 0) ai_choose_ferryboat(pplayer, mycity, &choice);
 
 #ifdef DEBUG
-  if (best_newv != save_newv) {
+  if ((best_newv != save_newv) ||
+      (map_get_terrain(punit->x, punit->y) == T_OCEAN)) {
     freelog(LOG_DEBUG,
-	    "Settler %d@(%d,%d) wants to %d at (%d,%d) with desire %d",
-	    punit->id, punit->x, punit->y, best_act, gx, gy, best_newv);
-  }
-  if (map_get_terrain(punit->x, punit->y) == T_OCEAN) {
-    freelog(LOG_DEBUG, "Punit %d@(%d,%d) wants to %d at (%d,%d) with desire %d",
-	    punit->id, punit->x, punit->y, best_act, gx, gy, best_newv);
+	    "%s %d@(%d,%d) wants to %s at (%d,%d) with desire %d",
+	    unit_name(punit->type), punit->id, punit->x, punit->y,
+	    get_activity_text(best_act), gx, gy, best_newv);
   }
 #endif
   
@@ -1161,9 +1228,9 @@ improving those tiles, and then immigrating shortly thereafter. -- Syela
   if (!punit->id) {
     /* has to be before we try to send_unit_info()! -- Syela */
     freelog(LOG_DEBUG,
-	    "%s (%d, %d) settler-want = %d, task = %d, target = (%d, %d)",
+	    "%s (%d, %d) settler-want = %d, task = %s, target = (%d, %d)",
 	    mycity->name, mycity->x, mycity->y, best_newv,
-	    best_act, gx, gy);
+	    get_activity_text(best_act), gx, gy);
     if (gx < 0 && gy < 0) {
       return(0 - best_newv);
     } else {
@@ -1192,7 +1259,7 @@ improving those tiles, and then immigrating shortly thereafter. -- Syela
 		      punit->id, punit->x, punit->y);
         if (!same_pos(x, y, punit->x, punit->y)) {
           auto_settler_do_goto(pplayer, punit, x, y);
-          if (!unit_list_find(&pplayer->units, save_id)) return(0); /* died */
+          if (!player_find_unit_by_id(pplayer, save_id)) return(0); /* died */
         }
         ferryboat = unit_list_find(&(map_get_tile(punit->x, punit->y)->units),
 				   punit->ai.ferryboat);
@@ -1217,16 +1284,18 @@ improving those tiles, and then immigrating shortly thereafter. -- Syela
 		  && could_unit_move_to_tile(punit, punit->x, punit->y,
 					     gx, gy)))) {
         auto_settler_do_goto(pplayer, punit, gx, gy);
-        if (!unit_list_find(&pplayer->units, save_id)) return(0); /* died */
+        if (!player_find_unit_by_id(pplayer, save_id)) return(0); /* died */
         punit->ai.ferryboat = 0;
       }
     }
   } else punit->ai.control=0;
+ /* The above line makes non-AI autosettlers go off auto When they run
+    out of squares to improve. I would like keep them on, prepared for
+    future pollution and warming, but there wasn't consensus to do so. */
 
   if (punit->ai.control
       && punit->moves_left
       && punit->activity == ACTIVITY_IDLE) {
-    /* if players can build with 0 moves left, so should the AI */
     if (same_pos(gx, gy, punit->x, punit->y)) {
       if (best_act == ACTIVITY_UNKNOWN) {
         remove_city_from_minimap(gx, gy); /* yeah, I know. -- Syela */
@@ -1234,19 +1303,24 @@ improving those tiles, and then immigrating shortly thereafter. -- Syela
         return(0);
       }
       set_unit_activity(punit, best_act);
-      send_unit_info(0, punit, 0);
+      send_unit_info(0, punit);
       return(0);
     }
   }
   return(0);
 }
 
+/**************************************************************************
+...
+**************************************************************************/
 void initialize_infrastructure_cache(struct city *pcity)
 {
   int i, j;
   struct player *pplayer = &game.players[pcity->owner];
+  int best = best_worker_tile_value(pcity);
   city_map_iterate(i, j) {
-    pcity->ai.detox[i][j] = ai_calc_pollution(pcity, pplayer, i, j);
+    pcity->ai.detox[i][j] = ai_calc_pollution(pcity, pplayer, i, j, best);
+    pcity->ai.derad[i][j] = ai_calc_fallout(pcity, pplayer, i, j, best);
     pcity->ai.mine[i][j] = ai_calc_mine(pcity, pplayer, i, j);
     pcity->ai.irrigate[i][j] = ai_calc_irrigate(pcity, pplayer, i, j);
     pcity->ai.transform[i][j] = ai_calc_transform(pcity, pplayer, i, j);
@@ -1261,7 +1335,7 @@ as punits arrive at adjacent tiles and start laying road -- Syela */
   run through all the players settlers and let those on ai.control work 
   automagically
 **************************************************************************/
-void auto_settlers_player(struct player *pplayer) 
+static void auto_settlers_player(struct player *pplayer) 
 {
   static struct timer *t = NULL;      /* alloc once, never free */
 
@@ -1306,11 +1380,11 @@ static void assign_settlers_player(struct player *pplayer)
         ptile->assigned = ptile->assigned | i; /* assigned for us only */
       } else {
         ptile = map_get_tile(punit->x, punit->y);
-        ptile->assigned = 32767; /* assigned for everyone */
+        ptile->assigned = 0xFFFFFFFF; /* assigned for everyone */
       }
     } else {
       ptile = map_get_tile(punit->x, punit->y);
-      ptile->assigned = ptile->assigned | (32767 ^ i); /* assigned for everyone else */
+      ptile->assigned = ptile->assigned | (0xFFFFFFFF ^ i); /* assigned for everyone else */
     }
   unit_list_iterate_end;
 }
@@ -1327,17 +1401,20 @@ static void assign_settlers(void)
   }
 }
 
-static void assign_region(int x, int y, int n, int d, int s)
+static void assign_region(int x, int y, int player_no, int distance, int s)
 {
   int i, j;
-  for (j = MAX(0, y - d); j <= MIN(map.ysize - 1, y + d); j++) {
-    for (i = x - d; i <= x + d; i++) {
+  for (j = MAX(0, y - distance); j <= MIN(map.ysize - 1, y + distance); j++) {
+    for (i = x - distance; i <= x + distance; i++) {
       if (!s || is_terrain_near_tile(i, j, T_OCEAN))
-        territory[map_adjust_x(i)][j] &= (1<<n);
+        territory[map_adjust_x(i)][j] &= (1<<player_no);
     }
   }
 }
 
+/**************************************************************************
+FIXME: we currently see even allies as neemies here in this routine
+**************************************************************************/
 static void assign_territory_player(struct player *pplayer)
 {
   int n = pplayer->player_no;
@@ -1368,7 +1445,7 @@ static void assign_territory(void)
   int i, x, y;
   for (x = 0; x < map.xsize; x++)
     for (y = 0; y < map.ysize; y++)
-      territory[x][y] = 65535;
+      territory[x][y] = 0xFFFFFFFF;
 
   for (i = 0; i < game.nplayers; i++) {
     assign_territory_player(&game.players[i]);
@@ -1399,6 +1476,9 @@ void contemplate_settling(struct player *pplayer, struct city *pcity)
 
   memset(&virtualunit, 0, sizeof(struct unit));
   virtualunit.id = 0;
+  /* note virtual unit is not added to unit lists (eg pplayer->units),
+     so equivalently don't need to call idex_register_unit()  --dwp
+  */
   virtualunit.owner = pplayer->player_no;
   virtualunit.x = pcity->x;
   virtualunit.y = pcity->y;
@@ -1429,3 +1509,22 @@ void contemplate_settling(struct player *pplayer, struct city *pcity)
     pcity->ai.founder_want = want;
   }
 }
+
+/**************************************************************************
+Return true iff a city exists within a city radius of the given location.
+(Not including a city at the given location.  I.e., if only city within
+radius is at (x,y), then returns false.)
+**************************************************************************/
+static int city_exists_within_city_radius(int x, int y)
+{
+  int dx, dy;
+
+  city_radius_iterate(dx, dy) {
+    if (!(dx==0 && dy==0)) {
+      if (map_get_city(x+dx, y+dy))
+	return 1;
+    }
+  }
+
+  return 0;
+}
