@@ -13,7 +13,9 @@
 #ifndef FC__SETTLERS_H
 #define FC__SETTLERS_H
 
-#include "unit.h"
+struct player;
+struct unit;
+struct city;
 
 int auto_settler_do_goto(struct player *pplayer, struct unit *punit, 
 			 int x, int y);
@@ -23,16 +25,10 @@ int find_boat(struct player *pplayer, int *x, int *y, int cap);
 #define MORT 24
 
 const char *get_a_name(struct player *pplayer);
-int amortize(int b, int d);
+int amortize(int benefit, int delay);
 int city_desirability(struct player *pplayer, int x, int y);
 void ai_manage_settler(struct player *pplayer, struct unit *punit);
 
-int ai_calc_pollution(struct city *pcity, struct player *pplayer, int i, int j);
-int ai_calc_mine(struct city *pcity, struct player *pplayer, int i, int j);
-int ai_calc_road(struct city *pcity, struct player *pplayer, int i, int j);
-int ai_calc_railroad(struct city *pcity, struct player *pplayer, int i, int j);
-int ai_calc_irrigate(struct city *pcity, struct player *pplayer, int i, int j);
-int ai_calc_transform(struct city *pcity, struct player *pplayer, int i, int j);
 int in_city_radius(int x, int y);
 int is_ok_city_spot(int x, int y); /* laughable, really. */
 int make_dy(int y1, int y2);
